/**
 * Accordion is a class to create an accordion widget
 * @param {Boolean} The boolean value true or false is to define whether all children have the same width or height
 * @base gtk.Widget
 * @constructor
 */
gtk.Accordion = function(homogeneous){
    gtk.Widget.apply(this);
    
    this._minWidth  = this._width  = 200;
    this._minHeight = this._height = 30;
    this._accordionHeaderHeight = 22;
    this._children = [];
    this._homogeneous = homogeneous || true;
    this._activeIndex = null;
    
    /** A registered signal name 'activated' */
    this.ACCORDION_ACTIVATED = '';
    /** A registered signal name 'deactivated' */
    this.ACCORDION_DEACTIVATED = '';
    
    this._render = function(){
        this._domObj.className += " gtk_accordion";
    };
    this._render();
};

gtk.Accordion.prototype = new gtk.Widget();
gtk.Accordion.prototype.constructor = gtk.Accordion;

/**
 * Method to add or append content to to each tab of accordion
 * @param {String} title The title of the accordion tab
 * @param {Object} childWidget The DOM object of the child content
 */
gtk.Accordion.prototype.appendChild = function(title, childWidget){
    childWidget.setParent(this);
    
    var childObject = this._createChildObject({"title": title, "childWidget": childWidget});
    this._children.push(childObject);
    
    this._appendAccordionNodeHolder(childObject["accordionNodeHolder"]);
    
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
    
    this.activate(0);
};

/**
 * Method to set deactivation unto any tab of accordion
 * @param {Integer} index The index value of the accordion tab
 */
gtk.Accordion.prototype.deActivate = function(index){
    this._activeIndex = null;
    for(var i = 0, len = this._children.length; i < len; i++){
        if(i === index) {
            jsgtk.Dom.hideElement(this._children[index]["childNodeHolder"]);
            var imgNode = this._children[index]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_collapseimg";
            this.emit("deactivated", index);
        }
    }
};

/**
 * Method to set activation unto any tab of accordion
 * @param {Integer} The index value of the accordion tab
 */
gtk.Accordion.prototype.activate = function(index){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(i === index) {
            jsgtk.Dom.showElement(this._children[index]["childNodeHolder"]);
            var imgNode = this._children[index]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_expandimg";
            this._activeIndex = index;
            this.emit("activated", index);
        }
        else {
            jsgtk.Dom.hideElement(this._children[i]["childNodeHolder"]);
            var imgNode = this._children[i]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_collapseimg";
            this._activeIndex = null;
        }
    }
};

/** @private */
gtk.Accordion.prototype._getAccordionIndex = function(headerNodeHolder){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["headerNodeHolder"] === headerNodeHolder) return i;
    }
};

/** @private */
gtk.Accordion.prototype._createAccordionNodeHolder = function(headerNodeHolder, childNodeHolder){
    var accordionNodeHolder = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_node_holder"});
    accordionNodeHolder.appendChild(headerNodeHolder);
    accordionNodeHolder.appendChild(childNodeHolder);
    
    return accordionNodeHolder;
}

/** @private */
gtk.Accordion.prototype._createHeaderNodeHolder = function(title){
    var headerNode = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_header"});
        var titleNode = jsgtk.Dom.create({"append": headerNode, "text": title,"tag": "div", "className": "gtk_accordion_title"});
        var imgNode = jsgtk.Dom.create({"append": headerNode, "tag": "span", "className": "gtk_accordion_collapseimg"});
      
    this._initEvent(headerNode);
    
    return headerNode;
};

/** @private */
gtk.Accordion.prototype._createChildNodeHolder = function(childWidget){
    var childNodeHolder = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_child_node_holder"});
    childNodeHolder.appendChild(childWidget.getNode());
    
    return childNodeHolder;
};

/** @private */
gtk.Accordion.prototype._createChildObject = function(childProperties){
    var headerNodeHolder    = this._createHeaderNodeHolder(childProperties["title"]);
    var childNodeHolder     = this._createChildNodeHolder(childProperties["childWidget"]);
    var accordionNodeHolder = this._createAccordionNodeHolder(headerNodeHolder, childNodeHolder);
    
    var childObject = {
        "accordionNodeHolder": accordionNodeHolder,
        "headerNodeHolder": headerNodeHolder,
        "childNodeHolder": childNodeHolder,
        "childWidget": childProperties["childWidget"]
    };
    
    return childObject;
};

/** @private */
gtk.Accordion.prototype._appendAccordionNodeHolder = function(accordionNodeHolder){
    this._domObj.appendChild(accordionNodeHolder);
};

/** @private */
gtk.Accordion.prototype._getAccordionsHeaderHeight = function(){
    return this._children.length * this._accordionHeaderHeight;
};

/** @private */
gtk.Accordion.prototype._resizeChildrenNodeWidth = function(minWidth){
    for(var i = this._children.length; i--;) 
        this._children[i]["childWidget"].resizeWidthComposite(minWidth);
};

/** @private */
gtk.Accordion.prototype._resizeChildrenNodeHeight = function(minHeight){
    for(var i = this._children.length; i--;) 
        this._children[i]["childWidget"].resizeHeightComposite(minHeight);
};

/**
 * Method to get minimum height of childNodeHolder
 * @return {Integer} The minimum height of it
 * @type Integer
 */
gtk.Accordion.prototype.getMinHeightComposite = function(){
    var minHeight = 0
    for(var i = this._children.length; i--;){
        var childWidgetMinHeight = this._children[i]["childWidget"].getMinHeightComposite()
        minHeight = minHeight < childWidgetMinHeight ? childWidgetMinHeight : minHeight;
    }
    
    minHeight += this._children.length * this._accordionHeaderHeight;

    this._minHeight = this._minHeight < minHeight ? minHeight : this._minHeight;
    return this._minHeight;
};

/**
 * Method to get minimum width of childNodeHolder
 * @return {Integer} The minimum width of it
 * @type Integer
 */
gtk.Accordion.prototype.getMinWidthComposite = function(){
    for(var i = this._children.length; i--;){
        var childWidgetMinWidth = this._children[i]["childWidget"].getMinWidthComposite();
        this._minWidth = this._minWidth < childWidgetMinWidth ? childWidgetMinWidth : this._minWidth;
    }
    return this._minWidth;
};

/**
 * Method to resize width composite of it
 * @param {Integer} width The new width needs setting to the widget
 */
gtk.Accordion.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);
    this._resizeChildrenNodeWidth(minWidth);
};

/**
 * Method to resize height composite of it
 * @param {Integer} height The new height needs setting to the widget
 */
gtk.Accordion.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;
    this._resizeHeight(minHeight);
    this._resizeChildrenNodeHeight(minHeight - this._getAccordionsHeaderHeight());
};

/** @private */
gtk.Accordion.prototype._initEvent = function(headerNode){
    var self = this;
    //var imgNode = headerNode.childNodes[1];
    jsgtk.Event.addDomListener(headerNode, "click", function(e){
        var index = self._getAccordionIndex(headerNode);

        if(index === self._activeIndex) {
        	self.deActivate(index);
        	self.emit("deactivated", index);
        }
        else {
        	self.activate(index);
        	self.emit("activated", index);
        }
        
    }, false);
};

gobject.signalNew("activated", gtk.Accordion, null, null, []);
gobject.signalNew("deactivated", gtk.Accordion, null, null, []);
