/**
 * Bubble is a class to create Bubble or Tooltip
 * @param {Object} dom DOM object of its child content
 * @param {Object} callerId DOM object of parent to show/display bubble or tooltip
 * @param {String} title The title of the Bubble
 * @param {Integer} tailDistance An integer value for tail of bubble
 * @param {Boolean} isVertical The boolean value true or false to define if it is horizontal or vertical
 * @param {Integer} tailLength An integer value of tail length
 * @base gtk.Widget
 * @constructor
 */
gtk.Bubble = function(dom, callerId, title, tailDistance, isVertical, tailLength){
    gtk.Widget.apply(this);

    this._domContent = dom;

    this._callerDom = (typeof callerId == "object") ? callerId : document.getElementById(callerId);

    this._isVertical = isVertical || false;
    this._tailDistance = tailDistance;
    this._title = title || "";
    this._closeButton = null;
    this._bubbleShape = null;
    this._contentWidth = 0;
    this._contentHeight = 0;
    this._minHeight = 25;
    this._minWidth = 100;
    this._titleHeight = 0;
    this._tailSide = "left";
    this._tailWidth = tailLength || 0;
    this._tailDistance = tailDistance || 30;
    
    /** A registered signal name 'closed' */
    this.BUBBLE_CLOSED = "";
    
    dom.className = "gtk_bubble_content";
    this._contentWidth = this._calculateDomWidth(dom);
    this._contentHeight = this._calculateDomHeight(dom);

	this._render = function(dom, callerId, title, tailDistance, isVertical, tailLength){
	    var bubbleDom = this._domObj;
	    bubbleDom.className += " gtk_bubble";
	    bubbleDom.setAttribute("id", "gtk_bubble");
	    bubbleDom.style.position = "absolute";

        this._closeButton = jsgtk.Dom.create({"tag": "p", "append": bubbleDom, "className": "gtk_bubble_close_button"});
        this._closeButton.innerHTML = " ";

        this._spanTitle = this._createTitle(bubbleDom);

        var callerDomProperties = this._getDomProperties(this._callerDom);

        this._tailSide = callerDomProperties.tailSide;

		this._positionBubbleTail(callerDomProperties);

        this._bubbleShape = new mylabs.Shape("bubble", 0, 0, this._contentWidth, this._getBubbleHeight(), this._tailSide, this._tailDistance);

        var domObjStyle = bubbleDom.style;
        domObjStyle.height = this._contentHeight + "px";
        domObjStyle.width = this._contentWidth + "px";

		document.body.appendChild(bubbleDom);
        this._setContent(this._domContent);
        
        bubbleDom.insertBefore(this._bubbleShape.getNode(), bubbleDom.firstChild);
    };

	this._render(dom, callerId, title, tailDistance, isVertical, tailLength);

	this._initEvents();
};

gtk.Bubble.prototype = new gtk.Widget();
gtk.Bubble.prototype.constructor = gtk.Bubble;

/** @private */
gtk.Bubble.prototype._initEvents = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this._closeButton, 'click', function(e){
        _self.close();
        e.cancelBubble = true;
    });
};

/** @private */
gtk.Bubble.prototype._setContent = function(content){
    if(content){
        this._domContent.className = "gtk_bubble_content";
        this._domObj.appendChild(content);
    }
};

/** @private */
gtk.Bubble.prototype._setContentWidth = function(width){
    this._contentWidth = width > this._minWidth ?  width : this._minWidth;
};

/** @private */
gtk.Bubble.prototype._getContentWidth = function(){
    return this._contentWidth;
};

/** @private */
gtk.Bubble.prototype._setContentHeight = function(height){
    this._contentHeight = height;
};

/** @private */
gtk.Bubble.prototype._getContentHeight = function(){
    return this._contentHeight;
};

/** @private */
gtk.Bubble.prototype._getBubbleHeight = function(){
    var bubbleHeight = this._domContent ? this._getContentHeight() + this._titleHeight : this._titleHeight + 10;

    if(bubbleHeight > this._minHeight){
        this._setContentHeight(bubbleHeight);
    }else{
        this._setContentHeight(this._minHeight);
        bubbleHeight = this._getContentHeight();
    }
    return bubbleHeight;
};

/** @private */
gtk.Bubble.prototype._createTitle = function(domBubble){
    if(this._title){
        this._updateTailWidth();
        var spanTitle = jsgtk.Dom.create({"append": domBubble, 'tag': "span", "className": "gtk_bubble_title"});
        spanTitle.innerHTML = this._title;
        spanTitle.style.width = this._getContentWidth() + "px";
        this._titleHeight = this._calculateDomHeight(domBubble.childNodes[1]);

        return spanTitle;
    }
};

/** @private */
gtk.Bubble.prototype._updateTailWidth = function(){
    var height = this._titleHeight? this._titleHeight : 0;

    this._tailWidth = this._getContentHeight() + height < 70 ? 10 : 20;
};

/** 
 * Method to set position of bubble or tooltip
 * @param {Object} caller The DOM object of caller or parent
 */
gtk.Bubble.prototype.setPosition = function(caller){
	this._callerDom = (typeof caller == "object") ? caller : document.getElementById(caller);
	var callerDomProperties = this._getDomProperties(this._callerDom);
	this._positionBubbleTail(callerDomProperties);
	this._resize();
};

/** 
 * Method to set position of bubble or tooltip by mouse position
 * @param {Event} e The event to be passed
 */
gtk.Bubble.prototype.setPositionByMouse = function(e){
    this._domObj.style.left = e.clientX + 5 + "px";
    this._domObj.style.top = e.clientY - 12 + "px";
};

/** @private */
gtk.Bubble.prototype._positionBubbleTail = function(callerDomProperties){
	var domBubble = this._domObj;
    var domStyle = domBubble.style;
    this._updateTailWidth();

    if(callerDomProperties.tailSide == "right"){
        domStyle.left = callerDomProperties.left - (this._tailWidth) - this._getContentWidth() + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4  +"px";
        domStyle.top = callerDomProperties.top + parseInt(callerDomProperties.height / 2) - this._tailDistance - (this._tailWidth/2)+ "px";
    }
    else if(callerDomProperties.tailSide == "left"){
        domStyle.left = callerDomProperties.width + callerDomProperties.left + "px";
        domStyle.paddingLeft = this._tailWidth + "px";
        this._closeButton.style.left = this._getContentWidth() + this._tailWidth - 16 +"px";
        domStyle.top = callerDomProperties.top + parseInt(callerDomProperties.height / 2) - this._tailDistance - (this._tailWidth/2)+ "px";
    }
    else if(callerDomProperties.tailSide == "up"){
        domStyle.top = callerDomProperties.top + callerDomProperties.height  + "px";
        domStyle.left = callerDomProperties.width/2 + callerDomProperties.left - this._tailDistance - 8 + "px";
        this._spanTitle.style.top = this._tailWidth + "px";
        if(this._domContent) this._domContent.style.top = this._tailWidth + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4 + "px";
        this._closeButton.style.top = this._tailWidth - 4 + "px";
    }
    else{
        var contentHeight = this._getContentHeight();
        domStyle.top = callerDomProperties.top - this._titleHeight - this._tailWidth - contentHeight  + "px";
        domStyle.left = callerDomProperties.width/2 - this._tailDistance + callerDomProperties.left - 8 + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4 + "px";
        this._closeButton.style.top = -4 + "px";
    }
};

/** @private */
gtk.Bubble.prototype._initEvents = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._closeButton, 'click', function(e){
        self.close();
        e.cancelBubble = true;
    });
};


/**
 * Method to show the bubble after initiated
 */
gtk.Bubble.prototype.show = function(){
    if(this._intervalID) { clearInterval(this._intervalID); }
    this.setPosition(this._callerDom);
    this._domObj.style.display = "";
};

/**
 * Method to set a newa title of the bubble
 * @param {String} title The string title is set to bubble
 */
gtk.Bubble.prototype.setTitle = function(title){
    if(!this._spanTitle){
        this._spanTitle = jsgtk.Dom.create({"tag": "span", "className": "gtk_bubble_title"});
        this._spanTitle.className += this._tailSide == "left" ? " leftTailBubble": " rightTailBubble";
        var svgDom = document.getElementsByTagName("path")[0].parentNode;
        this._domObj.insertBefore(this._spanTitle, svgDom);
    }

    this._title = title;
    this._spanTitle.innerHTML = title;
    var titleHeight = this._calculateDomHeight(this._spanTitle);
    this._resize();
};

/**
 * Method to get or return the title of the bubble
 * @return {String} The title of of the bubble if applicable
 */
gtk.Bubble.prototype.getTitle = function(){
    return this._title;
};

/**
 * Method to close or remove the bubble
 */
gtk.Bubble.prototype.close = function(){
    if(this._showConfirm){
        var quit = window.confirm(this._message);
        if(quit){
            jsgtk.Event.trigger(this, "confirm");
        }
    }
    
    this._removeModal();
    if(this._domObj.parentNode !== null) this._domObj.parentNode.removeChild(this._domObj);
        
    this._showConfirm = false;
    this.emit("closed");
};

/** @private */
gtk.Bubble.prototype._removeModal = function(){
    var modal = document.getElementById("gtk_bubble_modal");
    if(modal){ document.body.removeChild(modal); }
};

/**
 * Method to add Dom object into the bubble
 * @param {Dom} content The Dom object to be passed
 */
gtk.Bubble.prototype.addContent = function(content){
    this.removeContent();
    content.className = "gtk_bubble_content";
    this._domContent = content;
    this._contentWidth = this._calculateDomWidth(content);
    this._contentHeight = this._calculateDomHeight(content);
    this._domObj.style.width = this._contentWidth + "px";
    this._domObj.style.height = this._contentHeight + "px";
    this._domObj.appendChild(content);
    this._resize();
};

/**
 * Method to remove all of its contents
 */
gtk.Bubble.prototype.removeContent = function(){
    var childNodes = this._domObj.childNodes;
    var count = childNodes.length;
    var content ;
    for(var i = 0; i <count; i++){
        if(childNodes[i].className == "gtk_bubble_content"){
            content = childNodes[i];
        }
    }

    if(content){
        this._domObj.removeChild(content);
    }
};

 /**
 * Method to set content as tooltip
 * @param {DOm} domContent The DOM content value
 */
gtk.Bubble.prototype.setTooltip = function(domContent){
    if(domContent){
        this.addContent(domContent);
    }
    if(this._title) this.removeTitle();
    this._closeButton.style.display = "none";
};

/** @private */
gtk.Bubble.prototype._resize = function(){
    var nodeStyle = this._domObj.style;
    nodeStyle.height = "";
    nodeStyle.width = this._contentWidth + "px";
    var bubbleHeight = this._calculateDomHeight(this._domObj);
    nodeStyle.height = bubbleHeight + "px";

    callerTop = jsgtk.Util.pageY(this._callerDom);
    callerHeight = this._callerDom.clientHeight;
    this._contentHeight = this._calculateDomHeight(this._domContent);
    
    if(this._spanTitle){
    	this._titleHeight = this._calculateDomHeight(this._spanTitle);
    	this._spanTitle.style.width = this._contentWidth + "px";
    }
    

    var callerProperties = this._getDomProperties(this._callerDom);
    this._tailSide = callerProperties.tailSide;
    
    this._bubbleShape.remove();

    this._bubbleShape = new mylabs.Shape("bubble", 0, 0, this._contentWidth , bubbleHeight, this._tailSide, this._tailDistance);

    this._domObj.insertBefore(this._bubbleShape.getNode(), this._domObj.firstChild);

    if(this._tailSide == "right"){
        nodeStyle.left = callerProperties.left - (this._tailWidth + 6) - this._getContentWidth() + "px";
        nodeStyle.paddingLeft = "";
        this._closeButton.style.left = this._contentWidth - this._tailWidth + 6 + "px";
    }
    else if(this._tailSide == "up"){
        this._domContent.style.top = this._tailWidth + "px";
        nodeStyle.paddingLeft = "0px";
        this._closeButton.style.left = this._contentWidth - 16 + "px";
        this._closeButton.style.top = this._tailWidth - 6 + "px";
        this._spanTitle.style.top = this._tailWidth + "px";
    }
    else if(this._tailSide == "down"){
        this._domObj.style.top = callerTop - bubbleHeight - callerHeight + "px";
        this._closeButton.style.left = this._contentWidth - 16 + "px";
        this._closeButton.style.top = "0px";
        this._spanTitle.style.top = "0px";
        nodeStyle.paddingLeft = "0px";
    }
    else{
        this._closeButton.style.left = this._contentWidth + this._tailWidth - 16 + "px";
    }
};

 /**
  * Method to get the content from server then append to bubble
  * @param {String} url The URL of resource site
  * @param {Function} callback The method which will be executed after get response from server. Callback function should return a dom content
  */
gtk.Bubble.prototype.loadContent = function(url, callback){

    this._divLoader = jsgtk.Dom.create({"tag": "div", "className": "gtk_bubble_table"});
    var divLoaderStyle = this._divLoader.style;
    divLoaderStyle.height = MylabsSvgVml.type == "SVG" ? "70px" : "120px";
    divLoaderStyle.width = "300px";

    this._progressBarContainer = jsgtk.Dom.create({"append": this._divLoader, "tag": "span", "className": "gtk_bubble_progress_bar_container"});
        this._progressBar = jsgtk.Dom.create({"append": this._progressBarContainer, "tag": "div"});

    this.addContent(this._divLoader);

    this._updateTailWidth();
    if(this._tailSide == "right"){
        this._domObj.style.paddingLeft = "0px";
    }
    else if(this._tailSide == "down"){
        this._domObj.style.paddingLeft = "0px";
    }
    else if(this._tailSide == "up"){
        var top = this._tailWidth + "px";
        this._divLoader.style.top = top;
        this._closeButton.style.top = top;
        this._spanTitle.style.top = top;
    }
    else{
        this._domObj.style.paddingLeft = this._tailWidth + "px";
    }

    this._ajax(url, callback);

};

 /** @private */
gtk.Bubble.prototype._updateProgressBar = function(state){
    var self = this;
    this._barWidth = 0;
    var bar = this._progressBar;
    var x = window.setInterval(
        function(){
            self._barWidth += 10;
            bar.style.width = self._barWidth + "px";
            if( self._barWidth== 180) { clearInterval(x); }
        },
        40
    );
};

/** @private */
gtk.Bubble.prototype._appendContent = function(callback){
    var self = this;
    var time = 0, count = 50;
    var error = false;
    var y = window.setInterval(
        function(){

            time += count;
            if(self._barWidth == 180){
                if(parseFloat(self._progressBar.style.width) < 195) self._progressBar.style.width = (parseFloat(self._progressBar.style.width) + 0.035) + "px";
                if(self._response.state == 4){
                    clearInterval(y);
                    self._progressBar.style.width = "100%";
                    self._domObj.removeChild(self._divLoader);

                    var domContent = callback(eval( "(" + self._response.text + ")" ));
                    self.addContent(domContent);
                }
            }

            if(time > 15000 && !error){
                error = true;
                var errorSpan = jsgtk.Dom.create({"tag":"span"});
                errorSpan.innerHTML = "<i>This is taking longer than usual. </i><br/> Try again";
                self._divLoader.appendChild(errorSpan);

                self._divLoader.style.textAlign = "center";
            }
        },
        count
    );
};

/** @private */
gtk.Bubble.prototype._updateContent = function(content){
    var container = jsgtk.Dom.create({tag: "div"});
    container.innerHTML = content;
    this._domObj.removeChild(this._divLoader);
    this.addContent(container);
};

/** @private */
gtk.Bubble.prototype._ajax = function(url, callback){
	var method = "GET";
	var request;
	var self = this;

	if(window.XMLHttpRequest){
	    request = new XMLHttpRequest();
	}
	else if (window.ActiveXObject){
	    request = new ActiveXObject("Msxml2.XMLHTTP");
	    if (!request){
		    request = new ActiveXObject("Microsoft.XMLHTTP");
	    }
	}
	this._updateProgressBar();
	this._appendContent(callback);
	this._response = {};
	/** @private */
	request.onreadystatechange = function() {
		if (request.readyState == 4){
	        self._response = {
			    state: 4,
			    text: request.responseText
			};
		}
	};

	request.open(method, url, true);
	request.send(null);
};

/**
 * Method to remove title from bubble
 */
gtk.Bubble.prototype.removeTitle = function(){
    this._title = "";
    this._domObj.removeChild(this._spanTitle);
    this._titleHeight = 0;
    this._resize();
    this._spanTitle = null;
};

/** @private */
gtk.Bubble.prototype._getDomProperties = function(callerDom){
    var callerTemp = callerDom.cloneNode(true);
    var side, windowSize,
        widthFromLeft = jsgtk.Util.pageX(callerDom),
        heightFromTop = jsgtk.Util.pageY(callerDom),
        callerX = jsgtk.Util.fullWidth(callerTemp),
        callerY = jsgtk.Util.fullHeight(callerTemp);

    if(this._isVertical){
        windowSize = jsgtk.Util.getWindowHeight();
        side = (windowSize - heightFromTop - callerY > this._contentHeight + this._titleHeight + this._tailWidth) ? "up" : "down";
    }
    else{
        windowSize = document.body.clientWidth;
        side = (windowSize - (widthFromLeft + callerX + this._tailWidth) <  this._contentWidth) ? "right" : "left";
    }

    return {
        tailSide: side,
        left: widthFromLeft,
        top: heightFromTop,
        windowSize: windowSize,
        width: callerX,
        height: callerY
    };
};

/**
 * Method to show the modal modal window cover on top of page
 * @param {Function} callback The callback method
 * @param {String} msg The confirm message display in the confirmation popup box
 */
gtk.Bubble.prototype.confirmClose = function(callback, msg){
    var self = this;
    this._showConfirm = true;
    this._message = msg || "Are you sure you want to quit?";

    jsgtk.Event.addListener(this, "confirm", callback);
    document.onkeypress = function(e){
        e = (e)?e:((event)?event:null);
        if(e.keyCode == 27){
            self.close();
        }
    };
};

/**
 * Method to show the modal modal window cover on top of page
 */
gtk.Bubble.prototype.showModal = function(){
    this.show();

    var modal = jsgtk.Dom.create({"append": document.body, "tag": 'div', "id": "gtk_bubble_modal"});
    var pageSize = jsgtk.Util.getPageSize();
    modal.style.width = pageSize[0] + "px";
    modal.style.height = pageSize[1] + "px";
    this._domObj.style.display = "";
};

/** @private */
gtk.Bubble.prototype._calculateDomHeight = function(dom){
    if(dom){
        var tempDom = dom.cloneNode(true);
        tempDom.style.visibility = "hidden";
        tempDom.style.position = "absolute";
        var marginTop = parseInt(jsgtk.Util.getStyle(tempDom, "marginTop")) || 0;
        var marginBottom = parseInt(jsgtk.Util.getStyle(tempDom, "marginBottom")) || 0;
        var height = jsgtk.Util.fullHeight(tempDom) + marginTop + marginBottom;
        height = this._minHeight > height ? this._minHeight : height;
        return height;
    }
    return 0;
};

/** @private */
gtk.Bubble.prototype._calculateDomWidth = function(dom){
    if(dom){
        var tempDom = dom.cloneNode(true);
        tempDom.style.visibility = "hidden";
        tempDom.style.position = "absolute";
        var marginLeft = parseInt(jsgtk.Util.getStyle(tempDom, "marginLeft")) || 0;
        var marginRight = parseInt(jsgtk.Util.getStyle(tempDom, "marginRight")) || 0;
        var width = jsgtk.Util.fullWidth(tempDom) + marginLeft + marginRight;
        return width;
    }
    return 0;
};

/**
 * Method to hide the bubble with in an interval of time. When the user mouseover the bubble, it won't hide. When User mouseout the bubble, the bubble will be hide
 * @param {Integer} time The time interval which set to hide bubble. The default interval is 500.
 */
gtk.Bubble.prototype.hide = function(time){
    var self = this;
    if(!this._autoHide){
		this._domObj.style.display = "none";
		return;
	};
    this._intervalID = null;
    var timeout = time || 500;
    var modal = document.getElementById("gtk_bubble_modal");
    if(modal){
         return;
    }else{
        jsgtk.Event.addDomListener(this._domObj, "mouseover", function(){
            self._domObj.style.display = "";
            clearInterval(self._intervalID);
            jsgtk.Event.addDomListener(self._domObj, "mouseout", function(){

                if(!document.getElementById("gtk_bubble_modal")){ self._domObj.style.display = "none"; }
            });
        });

        var displayNone = function(){
            self._domObj.style.display = "none";
            clearInterval(self._intervalID);
        };

        this._intervalID = setInterval(displayNone, timeout);
    }
};

/** Method to set display auto hideElement
 *  @param {Boolean} The boolean true or false for auto display or hide
 */
gtk.Bubble.prototype.setAutoHide = function(autoHide){
	this._autoHide = autoHide;
};

/** 
 * Method to set fixed position
 */
gtk.Bubble.prototype.setFixedPosition = function(){
	this._domObj.style.position = "fixed";
};

gobject.signalNew("closed", gtk.Bubble, null, null, []);
