/**
 * Button is a class to create a button
 * @constructor
 * @base gtk.Widget
 * @param {String} Label text to be displayed by in the middle of button
 */
gtk.Button = function(label){
	gtk.Widget.apply(this);
	
	var self 			= this;
	this._label 		= label || "";
	this._labelWidget 	= null;
	this._imageWidget 	= null;
	this._isEnable 		= true;
	this._signalClicked = null;
	
	this._width 		= this._minWidth = 16;
	this._height 		= this._minHeight = 24;

    /** 'clicked' signal */
	this.BUTTON_CLICKED = "clicked";
	/** 'mouseout' signal */
	this.BUTTON_MOUSEOUT = "mouseout";
	/** 'mouseover' signal */
	this.BUTTON_MOUSEOVER = "mouseover";
	/** 'enterkeypressed' signal */
	this.BUTTON_ENTERKEYPRESSED = "enterkeypressed";

	/** @private */		
	this._render = function(){

	    this._domObj.className += " gtk_button";
	    var domObjStyle = this._domObj.style;
	    domObjStyle.position = "relative";
	    var jsgtkDom = jsgtk.Dom;
	        this._gradient = jsgtkDom.create({"append": this._domObj, "tag": "div", "class": "gtk_button_gradient"});
	        this._gradient.style.height = this._height / 2 + "px";
		    this._imageDom = jsgtkDom.create({"tag": "div", "style": "float: left; position: relative;"});
		    this._labelDom = jsgtkDom.create({"tag": "div", "style": "float: left; position: relative;"});

		domObjStyle.width = this._width - this.borderWidth + "px";
		domObjStyle.height = this._height - this.borderWidth + "px";
		if(label) this.setLabel(new gtk.Label(label));
	};
    this._render();
    this._initEvent();
};

gtk.Button.prototype = new gtk.Widget();
gtk.Button.prototype.constructor = gtk.Button;

/** @private */
gtk.Button.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    this._signalClicked = jsgtkEvent.addDomListener(this._domObj, "click", function(e){
		if(self._isEnable === true) self.emit("clicked");
	}, false);
	
    jsgtkEvent.addDomListener(this._domObj, "keyup", function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
	        if(self._isEnable === true) self.emit("enterkeypressed");
	    }
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseover", function(e){
		self.emit("mouseover");
	}, false);
	
	jsgtkEvent.addDomListener(this._domObj, "mouseout", function(e){
		self.emit("mouseout");
	}, false);
};

/** @private */
gtk.Button.prototype._positionWidget = function(widget, widgetDom){
    var top = (this._height  - widget.getHeight()) / 2;
    var left = (this._width - widget.getWidth() - this.borderWidth)/2;
 
    widgetDom.style.top = top + "px";
    widgetDom.style.left = left + "px";
};

/** @private */
gtk.Button.prototype._positionImageLabel = function(){
	var imageDomStyle = this._imageDom.style;
	var labelDomStyle = this._labelDom.style;

	var imageTop = (this._height / 2) - (this._imageWidget.getHeight() / 2);
	var imageLeft = (this._width - this._imageWidget.getWidth() - this._labelWidget.getWidth() - this.borderWidth)/2;
	imageDomStyle.top = imageTop + "px";
	imageDomStyle.left = imageLeft + "px";

	var labelTop = (this._height / 2) - (this._labelWidget.getHeight() / 2);
	labelDomStyle.top = labelTop + "px";
    labelDomStyle.left = parseInt(imageDomStyle.left) + "px";
};
/** @private */
gtk.Button.prototype.resize = function(width, height){
    if(this._imageWidget && this._labelWidget){
       	this._positionImageLabel();
        return;
    }
    
    if(this._imageWidget) this._positionWidget(this._imageWidget, this._imageDom);
 
    if(this._labelWidget) this._positionWidget(this._labelWidget, this._labelDom);
};

/** @private */
gtk.Button.prototype._getMaxChildrenHeight = function(){
    var maxHeight = this._minHeight;
    if(this._imageWidget) maxHeight = maxHeight > this._imageWidget.getMinHeightComposite() ? maxHeight : this._imageWidget.getMinHeightComposite();
    if(this._labelWidget) maxHeight = maxHeight > this._labelWidget.getMinHeightComposite() ? maxHeight : this._labelWidget.getMinHeightComposite();
    
    return maxHeight;
};

/** @private */
gtk.Button.prototype._getMaxChildrenWidth = function(){
    var maxWidth = 0;
    if(this._imageWidget) maxWidth += this._imageWidget.getMinWidthComposite();
    if(this._labelWidget) maxWidth += this._labelWidget.getMinWidthComposite();
    maxWidth += 15;
    maxWidth = maxWidth < this._minWidth ? this._minWidth : maxWidth;
    return maxWidth;
};

/** 
 * Set the image to be displayed next to the label
 * @param {gtk.Image} imageWidget
 */
gtk.Button.prototype.setImage = function(image){
    if(this._imageDom.childNodes) this._imageDom.innerHTML = "";
    this._imageWidget = image;
    
	this.resetMinSize(this._getMaxChildrenWidth(), this._getMaxChildrenHeight());
    this.resizeWidthHeightComposite(this._minWidth, this._minHeight);
    
    this._imageDom.appendChild(this._imageWidget.getNode());
    this._domObj.insertBefore(this._imageDom, this._domObj.childNodes[1]);
};
/**
 * Change the button shape to be image shape
 * @param {gtk.Image} imageWidget
 */
gtk.Button.prototype.setAsImage = function(imageObj){
	this._domObj.className += " gtk_button_with_image_shape";
	this._gradient.style.display = "none";
	this._domObj.style.backgroundImage = "url("+imageObj.getChildNode().src+")";
	this.resetMinSize(imageObj.getMinWidthComposite(), imageObj.getMinHeightComposite());
	this.resizeWidthHeightComposite(this._minWidth, this._minHeight);
};

/**
 * Returns the image widget
 * @return {gtk.Image} imageWidget
 */
gtk.Button.prototype.getImage = function(){
    return this._imageWidget;
};
/**
 * Sets label to be displayed in the button
 * @param {gtk.Label} labeWidget
 */
gtk.Button.prototype.setLabel = function(label){
    if(this._labelDom.childNodes) this._labelDom.innerHTML = "";
    this._labelWidget = label;
    this._label = label.getText();
    label.setStyle({cursor: "pointer"});
    
    this.resetMinSize(this._getMaxChildrenWidth(), this._getMaxChildrenHeight());
    this.resizeWidthHeightComposite(this._minWidth, this._minHeight);
    
    this._labelWidget.setJustify("center");

    this._labelDom.appendChild(this._labelWidget.getNode());
    this._domObj.appendChild(this._labelDom);
};
/**
 * Set text to button
 * @param {String} strName name of button
 */
gtk.Button.prototype.setTextLabel = function(strName){
    if(strName != null) this.setLabel(new gtk.Label(strName));
};
/**
 * Returns the text from the label of the button
 * @return {gtk.Label} labelWidget
 */
gtk.Button.prototype.getLabel = function(){
    return this._labelWidget;
};
/** @private */
gtk.Button.prototype._resizeHeight = function(height){
	this._domObj.style.height = height + "px";
	this._domObj.childNodes[0].style.height = height / 2 + "px";
	this._height = height;
};
/** @private */
gtk.Button.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this.borderWidth + "px";
	this._width = width;
};
/**
 * show loading Image
 */
gtk.Button.prototype.showLoading = function(){
	this._labelDom.style.display = "none";
	this._imageDom.style.display = "none";
	jsgtk.Event.removeListener(this._signalClicked);
	
	if(!this._loading) this._loading = jsgtk.Dom.create({tag:"div", className: "gtk_button_loading", append: this._domObj});
};
/**
 * Hide loading Image
 */
gtk.Button.prototype.hideLoading = function(){
	jsgtk.Event.registerHandler(this._signalClicked);
	this._labelDom.style.display = "";
	this._imageDom.style.display = "";
	if(this._loading){
		this._domObj.removeChild(this._loading);
		this._loading = null;
	} 
};
/**
 * Set tab index
 * @param {Integer} index
 */
gtk.Button.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};
/**
 * Disable button button no longer function
 */
gtk.Button.prototype.disable = function(){
	this._isEnable = false;
	this._domObj.className +=" gtk_button_disabled";
	this._gradient.className += " gradisable";
	if(this._labelWidget !== null) this._labelWidget.setStyle({cursor: "default"});
};
/**
 * Enable button button functions normally
 */
gtk.Button.prototype.enable = function(){
	this._isEnable = true;
	this._domObj.className = "gtk_widget gtk_container gtk_button";
	this._gradient.className = "gtk_button_gradient";
	if(this._labelWidget !== null) this._labelWidget.setStyle({cursor: "pointer"});
};

/** @private */
gtk.Button.prototype.resizeWidthHeightComposite = function(width, height){
	var minHeight = height > this._minHeight ? height : this._minHeight;
	var minWidth = width > this._minWidth ? width : this._minWidth;
	this._resizeWidth(minWidth);
	this._resizeHeight(minHeight);
	this.resize(minWidth, minHeight);
};

/** @private */
gtk.Button.prototype.resizeWidthComposite = function(width){
	var minWidth = width > this._minWidth ? width : this._minWidth;
    this._resizeWidth(minWidth);
    this.resize(minWidth, this._height);
};
/** @private */
gtk.Button.prototype.resizeHeightComposite = function(height){
	var minHeight = height > this._minHeight ? height : this._minHeight;
    this._resizeHeight(minHeight);
    this.resize(this._width, minHeight);
};
gobject.signalNew("clicked", gtk.Button, null, null, []);
gobject.signalNew("mouseout", gtk.Button, null, null, []);
gobject.signalNew("mouseover", gtk.Button, null, null, []);
gobject.signalNew("enterkeypressed", gtk.Button, null, null, []);
