/**
 * Dom is a class to create a text or an HTML container. It can contain any HTML string value.
 * @param {Integer} width The integer width value in pixel to be set
 * @param {Integer} height The integer height value in pixel to be set
 * @constructor
 * @base gtk.Widget
 */
gtk.Dom = function(width, height) {
	gtk.Widget.apply(this);

	/** 'clicked' signal */
	this.DOM_CLICKED = "clicked";
	/** 'doubleclicked' signal */
	this.DOM_DOUBLECLICKED = "doubleclicked";
	/** 'mouseover' signal */
	this.DOM_MOUSEOVER = "mouseover";
	/** 'mouseup' signal */
	this.DOM_MOUSEUP = "mouseup";
	/** 'mouseout' signal */
	this.DOM_MOUSEOUT = "mouseout";

	this._attribute = null;

	this._width  = this._minWidth  = this._initWidth	= width  || 16;
	this._height = this._minHeight = this._initHeight	= height || 16;
	
	this._dom = null;
	
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className   += " gtk_dom";
	};
	
    this._render();
    this._initEvent();
};

gtk.Dom.prototype = new gtk.Widget();
gtk.Dom.prototype.constructor = gtk.Dom;

/** 
* Append the child widget to gtk.Dom
* @param {gtk.Widget} dom The Dom object to be added
*/
gtk.Dom.prototype.appendChild = function(dom){
    if(this._dom) this._domObj.replaceChild(dom, this._dom);
    else this._domObj.appendChild(dom);
    
    this._dom = dom;
    
    this._resetSize();    
};

/**
 * Accept any HTML string
 * @param {String} value The HTML value to be set
 */
gtk.Dom.prototype.setInnerHTML = function(value){
	var newDom = jsgtk.Dom.create({'tag': 'div'});
	newDom.innerHTML = value;
	this.appendChild(newDom);
};

/**
 * Return HTML string
 * @return {String} The HTML string value inside the Dom
 * @type String
 */
gtk.Dom.prototype.getValue = function(){
    return this._dom.innerHTML;
};

/** @private */
gtk.Dom.prototype._resetSize = function(){
	var size = this._getChildSize();
	this._minWidth  = size[0];
	this._minHeight = size[1];
	this.resize(size[0], size[1]);
};

/** @private */
gtk.Dom.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    
    jsgtkEvent.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "dblclick", function(e){
		self.emit("doubleclicked", self.getValue());
	}, false);
	
	jsgtkEvent.addDomListener(this._domObj, "mouseover", function(e){
		self.emit("mouseover", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseout", function(e){
		self.emit("mouseout", self.getValue());
	}, false);
	
	jsgtkEvent.addDomListener(this._domObj, "mouseup", function(e){
		self.emit("mouseup", self.getValue());
	}, false);
};

/** @private */
gtk.Dom.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);  
};

/** @private */
gtk.Dom.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
	this._dom.style.width = width + 'px';
};

/** @private */
gtk.Dom.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = height + 'px';
	this._dom.style.height = height + "px";
};

/**
 * Set style or apply attribute 
 * @param {Object} attribute A collection of object literal of styles
 */
gtk.Dom.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.Dom.prototype._applyAttribute = function(){
	for(var el in this._attribute){
	    this._domObj.style[el] = this._attribute[el];
	}
};

/** @private */
gtk.Dom.prototype._getChildSize = function(){
	var jsgtkUtil = jsgtk.Util;
	var tmpNode = this._dom.cloneNode(true);
	tmpNode.style.width = this._width + "px";
	var height   = jsgtkUtil.fullHeight(tmpNode) ? jsgtkUtil.fullHeight(tmpNode) : this._height;
	return [this._width, height];
};

/** @private */
gtk.Dom.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._resizeWidth(minWidth);
};

/** @private */
gtk.Dom.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeHeight(minHeight);
};

gobject.signalNew("clicked", gtk.Dom, null, null, []);
gobject.signalNew("doubleclicked", gtk.Dom, null, null, []);
gobject.signalNew("mouseover", gtk.Dom, null, null, []);
gobject.signalNew("mouseup", gtk.Dom, null, null, []);
gobject.signalNew("mouseout", gtk.Dom, null, null, []);
