/**
 * EntryDropDown is a class to create a text entry dropdown or text entry with auto suggestion.
 * The EntryDropDown store data as a model from gtk.TreeStore
 * @constructor
 * @param {Integer} maximum length of characters
 * @extends gtk.Entry
 * @see gtk.TreeStore
 */
gtk.EntryDropDown = function(maxLength){
	gtk.Entry.apply(this, [maxLength]);

	/** 'selected' signal */
	this.ENTRYDROPDOWN_SELECTED = "selected";

	this._selectedRowText = "";
	this._selectRowIndex = 0;
	this._filteredData = [];
	this._selectedRowValue = null;
	this._hasScroller = false;
	/** @private */ 
	this._render = function(){
		var _self = this;
		this._domObj.className += " gtk_entry_drop_down";
			
		this.__dropDown = jsgtk.Dom.create({tag: "div", className: "gtk_entry_drop_down_holder", append: document.body});
		this._hideDropDown();
	};

	this._render();
	this._bindInputEvent();
	this._bindDropDownEvent();
};

gtk.EntryDropDown.prototype = new gtk.Entry();
gtk.EntryDropDown.prototype.constructor = gtk.EntryDropDown;

/**
 * A method to set selected item by any value
 * @param {String} key to search for
 * @param {any type} value that will match the value of given key
 */
gtk.EntryDropDown.prototype.setSelectedItemBy = function(valueMember, value){
	this.emit("_searchModelByValueMember", valueMember, value);
};

/** @private */
gtk.EntryDropDown.prototype._bindDropDownEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;

	jsgtkEvent.addDomListener(this.__dropDown, "click", function(e){
		var node = e.target;
		if(/gtk_entry_drop_down_element/.test(node.className)){          
			self.emit("_searchbypath", self._selectedRowNode.id);
        }
        jsgtkEvent.stopBubble(e);    
	});
	
	jsgtkEvent.addDomListener(this.__dropDown, "mouseover", function(e){
		var node = e.target;
        if(/gtk_entry_drop_down_element/.test(node.className)){
            self._deHilightRow(self._selectedRowNode);
		    self._selectedRowNode = node;
		    self._hilightRow(node);
		    self._selectRowIndex = self._getSelectedIndex(node);
        }
        node.className += " gtk_entry_drop_down_hover";
	});
	
	jsgtkEvent.addDomListener(document.body, "click", function(e){
		self._hideDropDown();
		jsgtkEvent.stopBubble(e);
	});
};

/**
 * Set model to Entry
 * @param {gtk.ListStore} model ListStore model to be set to Entry
 */
gtk.EntryDropDown.prototype.setModel = function(model){
	this._model = model;
	new gtk.EntryDropDownController(this, model);
};

/** @private */
gtk.EntryDropDown.prototype._constructRowNode = function(data){
	this._filteredData  = data;
	var docFragment = document.createDocumentFragment();
    for(var i = 0; i < data.length; i++){
		var dataObj = data[i]
        var element = jsgtk.Dom.create({tag: "div", append: docFragment, text: dataObj["display"], className: "gtk_entry_drop_down_element"});
        element.id = dataObj.path;
	}
	return docFragment;
};

//call by controller
gtk.EntryDropDown.prototype._renderDropDown = function(data){
	var rowsNode = this._constructRowNode(data);
	this._appendToDropDown(rowsNode);
};

/** @private */
gtk.EntryDropDown.prototype._appendToDropDown = function(dropDown){
	this.__dropDown.innerHTML = "";
	if(dropDown.textContent != "")
	    this.__dropDown.appendChild(dropDown);
	else this._hideDropDown();
};

/** @private */
gtk.EntryDropDown.prototype._deHilightRow = function(selectedRow){
	if(selectedRow) selectedRow.className = "gtk_entry_drop_down_element";
};

/** @private */
gtk.EntryDropDown.prototype._hilightRow = function(selectedRow){
	if(selectedRow) selectedRow.className += " gtk_entry_drop_down_selected";
};

/** @private */
gtk.EntryDropDown.prototype._hilightRowByDownArrowKey = function(){
	if(this._selectedRowNode !==  null){
	    if(this._selectedRowNode.nextSibling === null){
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this.__dropDown.childNodes[0];
	        this._selectRowIndex = 0;
	    }
	    else{
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this._selectedRowNode.nextSibling;
	        this._selectRowIndex = this._selectRowIndex + 1;
	    }
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";  
	}
	else{
	    this._selectedRowNode = this.__dropDown.childNodes[0];
	    this._selectRowIndex = 0;
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}	
};

/** @private */
gtk.EntryDropDown.prototype._hilightRowByUpArrowKey = function(){
	if(this._selectedRowNode !==  null){
	    if(this._selectedRowNode.previousSibling === null){
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this.__dropDown.lastChild;
	        this._selectRowIndex = this.__dropDown.childNodes.length - 1;
	    }
	    else{
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this._selectedRowNode.previousSibling;
	        this._selectRowIndex = this._selectRowIndex - 1;
	    }
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
	else{
	    this._selectedRowNode = this.__dropDown.lastChild;
	    this._selectRowIndex = this.__dropDown.childNodes.length - 1;
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
};

/** @private */
gtk.EntryDropDown.prototype._bindInputEvent = function(){
	var jsgtkEvent = jsgtk.Event;
    var self = this;
    
    var entry = this._domObj.firstChild.firstChild;
	jsgtkEvent.addDomListener(entry, "keydown", function(e){
        var keyInCode = e.keyCode;
		switch(keyInCode){
			case gtk.DOWN_ARROW_KEY:
			    self._changeSelectedByArrowDown();
				return;
			case gtk.UP_ARROW_KEY:
			    self._changeSelectedByArrowUp();
				return;
			case gtk.ESC_KEY:
                self._hideDropDown();
				return;
			case gtk.TAB_KEY:
                if(self._selectedRowNode) self.emit("_searchbypath", self._selectedRowNode.id);
				return;
			case gtk.ENTER_KEY:
				if(self._selectedRowNode) self.emit("_searchbypath", self._selectedRowNode.id);
				jsgtkEvent.stopBubble(e);
				return;
		}
		
	});
		
	this.connect("changed", function(value){
	    self._selectedRowNode = null;
	    self._selectedRowValue = null;
        self._showDropDown();
        self.emit("_modelfiltered", value);
	});
};

/** @private */
gtk.EntryDropDown.prototype._scrollByArroyKey = function(index){
    if(index == 0) this.__dropDown.scrollTop = 0;
    else this.__dropDown.scrollTop = index * 20 - 20;
};

/** @private */
gtk.EntryDropDown.prototype._changeSelectedByArrowUp = function(){
        if(this._filteredData.length > 0){
        	this._scrollByArroyKey(this._selectRowIndex);
            this._showDropDown();
            this._hilightRowByUpArrowKey();
		    this.setText(this._selectedRowNode.textContent);
        }
        
};

/** @private */
gtk.EntryDropDown.prototype._changeSelectedByArrowDown = function(){
    if(this._filteredData.length > 0){
    	this._scrollByArroyKey(this._selectRowIndex);
        this._showDropDown();
	    this._hilightRowByDownArrowKey();
	    this.setText(this._selectedRowNode.textContent);
    }
};

/** @private */
gtk.EntryDropDown.prototype._getSelectedIndex = function(){
	var tempNodes = this.__dropDown.cloneNode(true).childNodes;
	var found = false;
	for(var i = 0, len = tempNodes.length; i < len; i++){
		if(/gtk_entry_drop_down_selected/.test(tempNodes[i].className))
			break;
	}
	return i;
};

/**
 * Set display default value into EntryDropDown
 * @param {String} text value to be set
 */
gtk.EntryDropDown.prototype.setText = function(text){
    var input = this._domObj.firstChild.firstChild;
	input.value = text === undefined ? "" : text;
};

/**
* Return the text display in gtk.Entry
* @return {string} text
*/
gtk.EntryDropDown.prototype.getText = function(text){
    return this._domObj.firstChild.firstChild.value;
};

/**
 * Set selected value to display by default
 * @param {Integer} index an integer value to be passed
 */
gtk.EntryDropDown.prototype.setSelectedIndex = function(index){
	this.emit("_searchbyindex", index);
};

/** @private */
gtk.EntryDropDown.prototype._setSelectedRowValue = function(value){
	this._selectedRowValue = value;
};

/**
 *  Method to get/return an object of selected row value
 *  @param {Object}, it return selected object value
 */
gtk.EntryDropDown.prototype.getSelectedRowValue = function(){
	return this._selectedRowValue !== null ? this._selectedRowValue : this.getText();
};

/**
 * Get/Return an object valau if exists
 */
gtk.EntryDropDown.prototype.getValue = function(){
    return this.getSelectedRowValue();
};

/** @private */
gtk.EntryDropDown.prototype._hideDropDown = function(){
	this.__dropDown.style.display = "none";
	this._selectedRowNode = null;
	this._filteredData  = [];
};

/** @private */
gtk.EntryDropDown.prototype._showDropDown = function(){
    this.__dropDown.style.width = this._width - 2 + "px";
    this.__dropDown.style.left = "-1px";
    this.__dropDown.style.top = "1px";
	this.__dropDown.style.display = "";
	this._setPosition();
};
/** @private */
gtk.EntryDropDown.prototype._setPosition = function(e){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);
    var header = parseInt(this._height);
    var scrollY = this._getNearestScrollerY();   
    this.__dropDown.style.top = top + header + 2 - scrollY + "px";
    this.__dropDown.style.left = left + "px";
};

/** @private */
gtk.EntryDropDown.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode.parentNode != null){
        if(/gtk_scrolled_window/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._hasScroller = true;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};

/** @private */
gtk.EntryDropDown.prototype._addEventMouseWheel = function(scrollObj){
    var _self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    _self._hideDropDown();
	}, false);
};

gobject.signalNew("_searchbyindex", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_searchbypath", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_checkvaluechanged", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_searchModelByValueMember", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_modelfiltered", gtk.EntryDropDown, null, null, []);
gobject.signalNew("selected", gtk.EntryDropDown, null, null, []);
