/**
 * Contents is a class to create contents for NoteBook
 * @param {Integer} width The width of the widget to be set
 * @param {Integer} height The height of the widget
 * @constructor
 * @base gtk.Widget
 */
gtk.Contents = function(width, height) {
	gtk.Widget.apply(this);
	
	this._width = width;
	this._height = height;
	this._children = [];
	this._activeContent = null;
	this._render = function(){
		this._domObj.className += " gtk_contents";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
	};
	
	this._render();
};

gtk.Contents.prototype = new gtk.Widget();
gtk.Contents.prototype.constructor = gtk.Contents;

/** 
 * Method to hide border of widget
 */
gtk.Contents.prototype.hideBorder = function(){
    this._domObj.style.border = "0px";
};

/** 
 * Method to show border of widget
 */
gtk.Contents.prototype.showBorder = function(){
    var domStyle = this._domObj.style;
    domStyle.borderBottom = "1px solid #EFC169";
    domStyle.borderLeft   = "1px solid #EFC169";
    domStyle.borderRight  = "1px solid #EFC169";
};

/** 
 * Method to set activated as default
 */
gtk.Contents.prototype.activateDefault = function(){
	this._activeContent = this._children[0]["content"];
	this._domObj.appendChild(this._activeContent.getNode());
};

/** 
 * Method to set activated by index value
 * @param {Integer} index The index value to be set
 */
gtk.Contents.prototype.activate = function(index){
	var content = this._children[index]["content"];
	this._domObj.innerHTML = "";
	this._domObj.appendChild(content.getNode());
	this._activeContent = content;
	return content;
};

/** 
 * Method to set activated by object value
 * @param {Object} content The object value to be passed
 */
gtk.Contents.prototype.activateByObject = function(content){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["content"] === content) break;
    }
    this.activate(i);
};

/** 
 * Method to set activated its content
 * @param {Integer} id The id value as integer
 */
gtk.Contents.prototype.activateContent = function(id){
	var content;
	for(var i = 0, len = this._children.length; i < len; i++){
		content = this._children[i]["content"];
		if(id === this._children[i]["id"]) break;
	}
	this._domObj.innerHTML = "";
	this._domObj.appendChild(content._domObj);
	this._activeContent = content;
	return content;
};

/** 
 * Method to get/return active content
 * @return {Object} The active object value
 * @type Object
 */
gtk.Contents.prototype.getActiveContent = function(){
    return this._activeContent;  
};

/** 
 * Method to update its content
 * @param {Object} content The object value of the content
 * @param {Integer} index The integer value of the content tab index
 */
gtk.Contents.prototype.updateContent = function(content, index){
    this._children[index]["content"] = content;
    content.setParent(this);
};

/** 
 * Method to add child object into its body
 * @param {Object} contentObj The object value of the content
 */
gtk.Contents.prototype.addChild = function(contentObj){
    contentObj["padding"] = contentObj["padding"] === undefined || contentObj["padding"] === null ? 1 : contentObj["padding"];
    contentObj["content"].setParent(this);
	this._children.push(contentObj);
};

/** 
 * Method to insert an object value at a specific position
 * @param {Object} contentObj The object value of the content
 * @param {Integer} index The index value of the content tab
 */
gtk.Contents.prototype.insertAt = function(contentObj, index){
	this._children.splice(index, 0, contentObj);
};

/** 
 * Method to remove child from it body given an index value
 * @param {Integer} index The index value to be set
 */
gtk.Contents.prototype.removeChild = function(index){
    jsgtk.Util.removeArrayElements(this._children, index);
    this._domObj.removeChild(this._domObj.childNodes[0]);
};

/** 
 * Method to get or return all children
 * @return {Object} The array objects of its contents or children
 * @type Object
 */
gtk.Contents.prototype.getChildren = function(){
    return this._children;    
};

/** @private */ 
gtk.Contents.prototype._resizeChildrenHeight = function(height){

    for(var i = 0, len = this._children.length; i < len; i++){
        var childWiget = this._children[i]["content"];
        var padding    = this._children[i]["padding"];
        var childHeight = height - padding * 2;
        childWiget.resizeHeightComposite(childHeight);
        childWiget.getNode().style.top = padding + "px";
    }
};

/** @private */ 
gtk.Contents.prototype._resizeChildrenWidthComposite = function(width){
    for(var i = this._children.length; i--;){
        var childWiget = this._children[i]["content"];
        var padding    = this._children[i]["padding"];
        var childWidth = width - padding * 2;
        childWiget.resizeWidthComposite(childWidth);
        childWiget.getNode().style.left = padding + "px";
    }
};

/** 
 * Method to get/return minimum composite width
 * @return {Integer} The minimum composite width of the content
 * @type Integer
 */
gtk.Contents.prototype.getMinWidthComposiste = function(){
	var children = this._children;
	var minWidth = this._minWidth;

	for(var i = children.length; i--;){
		var width = children[i]["content"].getMinWidthComposite() + children[i]["padding"] * 2;
		if( width > minWidth & width != undefined ) minWidth = width;
	}

	this._minWidth = minWidth;
	return this._minWidth;
};

/** 
 * Method to get or return minimum composite height
 * @return {Integer} The minimum composite height of the content
 * @type Integer
 */
gtk.Contents.prototype.getMinHeightComposite = function(){
	var children = this._children;
	var minHeight = this._minHeight;
	for(var i = children.length; i--;){
		var height = children[i]["content"].getMinHeightComposite() + children[i]["padding"] * 2;
		if( height > minHeight & height != undefined ) minHeight = height;
	}

	this._minHeight = minHeight;
	return this._minHeight;
};

/** 
 * Method to resize width 
 * @param {Integer} width The width to be passed
 */
gtk.Contents.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
    this._resizeWidth(minWidth);
    this._resizeChildrenWidthComposite(minWidth);
};

/** 
 * Method to resize height 
 * @param {Integer} height The height to be passed
 */
gtk.Contents.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeHeight(minHeight);
    this._resizeChildrenHeight(minHeight);
};
