/**
 * Tabs is a class to create tab for NoteBook
 * @constructor
 * @param {Integer} width The width of the widget
 * @param {Integer} height The height of the widget
 * @param {Integer} vspaceWidth The vertical space
 * @base gtk.Widget
 * @see gtk.NoteBook
 * @see gtk.Contents
 */
gtk.Tabs = function(width, height, vspaceWidth) {
	gtk.Widget.apply(this);
	
	this._children        = [];
	this._width           = width;
	this._height          = height;
	this._vspaceWidth     = vspaceWidth;
	this._activeTabDom    = null;
	this._activeTab       = "";
	this._border          = 2;
    this._loadingImgWidth = 16;
    this._tabs = [];
	this._activeTabId 	  = null;
	
	/** A registered signal name 'active' */
	this.TABS_ACTIVE 	= "";
	
	this._render = function(){
		this._domObj.className   += " gtk_tabs";
		this._domObj.style.width  = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
	};
	
	this._render();
};

gtk.Tabs.prototype = new gtk.Widget();
gtk.Tabs.prototype.constructor = gtk.Tabs;

/** 
 * Method to show loading given tab object
 * @param {Object} tab The object value of the tab content
 */
gtk.Tabs.prototype.showLoading = function(tab){
    var pageObj     = this._getPage(tab);
    var page        = tab.getNode().parentNode.parentNode;
    var loadingImg  = page.lastChild;
    var pageWrapper = page.firstChild;
    
    if(jsgtk.Dom.isShowed(loadingImg)) return;
    
    var extraWidth = this._loadingImgWidth > pageObj["padding"] / 2 ? this._loadingImgWidth : 0;

    var width = parseInt(page.style.width);    
    page.style.width = width + extraWidth + "px";
    pageWrapper.style.width = width + extraWidth + "px";

    loadingImg.style.left = pageObj["padding"] / 2 + tab.getMinWidthComposite() + "px";
    jsgtk.Dom.showElement(loadingImg);
};

/** 
 * Method to hide loading given tab object
 * @param {Object} tab The object value of the tab content
 */
gtk.Tabs.prototype.hideLoading = function(tab){
    var pageObj     = this._getPage(tab);
    var page        = tab.getNode().parentNode.parentNode;
    var loadingImg  = page.lastChild;
    var pageWrapper = page.firstChild;
    
    if(!jsgtk.Dom.isShowed(loadingImg)) return;
    
    var extraWidth = this._loadingImgWidth > pageObj["padding"] / 2 ? this._loadingImgWidth : 0;

    var width = parseInt(page.style.width);    
    page.style.width = width - extraWidth + "px";
    pageWrapper.style.width = width - extraWidth + "px";

    jsgtk.Dom.hideElement(loadingImg);
};

/** 
 * Method to set activated as default
 */
gtk.Tabs.prototype.activateDefault = function(){
    this.activate(0);
};

/** @private */ 
gtk.Tabs.prototype._deactivateTab = function(){
    this._activeTabDom.className = "gtk_page";
    jsgtk.Dom.applyAttributes(this._activeTabDom.firstChild, {"borderBottom": "1px solid #D5D5D5"});
};

/** @private */ 
gtk.Tabs.prototype._activeateTab = function(index){
    this._activeTabDom = this._children[index]["tab"]._domObj.parentNode.parentNode;
    this._activeTabDom.className = "tab_active";
    this._activeTab = this._children[index]["tab"];
    this._activeTabId = this._children[index]["id"];
    jsgtk.Dom.applyAttributes(this._activeTabDom.firstChild, {"borderBottom": "0px"});
};

/** 
 * Method to set activated given an index value
 * @param {Integer} index The index value of the tab
 */
gtk.Tabs.prototype.activate = function(index){
    if(this._activeTabDom) this._deactivateTab();
	this._activeateTab(index);
};

/** 
 * Method to set activated given an object value
 * @param {Object} label The object value
 */
gtk.Tabs.prototype.activateByObject = function(label){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === label) break;
    }
    this.activate(i);
};

/** 
 * Method to get or return an index value
 * @return {Integer} tab The index value of the tab
 */
gtk.Tabs.prototype.getIndex = function(tab){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === tab) return i;
    }
};

/** 
 * Method to add new tab given an object value
 * @param {Object} pageObj The object value
 */
gtk.Tabs.prototype.addTab = function(pageObj){
    pageObj["padding"] = pageObj["padding"] || 0;
	this._children.push(pageObj);
	this._domObj.appendChild(this._createPage(pageObj));
	this._domObj.appendChild(this._createVerticalSpace());
};

/** 
 * Method to insert a tab in the specific value
 * @param {Object} pageObj The object value
 * @param {Integer} index The index value
 */
gtk.Tabs.prototype.insertAt = function(pageObj, index){
	this._children.splice(index, 0, pageObj);
    var domSibbling = this._domObj.childNodes[index * 2];
    var vspace = this._createVerticalSpace();
    this._domObj.insertBefore(vspace, domSibbling);
    this._domObj.insertBefore(this._createPage(pageObj), vspace);
};

/** 
 * Method to remove tab from body given an index value
 * @param {Integer} index The index value to be removed
 */
gtk.Tabs.prototype.removeTab = function(index){
    jsgtk.Util.removeArrayElements(this._children, index);
    var tabIndex = index * 2;
    this._domObj.removeChild(this._domObj.childNodes[tabIndex]);
    this._domObj.removeChild(this._domObj.childNodes[tabIndex]);
};

/** @private */
gtk.Tabs.prototype._resizePage = function(pageObj, pageNode, wrapper){

	var tabWidget = pageObj["tab"];
	var size = tabWidget.getSize();
	
	pageNode.style.width  = size[0] + this._border + pageObj["padding"] + 'px';
	pageNode.style.height = size[1] + 12 + 'px';
    
    jsgtk.Util.detect.init();
    if(jsgtk.Util.detect.OS === "Windows") 	pageNode.style.top = 6 + 'px';
    else pageNode.style.top = 5 + 'px';
    
		wrapper.style.width = size[0] + this._border + pageObj["padding"] + 'px';
		wrapper.style.height = size[1] + 11 + 'px';
		
		    var labelNode = tabWidget.getNode();
		    labelNode.style.left = pageObj["padding"] / 2 + "px";
}

/** @private */
gtk.Tabs.prototype._createPage = function(pageObj){
	var pageNode = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_page','tabindex':this._tabindex});
	    this._tabs.push(pageNode)
		var wrapperNode = jsgtk.Dom.create({'append': pageNode, 'tag': 'div', 'class': 'gtk_tabs_wrapper'});		    
	    wrapperNode.appendChild(pageObj["tab"].getNode());
	    
	    var loadingImg = jsgtk.Dom.create({'append': pageNode, 'tag': 'div', 'class': 'gtk_page_loading_img'});
        jsgtk.Dom.hideElement(loadingImg);
        
    this._resizePage(pageObj, pageNode, wrapperNode);
    this._tabindex++;
    this._initPageEvents(pageNode, pageObj);
	return pageNode;
};

/** @private */
gtk.Tabs.prototype._initPageEvents = function(page, pageObj){
    var self = this;
    jsgtk.Event.addDomListener(page, 'mousedown', function(e){
        if(self._activeTabDom === page) return;

	    if(self._activeTabDom) self._deactivateTab();
	    
	    self._activeateTab(self.getIndex(pageObj["tab"]));
	    
		self.emit('active', pageObj["id"]);
	});
	
};

/** @private */
gtk.Tabs.prototype._setFocus = function(tab){
    tab.className +=" gtk_label_focus";
};

/** @private */
gtk.Tabs.prototype._unFocus = function(tab){
    tab.className +="gtk_widget gtk_label";
};

/** @private */
gtk.Tabs.prototype._createVerticalSpace = function(){
	var vspace = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_vspace'});
	vspace.style.width = this._vspaceWidth + "px";
	vspace.style.height = "100%";
	return vspace;
};

/** 
 * Method to get or return the active tab
 * @return {Object} The active object value
 * @type Object
 */
gtk.Tabs.prototype.getActiveTab = function(){
    return this._activeTab;
};

/** 
 * Method to get/return the active tab
 * @return {Integer} The index value
 * @type Integer
 */
gtk.Tabs.prototype.getActiveTabId = function(){
	return this._activeTabId;
}

/** 
 * Method to get/return all children belong to the class
 * @return {Object} The array of objects
 * @type Object
 */
gtk.Tabs.prototype.getChildren = function(){
    return this._children;  
};

/** @private */ 
gtk.Tabs.prototype._getPage = function(tab){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === tab) return this._children[i];
    }  
};

/** 
 * Method to resize width 
 * @return {Integer} The width to be passed
 */
gtk.Tabs.prototype.getMinWidthComposite = function(){
	var tabs = this._children;
	if(tabs.length > 0){
		var minWidth = 0;
		for(var i = tabs.length; i--;){
			minWidth += tabs[i]["tab"].getMinWidthComposite() + this._border + tabs[i]["padding"];
		}
		var extraWidth = this._children.length * (this._border + this._vspaceWidth) ;
		this._minWidth = minWidth + extraWidth;
		return this._minWidth;
	}
};

/** 
 * Method to resize width 
 * @param {Integer} The width to be passed
 */
gtk.Tabs.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	
	this._resizeWidth(minWidth);
};

gobject.signalNew('active', gtk.Tabs, [], null, null);
