/**
 * Frame is a class to create a frame box to display text or image
 * @param {String} label The label or text display as title of the frame
 * @base gtk.Widget
 * @constructor
 */
gtk.Frame = function(label){
	gtk.Widget.apply(this);
	
	this._label = label || " ";
    this._labelXAlign = 0;
    this._labelYAlign = 0.5;
    this._child = null;

	this._minWidth = this._minContainerWidth = this._width = 1;
	this._minHeight = this._minContainerHeight = this._height = 1;
	this._childOffsetWidth = 18;
	this._childOffsetHeight = 39;
	this._attribute;
	/** @private */
	this._render = function(){
		var domObj = this._domObj;
		domObj.className += ' gtk_frame';
		    this._labelDom = jsgtk.Dom.create({'append': domObj, 'tag' : 'div'});
		        this._label = new gtk.Label(label);
    		    this._label.setJustify(gtk.JUSTIFY_CENTER);
    		    this._labelDom.appendChild(this._label.getNode());
	
		this._minWidth = this._minContainerWidth = this._width = this._label.getWidth() + 2;
		this._minHeight = this._minContainerHeight = this._height = this._label.getHeight() + 24;

		    this._frameArea = jsgtk.Dom.create({"append": domObj,"tag":"div", "className": "gtk_frame_area"});
		    var frameStyle = this._frameArea.style;
		    frameStyle.width = this._width - 2 + "px";
		    frameStyle.height = this._height - 10 + "px";
			    this._widgetContainer = jsgtk.Dom.create({"append": this._frameArea, 'tag': 'div', 'style': 'position: relative'});
			    this._widgetContainer.style.left = "7px";

		domObj.style.height = this._height + "px";
		domObj.style.width = this._width + "px";
		this.setLabelAlign(this._labelXAlign, 0.5);
	};
	
	this._render();
};

gtk.Frame.prototype = new gtk.Widget();
gtk.Frame.prototype.constructor = gtk.Frame;

/**
 * A method to add or append child inside its body
 * @param {Dom} widget The Dom widget to be set
 */
gtk.Frame.prototype.appendChild = function(widget){
	if(typeof widget != 'undefined'){
        if(this._child !== null){
            this._widgetContainer.removeChild(this._child.getNode());
        }
        this._child = widget;
        this._widgetContainer.appendChild(widget.getNode());
            
        var size = this._getSize();
        this.resize(size[0], size[1]);
    }    
};

/**
 * A method to remove a child from its body
 * @param {Dom} widget The Dom widget to be remove
 */
gtk.Frame.prototype.removeChild = function(widget){
        this._widgetContainer.removeChild(widget.getNode());
        this._child = gtk.NullObject;
};

/**
 * A method to add the child widget into Frame
 * @param {gtk.widget} widget A child widget to be displayed in frame
 */
gtk.Frame.prototype.add = function(widget){
    this.appendChild(widget);
	console.log("gtk.Frame.add() will be deprecated, please use appendChild()");
};

/**
 * A method to set the text of the label as specified by label. If label is None the current label is removed
 * @param {String} label A string to be used as the label text
 */

gtk.Frame.prototype.setLabel = function(label){
    this._label.setText(label);
    this._resetMinWidthMinHeight(this._label);
    this.setLabelAlign(this._labelXAlign, this._labelYAlign);
};

/**
 * A method to return the text in the label widget
 * @return {String} The text in the label, or None if there is no label widget.
 * @type String
 */
gtk.Frame.prototype.getLabel = function(){
    return this._label.getText();
};

/**
 * A method to set the label widget (usually to something other than a gtk.Label widget) for the frame
 * @param {gtk.Widget} widget The new label widget
 */
gtk.Frame.prototype.setLabelWidget = function(widget){
    this.setLabel(widget.getText());
    this._labelWidget = widget;
	this._domObj.style.width = 	this._labelWidget._minWidth + 2 + "px";
	this._frameArea.style.width = this._labelWidget._minWidth + "px";
};

/**
 * A method to retrieve the label widget for the frame
 * @return {Dom} The label widget, or null if there is no label widget
 * @type Dom
 */
gtk.Frame.prototype.getLabelWidget = function(){
    return this._labelWidget;
};

/**
 * A method to set the alignment of the frame's label widget and decoration (defaults are 0.0 and 0.5) as specified by xalign and yalign
 * @param {Double} xAlign The horizontal alignment of the label widget along the top edge of the frame (in the range of 0.0 to 1.0)
 * @param {Double} yAlign The vertical alignment of the decoration with respect to the label widget (in the range 0.0 to 1.0)
 */
gtk.Frame.prototype.setLabelAlign = function(xAlign, yAlign){
    this._labelXAlign = xAlign;
    this._labelYAlign = yAlign;
    var left = (this._width - this._label.getWidth()) * xAlign -1; 
    if(left < 1) left = 1;
    this._labelDom.style.left = left + "px";
};

/**
 * A method to get or return an array containing the X and Y alignment of the frame's label widget and decoration
 * @return {Array} An Array containing the x and y alignments of the frame's label widget
 * @type Array
 */
gtk.Frame.prototype.getLabelAlign = function(){
    return [this._labelXAlign, this._labelYAlign];
}; 

/** @private */ 
gtk.Frame.prototype._getSize = function(){
    var width  = this._child.getMinWidthComposite();
    var height = this._child.getMinHeightComposite();
    width      = this._minContainerWidth  > width  ? this._minContainerWidth  : width;
    height     = this._minContainerHeight > height ? this._minContainerHeight : height;
    return [width + this._childOffsetWidth, height + this._childOffsetHeight];
};

/** @private */ 
gtk.Frame.prototype._resetMinWidthMinHeight = function(label){
    if(this._minWidth < label.getWidth()) this._minWidth = label.getWidth();
    if(this._minHeight < label.getHeight()) this._minHeight = label.getHeight();
};

/** @private */ 
gtk.Frame.prototype._setLabelXAlign = function(){
    this._labelXAlign = (this._width/2) - parseInt(this._labelDom.style.width)/2;
};

/** @private */ 
gtk.Frame.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
    this._resizeChildren(width, height);
    this.setLabelAlign(this._labelXAlign, this._labelYAlign);
};

/** @private */ 
gtk.Frame.prototype._resizeWidth = function(width){
    if(this._minWidth > width) return;
    
    if(this._width < width) this._width = width;
    else width = this._width;
    
    this._domObj.style.width = width - 2 + "px";
        this._frameArea.style.width = width - 4 + "px";
            this._widgetContainer.style.width = width - this._childOffsetWidth + 'px';
};

/** @private */ 
gtk.Frame.prototype._resizeHeight = function(height){       
    this._domObj.style.height = height - 2 + "px";
        this._frameArea.style.height = height - 12 + "px" ;
            this._widgetContainer.style.height = height - this._childOffsetHeight + 'px';
    this._height = height;
};

/** @private */ 
gtk.Frame.prototype._resizeChildren = function(width, height){
    this._child.resizeWidthComposite(width - this._childOffsetWidth);
    this._child.resizeHeightComposite(height - this._childOffsetHeight);
};

/**
 * A method to get the minimum width of frame
 * @return {Integer} The minimum width of frame
 */
gtk.Frame.prototype.getMinWidthComposite = function(){
	var minWidth = this._child.getMinWidthComposite();
    if(this._minWidth < minWidth)
		this._minWidth = minWidth;
	return this._minWidth + this._childOffsetWidth;
};

/**
 * A method to get the minimum height of frame
 * @return {Integer} The minimum height of frame
 */
gtk.Frame.prototype.getMinHeightComposite = function(){
	var minHeight = this._child.getMinHeightComposite();
    if(this._minHeight < minHeight)
		this._minHeight = minHeight;
	return this._minHeight + this._childOffsetHeight;
};

/**
 * A method to resize the width of frame
 * @param {Interger} width The length of width to be resize.
 */
gtk.Frame.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this.setWidth(minWidth);
	this.resize(minWidth, this._height);
};

/**
 * A method to resize the height of frame
 * @param {Interger} height The length of width to be resize.
 */
gtk.Frame.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite? height : minHeightComposite;
	this.resize(this._width, minHeight);
};
/**
 * A method to set and apply style given an object literal of an object
 * @param {Array} The array of attributes object styles
 */
gtk.Frame.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.Frame.prototype._applyAttribute = function(){
	for(var el in this._attribute){
	   if(el == "border"){
	    this._frameArea.style.border = this._attribute[el];
	   }else if(el == "backgroundColor"){
	        this._domObj.style[el] = this._attribute[el];
	   }
	   else{
	        this._label.getNode().childNodes[0].childNodes[0].childNodes[0].childNodes[0].style[el] = this._attribute[el];
	   }
	}
};
