/**
 * A class to create horizontal box or container
 * @constructor
 * @base gtk.Widget
 * @param {Integer} spacing
 * @param {Integer} padding
 */
gtk.HBox = function(spacing, padding) {
	gtk.Widget.apply(this);

	this._padding = (padding === undefined || padding == 0) ? 0 : padding;
	this._spacing = (spacing === undefined || spacing == 0) ? 0 : spacing;
	this._children = [];
	this._width = this._minWidth = 10;
	this._height = this._minHeight = 10;
    this._isSizeLocked = false;
    
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " hbox";
		domObj.style.width = this._width + 'px';
		domObj.style.height = this._height + 'px';
	};
	
	this._render();
	this._initEvent();
};

gtk.HBox.prototype = new gtk.Widget();
gtk.HBox.prototype.constructor = gtk.HBox;

gtk.HBox.prototype._initEvent = function(){
	var self = this;
	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked");
    });
};

/** 
 * Return all children widget
 * @return {array} children inside the HBox
 */
gtk.HBox.prototype.getChildren = function(){
	var childWidgets = [],
		children = this._children,
	 	len = children.length;
	for(var i = 0; i < len; i++){
		childWidgets.push(children[i].childWidget);
	}
	return childWidgets;
}

/** 
 * Return the first child
 * @return {gtk.Widget} widget first child inside the HBox
 */
gtk.HBox.prototype.getFirstChild = function(){
    return this._children[0]["childWidget"];
};

/** 
 * Method to set enabled
 */
gtk.HBox.prototype.enable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].enable();
    }
};

/** 
 * Method to set disabled
 */
gtk.HBox.prototype.disable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].disable();
    }
};

/** 
 * Return the last child
 * @return {gtk.Widget} widget the last child inside the HBox
 */
gtk.HBox.prototype.getLastChild = function(){
    return this._children[this._children.length - 1]["childWidget"]; 
};

/** 
 * Return a next child widget given current widget as reference
 * @return {gtk.Widget} widget the child to be returned
 */
gtk.HBox.prototype.getNextSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === this._children.length - 1) return null;
    return this._children[childIndex + 1]["childWidget"];
};

/** 
 * Return a previous child widget given a current widget as reference
 * @return {Object} widget the child to be returned
 */
gtk.HBox.prototype.getPreviousSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === 0) return null;
    return this._children[childIndex - 1]["childWidget"];
};

/** 
 * Method to refresh HBox
 * if gtk.HBox.lockSize() has been called, then the size of HBox will not decrease when some child have been remove
 * else vice versa
 */
gtk.HBox.prototype.refresh = function(){
	this._refreshSelf();
	this._refreshChildren();
};

/** @private */
gtk.HBox.prototype._refreshChildren = function(){
	for(var i = this._children.length; i--;){
		this._children[i]['childWidget'].refresh();
	}
};

/** 
 * Make sure, the HBox size will not decrease when some children have been remove
 */
gtk.HBox.prototype.lockSize = function(){
    this._isSizeLocked = true;
};

/** 
 * HBox size will decrease when some children have been remove
 */
gtk.HBox.prototype.unlockSize = function(){
    this._isSizeLocked = false;
};

/** @private */
gtk.HBox.prototype._refreshSelf = function(){
	var minWidth = this.getMinWidthComposite();
	if(this._isSizeLocked) minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	if(this._isSizeLocked) minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);    
};

/** 
 * Append child gtk.Widget to HBox
 * @param {Object} widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.appendChild = function(childWidget, packOption) {
    
    childWidget.setParent(this);

	this._children.push(this._createChildObject(childWidget, packOption));
	
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/** 
 * Method to add a separator between children widget inside HBox
 */
gtk.HBox.prototype.addSeparator = function(){
    var space = this._children[this._children.length - 1]["space"];
    jsgtk.Dom.showElement(space.childNodes[0]);
};

/** 
 * Method to clone the child widget
 * @return {Object} widget new cloned widget
 */
gtk.HBox.prototype.cloneWidget = function(){
    return jsgtk.Util.clone(this);
};

/** 
 * Insert a child widget infront of give child reference
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, packOption){
    
    childWidget.setParent(this);
	
    var index = this._isChild(oldChildWidget);
    this._children.splice(index,0, this._createChildObject(newChildWidget, packOption));
    
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/** 
 * Replace an existing child widget with new child widget
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.replaceChild = function(newChildWidget, oldChildWidget, packOption){
    newChildWidget.setParent(this);
    
    var index = this._isChild(oldChildWidget);
    if(!index) jsgtk.Util.debug("target child widget not found");
    
    var child = this._children[index];
    child["element"].insertBefore(newChildWidget.getNode(), oldChildWidget.getNode());
    child["element"].removeChild(oldChildWidget.getNode());

    child["childWidget"]    = newChildWidget;
	child["expand"]         = packOption["expand"]         === undefined ? child["expand"]         : packOption["expand"];
    child["fill"]           = packOption["fill"]           === undefined ? child["fill"]           : packOption["fill"];
	child["padding"]        = packOption["padding"]        === undefined ? child["padding"]        : packOption["padding"];
	child["resizableHeight"] = packOption["resizableHeight"] === undefined ? child["resizableHeight"] : packOption["resizableHeight"];    

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/** 
 * Remove a child widget from the HBox
 * @param {Object} widget
 */
gtk.HBox.prototype.removeChild = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    
    this._resizeWidth(this._width - parseInt(this._children[childIndex]["element"].style.width));
    
    this._domObj.removeChild(this._children[childIndex]["element"]);
    this._domObj.removeChild(this._children[childIndex]["space"]);
    
    this._children.splice(childIndex, 1);
};

/** 
 * Remove a child widget given an index
 * @param {Integer} index value of the child to be removed
 */
gtk.HBox.prototype.removeChildByIndex = function(index){
    if(!this._children[index]) jsgtk.Util.debug("The index is out of range");
    
    this._resizeWidth(this._width - parseInt(this._children[index]["element"].style.width));
    
    this._domObj.removeChild(this._children[index]["element"]);
    this._domObj.removeChild(this._children[index]["space"]);
    
    this._children.splice(index, 1);
};

/**
 * Re-order or relocatte the child widget
 * @param{gtk.Widget} widget
 * @param{integer} position The new position for child in the children list of the box starting from 0
 */
gtk.HBox.prototype.reorderChild = function(childWidget, position) {

    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    if( position >= this._children.length || position < 0 ) jsgtk.Util.debug("Position is undefined or out of range");
    if( childIndex === position ) return;

    var child = this._children[childIndex];
    var targetChild = this._children[position];

    //swapping the array
    this._children[position] = child;
    this._children[childIndex] = targetChild;

    //swapping the dom
	var tmpNode1 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});
	var tmpNode2 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});

	this._domObj.replaceChild(tmpNode2, targetChild["element"]);
	this._domObj.replaceChild(tmpNode1, child["element"]);

	this._domObj.appendChild(child["element"]);
	this._domObj.appendChild(targetChild["element"]);

	this._domObj.replaceChild(child["element"], tmpNode2);
	this._domObj.replaceChild(targetChild["element"], tmpNode1);
};

/** @private */
gtk.HBox.prototype._createChildObject = function(childWidget, packOption){

    var element = this._createChildNodeHolder(childWidget.getNode());

	var space = this._createSpace();

	this._appendChild(element, space);
	
	var childObject = {
		"element" : element,
		"childWidget"   : childWidget,
		"space"   : space,
		"expand"  : packOption["expand"] === undefined ? true : packOption["expand"],
		"fill"    : packOption["fill"] === undefined ? true : packOption["fill"],
		"padding" : packOption["padding"] === undefined ? 0 : packOption["padding"],
		"resizableHeight" : packOption["resizableHeight"] === undefined ? true : packOption["resizableHeight"] 
	};
	
	return childObject;
};

/** 
 * Return number of children inside HBox
 * @return {integer} length
 */
gtk.HBox.prototype.getChildrenLength = function(){
    return this._children.length;
};

/** @private */
gtk.HBox.prototype._isChild = function(child){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === child) return i;
    }
    return false;
};

/** @private */
gtk.HBox.prototype._appendChild = function(element, space){
    this._domObj.appendChild(element);
    this._domObj.appendChild(space);
};

/** @private */
gtk.HBox.prototype._createChildNodeHolder = function(childNode) {
    var element = jsgtk.Dom.create({tag: 'div', style: 'position: relative; float: left;'});
    element.appendChild(childNode);
	return element;
};

/** @private */
gtk.HBox.prototype._createSpace = function(){
    var space = jsgtk.Dom.create({tag: 'div', "class": "gtk_hbox_space"});
        var separator = jsgtk.Dom.create({"append": space , "tag": 'div', "class": "gtk_hbox_separator"});
	return space;
};

/** @private */
gtk.HBox.prototype._hideLastSpace = function(){
	if(this._domObj.childNodes.length >= 1)
		this._domObj.childNodes[this._domObj.childNodes.length - 1].style.width = "0px";
};

/** @private */
gtk.HBox.prototype._resizeChildrenHeight = function(height){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["element"].style.top = this._padding + "px";
        child["space"].style.top = this._padding + "px";
        if(child["resizableHeight"]){
            child["element"].style.height = height + "px";
            child["childWidget"].resizeHeightComposite(height);
            child["space"].style.height = height + "px";   
        }else{
            var childHeight = child["childWidget"].getMinHeightComposite();
            child["element"].style.height = childHeight + "px";
            child["space"].style.height = childHeight + "px";   
        }
    }
};

/** @private */
gtk.HBox.prototype.getMinHeightComposite = function(){
    var maxChildHeight = this._children[0]["childWidget"].getMinHeightComposite();
    var childHeight;
    for(var i = 1, len = this._children.length; i < len; i++){
        childHeight = this._children[i]["childWidget"].getMinHeightComposite();
        maxChildHeight = maxChildHeight < childHeight ? childHeight : maxChildHeight;
    }
    var paddings = this._padding * 2;
	
	var minHeight = maxChildHeight + paddings;
	minHeight	  = minHeight < this._minHeight ? this._minHeight : minHeight;

    return minHeight;
};

/** @private */
gtk.HBox.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;
    this._resizeHeight(minHeight);
    this._resizeChildrenHeight(minHeight - this._padding * 2);
};
gobject.signalNew('clicked', gtk.HBox, [], null, null);
