/**
 * @constructor
 * @base gtk.Widget
 * @param {String} text The text to be displayed in label
 */
gtk.Label = function(text) {
	gtk.Widget.apply(this);
	
	/** 'clicked' signal */
	this.LABEL_CLICKED = "clicked";
	/** 'mouseover' signal */
	this.LABEL_MOUSEOVER = "mouseover";
	/** 'mouseout' signal */
	this.LABEL_MOUSEOUT = "mouseout";

	this._dom = null;
	this._attribute = null;
	this._isSelectable = false;
    this._isEnable = true;

	this._render = function(){
		this._domObj.className += " gtk_label";
			var table = jsgtk.Dom.create({append: this._domObj, tag: "table", style: 'width: 100%; height: 100%;'});
				var tr = jsgtk.Dom.create({append: table, tag: "tr"});
					var td = jsgtk.Dom.create({append: tr, tag: "td", "className": "gtk_label_align"});
						this._dom = jsgtk.Dom.create({append: td, tag: "label", 'className': 'gtk_none_selectable'});
						this._dom.innerHTML = text || "";

		this._disabledLabel = jsgtk.Dom.create({append: this._domObj, tag: "div", "class": "gtk_label_disabled"});
        
        this._minWidth = this._width = this._calculateMinWidthMinHeight()[0];
        this._minHeight = this._height = this._calculateMinWidthMinHeight()[1];

        this.resizeWidthComposite(this._minWidth);
        this.resizeHeightComposite(this._minHeight);
	};
	
	this._render();
	this._initEvent();
};

gtk.Label.prototype = new gtk.Widget();
gtk.Label.prototype.constructor = gtk.Label;

gtk.Label.prototype._initEvent = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._domObj, "click", function(e){
       self.emit("clicked");
    }, false);
    
	jsgtk.Event.addDomListener(this._domObj, "mouseover", function(e){
       self.emit("mouseover");
    }, false);
    
    jsgtk.Event.addDomListener(this._domObj, "mouseout", function(e){
        if(self._isEnable) self.emit("mouseout");
    }, false);
};
/**
 * Set text to label
 * @param {String} text
 */
gtk.Label.prototype.setText = function(text){
	this._dom.innerHTML = text;
	var size = this._calculateMinWidthMinHeight();
	this.resetMinSize(size[0], size[1]);
    this.resizeWidthComposite(size[0]);
};
/**
 * Get text from label
 * @return {String} text
 */
gtk.Label.prototype.getText = function(){
	return this._dom.innerHTML;
};
/** @private */
gtk.Label.prototype.disable = function(){
	this._isEnable = false;
    this._domObj.className = "gtk_widget gtk_label gtk_label_disable";
   
	if(this._attribute) jsgtk.Dom.applyAttributes(this._dom, {"cursor":"default","color":"#ccc"});

	var domHref = this._dom.firstChild;
	if(domHref.tagName == "A") jsgtk.Dom.applyAttributes(domHref, {"cursor":"default","color":"#ccc"});	
	jsgtk.Dom.showElement(this._disabledLabel);
};
/** @private */
gtk.Label.prototype.enable = function(){
	this._isEnable = true;
	this._domObj.className = "gtk_widget gtk_label";

	if(this._isHover) this.enableHoverStyle();
	this.setAttribute(this._attribute);

	var domHref = this._dom.firstChild;
	if(domHref.tagName == "A") jsgtk.Dom.applyAttributes(domHref, {"cursor":"","color":""});
	jsgtk.Dom.hideElement(this._disabledLabel);
};
/** @private */
gtk.Label.prototype.resize = function(width, height){
	this._resizeWidth(width);
	this._resizeHeight(height);
};
/** set tab index */
gtk.Label.prototype.setTabindex = function(index){
	this._domObj.tabIndex = index;
};

/** @private */
gtk.Label.prototype._calculateMinWidthMinHeight = function(){
    var tempDom = this._dom.cloneNode(true);
        var width = jsgtk.Util.fullWidth(tempDom);
	    var height = jsgtk.Util.fullHeight(tempDom);
   
	return [width , height];
};
/** display border left yellow when mouse hovered label text */
gtk.Label.prototype.enableHoverStyle = function(){
    this._isHover = true;
    this._domObj.className += " gtk_label_enable_hover";
};
/** @private */
gtk.Label.prototype._calculateHeight = function(){
    var tempDom = this._dom.cloneNode(true);
    var tempSpan = jsgtk.Dom.create({tag: "div"});
    tempSpan.style.width = this._width + "px";
    tempSpan.appendChild(tempDom);
	var height = jsgtk.Util.fullHeight(tempSpan);
	
	return height;
};
/**
 * Set attributes(style) to label
 * @param {array} array of attributes object
 */
gtk.Label.prototype.setAttribute = function(attribute){
    if(this._isEnable){
        this._attribute = attribute;
	    jsgtk.Dom.applyAttributes(this._dom, this._attribute);

	    var size = this._calculateMinWidthMinHeight();
		this.resetMinSize(size[0], size[1]);
        this.resizeWidthComposite(size[0]);
        this.resizeHeightComposite(size[1]);
    }
};
/**
 * Get attributes
 * @return {Array} array of attributes object
 */
gtk.Label.prototype.getAttribute = function(){
	return this._attribute;
};
/**
 * Set label's text selectable
 * @param {boolean} isSelectable 
 */
gtk.Label.prototype.setSelectable = function(isSelectable){
	this._isSelectable = isSelectable;
    this._dom.className = this._isSelectable ? "gtk_selectable" : "gtk_none_selectable";
};
/**
 * Get label's text selectable
 * @return {boolean}
 */
gtk.Label.prototype.getSelectable = function(){
	return this._isSelectable;
};
/** @private */
gtk.Label.prototype.show = function(){
	if(this._domObj.style.display != 'inline') this._domObj.style.display = 'block';
};
/**
 * Set the width of Label
 * @param {Interger} width The length of width to be set
 */
gtk.Label.prototype.setWidthChars = function(width){
    this._minWidth = this._width = width;
    this._minHeight = this._height = this._calculateHeight();

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};
/**
 * Set the justify of Label
 * @param {String} justify The justify type to be set to label
 */
gtk.Label.prototype.setJustify = function(justify){
    this._dom.parentNode.style.textAlign = justify;
};
/**
 * Set the label style
 * @param {Object} properties The properties to be set to Label style
 */
gtk.Label.prototype.setStyle = function(properties){
	var domStyle = this._domObj.style;
    var labelStyle = this._dom.style;
    for(var i in properties){
    	domStyle[i] = properties[i];
    	if(i == "cursor") labelStyle[i] = properties[i];
    }
};
/** @private */
gtk.Label.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + 1 + "px";
};
/** @private */
gtk.Label.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
};
/** @private */
gtk.Label.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = width > minWidth ? width : minWidth;
    this._resizeWidth(minWidth);
};
/** @private */
gtk.Label.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = height > minHeight ? height : minHeight;
    this._resizeHeight(minHeight);
};
gobject.signalNew("clicked", gtk.Label, null, null, []);
gobject.signalNew("mouseover", gtk.Label, null, null, []);
gobject.signalNew("mouseout", gtk.Label, null, null, []);
