/**
 * @constructor
 * @base gtk.Widget
 * @param {integer} width of liquidboxWidget
 * @param {integer} space between child widget
 */
gtk.LiquidBox = function(width, space) {
	gtk.Widget.apply(this);

	/** 'refreshed' signal */
	this.LIQUIDBOX_REFRESHED = "refreshed";

	this._width          = this._minWidth = width;
	this._height         = this._minHeight = 10;
    this._spacing        = space || 0;
    this._children       = [];
    this._maxChildWidth  = 0;
    this._maxChildHeight = 0;
    this._columns        = 0;
    this._rows           = 1;
    
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " liquidbox";
	};/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} childWidget 
 * @param {object} options {'fill': true || false, 'padding': any number}
 */
	
	this._render();
};

gtk.LiquidBox.prototype = new gtk.Widget();
gtk.LiquidBox.prototype.constructor = gtk.LiquidBox;

/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} childWidget 
 * @param {object} options {'fill': true || false, 'padding': any number}
 */
gtk.LiquidBox.prototype.appendChild = function(childWidget, options){
    childWidget.setParent(this);
    
    var nodeHolder = this._createChildNodeHolder();
    nodeHolder.appendChild(childWidget.getNode());
    
    this._children.push({
        "childWidget": childWidget,
        "nodeHolder" : nodeHolder,
        "fill"       : options["fill"] || false,
        "padding"    : options["padding"] || 0
    });

    this._resetMaxChildWidth(childWidget, options["padding"] || 0 );
    this._resetMaxChildHeight(childWidget, options["padding"] || 0);
    
    this.resizeWidthComposite(this._width);
    
    var rows = Math.ceil(this._children.length / this._columns);
    if(rows > this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/**
 * Return the length of chidren
 * @return {integer} 
 */
gtk.LiquidBox.prototype.getChildrenLength = function(){
	return this._children.length;
};

/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} newChildWidget
 * @param {gtk.Widget} oldChildWidget
 * @param {object} options {'fill': true || false, 'padding': any number}
 */
gtk.LiquidBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, options){
	this.appendChild(newChildWidget, options);
	
	this._swapChildWidgetsDom(newChildWidget, oldChildWidget);
	
	this.refresh();
};

/**
 * Refresh the gtk.LiquidBox
 */
gtk.LiquidBox.prototype.refresh = function(){
    this._resetMaxChildrenWidthAndMaxChildrenHeight();
    this.resizeWidthComposite(this._width);  
};

/** @private */
gtk.LiquidBox.prototype._swapChildWidgetsDom = function(newChildWidget, oldChildWidget){
	var newChildWidgetObject = this._children[this._getIndexOfChild(newChildWidget)];
	var newChildWidgetNodeHolder = newChildWidgetObject["nodeHolder"];
	this._domObj.removeChild(newChildWidgetNodeHolder);
	
	var targetChildWidgetObject = this._children[this._getIndexOfChild(oldChildWidget)];
	var targetChildWidgetNodeHolder = targetChildWidgetObject["nodeHolder"];
	this._domObj.insertBefore(newChildWidgetNodeHolder, targetChildWidgetNodeHolder);
	
	jsgtk.Util.arraySwapElements(this._children, newChildWidgetObject, targetChildWidgetObject);
};

/**
 * Remove child from gtk.LiquidBox by index
 * @param {integer} index
 */
gtk.LiquidBox.prototype.removeByIndex = function(index){
    var childNode = this._children[index]["nodeHolder"];
    childNode.parentNode.removeChild(childNode);
    
    this._children.splice(index, 1);
    this.resizeWidthComposite(this._width);
    
    var rows = Math.ceil(this._children.length / this._columns);
    if(rows < this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/** @private */
gtk.LiquidBox.prototype._getIndexOfChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === childWidget) return i;
    }
    return -1;
}

/**
 * Remove child from gtk.LiquidBox
 * @param {gtk.Widget} widget
 */
gtk.LiquidBox.prototype.removeChild = function(object){
    var index = this._getIndexOfChild(object);
    if(index === -1) jsgtk.Util.debug("The child does not belong to the parent");
    
    this._children.splice(index, 1);
    this._domObj.removeChild(object.getNode().parentNode);
    
    this.resizeWidthComposite(this._width);
    
    var rows = Math.ceil(this._children.length / this._columns);
    if(rows < this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/** @private */
gtk.LiquidBox.prototype._resetColumns = function(width){
    this._columns = Math.floor(width / this._maxChildWidth);
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildWidth = function(childWidget, padding){
    var childWidth = childWidget.getMinWidthComposite() + padding * 2;
    this._maxChildWidth = childWidth > this._maxChildWidth ? childWidth : this._maxChildWidth;
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildHeight = function(childWidget, padding){
    var childHeight = childWidget.getMinHeightComposite() + padding * 2;
    this._maxChildHeight = childHeight > this._maxChildHeight ? childHeight : this._maxChildHeight;
};

/** @private */
gtk.LiquidBox.prototype._handleFilledChild = function(child){
    child["childWidget"].resizeWidthComposite(this._maxChildWidth - child["padding"] * 2);
    child["childWidget"].resizeHeightComposite(this._maxChildHeight - child["padding"] * 2);
    child["childWidget"].getNode().style.left = child["padding"] + "px";
    child["childWidget"].getNode().style.top  = child["padding"] + "px";
};

/** @private */
gtk.LiquidBox.prototype._handleNoneFillChild = function(child){
    var childWidth = child["childWidget"].getMinWidthComposite();
    var childHeight = child["childWidget"].getMinHeightComposite();
    child["childWidget"].getNode().style.left = this._maxChildWidth / 2 - childWidth / 2 + "px";
    child["childWidget"].getNode().style.top  = this._maxChildHeight / 2 - childHeight / 2 + "px";  
};

/** @private */
gtk.LiquidBox.prototype._resizeChildNodeHolder = function(nodeHolder){
    nodeHolder.style.width  = this._maxChildWidth + "px";
    nodeHolder.style.height = this._maxChildHeight + "px"
};

/** @private */
gtk.LiquidBox.prototype._positionChildNodeHolder = function(nodeHolder, column, rows){
    var vspacings = column > 0 ? this._spacing : 0;
    var hspacings = rows > 0 ? this._spacing : 0;
    nodeHolder.style.left = (this._maxChildWidth + vspacings) * column +"px";
    nodeHolder.style.top  = (this._maxChildHeight + hspacings) * rows + "px";
};

/** @private */
gtk.LiquidBox.prototype._resize = function(){
    
    var column = 0;
    var rows   = 0;
    
    var docFragment = document.createDocumentFragment();
    for(var i = 0, len = this._children.length; i < len; i++){

        var child = this._children[i];
        var nodeHolder = child["nodeHolder"];
        
        docFragment.appendChild(nodeHolder);
        
        this._positionChildNodeHolder(nodeHolder, column, rows);
        this._resizeChildNodeHolder(nodeHolder);
        
        if(child["fill"]) this._handleFilledChild(child);
        else this._handleNoneFillChild(child);

        var spacings = this._spacing * (this._children.length);
        this._minHeight = (this._maxChildHeight * (rows+1) + spacings);
        this._resizeHeight(this._minHeight);
        if(column < this._columns - 1){
            column++;
        }else{
            column = 0;
            rows += 1;
        }
    }
    this._domObj.appendChild(docFragment);
};

/** @private */
gtk.LiquidBox.prototype._createChildNodeHolder = function(){
    var nodeHolder = jsgtk.Dom.create({"tag": "div", "class": "gtk_liquid_box_childnode_holder"});
    nodeHolder.style.position = "absolute";
    return nodeHolder;
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildrenWidthAndMaxChildrenHeight = function(){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        this._resetMaxChildWidth(child["childWidget"], child["padding"]);
        this._resetMaxChildHeight(child["childWidget"], child["padding"]);
    }
};

/** @private */
gtk.LiquidBox.prototype.getMinHeightComposite = function(){
    return this._minHeight + this._spacing * (this._children.length - 1);
};

/** @private */
gtk.LiquidBox.prototype.getMinWidthComposite = function(){
    return this._minWidth + this._spacing * (this._children.length - 1);
};

/** @private */
gtk.LiquidBox.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);
    
    var spacings = this._spacing * (this._children.length - 1);
    this._resetColumns(minWidth - spacings);
    this._resize();
};

gobject.signalNew('refreshed', gtk.LiquidBox, null, null, []);
