/**
 * @constructor
 * @param {array} headers columns of header
 * @base gtk.Object
 */
gtk.ListStore = function(headers){
 	gtk.Object.apply(this);
	this._modelType = gtk.ListStore;
 	this.children   = new Array();
	this._headers   = headers || [];
 };
 
gtk.ListStore.prototype = new gtk.Object();
gtk.ListStore.prototype.constructor = gtk.ListStore;

/**
 * appends a new row to the liststore
 * @param {object} json object
 * @return {gtk.TreeIter}
 * @see gtk.TreeIter
 */
gtk.ListStore.prototype.append = function(row){
    var self = this;
	row["value"] = this._filterRow(row["value"]);
	var iter = new gtk.TreeIter(this.children.length.toString() + ':');
	iter.setRow(row);
	this.add(iter);
	iter.connect("valueupdated", function(path, value){
        self.emit("rowupdated", path, value);
	});
	return iter;
};

/** @private */
gtk.ListStore.prototype.appendToHeader = function(header){
    this._headers.push(header);
};

/**
 * appends new rows to the liststore
 * @param {array} arrow of json object
 * @return {array} array of gtk.TreeIter object
 * @see gtk.TreeIter
 */
gtk.ListStore.prototype.appendRows = function(rows){
    if(rows.length > 0){
        var payloadRows = [];
        
        for(var i = 0, len = rows.length; i < len; i++){
            payloadRows.push(this.append(rows[i]));
        }
        
        this.emit("rowsupdated", payloadRows);
    }
	return payloadRows;
};

/**
 * Return gtk.TreeIter object
 * @param {string} path which is id of each gtk.TreeIter object 
 * @return {gtk.TreeIter}
 */
gtk.ListStore.prototype.getChildByPath = function(path){
    for(var i = this.children.length; i--;){
        var child = this.children[i];
        if(child.getPath() === path) return child;
    }
};

/** @private */
gtk.ListStore.prototype.getModelType = function(){
	return this._modelType;
};

/** @private */
gtk.ListStore.prototype.getColumnsWidth = function(){
    var widths = [];
    for(var i = 0, len = this._headers.length; i < len; i++){
        widths.push(this._headers[i].width);
    }
    return widths;
};

/** @private */
gtk.ListStore.prototype.getWidths = function(){
    var width = 0;
    for(var i = this._headers.length; i--;){
        width += this._headers[i].width;
    }
    return width;
};

/** @private */
gtk.ListStore.prototype._filterRow = function(row){
	var filters = [];
	for(var i = 0, len = this._headers.length; i < len; i++){
		if(row[i] === null || row[i] === "" || row[i] === " " || row[i] === undefined){
			filters.push('N/A');
			continue;
		}
		filters.push(row[i]);
 	}
 	return	filters;
};

/**
 * Return all gtk.TreeIter
 * @return {array}
 */
gtk.ListStore.prototype.getChildren = function(){
	return this.children;
};

/** @private */
gtk.ListStore.prototype.getHeaders = function(){
	return this._headers;
};

/** @private */
gtk.ListStore.prototype.add = function(child){
	this.children.push(child);
};

/**
 * Remove gtk.TreeIter from gtk.ListStor
 * @param {gtk.TreeIter}
 */
gtk.ListStore.prototype.remove = function(child){
	for(var i = this.children.length; i--;){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
 * Return gtk.TreeIter by index
 * @param {integer} index
 */
gtk.ListStore.prototype.getChild = function(index){
	return this.children[index] || null;
};

/** @private */
gtk.ListStore.prototype._getColumn = function(valueMember){
    for(var i = 0, len = this._headers.length; i < len; i++){
        if(this._headers[i]["title"] === valueMember) return i;
    }
    return 0;
};

/**
 * @private
 */
gtk.ListStore.prototype.search = function(valueMember, value){
    var column = this._getColumn(valueMember);
    var re = new RegExp("^" + value.toString().toLowerCase());
    var children = this.children;
    var result = "";
    for(var i = 0, len = children.length; i < len; i++){
        result = children[i].getValue().value[column].toString().toLowerCase().match(re);
        if(result){
            return children[i];   
        }        
    }
    return children[0];
};

/** @private */
gtk.ListStore.prototype.filter = function(value, valueMember){
    var column = this._getColumn(valueMember);
    var filteredChildren = [];
    var re = new RegExp("^" + value.toString().toLowerCase());
    var children = this.children;
    var result = "";
    
    for(var i = 0, len = children.length; i < len; i++){
        result = children[i].getValue().value[column].toString().toLowerCase().match(re);
        if(result){
            filteredChildren.push(children[i]);   
        }        
    }
    return filteredChildren;
};

gobject.signalNew('rowsupdated', gtk.ListStore, [], null, null);
gobject.signalNew('rowupdated', gtk.ListStore, [], null, null);
