/**
 * Menu is a class to create a list of menu items
 * @base gtk.Widget
 * @constructor
 * @see gtk.MenuItem
 */
gtk.Menu = function() {
	gtk.Widget.apply(this);
    var self = this;
	this._menuItems        = [];
	this._separatorDoms     = [];
	this._minWidth         = 20;
	this._minHeight        = 20;
	this._menuItemActive   = false;
	this._menuItemDeactive = true;
	this._isActive         = false;
	this._width            = this._minWidth;
	this._height           = this._minHeight;
	this._selectedItem     = null;
	this._cellHeight       = 22;
	this._zIndex           = 9000;
	this._index            = -1;
	this._currentHilighted = null;
	
	/** A registered signal name 'menuselected' */
	this.MENU_MENUSELECTED		= "";
	/** A registered signal name 'menudeactivated' */
	this.MENU_MENUDEACTIVATED	= "";
	/** A registered signal name 'menuactivated' */
	this.MENU_MENUACTIVATED		= "";
	
	this._render = function(){
		this._domObj.className += " gtk_menu_shell";
		    this._hiddenEntry = jsgtk.Dom.create({"append": this._domObj, "tag": "input", "type": "text", "class": "gtk_menu_shell_hidden_textbox"});
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		this._domObj.style.zIndex = this._zIndex;
		this._domObj.style.position = "absolute";
		this.hide();
	};

	this._render();
	
	jsgtk.Event.addDomListener(this._hiddenEntry, "keyup", function(e){
	    var topIndex = self._menuItems.length - 1;
	    
	    if(e.keyCode === 13){
	        var item = self._menuItems[self._index];
	        if(!item.getSubMenu())item.activateItem(item.getText());
	    }
	    
        if(e.keyCode === 40){
            if(self._index === topIndex) self._index = -1;
            self._index++;
            self._hilightCell(self._index);
            return;
        }
        
        if(e.keyCode === 38){
            if(self._index <= 0) self._index = topIndex + 1;
            self._index--;
            self._hilightCell(self._index);
            return;
        }
        
        if(e.keyCode === 37 || e.keyCode === 39){
            var item = self._menuItems[self._index];
            if(item === undefined) return;
            if(!item.getSubMenu()){
                self.emit("menudeactivated");
                return;    
            }
            item.showSubMenu();
        }
        
        jsgtk.Event.stopBubble(e);
	});
};

gtk.Menu.prototype = new gtk.Widget();
gtk.Menu.prototype.constructor = gtk.Menu;

/**
 * A method to add or append a menu item
 * @param {Dom} menuItem The Dom object of the menu item to be set
 */
gtk.Menu.prototype.append = function(menuItem){
	var self = this;
	var domItem = menuItem.getNode();
	var separatorMargin = 0;
	this._domObj.appendChild(domItem);
	
	this._menuItems.push(menuItem);
	this._minWidth  = this._getMaxChildWidth();
	
	if(this._separatorDoms.length > 0){
		separatorMargin = (3 * this._separatorDoms.length) ;
	} 
	
	this._minHeight = (this._cellHeight * this._menuItems.length) + separatorMargin;

	this.resize(this._minWidth, this._minHeight);
	
	menuItem.connect("menumouseover", function(item){
        self._hideSubMenu(menuItem);
        self._index = self._getIndex(item);
        self._hilightCell(self._index);
        self._hiddenEntry.focus();
	});
	
	menuItem.connect("menuactivated", function(){
	    self.show();
	    self._hiddenEntry.focus();
	});
	
	menuItem.connect("menuselected", function(text){
	    self.emit("menuselected", text);
	    self._isActive = false;
	    self.hideMenuComposite();
	});
};

/** @private */
gtk.Menu.prototype._hilightCell = function(index){
    if(this._currentHilighted) jsgtk.Dom.applyAttributes(this._currentHilighted, {"backgroundColor": ""});
    this._currentHilighted = this._menuItems[index].getNode();
    jsgtk.Dom.applyAttributes(this._currentHilighted, {"backgroundColor": "#FFDD00"});
};

/** @private */
gtk.Menu.prototype._getIndex = function(item){
    for(var i = 0, len = this._menuItems.length; i < len; i++){
        if(this._menuItems[i] === item) return i;
    }
};

/**
 * A method to set selected on any menu item
 * @param {Dom} menuItem The Dom object of the menu item to be set
 */
gtk.Menu.prototype.selectItem = function(menuItem){
	for (var i = 0; i < this._menuItems.length; i++){
			if(this._menuItems[i] === menuItem) this._index = i;
	}
	this._hilightCell(this._index);	
};

/**
 * A method to hide or deactivate the menu
 */
gtk.Menu.prototype.deactivate = function(){
	this.hide();
};

/**
 * A method to add a separator amongst each menu items
 */
gtk.Menu.prototype.addSeparator = function(){
	var separator = jsgtk.Dom.create({'append':this._domObj, 'tag' : 'div'});
	separator.className += ' separator';
	this._separatorDoms.push(separator);
};

/** @private */
gtk.Menu.prototype._getMaxChildWidth = function(){
	var maxWidth = 0;
	for(var i = this._menuItems.length; i--;){
	    var itemWidth = this._menuItems[i].getMinWidthComposite();
		if(itemWidth > maxWidth) maxWidth = itemWidth;
	}
	return maxWidth;
};

/** @private */
gtk.Menu.prototype.resize = function(width,height){
	this._resizeWidth(width);
    this._resizeHeight(height);
    
    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        item.resizeWidthComposite(width);
    }
};

/** @private */
gtk.Menu.prototype._hideSubMenu = function(menuItem){
    var subMenu;
    for(var i = 0, len = this._menuItems.length; i < len; i++){
        var item = this._menuItems[i];
        if(item !== menuItem){
            subMenu = item.getSubMenu();
            if(subMenu) subMenu.hideMenuComposite();
        }
    }
};

/**
 * A method to hide the menu item
 */
gtk.Menu.prototype.hideMenuComposite = function(){
    var subMenu;
    this.hide();
    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        subMenu = item.getSubMenu();
        if(subMenu) subMenu.hideMenuComposite();
    }
};

/**
 * A method to show menu item
 */
gtk.Menu.prototype.showMenuComposite = function(){
    var subMenu;
    this.show();
    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        subMenu = item.getSubMenu();
        if(subMenu) subMenu.showMenuComposite();
    }
};

/**
 * A method to set anchor of the menu
 * @param {Integer} x The x-axis value 
 * @param {Integer} y The y-axis value 
 * @param {Integer} index The index value of the menu item
 */
gtk.Menu.prototype.anchorTo = function(x, y, index){
    if(!this._isActive){
        this.show();
        this._hiddenEntry.focus();
        
        index = index || 0;
        this.show();
        var left,top;
        
        this._zIndex += index;
        this._domObj.style.zIndex = this._zIndex;
        
        var windowWidth = jsgtk.Util.getWindowWidth();
        var newX = windowWidth - x;
        
        if(this._width > newX) left = x - this._width;
        else left = x;
        
        var windowHeight = jsgtk.Util.getWindowHeight();
        var newY = windowHeight - y;
        
        if(this._height > newY) top = y - this._height;
        else top = y;
        
        this._domObj.style.top = top + "px";
        this._domObj.style.left = left + "px";
        
        this._isActive = true;
    }else{
        this.hideMenuComposite();
        this._isActive = false;   
    }
};

/**
 * A method to resize its composite width
 * @param {Integer} width The integer width value to be set
 */
gtk.Menu.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this.resize(minWidth, this._height);
};

/**
 * A method to resize its composite height
 * @param {Integer} height The integer height value to be set
 */
gtk.Menu.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this.resize(this._width, minHeight);
};

gobject.signalNew("menuactivated", gtk.Menu, null, null, []);
gobject.signalNew("menudeactivated", gtk.Menu, null, null, []);
gobject.signalNew("menuselected", gtk.Menu, null, null, []);
