/**
 * MenuItem is a class to create a menu item
 * @param {gtk.Image} img The gtk.Image object to be set
 * @param {String} text The string text of the menu item
 * @base gtk.Item
 * @constructor
 * @see gtk.Menu
 */
gtk.MenuItem = function(img, text){
	gtk.Item.apply(this);
    var self = this;
    this._width            = this._minWidth  = 50;
    this._height           = this._minHeight = 22;
    this._imageWidgetWidth = 16;
	this._imageWidget      = img || null;
    this._labelWidget      = new gtk.Label( text || "Item");
    this._labelWidget.show();
    
	this._toggle           = false;
	this._domMenuItemLabel = null;
	this._isSubMenu        = false;
	this._subMenu          = null;
	this._callBack		   = function(){};
	this._leftPadding      = 5;
	this._rightPadding     = 10;
	this._labelPadding     = 5;
	
	/** A registered signal name 'menuselected' */
	this.MENUITEM_MENUSELECTED	= "";
	/** A registered signal name 'menumouseover' */
	this.MENUITEM_MENUMOUSEOVER		= "";
	/** A registered signal name 'menuactivated' */
	this.MENUITEM_MENUACTIVATED		= "";
	
	this._render = function(){
		this._domObj.className += ' menu_item';		    
		this._domObj.style.width = this._width + "px";
		this._domObj.style.height = this._height + "px";
		
		this._appendChildren();
	};
	
	this._render();
	
	jsgtk.Event.addDomListener(this._domObj,'mouseover',function(e){
	    self.emit("menumouseover", self);
	    jsgtk.Event.stopBubble(e);
	    if(self._subMenu){
	        self._subMenu.show();
			var position = self._getSubMenuPosition(this, self._subMenu);
		    self._subMenu.anchorTo(position[0]-3, position[1]-3, 1);
	    }
	}, false);
	
	jsgtk.Event.addDomListener(this._domObj, 'click', function(e){
	    if(!self._subMenu) self.activateItem(self._labelWidget.getText());
		jsgtk.Event.stopBubble(e);
	}, false);
};

gtk.MenuItem.prototype = new gtk.Item();
gtk.MenuItem.prototype.constructor = gtk.MenuItem;

/**
 * A method to display the sub menu
 */
gtk.MenuItem.prototype.showSubMenu = function(){
    if(this._subMenu){
        this._subMenu.show();
		var position = this._getSubMenuPosition(this._domObj, this._subMenu);
	    this._subMenu.anchorTo(position[0]-3, position[1]-3, 1);
    }
};

/** @private */
gtk.MenuItem.prototype._getSubMenuPosition = function(dom, subMenu){
    var parentNode = dom.parentNode;
    var menuItems = parentNode.childNodes;
        
    var left = ( parseInt(parentNode.style.left) || 0) + parseInt(dom.style.width) + 6;
    var windowWidth = jsgtk.Util.getWindowWidth();
    var subMenuWidth = subMenu.getMinWidthComposite();    
    var newLeft = windowWidth - left;
	if(newLeft < subMenuWidth){
        left = (parseInt(parentNode.style.left) || 0) - subMenuWidth;
    }

    var height = 0;
    for(var i = 0; i < menuItems.length; i++){
        if(this._domObj === menuItems[i]){
            break;
        }
        height += parseInt(menuItems[i].style.height) ? parseInt(menuItems[i].style.height) : 0;
    }
    var parentTop  = parseInt(parentNode.style.top);
    var windowHeight = jsgtk.Util.getWindowHeight();
    var subMenuHeight = subMenu.getMinHeightComposite();    
    var top = height + parentTop;
    var newTop = windowHeight - top;
	if(newTop < subMenuHeight){
		top = (windowHeight || 0) - subMenuHeight;
	}
	
	return [left, top];
};

/** @private */
gtk.MenuItem.prototype._appendChildren = function(){
    var width = this._labelWidget.getMinWidthComposite() + this._imageWidgetWidth + this._leftPadding + this._rightPadding + this._labelPadding;
    this._minWidth = width > this._minWidth ? width : this._minWidth;
    this.resize(this._minWidth, this._minHeight);
    
    if(this._imageWidget) {
        var imageNode = this._imageWidget.getNode();
        imageNode.style.left = this._leftPadding + "px";
        imageNode.style.top  = "2px";
        this._domObj.appendChild(imageNode);
    }
    
    var labelNode = this._labelWidget.getNode();
    this._domObj.appendChild(labelNode);
    labelNode.style.position = "absolute";
    labelNode.style.left = this._leftPadding + this._imageWidgetWidth + this._labelPadding + "px";
    labelNode.style.top  = "1px";
};

/**
 * A method to set the menu as sub menu
 * @param {Dom} menu The Dom object to be set
 */
gtk.MenuItem.prototype.setSubMenu = function(menu){
	var self = this;
	this._isSubMenu = true;
	this._subMenu = menu;
	
	this._subMenu.getNode().className += ' sub_menu';
	document.body.appendChild(this._subMenu.getNode());
	
	this._imageSubMenu = jsgtk.Dom.create({'append' : this._domObj, 'tag' : 'img', 'src' : '../../src/images/test/toSubmenu.png'});
	this._imageSubMenu.className = 'image_sub_menu';
	this._imageSubMenu.style.left = parseInt(this._domObj.style.width) - 10 + "px"; 
	
	this._subMenu.connect("menudeactivated", function(){
        self.emit("menuactivated");
        self._subMenu.hide();
	});
	
	this._subMenu.connect("menuselected", function(text){
    	self.activateItem(text);
	});
};

/** @private */
gtk.MenuItem.prototype._setSubMenuImage = function(imagePath){
	this._imageSubMenu.src = imagePath;
};

/**
 * A method to get or return menu text
 * @return {String} The string text of the menu item
 * @type String
 */
gtk.MenuItem.prototype.getText = function(){
    return this._labelWidget.getText();
};

/**
 * A method to get or return the string text of sub menu
 * @return {String} The string text of the sub menu item
 * @type String
 */
gtk.MenuItem.prototype.getSubMenu = function(){
    return this._subMenu;
};

/** @private */
gtk.MenuItem.prototype.resize = function(width, height){
	this._resizeWidth(width);
	this._resizeHeight(height);
};

/** @private */
gtk.MenuItem.prototype.removeSubMenu = function(){
	document.body.removeChild(this._subMenu.getNode());
	this._domObj.removeChild(this._imageSubMenu);
	this._subMenu = null;
};

/** @private */
gtk.MenuItem.prototype.activateItem = function(text){
    this.emit("menuselected", text);
};

/**
 * A method to resize width composite of the menu
 * @param {Integer} width The width composite of the menu
 */
gtk.MenuItem.prototype.resizeWidthComposite = function(width){
    var minWidth = width > this._minWidth ? width : this._minWidth;
    this.resize(minWidth, this._height);
};

gobject.signalNew("menumouseover", gtk.MenuItem, null, null, []);
gobject.signalNew("menuselected", gtk.MenuItem, null, null, []);
gobject.signalNew("menuactivated", gtk.MenuItem, null, null, []);
