/**
 * Panel is a class to create a container called panel
 * @param {String} title The header title of the panel
 * @param {Integer} padding The integer value of padding of the panel
 * @constructor
 * @base gtk.Widget
 * @extends gtk.Widget
 */
gtk.Panel = function(title, padding) {
	gtk.Widget.apply(this);
	
	/** 'minimzed' signal */
	this.PANEL_MINIMIZED = "minimized";
	/** 'minimizedpanelmouseovered' signal */
	this.PANEL_MINIMIZEDPANEL_MOUSEOVERED = "minimizedpanelmouseovered";
	/** 'minimizedpanelmouseovered' signal */
	this.PANEL_MINIMIZEDPANEL_MOUSEOUT = "minimizedpanelmouseout";
	/** 'restored' signal */
	this.PANEL_RESTORED = "restored";
	/** 'closed' signal */
	this.PANEL_CLOSED = "closed";

	this._width               = this._minWidth  = 20;
	this._height              = this._minHeight = 20;
	this._title               = title || "gtk Title";
	this._minimizedWidth      = 150;
	this._inconsWidth         = 45;
	this._headerHeight        = 22;
	this._padding             = padding === undefined ? 10 : padding * 2;
	this._bodyHeight          = this._height - this._headerHeight;
	this._child               = null;
	this._isCloseButtonActive = false; 
	this._minimizedPanel      = null; 
	var self = this;
	this._createTooltip = false;
	
	/** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		var domObj = this._domObj;
		
		domObj.id = new UUID().id;
		domObj.className   += " gtk_panel";
        domObj.style.position = "absolute";
		
		    this._header = jsgtkDom.create({'append': domObj, 'tag': 'div', 'className': 'gtk_panel_header'});
    		        this._domTitle = jsgtkDom.create({"append": this._header, "tag": "div", "text": this._title, 'className': 'gtk_none_selectable gtk_panel_header_title'});
                
                this._closeButton = jsgtkDom.create({"append": this._header, "tag": "div", "title": "Close Panel","className": "gtk_panel_close_button", "style": "display:none;"});
                this._minimizeButton = jsgtkDom.create({'append': this._header, 'tag': 'div', "title": "Minimize Panel", 'className': 'gtk_panel_minimize_button'});
		    
		    this._body = jsgtk.Dom.create({'append': domObj, 'tag': 'div', 'className': 'gtk_panel_body'});
	};
	
    this._render();

    jsgtk.Event.addDomListener(this._minimizeButton, "mousedown", function(e){
        self.minimize();
        jsgtk.Event.stopBubble(e);
    }, false);
};

gtk.Panel.prototype = new gtk.Widget();
gtk.Panel.prototype.constructor = gtk.Panel;

/**
 * A method to set the top most given a boolean value
 * @param {Boolean} isTop The boolean value to be topmost of not
 */
gtk.Panel.prototype.setTopMost = function(isTop){
    if(isTop) this._domObj.style.zIndex = "1001";
    else this._domObj.style.zIndex = "0";
};

/** 
 * A method to hide the minimized button from the panel
 */
gtk.Panel.prototype.hideMinimizeButton = function(){
    this._minimizeButton.style.display = "none";
};

/** @private */
gtk.Panel.prototype._isNodeExistInMinimizedHolder = function(holder){
    var clonedHolder = holder.cloneNode(true);
    for(var i = 0, len = clonedHolder.childNodes.length; i < len; i++){
        if(this._domObj.id == clonedHolder.childNodes[i].id) return true;
    }
    return false;
}

/** @private */
gtk.Panel.prototype._createMinimizedHolder = function(){
    var id = "gtk_minimized_widget_holder";
    var holder = document.getElementById(id);
    if(!holder) var holder = jsgtk.Dom.create({"append": document.body, "tag": "div", "id": id});
    return holder;
};

/**
 * A method to call setting the panel to minimized state
 */
gtk.Panel.prototype.minimize = function(){
    jsgtk.Dom.hideElement(this._domObj);

    var holder = this._createMinimizedHolder();
    if(this._isNodeExistInMinimizedHolder(holder)){
        jsgtk.Dom.showElement(this._minimizedPanel);    
        return;   
    }

    this._minimizedPanel = this._createMinimizedPanel();
    holder.appendChild(this._minimizedPanel);    
    
    this._initMinimizedEvents();
};

/** @private */
gtk.Panel.prototype._initMinimizedEvents = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    var restoreButton = this._minimizedPanel.childNodes[2];
    jsgtkEvent.addDomListener(restoreButton, "mousedown", function(){
		self.restore();
    });
    
    jsgtkEvent.addDomListener(restoreButton, "mouseover", function(){
		restoreButton.setAttribute("title", "Restore Panel");
    });
    	
	var closeButton = this._minimizedPanel.childNodes[1];
	jsgtkEvent.addDomListener(closeButton, "mousedown", function(e){
	    self._minimizedPanel.parentNode.removeChild(self._minimizedPanel);
	    self.emit("closed", self);
	    jsgtkEvent.stopBubble(e);
	});
    
    
    jsgtkEvent.addDomListener(this._minimizedPanel, "dblclick", function(e){
        self.restore();
    });
    jsgtk.Event.addDomListener(this._minimizedPanel, "mouseover", function(e){
        self.emit("minimizedpanelmouseovered", self._title);
        jsgtk.Event.stopBubble(e);
    });
    jsgtk.Event.addDomListener(this._minimizedPanel, "mouseout", function(e){
        self.emit("minimizedpanelmouseout", self._title);
        jsgtk.Event.stopBubble(e);
    });
    self.emit('minimized', self);
};

/** 
 * A method to justified title of the panel
 * @param {String} justify The contant varaiable of gtk
 */
gtk.Panel.prototype.setTitleJustify = function(justify){
    this._header.childNodes[0].style.textAlign = justify;
};

/** @private */
gtk.Panel.prototype._isPanelRestored = function(id){
    if(!document.getElementById(id)){
        this._domObj.id = id;
        return true;
    }
    return false;
};

/** 
 * A method to restore the state of the panel to the previsous state
 */
gtk.Panel.prototype.restore = function(){
    var id = this._domObj.id;
    this._domObj.id = "";
    if(this._isPanelRestored(id)) return;
    this.emit('restored', this);

    jsgtk.Dom.showElement(this._domObj);
    this._resizeDomTitleWidth(this._width - this._inconsWidth);
    
    jsgtk.Dom.hideElement(this._minimizedPanel);
    this._domObj.id = id;
};

gtk.Panel.prototype.setCloseButton = function(isClose){
	isClose ? this.hideCloseButton() : this.showCloseButton();
};

/** @private */
gtk.Panel.prototype.hideCloseButton = function(){
    if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "5px";		
	}
	else{
		this.hideCloseButton();
	} 
};

/** @private */
gtk.Panel.prototype.showCloseButton = function(){
    var self = this,
        jsgtkEvent = jsgtk.Event, 
        jsgtkDom = jsgtk.Dom;
	
    jsgtkDom.showElement(this._closeButton);
	this._minimizeButton.style.right = "22px";
	
	jsgtkEvent.addDomListener(this._closeButton, "click", function(e){
	    self._domObj.parentNode.removeChild(self._domObj);
	    self.emit("closed", self);
	    self._domObj = null;
	    jsgtkEvent.stopBubble(e);
	});
};

/**
 * A method to hide/remove close button from panel
 */
gtk.Panel.prototype.hideCloseButton = function(){
	if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "5px";		
	}
};

/**
 * A method to get/return the title of the panel
 * @return {String} The name of the panel
 * @type String
 */
gtk.Panel.prototype.getTitle = function(){
    return this._title;
};

/** 
 * A method to append or add a new child into the panel
 * @param {Dom} widget The dom widget to be passed for adding
 */ 
gtk.Panel.prototype.appendChild = function(widget){
    this._child = widget;
    this._child.setParent(this);
    this._positionChild(widget.getNode());
    
    this.resizeHeightComposite(this._height);
    this.resizeWidthComposite(this._width);
};

/** @private */
gtk.Panel.prototype._positionChild = function(childNode){
    this._body.appendChild(childNode);
    var childNodeStyle = childNode.style;
    childNodeStyle.left = this._padding / 2 + "px";
    childNodeStyle.top  = this._padding / 2 + "px";
};

/**
 * A method to flush the child inside the panel
 */
gtk.Panel.prototype.flushChild = function(){
    this._body.removeChild(this._child.getNode());
    this._child = null;  
};

/**
 * A method to set the panel title to bold
 * @param {Boolean} bold The boolean value to display bold or not
 */
gtk.Panel.prototype.setBoldTitle = function(bold){
	if(bold){
		this._domTitle.style.fontWeight = "bold";
	} else 	this._domTitle.style.fontWeight = "normal";
};

/** @private */
gtk.Panel.prototype._resetTitle = function(title){
    this._domTitle.innerHTML = title;
};

/** 
 * A method to set the new title to the panel
 * @param {String} title The string title of the panel
 * @param {String} xalign The constant variable of jsgtk
 */
gtk.Panel.prototype.setTitle = function(title, xalign){
    this._title = title;
    var xalign = xalign || 1.0;
    this._domTitle.innerHTML = this._title;
};

/** @private */
gtk.Panel.prototype._resizeDomTitleWidth = function(width){
    this._domTitle.style.width = width + "px";
};

/** @private */
gtk.Panel.prototype._createMinimizedPanel = function(){
    this._resizeDomTitleWidth(this._minimizedWidth - this._inconsWidth);

    var minimizePanel = this._header.cloneNode(true);
	minimizePanel.className = "gtk_panel_minimize";
    minimizePanel.removeAttribute("style");
    minimizePanel.style.width = this._minimizedWidth + "px";
    minimizePanel.id = this._domObj.id;
        
    return minimizePanel;
};

/** @private */
gtk.Panel.prototype._getSubElementsHeight = function(){
    return this._headerHeight + this._subHeaderHeight + this._footerHeight;  
};

/**
 * A method to resize both width and height the panel
 * @param {Integer} width The integer width value to be set
 * @param {Integer} height The integer height value to be set
 */
gtk.Panel.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Panel.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width    = this._width - 2 + 'px';
	    this._domTitle.style.width = this._width - this._inconsWidth + "px";
    	this._body.style.width      = this._width - 3 + 'px';
};

/** @private */
gtk.Panel.prototype._resizeHeight = function(height){
	this._height = height;
	this._bodyHeight = this._height - this._headerHeight;
	this._domObj.style.height    = this._height - 2 + "px";
	this._body.style.height      = this._bodyHeight + "px";
};

/**
 * A method to get/return body height of the panel
 * @return {Integer} The body height of the panel
 * @type Integer
 */
gtk.Panel.prototype.getBodyHeight = function(){
    return this._bodyHeight;
};

/**
 * A method to get/return the minimum composite width
 * @return {Integer} The minimum composite width of the panel
 * @type Integer
 */
gtk.Panel.prototype.getMinWidthComposite = function(){
    return this._child.getMinWidthComposite() + this._padding;
};

/**
 * A method to get/return the minimum composite height
 * @return {Integer} The minimum composite height of the panel
 * @type Integer
 */
gtk.Panel.prototype.getMinHeightComposite = function(){
    return this._child.getMinHeightComposite() + this._headerHeight + this._padding;
};

/**
 * A method to resize its composite width
 * @param {Integer} The composite width to be set
 */
gtk.Panel.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._child.resizeWidthComposite(minWidth - this._padding);
    this._resizeWidth(minWidth);
};

/**
 * A method to resize its composite height
 * @param {Integer} The composite height to be set
 */
gtk.Panel.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._child.resizeHeightComposite(minHeight - this._padding - this._headerHeight);
    this._resizeHeight(minHeight);
};

gobject.signalNew('minimized', gtk.Panel, [], null, null);
gobject.signalNew('minimizedpanelmouseovered', gtk.Panel,[], null, null);
gobject.signalNew('minimizedpanelmouseout', gtk.Panel, [], null, null);
gobject.signalNew('restored', gtk.Panel, [], null, null);
gobject.signalNew('closed', gtk.Panel, [], null, null);
