/**
 * Scroller is a class to create a scroller with horizontal and vertical scrollbars
 * @constructor
 * @base gtk.Widget
 */
gtk.Scroller = function() {
	gtk.Widget.apply(this);

	/** 'scrolled' signal */
	this.SCROLLER_SCROLLED = "scrolled";

	this._width          = this._minWidth  = 100;
	this._height         = this._minHeight = 50;
	this._childResizable = false;
	this._isScrolledX    = true;
	this._isScrolledY    = true;
	this._vScrolledBar   = null;
	this._child          = null;
	this._scrollYDefault = 0;
	this._attribute		= null;
	this._padding	= 0;
	
	this._render = function(){
        this._domObj.className       += " gtk_scroller_base";
        this._domObj.style.width      = this._width + "px";
        this._domObj.style.height     = this._height + "px";
	};
	this._render();
	this._initEvent();
};

gtk.Scroller.prototype = new gtk.Widget();
gtk.Scroller.prototype.constructor = gtk.Scroller;

/**
 * Append child to scroller
 * @param {gtk.Widget} widget
 * @param {boolean} resizable true child expand false vice versa
 * @param {integer} padding
 */
gtk.Scroller.prototype.appendChild = function(widget, resizable, padding){
    this._childResizable = resizable === undefined || resizable === null ? false : resizable;
    
   this._padding = (typeof padding !== "undefined") ? padding : 0;
      
	this._child = widget;
	this._domObj.appendChild(widget._domObj);
	
	if(typeof padding !== "undefined") this.setAttribute({padding: padding + "px"});
	
    this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);

	this._child.setParent(this);
};

/** @private */
gtk.Scroller.prototype.add = function(widget, resizable){
	this.appendChild(widget, resizable);
	console.log("gtk.Scroller.add(), will be depricated in next release, please use gtk.Scroller.appendChild() instead");
};

/**
 * Scroll to top
 */
gtk.Scroller.prototype.scrollToTop = function(){
    this.scrollTo(null, 0);
};

/**
 * Scroll to bottom
 */
gtk.Scroller.prototype.scrollToBottom = function(){
	var childHeight = this._child.getMinHeightComposite();

    this.scrollTo(null, this._height > childHeight ? this._height : childHeight);
};

/**
 * Scroll to left
 */
gtk.Scroller.prototype.scrollToLeft = function(){
    this.scrollTo(0);
};

/**
 * Scroll to right
 */
gtk.Scroller.prototype.scrollToRight = function(){
    this.scrollTo(this._width);
};

/**
 * Scroll to right x pixel, Scroll to bottom y pixel
 * @param {integer} x
 * @param {integer} y
 */
gtk.Scroller.prototype.scrollTo = function(x , y){
    this._domObj.scrollTop = y;
    this._domObj.scrollLeft = x;
};

/**
 * Return child widget gtk.Widget
 * @return {gtk.Widget}
 */
gtk.Scroller.prototype.getChildren = function(){
	return this._child;
};

/** @private */
gtk.Scroller.prototype._initEvent = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this._domObj, "scroll", function(e){
        var scrollY = _self._domObj.scrollTop;
        var scrollHeight = _self._domObj.scrollHeight;
        var minScrolledY = scrollHeight-(scrollHeight - _self._domObj.scrollTop);
        if(scrollY > 0 && minScrolledY >= 15){
            _self.emit("scrolled");
        }
    });
};

/**
 * Refresh gtk.Scroller and its child widget
 */
gtk.Scroller.prototype.refresh = function(){
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

/**
 * Show loading bar
 */
gtk.Scroller.prototype.showLoading = function(){
    if(this._isScrolledY) this._domObj.style.overflowY = "hidden";
    if(this._isScrolledX) this._domObj.style.overflowX = "hidden";

    if(!this._loadingImgContainer){
		this._loadingImgContainer = this._constructLoading();
        this._appendLoading();
    }else this._loadingImgContainer.style.display = "table";
};


/**
 * Hide loading bar
 */
gtk.Scroller.prototype.hideLoading = function(){
    if(this._isScrolledY) this._domObj.style.overflowY = "auto";
    if(this._isScrolledX) this._domObj.style.overflowX = "auto";
    
    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * Remove event from scroller
 */
gtk.Scroller.prototype.removeScrolledEvent = function(){
    jsgtk.Event.removeListener(this._onScroll);
};

/** @private */
gtk.Scroller.prototype.getScrolledDom = function(){
    return this._domObj;
};

/**
 * Set style to the scroller
 * @param {object} attribute {'height': ..px, 'border': '...', 'fontColor': ..}
 */
gtk.Scroller.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
	
	this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);
};

/** @private */
gtk.Scroller.prototype._applyAttribute = function(){
	for(var el in this._attribute){
		if(el == "padding") this._padding = parseInt(this._attribute[el]);
		this._domObj.style[el] = this._attribute[el];
	}
		
};

/**
 * Disable horizontal scrollbar
 */
gtk.Scroller.prototype.disableHScrolled = function(){
    this._domObj.style.overflowX = "hidden";
    this._isScrolledX = false;  
};

/**
 * Disable vertical scrollbar
 */
gtk.Scroller.prototype.disableVScrolled = function(){
    this._domObj.style.overflowY = "hidden";
    this._isScrolledY = false;
};

/**
 * Enable horizontal scrollbar
 */
gtk.Scroller.prototype.enableHScrolled = function(){
    this._domObj.style.overflowX = "auto";
    this._isScrolledX = true;
};

/**
 * Enable vertical scrollbar
 */
gtk.Scroller.prototype.enableVScrolled = function(){
    this._domObj.style.overflowY = "auto";
    this._isScrolledY = true;
    this._width = this._width - 200;
    this._resize(this._width, this._height);
};

/** @private */
gtk.Scroller.prototype.getScrollHeight = function(){
    return this._child.getHeight();  
};

/** @private */
gtk.Scroller.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Scroller.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width            = width + "px";
};

/** @private */
gtk.Scroller.prototype._resizeHeight = function(height){
    this._height = height;
	this._domObj.style.height            = height + "px";
	if(!this._isScrolledY && this._child !== null){
	    this._child._domObj.style.height = this._height - 2 + "px";
	}
};

/** @private */
gtk.Scroller.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/** @private */
gtk.Scroller.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.Scroller.prototype.resizeWidthComposite = function(width){
	var _width = width - this._padding*2;
	
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = _width > minWidthComposite ? _width : minWidthComposite;

	this._resizeWidth(minWidth);

	if(this._childResizable){
	    var childMinWidth = this._child.getMinWidthComposite();
	    if(minWidth > childMinWidth) this._child.resizeWidthComposite(minWidth);
	    else this._child.resizeWidthComposite(childMinWidth);
	}
	if(this._child instanceof gtk.ListView) this._child.setParentWidth(minWidth);
};

/** @private */
gtk.Scroller.prototype.resizeHeightComposite = function(height){
	var _height = height - this._padding*2;
	
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = _height > minHeightComposite ? _height : minHeightComposite;

	this._resizeHeight(minHeight);
	
    if(this._childResizable){
	    var childMinHeight = this._child.getMinHeightComposite();
	    if(minHeight > childMinHeight) this._child.resizeHeightComposite(minHeight);
	    else this._child.resizeHeightComposite(childMinHeight);
	}
};

gobject.signalNew("scrolled", gtk.Scroller, null, null, []);
