/**
 * TreeStore is a helper or model or subordinate class used to create a model for other class applying in
 * MVC (Model View Controller) and MVP (Model View Presenter) as development concept.
 * @constructor
 */
gtk.TreeStore = function(header){
 	gtk.Object.apply(this);
 	
	this._modelType = gtk.TREESTORE;
	this._header = header || "Header";
 	this.children = [];
};
 
gtk.TreeStore.prototype = new gtk.Object();
gtk.TreeStore.prototype.constructor = gtk.TreeStore;

/**
 * Method to append a new row to the liststore
 * @param {Object} parent An object instantiated from gtk.TreeIter
 * @param {Object} row A tuple or list containing ordered column values to be set in the new row
 * @return {object} A gtk.TreeIter pointing to the new row
 * @type Object
 */
gtk.TreeStore.prototype.append = function(parent, row){
	if(parent){
		var path = parent.getPath() + parent.getChildren().length.toString() + ':';
		var iter = new gtk.TreeIter(path);
		iter.setRow(row);
		iter.hasNode = row["hasChild"] || false;
		parent.add(iter);
		parent.hasNode = true;
	}else{
		var iter = new gtk.TreeIter(this.children.length.toString() + ':');
		iter.setRow(row);
		iter.hasNode = row["hasChild"] || false;
		this.add(iter);
	}
	return iter;
};

/**
 * Method to append a new node
 * @param {Object} parent An object instantiated from gtk.TreeIter
 * @param {Object} row A tuple or list containing ordered column values to be set in the new row
 * @return {Object} All children models
 * @type Object
 */
gtk.TreeStore.prototype.appendNodes = function(parent, rows){
    if(rows.length === 0) return;
    var childrenModel = [];
    for(var i = 0, len = rows.length; i < len; i++){
        childrenModel.push(this.append(parent, rows[i]));
    }
    this.emit("nodesappended", parent);
    return childrenModel;
};

/**
 * Method to get a model type or name
 * @return {String} A type of model name
 * @type String
 */
gtk.TreeStore.prototype.getModelType = function(){
	return this._modelType;
};

/**
 * Method to get a child given its value member
 * @param {String} valueMember A name of its valued member
 * @param {String} value A value to be set
 * @return {Object} A model of the child
 * @type Object
 */
gtk.TreeStore.prototype.getChldBy = function(valueMember, value){
	for(var i = this.children.length; i--;){
		var childModel = this.children[i].getChldBy(valueMember, value);
		if(childModel) return childModel;
	}
};

/**
 * Method to get a child given its index value
 * @param {Integer} index An index value to be passed
 * @return {Object} An object of a child
 * @type Object
 */
gtk.TreeStore.prototype.getChildByIndex = function(index){
	if(index < 0 || index > this.children.length) return null;
	return this.children[index];
};

/**
 * Method to get a child given its path value
 * @param {String} path A path value to be looked for
 * @return {Object} An object of a child
 * @type Object
 */
gtk.TreeStore.prototype.getChildByPath = function(path){
    var paths = path.split(":");
    var child = this._getChildByPath( parseInt(paths[0]) );

    paths.splice(0,1);
    if(paths.length === 1) return child;
    
    return child.getChildByPath(paths.join(":"));
};

/** @private */
gtk.TreeStore.prototype._getChildByPath = function(path){
	for(var i = this.children.length; i--;){
		var childPath = this.children[i].path.split(":");
		if(parseInt(childPath[0]) === path) return this.children[i];
	}
	return null;
};

/**
 * Method to get all children
 * @return {Array} An array of objects of a children
 * @type Array
 */
gtk.TreeStore.prototype.getChildren = function(){
	return this.children;
};

/**
 * Method to push or append a child to be the last child in an array
 * @param {Object} childModel The object of a child to be passed
 */
gtk.TreeStore.prototype.add = function(child){
	this.children.push(child);
};

/** @private */
gtk.TreeStore.prototype._searchByValueMember = function(valueMember, value, children){
	var child;
	
	var children = children || this.children;
	for(var i = 0, len = children.length; i < len; i++){
		child = children[i];
		if(child._value[valueMember] === value){
			return child;
		}
	}
	
	for(var i = 0, len = children.length; i < len; i++){
		child = children[i];
		if(child.hasNode) {
			child = this._searchByValueMember(valueMember, value, child.getChildren());
			if(child) return child;
		}
	}
};

/**
 * Method to search and filter child/children
 * @param {String} value The object of a child to be passed
 * @return {Object} A found object from all children
 * @type Object
 */
gtk.TreeStore.prototype.filter = function(value){
    if(value.length === 0) return null;
    
    var matchedChildren = [];
    var re = new RegExp("^" + value.toLowerCase());
    for(var i = 0, len = this.children.length; i < len; i++){
    	var child = this.children[i];
    	result = child.getValue()["display"].toLowerCase().match(re);
    	if(result) matchedChildren.push(child);
    }
    return matchedChildren;
};

/**
 * Method to remove or delete a child from the list of children
 * @param {Object} child The object of a child to be removed
 */
gtk.TreeStore.prototype.remove = function(child){
	for(var i = 0, len = this.children.length; i < len; i++){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
 * Method to get a child given an index value
 * @param {Integer} index The integer value as an index of the to-be-wanted child
 * @return {Object} The object of the child to be returned
 * @type Object
 */
gtk.TreeStore.prototype.getChild = function(index){
	return this.children[index] || null;
};

/**
 * Method to search through all children given text and all children
 * @param {String} text The string value to be wanted to search for
 * @param {Array} children The array of children objects
 * @return {Object} The object of the found child
 * @type Object
 */
gtk.TreeStore.prototype.search = function(text, children){
    var re = new RegExp("^" + text.toLowerCase());
    var children = children || this.children;
    var result = "";

    for(var i = 0, len = children.length; i < len; i++){
    	var child = children[i];
		result = child._search(text);
		if(result) return result;
    }
};

gobject.signalNew('nodesappended', gtk.TreeStore, [], null, null);
