/**
 * UploadButton is a class to create an upload button
 * @param {String} label The label or text display inside the upload button
 * @param {String} stock The stock value to be set
 * @base gtk.Button
 * @constructor
 */
gtk.UploadButton = function(label){
    gtk.Button.apply(this, [label]);
    
    var self = this;
    this._imageShapeDom = null;
    
    /** A registered signal name '<b>filechanged</b>' when file is changed */
    this.UPLOADBUTTON_FILECHANGED 	= "";
    /** A registered signal name '<b>mouseover</b>' */
    this.UPLOADBUTTON_MOUSEOVER 	= "";
    /** A registered signal name '<b>mouseout</b>' */
    this.UPLOADBUTTON_MOUSEOUT 	= "";
    
    /** @private */
    this._render = function(){
	    this._domObj.className += " gtk_upload_button";
	    this._domObj.style.display = "";

	    this._domImage = jsgtk.Dom.create({"tag": "div", "append": this._domObj, "className": "gtk_uploadbutton_image"});
	    this._className = this._domObj.className;
	    this._inputFile = jsgtk.Dom.create({"tag": "input", "type": "file", "append": this._domObj, "className": "gtk_upload_button_input_file"});
	    this._inputFile.style.width = this._width + "px";
	    this._inputFile.style.height = this._height + "px";
	};
   this._render(); 
   
   jsgtk.Event.addDomListener(this._inputFile, "change", function(){	
		self.emit("filechanged", self._inputFile.value);
	});

	jsgtk.Event.addDomListener(this._domObj, "mouseover", function(){	
		self.emit("mouseover");
	});
	
	jsgtk.Event.addDomListener(this._domObj, "mouseout", function(){	
		self.emit("mouseout");
	});
};

gtk.UploadButton.prototype = new gtk.Button();
gtk.UploadButton.prototype.constructor = gtk.UploadButton;

/**
 * A method to set the name of upload button
 * @param {String} name The name of upload button
 */
gtk.UploadButton.prototype.setName = function(name){
    this._inputFile.setAttribute("name", name);
};

/**
 * A method to set the text of upload button
 * @param {String} strName The name of upload button
 */
gtk.UploadButton.prototype.setText = function(strName){
    if(strName != null || typeof strName !== 'undefined') {
		this.setTextLabel(strName);
		this._inputFile.style.width = this._width - 2 + "px";
	}
};

/**
 * A method to get an upload object as Dom
 * @return {Dom} The Dom object of upload button
 * @type Dom
 */
gtk.UploadButton.prototype.getUploadNode = function(){
    console.log("gtk.UploadButton.getUploadNode() will be depricated, please use gtk.UploadButton.getChildNode()");
    return this.getChildNode();
};

/**
 * A method to get object of the upload input file
 * @return {Dom} The object of upload file
 * @type Dom
 */
gtk.UploadButton.prototype.getChildNode = function(){
    return this._inputFile;
};

/**
 * A method to show loading image inside the button
 */
gtk.UploadButton.prototype.showLoading = function(){
	this._labelDom.style.display = "none";
	this._imageDom.style.display = "none";
	this._inputFile.style.display = "none";
	if(!this._loading) this._loading = jsgtk.Dom.create({tag:"div", className: "gtk_button_loading", append: this._domObj});
};

/**
 * A method to transform or chagne upload button to image
 * @param {gtk.Image} imageObj The gtk.Image object to display to be set
 */
gtk.UploadButton.prototype.setAsImage = function(imageObj){
	if(this._imageShapeDom !== null) {
		this._domImage.innerHTML = "";
		this._imageShapeDom = null;
	}
	//this._inputFile.value = null;
	this._imageShapeDom = imageObj;
	this._domObj.className += " gtk_button_with_image_shape";
	this._gradient.style.display = "none";
	this.setTextLabel("");
	this._domImage.appendChild(imageObj.getNode());
	imageObj.resetSize(imageObj._width, imageObj._height);

	this._width	= this._minWidth	= imageObj._width;
	this._height	= this._minHeight	= imageObj._height;

	this.resizeWidthComposite(imageObj._width);
	this.resizeHeightComposite(imageObj._height);
};

/** @private */
gtk.UploadButton.prototype._getActualImageSize = function(imageObj){
	var imageNode = imageObj.getChildNode();
	var tmpImgObj = new Image();
		 tmpImgObj.src = imageNode.src;
	return [tmpImgObj.width, tmpImgObj.height];
};

/** @private */
gtk.UploadButton.prototype._getPaddingAndBorderWidth = function(){
	var attribute = this._attribute;
	var paddingWidth 	= 0;
	var borderWidth	= 0;
	for(var el in attribute){
		if(el == "padding") paddingWidth = parseInt(attribute[el]) * 2;
		if(el == "border") borderWidth = parseInt(attribute[el]) * 2;
	}
	return [paddingWidth, borderWidth];
};

/**
 * A method to set attributes and apply styles
 * @param {Object} attribute The object literal that contain JavaScript styles
 */
gtk.UploadButton.prototype.setAttribute = function(attribute){
	this._attribute 	= attribute;
	var paddingWidth 	= 0;
	var borderWidth	= 0;
	for(var el in attribute){
		this._domObj.style[el] = attribute[el];
		if(el == "padding") paddingWidth = parseInt(attribute[el]);
		if(el == "border") borderWidth = parseInt(attribute[el]);	
	}	
	this._domImage.style.top = paddingWidth + "px";
	this._domImage.style.left = paddingWidth + "px";
	
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * A method to disable the control of upload button
 */
gtk.UploadButton.prototype.disable = function(){
	this._inputFile.disabled = false;
	if(this._domImage.childNodes.length > 0){
			this._domObj.className += " gtk_button_disabled gtk_upload_button_disabled";
			this._imageShapeDom.getChildNode().style.opacity = 0.50;
	}
	else this._domObj.className += " gtk_button_disabled";
	
	this._gradient.className += " gradisable";
	this._inputFile.style.display = "none";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "default"});
	
};

/**
 * A method to set enabled to the button
 */
gtk.UploadButton.prototype.enable = function(){
	this._inputFile.disabled = false;
	var uploadBtnClass = "gtk_widget gtk_container gtk_button gtk_upload_button";
	if(this._domImage.childNodes.length > 0) {
			uploadBtnClass = "gtk_widget gtk_container gtk_upload_button";
			this._imageShapeDom.getChildNode().style.opacity = 1;
	}
	this._domObj.className = uploadBtnClass;	
	this._gradient.className = "gtk_button_gradient";
	this._inputFile.style.display = "";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "default"});
	
};

/**
 * A method to hide loading image of upload button
 */
gtk.UploadButton.prototype.hideLoading = function(){
	this._labelDom.style.display = "";
	this._imageDom.style.display = "";
	this._inputFile.style.display = "";
	if(this._loading){
		this._domObj.removeChild(this._loading);
		this._loading = null;
	} 
};

/** @private */
gtk.UploadButton.prototype._resizeHeight = function(height){
	this._domObj.style.height = this._height + "px";
	this._domObj.childNodes[0].style.height = this._height / 2 + "px";	
	this._height = height;
};

/** @private */
gtk.UploadButton.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this.borderWidth + "px";
	if(typeof this._domImage !== 'undefined'){
		if(this._domImage.childNodes.length > 0) {
			this._domObj.style.width = width + "px";
		}
	}	
	this._width = width;
};

/**
 * A method to resize the width of the upload button
 * @param {Interger} width The length of width to be resized.
 */
gtk.UploadButton.prototype.resizeWidthComposite = function(width){
	this.setWidth(width);
	this.resize(width, this._height);
	this._inputFile.style.width = width + "px";
	this._domObj.style.width = width + "px";
	
	if(typeof this._domImage !== 'undefined'){
		if(this._domImage.childNodes.length > 0) {
			this._imageShapeDom.resetSize(width, this._height);
			this._domObj.style.top = "0px";
			//var actualImageWidth = this._imageShapeDom.width;
			//this._domImage.style.left = (width - actualImageWidth)/2 + "px";
		}
	}
};

/**
 * A method to resize the height of upload Button
 * @param {Interger} height The length of width to be resized.
 */
gtk.UploadButton.prototype.resizeHeightComposite = function(height){
	this.setHeight(height);
	this.resize(this._width, height);
	var inputFileStyle = this._inputFile.style;
	inputFileStyle.height = height + "px";
	inputFileStyle.fontSize = height + "px";
	
	this._domObj.style.height = height + "px";
		
	if(this._domImage.childNodes.length > 0) {
		this._imageShapeDom.resetSize(this._width, height);
		this._domObj.style.top = "0px";
	}
};

gobject.signalNew("filechanged", gtk.UploadButton, null, null, []);
gobject.signalNew("mouseover", gtk.UploadButton, null, null, []);
gobject.signalNew("mouseout", gtk.UploadButton, null, null, []);
