/**
 * A class to create vertical box or container
 * @constructor
 * @base gtk.Widget
 * @param {Integer} spacing
 * @param {Integer} padding
 */
gtk.VBox = function(spacing, padding) {
	gtk.Widget.apply(this);

	this._spacing = (spacing === undefined || spacing == 0) ? 0 : spacing;
	this._padding = (padding === undefined || padding == 0) ? 0 : padding;
	this._children = [];
	this._width = this._minWidth = 10;
	this._height = this._minHeight = 10;
	this._isSizeLocked = false;
	
	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " vbox";
	};
	
	this._render();
	this._initEvent();
};

gtk.VBox.prototype = new gtk.Widget();
gtk.VBox.prototype.constructor = gtk.VBox;

gtk.VBox.prototype._initEvent = function(){
	var self = this;
	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked");
    });
};

/** 
 * Return all children widget
 * @return {array} children inside the VBox
 */
gtk.VBox.prototype.getChildren = function(){
	var childWidgets = [], children = this._children, len = children.length;
	for(var i = 0; i < len; i++){
		childWidgets.push(children[i].childWidget);
	}
	return childWidgets;
};

/** 
 * Return the first child
 * @return {gtk.Widget} widget first child inside the HBox
 */
gtk.VBox.prototype.getFirstChild = function(){
    return this._children[0]["childWidget"];
};

/** 
 * Return the last child
 * @return {gtk.Widget} widget the last child inside the HBox
 */
gtk.VBox.prototype.getLastChild = function(){
    return this._children[this._children.length - 1]["childWidget"]; 
};

/** 
 * Return a next child widget given current widget as reference
 * @return {gtk.Widget} widget the child to be returned
 */
gtk.VBox.prototype.getNextSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === this._children.length - 1) return null;
    return this._children[childIndex + 1]["childWidget"];
};

/** 
 * Return a previous child widget given a current widget as reference
 * @return {Object} widget the child to be returned
 */
gtk.VBox.prototype.getPreviousSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === 0) return null;
    return this._children[childIndex - 1]["childWidget"];
};

/** 
 * Method to refresh HBox
 * if gtk.HBox.lockSize() has been called, then the size of HBox will not decrease when some child have been remove
 * else vice versa
 */
gtk.VBox.prototype.refresh = function(){
	this._refreshSelf();
	this._refreshChildren();
};

/** @private */
gtk.VBox.prototype._refreshChildren = function(){
	for(var i = this._children.length; i--;){
		this._children[i]['childWidget'].refresh();
	}
};

/** 
 * Make sure, the HBox size will not decrease when some children have been remove
 */
gtk.VBox.prototype.lockSize = function(){
    this._isSizeLocked = true;
};

/** 
 * HBox size will decrease when some children have been remove
 */
gtk.VBox.prototype.unlockSize = function(){
    this._isSizeLocked = false;
};

/** @private */
gtk.VBox.prototype._refreshSelf = function(){
	var minWidth = this.getMinWidthComposite();
	if(this._isSizeLocked) minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	if(this._isSizeLocked) minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);
};

/** 
 * Append child gtk.Widget to HBox
 * @param {Object} widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.appendChild = function(childWidget, packOption){
    childWidget.setParent(this);
    
    var childObject = this._createChildObject(childWidget, packOption);
	this._children.push(childObject);
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);    
};

/** 
 * Method to add a separator between children widget inside HBox
 */
gtk.VBox.prototype.addSeparator = function(){
    var space = this._children[this._children.length - 1]["space"];
    jsgtk.Dom.showElement(space.childNodes[0]);
};

/** 
 * Method to clone the child widget
 * @return {Object} widget new cloned widget
 */
gtk.VBox.prototype.cloneWidget = function(){
    return jsgtk.Util.clone(this);
};

/** 
 * Insert a child widget infront of give child reference
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, packOption){
    
    newChildWidget.setParent(this);
	
    var index = this._isChild(oldChildWidget);
    this._children.splice(index,0, this._createChildObject(newChildWidget, packOption));
    
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/** 
 * Replace an existing child widget with new child widget
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.replaceChild = function(newChildWidget, oldChildWidget, packOption){
    newChildWidget.setParent(this);
    
    var index = this._isChild(oldChildWidget);
    if(!index) jsgtk.Util.debug("target child widget not found");
    
    var child = this._children[index];
    child["element"].insertBefore(newChildWidget.getNode(), oldChildWidget.getNode());
    child["element"].removeChild(oldChildWidget.getNode());

    child["childWidget"]    = newChildWidget;
	child["expand"]         = packOption["expand"]         === undefined ? child["expand"]         : packOption["expand"];
    child["fill"]           = packOption["fill"]           === undefined ? child["fill"]           : packOption["fill"];
	child["padding"]        = packOption["padding"]        === undefined ? child["padding"]        : packOption["padding"];
	child["resizableWidth"] = packOption["resizableWidth"] === undefined ? child["resizableWidth"] : packOption["resizableWidth"];    

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/** 
 * Remove a child widget from the HBox
 * @param {Object} widget
 */
gtk.VBox.prototype.removeChild = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    
    this.removeChildByIndex(childIndex);
};

/** 
 * Remove a child widget given an index
 * @param {Integer} index value of the child to be removed
 */
gtk.VBox.prototype.removeChildByIndex = function(index){
    if(!this._children[index]) jsgtk.Util.debug("The index is out of range");

    this._resizeHeight(this._height - parseInt(this._children[index]["element"].style.height));
    
    this._domObj.removeChild(this._children[index]["element"]);
    this._domObj.removeChild(this._children[index]["space"]);
    
    this._children.splice(index, 1);
};

/**
 * Re-order or relocatte the child widget
 * @param{gtk.Widget} widget
 * @param{integer} position The new position for child in the children list of the box starting from 0
 */
gtk.VBox.prototype.reorderChild = function(childWidget, position) {

    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    if( position >= this._children.length || position < 0 ) jsgtk.Util.debug("Position is undefined or out of range");
    if( childIndex === position ) return;

    var child = this._children[childIndex];
    var targetChild = this._children[position];

    //swapping the array
    this._children[position] = child;
    this._children[childIndex] = targetChild;

    //swapping the dom
	var tmpNode1 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});
	var tmpNode2 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});

	this._domObj.replaceChild(tmpNode2, targetChild["element"]);
	this._domObj.replaceChild(tmpNode1, child["element"]);

	this._domObj.appendChild(child["element"]);
	this._domObj.appendChild(targetChild["element"]);

	this._domObj.replaceChild(child["element"], tmpNode2);
	this._domObj.replaceChild(targetChild["element"], tmpNode1);
};

/** @private */
gtk.VBox.prototype._createChildObject = function(childWidget, packOption){
    var element = this._createChildNodeHolder(childWidget.getNode());

	var space = this._createSpace();

	this._appendChild(element, space);
    
    var childObject = {
		"element" : element,
		"childWidget"   : childWidget,
		"space"   : space,
		"expand"  : packOption["expand"] === undefined ? true : packOption["expand"],
		"fill"    : packOption["fill"] === undefined ? true : packOption["fill"],
		"padding" : packOption["padding"] === undefined ? 0 : packOption["padding"],
		"resizableWidth" : packOption["resizableWidth"] === undefined ? true : packOption["resizableWidth"] 
	};
	return childObject;
};

/** @private */
gtk.VBox.prototype._isChild = function(child){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === child) return i;
    }
    return false;
};

/** @private */
gtk.VBox.prototype._appendChild = function(element, space){
    this._domObj.appendChild(element);
    this._domObj.appendChild(space);
};

/** 
 * Return number of children inside HBox
 * @return {integer} length
 */
gtk.VBox.prototype.getChildrenLength = function(){
    return this._children.length;
};

/** @private */
gtk.VBox.prototype._createChildNodeHolder = function(childNode) {
    var verticleElement = jsgtk.Dom.create({tag: 'div', style: 'position: relative; display: block;'});
    verticleElement.appendChild(childNode);
	return verticleElement;
};

/** @private */
gtk.VBox.prototype._createSpace = function(){
    var space = jsgtk.Dom.create({"tag": 'div', "class": "gtk_vbox_space"});
        var separator = jsgtk.Dom.create({"append": space , "tag": 'div', "class": "gtk_vbox_separator"});
	return space;
};

/** @private */
gtk.VBox.prototype._hideLastSpace = function(){
    this._domObj.childNodes[this._domObj.childNodes.length - 1].style.height = "0px";
};

/**
 * Assign property to enabled
 */
gtk.VBox.prototype.enable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].enable();
    }
};

/**
 * Assign property to disabled
 */
gtk.VBox.prototype.disable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].disable();
    }
};

/** @private */
gtk.VBox.prototype._handleHiddenChild = function(child){
	if(child["childWidget"].isHidden()) {
		jsgtk.Dom.hideElement(child["element"]);
		jsgtk.Dom.hideElement(child["space"]);
	}
	else {
		jsgtk.Dom.showElement(child["element"]);
		jsgtk.Dom.showElement(child["space"]);
	}
};

/** @private */
gtk.VBox.prototype.getMinWidthComposite = function(){
    var maxChildWidth = this._children[0]["childWidget"].getMinWidthComposite();
    var childWidth;
    for(var i = 1, len = this._children.length; i < len; i++){
        childWidth = this._children[i]["childWidget"].getMinWidthComposite();
        maxChildWidth = maxChildWidth < childWidth ? childWidth : maxChildWidth;
    }

	var minWidth = maxChildWidth + this._padding * 2;
	minWidth     = minWidth < this._minWidth ? this._minWidth : minWidth;

    return minWidth;
};

/** @private */
gtk.VBox.prototype._resizeChildrenWidth = function(width){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["element"].style.left = this._padding + "px";
        child["space"].style.left = this._padding + "px";
        if(child["resizableWidth"]){
            child["element"].style.width = width + "px";
            child["childWidget"].resizeWidthComposite(width);
            child["space"].style.width = width + "px";    
        }else{
            var childWidth = child["childWidget"].getMinWidthComposite();
            child["element"].style.width = childWidth + "px";
            child["space"].style.width = childWidth + "px";
        }
    }
};

/** @private */
gtk.VBox.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);
    this._resizeChildrenWidth(minWidth - this._padding * 2);
};

gobject.signalNew('clicked', gtk.VBox, [], null, null);
