/**
 * gtk is a conventional namespace for jsgtk. All classes must have prefix name gtk.
 * For example: gtk.Widget, gtk.Entry, ... and so on.
 * @namespace gtk namespace
 */
gtk = {};



/**
 * @constructor
 */
gobject = {
	_signals: [],
	_signalCount: 0,

	/**
	* The gobject.signalNew() registers specified object type to a specified signal_name
	* @param  {string}  signalName  : a string containing the name of the signal
    * @param  {object}  type 		: the object type that the signal is associated with
	* @return {integer} signalId    :a unique integer signal ID
	*/
	signalNew: function(signalName, type) {
		var signal = {};
		if(this.signalLookup(signalName, type)) throw("RuntimeError: this signal is already binned to the type");
		this._signalCount++;
		signal[signalName] = {'type': type, 'id': this._signalCount};
		this._signals.push(signal);
		return this._signalCount;
	},

	/**
	* find the specified the signal that is associated to the object type
	* @param {string} name       : the name of a signal
	* @param {object} type       : object type
    * @return {iteger} or {null} : the integer id of a signal supported by type or 0.
	*/
	signalLookup: function(name, type) {
		for(var i = this._signals.length; i--;){
			if(this._signals[i][name] && this._signals[i][name].type === type ) return this._signals[i][name].id;
			if(this._signals[i][name] && type instanceof this._signals[i][name].type ) return this._signals[i][name].id;
		}
		return null;
	}
};
/**
 * Base class of all widgets, handle signal binding and triggering
 * @constructor
 */
gobject.GObject = function(){
	this.props = {};
	this._signalHandlers = {};

	/**
	 * Trigger the the registered signal
	 * @param {String} detailedSignal The name of signal
	 * @param {Arguments} arguments value that will be used as argument by handler
	 */
	this.emit = function(detailedSignal){
		if(this._signalHandlers[detailedSignal]){
			var args = [];
			for(var i=1; i<arguments.length; i++) args.push(arguments[i]);
			for(var i=0; i<this._signalHandlers[detailedSignal].length; i++){
				this._signalHandlers[detailedSignal][i].handler.apply(this, args.concat(this._signalHandlers[detailedSignal][i].args));
			}
		}
	};

	/**
	 * Listen to the signal being triggered
	 * @param {String} detailedSignal The name of signal
	 * @param {Function} handler function to be executed when the signal is triggered
	 * @return The id of handler
	 */
	this.connect = function(detailedSignal, handler){
		this._signalCheck(detailedSignal);
		if(!this._signalHandlers[detailedSignal]){
			this._signalHandlers[detailedSignal] = [];
			this._handlerId++;
		}
		var args = [];
		var handlerId = new UUID().id;
		for(var i=2; i<arguments.length; i++) args.push(arguments[i]);
		this._signalHandlers[detailedSignal].push({'handler': handler, 'args': args, id: handlerId});
		return handlerId;
	};

    /**
     * Disconnect the function from the signal
     * @param {String} signal handlerId id
     */
	this.disconnect = function(handlerId){
		for(var el in this._signalHandlers){
			var handlers = this._signalHandlers[el];
			for(var i = 0, len = handlers.length; i < len; i++){
				if(handlers[i].id === handlerId) {
					handlers.splice(i, 1);
					break;
				}
			}
		}
	};

	/** @private */
	this._signalCheck = function(detailedSignal){
		var signal = gobject.signalLookup(detailedSignal, this);
		if(signal) return signal;
		throw("Undefined signal name: " + detailedSignal);
	};

    /** @private */
	this._removeSignal = function(handlerId){
		for (var el in this._signalHandlers){
			if(el == handlerId) delete this._signalHandlers.handlerId;
		}
	};
};
/**
 * Object is parent class of most of all widgets
 * @constructor
 * @base gobject.GObject
 */
gtk.Object = function() {
	gobject.GObject.apply(this);
};
/**
 * Widget is a main base class of all widgets, this widget inherit from gtk.Object.
 * It is used to create any widget such gtk.Entry, gtk.ComboBox, ... and so on.
 * @constructor
 * @base gtk.Object
 */
gtk.Widget = function() {
	gtk.Object.apply(this);

    var self 					= this;
	this._domObj;
	this._minWidth 				= 16;
	this._minHeight 			= 16;
	this._width 				= this._minWidth;
	this._height 				= this._minHeight;
	this._isMinWidthChanged 	= false;
	this._isMinHeightChanged 	= false;
	this._activated 			= false;
	this._tag 					= null;
	this._domObj;
	this._signalName 			= "";
	this._parent 				= null;
	this._tooltipObj 			= null;
	this._enabled				= true;
	this.borderWidth 			= 2 ;

	this._render = function(){
		this._domObj = jsgtk.Dom.create({tag: "div", className: "gtk_widget"});
		var domObjStyle = this._domObj.style;
		domObjStyle.width = this._width + 'px';
		domObjStyle.height = this._height + 'px';
		domObjStyle.boder = "1px";
        domObjStyle.position = "relative";
	};

	this._render();
};

gtk.Widget.prototype = new gtk.Object();
gtk.Widget.prototype.constructor = gtk.Widget;

/**
 * A method to show the loading image while loading
 */
gtk.Widget.prototype.showLoading = function(){
    if(!this._loadingImgContainer){
		this._loadingImgContainer = this._constructLoading();
        this._appendLoading();
    }else this._loadingImgContainer.style.display = "table";
};

/**
 * A method to hide loading image inside the widget
 */
gtk.Widget.prototype.hideLoading = function(){
    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * A method to enable widget, widget will turn to its normal state
 * @see gtk.Widget#disable
 * @see gtk.Widget#setEnable
 */
gtk.Widget.prototype.enable = function(){};

/**
 * A method to disable widget, widget will be no more functioning
 * @see gtk.Widget#enable
 * @see gtk.Widget#setEnable
 */
gtk.Widget.prototype.disable = function(){};

/**
 * A method to set enabled or disabled
 * @param {Boolean}, true to set enabled, false to set disabled
 * @see gtk.Widget#enable
 * @see gtk.Widget#disable
 */
gtk.Widget.prototype.setEnable = function(isEnabled){
	this._enabled = (typeof isEnabled != 'undefined' && typeof isEnabled == 'boolean') ? isEnabled : true;
    this._enabled ? this.enable() : this.disable();
};

/**
 * A method to set tag onto it
 * @param {String} tag The tag name to be set
 */
gtk.Widget.prototype.setTag = function(tag){
    this._tag = tag;
};

/**
 * A method to get its tag
 * @return {String} The tag name of the widget
 * @type String
 */
gtk.Widget.prototype.getTag = function(){
    return this._tag;
};

/**
 * A method to store the reference of widget's parent
 * @param {Object} widget The parent to be set
 */
gtk.Widget.prototype.setParent = function(widget){
    this._parent = widget;
};

/**
 * A method to get reference of wiget's parent widget
 * @return {Object} The parent object of the widget
 * @type Object
 */
gtk.Widget.prototype.getParent = function(){
    return this._parent;
};

/**
 * A method to show or hide tooltip on widget
 * @param {Boolean} isToShow The boolean value true is to show, false is not
 * @see gtk.Widget#hideTooltip
 * @see gtk.Widget#setTooltip
 */
gtk.Widget.prototype.showTooltip = function(isToShow){
	this._isToShow = (typeof isToShow != 'undefined' && typeof isToShow == 'boolean') ? isToShow : true;
	if(this._tooltipObj !== null){
		this._isToShow ? this._tooltipObj.show() : this._tooltipObj.hide();
	}
};

/**
 * A method to hide tooltip
 * @see gtk.Widget#showTooltip
 * @see gtk.Widget#setTooltip
 */
gtk.Widget.prototype.hideTooltip = function(){
	if(this._tooltipObj !== null) this._tooltipObj.hide();
};

/**
 * A method to set tooltip content into its body
 * @param {String} textMessage The content message inside the tooltip to be display
 * @see gtk.Widget#showTooltip
 * @see gtk.Widget#hideTooltip
 */
gtk.Widget.prototype.setTooltip = function(textMessage){
	var contentMessage = textMessage || "&nbsp;&nbsp;<span style='color: #7F7F7F'>Value is undefined</span>&nbsp;&nbsp;&nbsp;&nbsp;";
	var contentObj = jsgtk.Dom.create({"tag": "div","style": "display: block;"});
		contentObj.innerHTML = contentMessage;

	if(this._tooltipObj === null){
		this._tooltipObj = new gtk.Bubble(contentObj, this._domObj, "", 8);
		this._tooltipObj.setTooltip();
		this._tooltipObj.hide();

	}
	else {
		this._tooltipObj.addContent(contentObj);
		this._tooltipObj.hide();
	}
};

/**
 * A method to reload or refresh the contents inside the widget
 */
gtk.Widget.prototype.refresh = function(){
	var minWidth = this.getMinWidthComposite();
	minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);
};

/**
 * A method to set title to bold style
 * @param {Boolean} bold The boolean value true is bold and false is not
 */
gtk.Widget.prototype.setBoldTitle = function(bold){
	if(bold){
		this._domTitle.style.fontWeight = "bold";
	} else 	this._domTitle.style.fontWeight = "normal";
};

/**
 * A method to return Dom wiget as an object
 * @return {Object} The Dom object value to be appended to any HTML element
 * @type Object
 */
gtk.Widget.prototype.getNode = function(){
	return this._domObj;
};

/**
 * A method to show the widget
 * @see gtk.Widget#hide
 */
gtk.Widget.prototype.show = function(){
	this._domObj.style.display = "";
};

/**
 * A method to hide the widget
 * @see gtk.Widget#show
 */
gtk.Widget.prototype.hide = function(){
	this._domObj.style.display = "none";
};

/**
 * A method to return true if widget is hidden and vice versa
 * @return {Boolean} The boolean value, true or false
 */
gtk.Widget.prototype.isHidden = function(){
	return !jsgtk.Dom.isShowed(this._domObj);
};

/**
 * A method to emit the "activated" signal on the widget
 * @return {Boolean} The boolean value which is true or false
 * @type Boolean
 */
gtk.Widget.prototype.activate = function(){
	var signal = this.getActivateSignal();

	if(signal){
		this._activated = true;
		this.emit(signal);
	}
	else this._activated = false;

	return this._activated;
};
/**
 * A method to set the activate signal name
 * @param {String} signal The signal name to be set. And it must be from any conventional name of JSGTK
 */
gtk.Widget.prototype.setActivateSignal = function(signal){
	this._signalName = signal;
};

/**
 * A method to return the name of the signal that is emitted when the widget is activated.
 * @return {String} The signal name that is registered
 * @type String
 */
gtk.Widget.prototype.getActivateSignal = function(){
	return this._signalName;
};

/**
 * A method to reset the minimum width and height of widget
 * @param {Integer} width  The minimum width value to be set
 * @param {Integer} height The minimum height value to be set
 */
gtk.Widget.prototype.setSizeRequest = function(width, height){
	this._isMinWidthChanged = width ? true : false;
	this._isMinHeightChanged = height ? true : false;

	this._minHeight = height;
	this._minWidth = width;
};
/**
 * Reset the mininimum width and minimum height of widget
 * @param {Integer} width  The new minimum width value of the widget
 * @param {Integer} height  The new minimum height value of the widget
 */
gtk.Widget.prototype.resetMinSize = function(minWidth, minHeight){
	this.resetMinWidth(minWidth);
	this.resetMinHeight(minHeight);
};
/**
 * A method to reset the mininimum width of widget
 * @param {Integer} width  The new minimum width value of the widget
 */
gtk.Widget.prototype.resetMinWidth = function(width){
	this._isMinWidthChanged = width ? true : false;
	this._minWidth = width;
    this._width = width;
};
/**
 * A method to reset the mininimum height of widget
 * @param {Integer} height  The new minimum height value in pixel
 */
gtk.Widget.prototype.resetMinHeight = function(height){
	this._isMinHeightChanged = height ? true : false;
	this._minHeight = height;
    this._height = height;
};

/**
 * A method to set the width and height of widget
 * @param {Integer} width  The integer width value to be set
 * @param {Integer} height The integer height value to be set
 */
gtk.Widget.prototype.setSize = function(width, height){
	this._height = height;
	this._width = width;
};

/**
 * A method to set new width to widget
 * @param {Integer} width : new width of widget
 */
gtk.Widget.prototype.setWidth = function(width){
	this._width = width;
};

/**
 * A method to return the current width to widget
 * @return {Integer} The integer width value of the widget
 * @type Integer
 */
gtk.Widget.prototype.getWidth = function(){
	return this._width;
};

/**
 * A method to set new height to the widget
 * @param {Integer} height The integer height value in pixel to be set
 */
gtk.Widget.prototype.setHeight = function(height){
	this._height = height;
};

/**
 * A method to return the current height to widget
 * @return {Integer} The height value in pixel of the widget
 * @type Integer
 */
gtk.Widget.prototype.getHeight = function(){
	return this._height;
};

/**
 * A method to return the minimum width and height of widget
 * @return {Array} The array values of minimum width and height
 * @type Array
 */
gtk.Widget.prototype.getSizeRequest = function(){
	return [this._minWidth, this._minHeight];
};

/**
 * A method to get width and height
 * @return {Array} The array value contains width and height
 * @type Array
 */
gtk.Widget.prototype.sizeRequest = function(){
	return [
		this._width,
		this._height
	];
};

/**
 * A method to return an array of current width and height
 * @param {Array} The array values contains width and height
 * @type Array
 */
gtk.Widget.prototype.getSize = function(){
	return [
		this._width,
		this._height
	];
};

/** @private */
gtk.Widget.prototype._constructLoading = function(){
    var loadingImgContainer = jsgtk.Dom.create({tag: "table", 'class': 'gtk_loading_bar_container_widget'});
    	var tr = jsgtk.Dom.create({append: loadingImgContainer, tag: "tr"});
			var td = jsgtk.Dom.create({append: tr, tag: "td"});
                this._loadingImgFrame = jsgtk.Dom.create({"append": td, "tag": "div", "class": "gtk_loading_frame"});
                    this._loadingImg      = jsgtk.Dom.create({"append": this._loadingImgFrame, "tag": "div", "class": "gtk_loading_bar_widget"});
    return loadingImgContainer;
};

/** @private */
gtk.Widget.prototype._appendLoading = function(){
    var firstChild = this._domObj.firstChild;
    if(firstChild)
        this._domObj.insertBefore(this._loadingImgContainer, firstChild);
    else
        this._domObj.appendChild(this._loadingImgContainer);
};

/** @private */
gtk.Widget.prototype._resizeHeight = function(height){
	this._domObj.style.height = height + 'px';
	this._height = height;
};

/** @private */
gtk.Widget.prototype._resizeWidth = function(width){
	this._domObj.style.width = width + 'px';
	this._width = width;
};

/**
 * A method to return the minimum composite width, if the widget contains children,
 * the minWidth will be smallest with that the widget and its children can be shrinken
 * @return {Integer} The minimum width value in pixel
 * @type Integer
 */
gtk.Widget.prototype.getMinWidthComposite = function(){
	return this._minWidth;
};

/**
 * A method to return the minimum composite height, if the widget contains children,
 * the minHeight will be the smallest height that the widget and its children can be shrinken
 * @return {Integer} The minimum height value in pixel
 * @type Integer
 */
gtk.Widget.prototype.getMinHeightComposite = function(){
	return this._minHeight;
};

/**
 * A method to resize the wiget and its children's width
 * @param {Integer} width The integer width value in pixel to be set
 */
gtk.Widget.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
};

/**
 * A method to resize the wiget and its children's heights
 * @param {Integer} height The height in pixel value to be set
 */
gtk.Widget.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = this._height + 'px';
};

/**
 * A method to reset to default composite height
 */
gtk.Widget.prototype.resetHeightComposite = function(){
	var minSize = this.getSizeRequest();
	this.setHeight(minSize[1]);
};

/**
 * A method to reset to default composite width
 */
gtk.Widget.prototype.resetWidthComposite = function(){
	var minSize = this.getSizeRequest();
	this.setWidth(minSize[0]);
};
/**
 * Button is a class to create a button
 * @constructor
 * @base gtk.Widget
 * @param {String} Label text to be displayed by in the middle of button
 */
gtk.Button = function(label){
	gtk.Widget.apply(this);

	var self 			= this;
	this._label 		= label || "";
	this._widget1 		= null;
	this._widget2 		= null;
	this._isEnable 		= true;
	this._signalClicked = null;

	this._width 		= this._minWidth = 16;
	this._height 		= this._minHeight = 24;

    /** 'clicked' signal */
	this.BUTTON_CLICKED = "clicked";
	/** 'mouseout' signal */
	this.BUTTON_MOUSEOUT = "mouseout";
	/** 'mouseover' signal */
	this.BUTTON_MOUSEOVER = "mouseover";
	/** 'enterkeypressed' signal */
	this.BUTTON_ENTERKEYPRESSED = "enterkeypressed";

	/** @private */
	this._render = function(){

	    this._domObj.className += " gtk_button";
	    var domObjStyle = this._domObj.style;
	    domObjStyle.position = "relative";
	    var jsgtkDom = jsgtk.Dom;
	        this._gradient = jsgtkDom.create({"append": this._domObj, "tag": "div", "class": "gtk_button_gradient"});
	        this._gradient.style.height = this._height / 2 + "px";
		    this._imageDom = jsgtkDom.create({"tag": "div", "style": "float: left; position: relative;"});
		    this._labelDom = jsgtkDom.create({"tag": "div", "style": "float: left; position: relative;"});

		domObjStyle.width = this._width - this.borderWidth + "px";
		domObjStyle.height = this._height - this.borderWidth + "px";
		if(label) this.setLabel(new gtk.Label(label));
	};
    this._render();
    this._initEvent();
};

gtk.Button.prototype = new gtk.Widget();
gtk.Button.prototype.constructor = gtk.Button;

/** @private */
gtk.Button.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    this._signalClicked = jsgtkEvent.addDomListener(this._domObj, "click", function(e){
		if(self._isEnable === true) self.emit("clicked");
	}, false);

    jsgtkEvent.addDomListener(this._domObj, "keyup", function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
	        if(self._isEnable === true) self.emit("enterkeypressed");
	    }
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseover", function(e){
		self.emit("mouseover");
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseout", function(e){
		self.emit("mouseout");
	}, false);
};

/** @private */
gtk.Button.prototype._positionWidget = function(widget, widgetDom){
    var top = (this._height  - widget.getHeight()) / 2;
    var left = (this._width - widget.getWidth() - this.borderWidth)/2;

    widgetDom.style.top = top + "px";
    widgetDom.style.left = left + "px";
};

/** @private */
gtk.Button.prototype.resize = function(width, height){

    this._resizeWidth(width);
    this._resizeHeight(height);

    if(this._widget2 && this._widget1){
        var imageDomStyle = this._imageDom.style;
        var labelDomStyle = this._labelDom.style;
        if(this._imagePosition == "topPosition"){
            imageDomStyle.cssFloat = "";
            labelDomStyle.ccsFloat = "";
            labelDomStyle.top = "0px";
            imageDomStyle.left = (this._width / 2) - (this._widget2.getWidth() / 2) + "px";
            labelDomStyle.left = (this._width / 2) - (this._widget1.getWidth() / 2) + "px";
        }
        else{
            var imageTop = (this._height / 2) - (this._widget2.getHeight() / 2);
            var imageLeft = (this._width - this._widget2.getWidth() - this._widget1.getWidth() - this.borderWidth)/2;
            imageDomStyle.top = imageTop + "px";
            imageDomStyle.left = imageLeft + "px";

            var labelTop = (this._height / 2) - (this._widget1.getHeight() / 2);
            labelDomStyle.top = labelTop + "px";
            labelDomStyle.left = parseInt(imageDomStyle.left) + "px";
        }
        return;
    }

    if(this._widget2) this._positionWidget(this._widget2, this._imageDom);

    if(this._widget1) this._positionWidget(this._widget1, this._labelDom);
};

/** @private */
gtk.Button.prototype._getMaxChildrenHeight = function(){
    var maxHeight = this._height;
    if(this._widget2) maxHeight = maxHeight > this._widget2.getHeight() ? maxHeight : this._widget2.getHeight();
    if(this._widget1) maxHeight = maxHeight > this._widget1.getHeight() ? maxHeight : this._widget1.getHeight();

    return maxHeight;
};

/** @private */
gtk.Button.prototype._getMaxChildrenWidth = function(){
    var maxWidth = 0;
    if(this._widget2) maxWidth += this._widget2.getWidth();
    if(this._widget1) maxWidth += this._widget1.getWidth();
    maxWidth += 15;
    maxWidth = maxWidth < this._width ? this._width : maxWidth;
    return maxWidth;
};

/**
 * Set the image to be displayed next to the label
 * @param {gtk.Image} imageWidget
 */
gtk.Button.prototype.setImage = function(image){
    if(this._imageDom.childNodes) this._imageDom.innerHTML = "";
    this._widget2 = image;

    this._minWidth = this._getMaxChildrenWidth();
    this._minHeight = this._getMaxChildrenHeight();

    this.resize(this._minWidth, this._minHeight);

    this._widget2._domObj.style.display = "";

    this._imageDom.appendChild(this._widget2._domObj);
    this._domObj.insertBefore(this._imageDom, this._domObj.childNodes[1]);
};

/**
 * @deprecated
 */
gtk.Button.prototype.setButtonShape = function(imageObj){
	this.setAsImage(imageObj);
	console.log("gtk.Button.setButtonShape() will be soon renamed to gtk.Button.setAsImage()");
	console.log("Please change it immediately if you see this message!!!");
};

/**
 * Change the button shape to be image shape
 * @param {gtk.Image} imageWidget
 */
gtk.Button.prototype.setAsImage = function(imageObj){
	this._domObj.className += " gtk_button_with_image_shape";
	this._gradient.style.display = "none";
	this._domObj.style.backgroundImage = "url("+imageObj._dom.src+")";
	this.setSizeRequest(imageObj._width, imageObj._height);
	this.resize(imageObj._width, imageObj._height);
};

/**
 * Returns the image widget
 * @return {gtk.Image} imageWidget
 */
gtk.Button.prototype.getImage = function(){
    return this._widget2;
};

/**
 * Sets label to be displayed in the button
 * @param {gtk.Label} labeWidget
 */
gtk.Button.prototype.setLabel = function(label){
    if(this._labelDom.childNodes) this._labelDom.innerHTML = "";
    this._widget1 = label;
    this._label = label.getText();
    label.setStyle({cursor: "pointer"});

    this._minWidth = this._getMaxChildrenWidth();
    this._minHeight = this._getMaxChildrenHeight();

    this.resize(this._minWidth, this._minHeight);
    this._widget1.setJustify("center");
    this._widget1._domObj.style.display = "";

    this._labelDom.appendChild(this._widget1._domObj);
    this._domObj.appendChild(this._labelDom);
};

/**
 * Set text to button
 * @param {String} strName name of button
 */
gtk.Button.prototype.setTextLabel = function(strName){
    if(strName != null) this.setLabel(new gtk.Label(strName));
};

/**
 * Returns the text from the label of the button
 * @return {gtk.Label} labelWidget
 */
gtk.Button.prototype.getLabel = function(){
    return this._label;
};

/** @private */
gtk.Button.prototype._resizeHeight = function(height){
	this._domObj.style.height = height + "px";
	this._domObj.childNodes[0].style.height = height / 2 + "px";
	this._height = height;
};

/** @private */
gtk.Button.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this.borderWidth + "px";
	this._width = width;
};

/** @private */
gtk.Button.prototype._setImagePosition = function(position){
    this._imagePosition = position;
    if(position == "topPosition"){
    	var imageDomStyle = this._imageDom.style;
    	var labelDomStyle = this._labelDom.style;
        imageDomStyle.cssFloat = "";
        labelDomStyle.ccsFloat = "";
        labelDomStyle.top = "0px";
        imageDomStyle.left = (this._width / 2) - (this._widget2.getWidth() / 2) + "px";
        labelDomStyle.left = (this._width / 2) - (this._widget1.getWidth() / 2) + "px";

        var imageLabelHeight = this._widget1.getHeight() + this._widget2.getHeight();
        this._height = (this._height > imageLabelHeight)? this._height : imageLabelHeight;
        this._domObj.style.height = this._height + "px";
	    this._domObj.childNodes[0].style.height = this._height / 2 + "px";
    }
};

/**
 * show loading Image
 */
gtk.Button.prototype.showLoading = function(){
	this._labelDom.style.display = "none";
	this._imageDom.style.display = "none";
	jsgtk.Event.removeListener(this._signalClicked);

	if(!this._loading) this._loading = jsgtk.Dom.create({tag:"div", className: "gtk_button_loading", append: this._domObj});
};

/**
 * Hide loading Image
 */
gtk.Button.prototype.hideLoading = function(){
	jsgtk.Event.addDomListener(this._signalClicked.obj, this._signalClicked.type, this._signalClicked.callback);
	this._labelDom.style.display = "";
	this._imageDom.style.display = "";
	if(this._loading){
		this._domObj.removeChild(this._loading);
		this._loading = null;
	}
};

/**
 * Set tab index
 * @param {Integer} index
 */
gtk.Button.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};

/**
 * Disable button button no longer function
 */
gtk.Button.prototype.disable = function(){
	this._isEnable = false;
	this._domObj.className +=" gtk_button_disabled";
	this._gradient.className += " gradisable";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "default"});
};

/**
 * Enable button button functions normally
 */
gtk.Button.prototype.enable = function(){
	this._isEnable = true;
	this._domObj.className = "gtk_widget gtk_container gtk_button";
	this._gradient.className = "gtk_button_gradient";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "pointer"});
};

gtk.Button.prototype.resizeWidthComposite = function(width){
    this.setWidth(width);
    this.resize(this._width, this._height);
};

gtk.Button.prototype.resizeHeightComposite = function(height){
    this.setHeight(height);
    this.resize(this._width, this._height);
};

/** Button signal: clicked */
gobject.signalNew("clicked", gtk.Button, null, null, []);
gobject.signalNew("mouseout", gtk.Button, null, null, []);
gobject.signalNew("mouseover", gtk.Button, null, null, []);
gobject.signalNew("enterkeypressed", gtk.Button, null, null, []);
/**
 * @constructor
 * @base gtk.Widget
 * @param {String} text The text to be displayed in label
 */
gtk.Label = function(text) {
	gtk.Widget.apply(this);

	/** 'clicked' signal */
	this.LABEL_CLICKED = "clicked";
	/** 'mouseover' signal */
	this.LABEL_MOUSEOVER = "mouseover";
	/** 'mouseout' signal */
	this.LABEL_MOUSEOUT = "mouseout";

	this._dom = null;
	this._attribute = null;
	this._isSelectable = false;
    this._isEnable = true;

	this._render = function(){
		this._domObj.className += " gtk_label";
			var table = jsgtk.Dom.create({append: this._domObj, tag: "table", style: 'width: 100%; height: 100%;'});
				var tr = jsgtk.Dom.create({append: table, tag: "tr"});
					var td = jsgtk.Dom.create({append: tr, tag: "td", "className": "gtk_label_align"});
						this._dom = jsgtk.Dom.create({append: td, tag: "label", 'className': 'gtk_none_selectable'});
						this._dom.innerHTML = text || "";

		this._disabledLabel = jsgtk.Dom.create({append: this._domObj, tag: "div", "class": "gtk_label_disabled"});

        this._minWidth = this._width = this._calculateMinWidthMinHeight()[0];
        this._minHeight = this._height = this._calculateMinWidthMinHeight()[1];

        this.resizeWidthComposite(this._minWidth);
        this.resizeHeightComposite(this._minHeight);
	};

	this._render();
	this._initEvent();
};

gtk.Label.prototype = new gtk.Widget();
gtk.Label.prototype.constructor = gtk.Label;

gtk.Label.prototype._initEvent = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._domObj, "click", function(e){
       self.emit("clicked");
    }, false);

	jsgtk.Event.addDomListener(this._domObj, "mouseover", function(e){
       self.emit("mouseover");
    }, false);

    jsgtk.Event.addDomListener(this._domObj, "mouseout", function(e){
        if(self._isEnable) self.emit("mouseout");
    }, false);
};
/**
 * Set text to label
 * @param {String} text
 */
gtk.Label.prototype.setText = function(text){
	this._dom.innerHTML = text;
	var size = this._calculateMinWidthMinHeight();
	this.resetMinSize(size[0], size[1]);
    this.resizeWidthComposite(size[0]);
};
/**
 * Get text from label
 * @return {String} text
 */
gtk.Label.prototype.getText = function(){
	return this._dom.innerHTML;
};
/** @private */
gtk.Label.prototype.disable = function(){
	this._isEnable = false;
    this._domObj.className = "gtk_widget gtk_label gtk_label_disable";

	if(this._attribute) jsgtk.Dom.applyAttributes(this._dom, {"cursor":"default","color":"#ccc"});

	var domHref = this._dom.firstChild;
	if(domHref.tagName == "A") jsgtk.Dom.applyAttributes(domHref, {"cursor":"default","color":"#ccc"});
	jsgtk.Dom.showElement(this._disabledLabel);
};
/** @private */
gtk.Label.prototype.enable = function(){
	this._isEnable = true;
	this._domObj.className = "gtk_widget gtk_label";

	if(this._isHover) this.enableHoverStyle();
	this.setAttribute(this._attribute);

	var domHref = this._dom.firstChild;
	if(domHref.tagName == "A") jsgtk.Dom.applyAttributes(domHref, {"cursor":"","color":""});
	jsgtk.Dom.hideElement(this._disabledLabel);
};
/** @private */
gtk.Label.prototype.resize = function(width, height){
	this._resizeWidth(width);
	this._resizeHeight(height);
};
/** set tab index */
gtk.Label.prototype.setTabindex = function(index){
	this._domObj.tabIndex = index;
};

/** @private */
gtk.Label.prototype._calculateMinWidthMinHeight = function(){
    var tempDom = this._dom.cloneNode(true);
        var width = jsgtk.Util.fullWidth(tempDom);
	    var height = jsgtk.Util.fullHeight(tempDom);

	return [width , height];
};
/** display border left yellow when mouse hovered label text */
gtk.Label.prototype.enableHoverStyle = function(){
    this._isHover = true;
    this._domObj.className += " gtk_label_enable_hover";
};
/** @private */
gtk.Label.prototype._calculateHeight = function(){
    var tempDom = this._dom.cloneNode(true);
    var tempSpan = jsgtk.Dom.create({tag: "div"});
    tempSpan.style.width = this._width + "px";
    tempSpan.appendChild(tempDom);
	var height = jsgtk.Util.fullHeight(tempSpan);

	return height;
};
/**
 * Set attributes(style) to label
 * @param {array} array of attributes object
 */
gtk.Label.prototype.setAttribute = function(attribute){
    if(this._isEnable){
        this._attribute = attribute;
	    jsgtk.Dom.applyAttributes(this._dom, this._attribute);

	    var size = this._calculateMinWidthMinHeight();
		this.resetMinSize(size[0], size[1]);
        this.resizeWidthComposite(size[0]);
        this.resizeHeightComposite(size[1]);
    }
};
/**
 * Get attributes
 * @return {Array} array of attributes object
 */
gtk.Label.prototype.getAttribute = function(){
	return this._attribute;
};
/**
 * Set label's text selectable
 * @param {boolean} isSelectable
 */
gtk.Label.prototype.setSelectable = function(isSelectable){
	this._isSelectable = isSelectable;
    this._dom.className = this._isSelectable ? "gtk_selectable" : "gtk_none_selectable";
};
/**
 * Get label's text selectable
 * @return {boolean}
 */
gtk.Label.prototype.getSelectable = function(){
	return this._isSelectable;
};
/** @private */
gtk.Label.prototype.show = function(){
	if(this._domObj.style.display != 'inline') this._domObj.style.display = 'block';
};
/**
 * Set the width of Label
 * @param {Interger} width The length of width to be set
 */
gtk.Label.prototype.setWidthChars = function(width){
    this._minWidth = this._width = width;
    this._minHeight = this._height = this._calculateHeight();

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};
/**
 * Set the justify of Label
 * @param {String} justify The justify type to be set to label
 */
gtk.Label.prototype.setJustify = function(justify){
    this._dom.parentNode.style.textAlign = justify;
};
/**
 * Set the label style
 * @param {Object} properties The properties to be set to Label style
 */
gtk.Label.prototype.setStyle = function(properties){
	var domStyle = this._domObj.style;
    var labelStyle = this._dom.style;
    for(var i in properties){
    	domStyle[i] = properties[i];
    	if(i == "cursor") labelStyle[i] = properties[i];
    }
};
/** @private */
gtk.Label.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + 1 + "px";
};
/** @private */
gtk.Label.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
};
/** @private */
gtk.Label.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = width > minWidth ? width : minWidth;
    this._resizeWidth(minWidth);
};
/** @private */
gtk.Label.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = height > minHeight ? height : minHeight;
    this._resizeHeight(minHeight);
};
gobject.signalNew("clicked", gtk.Label, null, null, []);
gobject.signalNew("mouseover", gtk.Label, null, null, []);
gobject.signalNew("mouseout", gtk.Label, null, null, []);
/**
 * RadioButton is a class to create a radio button
 * @constructor
 * @base gtk.Widget
 * @param {gtk.RadioButton} group The group name that it belongs to
 * @param {string} label The text of radio button
 */
gtk.RadioButton = function(group, label){
    gtk.Widget.apply(this);

    this._activated = false;
    this._name = group ? group.getName() : new UUID().id;
	this._addToGroup(this);

    this._imageSize   = 16;
    this._space       = 4;
    this._imageClassUnchecked  = "gtk_radio_button_img_unchecked";
    this._imageClassChecked    = "gtk_radio_button_img_checked"
    this._isFocus  = false;
    this._labelWidget = new gtk.Label(label);

    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_radio_button";
            this._radioImgLabelContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_radio_button_img_label_container"});
                this._radioImage = jsgtk.Dom.create({"append": this._radioImgLabelContainer, "tag": "div", "className": this._imageClassUnchecked});
                this._radioLabel = jsgtk.Dom.create({"append": this._radioImgLabelContainer, "tag": "div", "class": "gtk_radio_button_label"});
                this._radioLabel.appendChild(this._labelWidget.getNode());

        this._width  = this._minWidth  = this._imageSize + this._labelWidget.getMinWidthComposite() + this._space;
        this._height = this._minHeight = this._labelWidget.getMinHeightComposite();
    };
    this._render();

    this._initEvent();

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

gtk.RadioButton.prototype = new gtk.Widget();
gtk.RadioButton.prototype.constructor = gtk.RadioButton;

/** @private */
gtk.RadioButton.group = [];

/** @private */
gtk.RadioButton.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    jsgtkEvent.addDomListener(this._domObj, "mouseup", function(){
            self._unMarkeOther(self);
            self._unFocusOther(self)
            self._select();
            self.emit("toggled");
    });
    jsgtkEvent.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
            self._unMarkeOther(self);
            self._unFocusOther(self)
            self.setFocus(self);
            self._select();
	    }
	});
	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
        self.setFocus(self);
    });
    jsgtkEvent.addDomListener(this._domObj, "blur", function(){
        self._unFocusOther(self)
    });
};

/**
 * A method to set the tab index value onto the gtk.RadioButton
 * @param {integer} index The integer index value to be set
 */
gtk.RadioButton.prototype.setTabindex = function(index){
	this._domObj.tabIndex = index;
};

/** @private */
gtk.RadioButton.prototype._unFocusOther = function(btn){
	var group = gtk.RadioButton.group;
    for(var i = group.length; i--;){
        if(group[i].name === btn.getName()) {
            for(var j = group[i].children.length; j--;){
                this._unFocus(group[i].children[j]);
            }
            return;
        }
    }
};

/** @private */
gtk.RadioButton.prototype._unFocus = function(btn){
	btn.getNode().firstChild.lastChild.className = "gtk_radio_button_label"
};

/**
 * A method to set focus onto the gtk.RadioButton
 */
gtk.RadioButton.prototype.setFocus = function(){
	this._domObj.firstChild.lastChild.className += " gtk_radio_focus"
};

/**
 * A method to set active onto the radio button
 * @param {Boolean} isActive The state to define whether it is true or false
 */
gtk.RadioButton.prototype.setActive = function(isActive){
	this._activated = isActive;
};

/**
 * A method to get/return the state of radio button whether it is activated or not
 * @return {Boolean} The state as true or false
 * @type Boolean
 */
gtk.RadioButton.prototype.getActive = function(){
	return this._activated;
};

/** @private */
gtk.RadioButton.prototype._select = function(){
    this._radioImage.className = this._imageClassUnchecked + " " + this._imageClassChecked;
	this.setActive(true);
};

/** @private */
gtk.RadioButton.prototype._deSelect = function(){
    this._radioImage.className = this._imageClassUnchecked;
	this.setActive(false);
};

/**
 * A method to get the group
 * @return {gtk.RadioButton} The group name as an object of gtk.RadioButton
 * @type gtk.RadioButton
 */
gtk.RadioButton.prototype.getGroup = function(){
    var group = gtk.RadioButton.group;
    var tmpGroup = [];
    for(var i = group.length; i--;){
        if(group[i].getName() === this.getName()) tmpGroup.push(group[i]);
    }
    return tmpGroup;
};

/**
 * A method to get the name of group that it belongs to
 * @return {String} The name of the gtk.RadioButton belongs to
 * @type String
 */
gtk.RadioButton.prototype.getName = function(){
    return this._name;
};

/**
 * A method to set the name of group that it is to belongs to
 * @param {String} name The name of the group
 */
gtk.RadioButton.prototype.setName = function(name){
    this._name = name;
};

/** @private */
gtk.RadioButton.prototype._addToGroup = function(group){
    var radioGroup = gtk.RadioButton.group;
    if(radioGroup != []){
        for(var i = radioGroup.length; i--;){
            if(radioGroup[i].name == group._name){
                radioGroup[i].children.push(group);
                return;
            }
        }
    }
    var groupObj = {name: group._name, children: [group]};
    radioGroup.push(groupObj);
};

/** @private */
gtk.RadioButton.prototype._unMarkeOther = function(btn){
    var group = gtk.RadioButton.group;
    for(var i = group.length; i--;){
        if(group[i].name === btn.getName()) {
            for(var j = group[i].children.length; j--;){
                group[i].children[j]._deSelect();
            }
            return;
        }
    }
};

/** @private */
gtk.RadioButton.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._radioImgLabelContainer.style.height = this._minHeight + "px";
};

/** @private */
gtk.RadioButton.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._radioImgLabelContainer.style.width = width + "px";
};

/**
 * A method to resize its composite width of gtk.Radiobutton
 * @param {Integer} width The integer value as width to be set
 */
gtk.RadioButton.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
};

/**
 * A method to resize its composite height of gtk.Radiobutton
 * @param {Integer} height The integer value as height to be set
 */
gtk.RadioButton.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    this._positionCheckBox();
};

/** @private */
gtk.RadioButton.prototype._positionCheckBox = function(height){
    this._radioImgLabelContainer.style.top = this._height / 2 - this._minHeight / 2 + "px";
};

/** It allows user to connect to a registered signal event "toggled" when click on the radio button */
gobject.signalNew("toggled", gtk.RadioButton, null, null, []);
/**
* @constructor
* @base gtk.Widget
* @param {String} Label text to be displayed by in the middle of button
*/
gtk.CheckButton = function(label){
    gtk.Widget.apply(this);

	/** 'checked' signal*/
	this.CHECKBUTTON_CHECKED = 'checked';
	/** 'unchecked' signal*/
	this.CHECKBUTTON_UNCHECKED = "unchecked";
	/** 'focused' signal*/
	this.CHECKBUTTON_FOCUSED = "focused";
	/** 'blurred' signal*/
	this.CHECKBUTTON_BLURRED = "blurred";

    this._isChecked   	= false;
    this._imageSize   	= 16;
    this._space       	= 7;
    this._imageClass  	= "gtk_check_button_img";
    this._disabledModal	= null;

    this._labelWidget = new gtk.Label(label);

    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_check_button";
            this._checkImgLabelContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_check_button_img_label_container"});
                this._checkImage = jsgtk.Dom.create({"append": this._checkImgLabelContainer, "tag": "div", "className": "gtk_check_button_img"});
                this._checkLabel = jsgtk.Dom.create({"append": this._checkImgLabelContainer, "tag": "div", "class": "gtk_check_button_label"});
                this._checkLabel.appendChild(this._labelWidget.getNode());

        this._width  = this._minWidth  = this._imageSize + this._labelWidget.getMinWidthComposite() + this._space;
        this._height = this._minHeight = this._labelWidget.getMinHeightComposite();
    };
    this._render();

    this._initEvent();

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

gtk.CheckButton.prototype = new gtk.Widget();
gtk.CheckButton.prototype.constructor = gtk.CheckButton;

/** @private */
gtk.CheckButton.prototype._initEvent = function(){
    var self = this;
	var jsgtkEvent = jsgtk.Event;

	this._mouseupHandler = jsgtkEvent.addDomListener(this._domObj, 'mouseup', function(){
	    if(!self._isChecked) self.setChecked();
        else self.setUnchecked();
	});
	this._mouseoverHandler = jsgtkEvent.addDomListener(this._checkImgLabelContainer, 'mouseover', function(){
		if(self._isChecked) self._checkImage.className = "gtk_check_button_img gtk_check_button_checked_hover";
		else self._checkImage.className = "gtk_check_button_img gtk_check_button_unchecked_hover";
	});

	this._mouseoutHandler = jsgtkEvent.addDomListener(this._checkImgLabelContainer, 'mouseout', function(){
		if(self._isChecked) self._checkImage.className = "gtk_check_button_img gtk_check_button_checked";
		else   self._checkImage.className = "gtk_check_button_img";
	});

	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
		self.emit("focused");
    });

    jsgtkEvent.addDomListener(this._domObj, "blur", function(){
    	self.emit("blurred");
    });

    this._keyupHandler = jsgtkEvent.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
           if(!self._isChecked) self.setChecked();
           else self.setUnchecked();
	    }
	});
};

/**
 * Method to set tab index value
 */
gtk.CheckButton.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};

/**
 * set default checked on check button
 */
gtk.CheckButton.prototype.setChecked = function(){
    this._checkImage.className = this._imageClass +" gtk_check_button_checked";
    this._isChecked = true;
    this.emit("checked", this._labelWidget.getText());
};

/**
 * uncheck the checked
 */
gtk.CheckButton.prototype.setUnchecked = function(){
    this._checkImage.className ="gtk_check_button_img gtk_check_button_unchecked";
	this._isChecked = false;
	this.emit("unchecked", this._labelWidget.getText());
};

/**
 * return state of checkbutton
 * @return {boolean}
 */
gtk.CheckButton.prototype.getCheckedState = function(){
    return this._isChecked;
};

/**
 * All functionalities are disabled
 */
gtk.CheckButton.prototype.disable = function(){
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.removeListener(this._mouseupHandler);
	jsgtkEvent.removeListener(this._keyupHandler);
	jsgtkEvent.removeListener(this._mouseoutHandler);
	jsgtkEvent.removeListener(this._mouseoverHandler);
	this._labelWidget.setAttribute({color: "#6F6F6F"});
};

/**
 * All functionalities are enabled
 */
gtk.CheckButton.prototype.enable = function(){
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.registerHandler(this._mouseupHandler);
	jsgtkEvent.registerHandler(this._keyupHandler);
	jsgtkEvent.registerHandler(this._mouseoutHandler);
	jsgtkEvent.registerHandler(this._mouseoverHandler);
	this._labelWidget.setAttribute({color: ""});
};

/** @private */
gtk.CheckButton.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._checkImgLabelContainer.style.height = this._minHeight + "px";
};

/** @private */
gtk.CheckButton.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._checkImgLabelContainer.style.width = width + "px";
};

/** @private */
gtk.CheckButton.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
};

/** @private */
gtk.CheckButton.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    this._positionCheckBox();
};

/** @private */
gtk.CheckButton.prototype._positionCheckBox = function(height){
    this._checkImgLabelContainer.style.top = this._height / 2 - this._minHeight / 2 + "px";
};

/** Signal in CheckButton widget: checked */
gobject.signalNew("checked", gtk.CheckButton, null, null, []);
gobject.signalNew("unchecked", gtk.CheckButton, null, null, []);
gobject.signalNew("focused", gtk.CheckButton, null, null, []);
gobject.signalNew("blurred", gtk.CheckButton, null, null, []);
/**
 * @constructor
 * @base gtk.Widget
 */
gtk.Image = function(imagePath, imageWidth, imageHeight) {
	gtk.Widget.apply(this);
	var self = this;

	/** 'clicked' signal */
	this.IMAGE_CLICKED = "clicked";

	this._size         = null;
	this._dom          = null;
	this._defaultImage = "images/icons/ajax_loader.gif";

	this._width        = this._minWidth = imageWidth || 16;
	this._height       = this._minHeight = imageHeight || 16;

    this._attribute;

	/** @private */
	this._render = function(){
		this._domObj.className += " gtk_image";
			this.header = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_image_header"});
				this.headerDom = jsgtk.Dom.create({"append": this.header, "tag": "div", "class": "gtk_image_header_dom"});
					this.closeButton = jsgtk.Dom.create({"append":this.headerDom, "tag": "div","className": "gtk_image_header_close_button"});

			var table = jsgtk.Dom.create({append: this._domObj, tag: "table", "className": "gtk_image_table"});
				var tr = jsgtk.Dom.create({append: table, tag: "tr"});
					var td = jsgtk.Dom.create({append: tr, tag: "td", "className": "gtk_image_table_td"});
						this._dom = jsgtk.Dom.create({append: td, tag: "img", src: this._defaultImage});

            this._disableLayer = jsgtk.Dom.create({append: this._domObj, tag: "div", 'class': "gtk_image_div_disable"});

		this._invisibleOverlay = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "className": "gtk_image_invisible_overlay"});

		if(imagePath !== undefined){
		    this.setFromFile(imagePath, imageWidth, imageHeight);
		}
	};

	this._render();

	jsgtk.Event.addDomListener(this._dom, "click", function(){
		self.emit("clicked");
	});

	jsgtk.Event.addDomListener(this._dom, "mouseover", function(){
		self.emit("mouseover");
	});

	jsgtk.Event.addDomListener(this._dom, "mouseout", function(){
		self.emit("mouseout");
	});
};

gtk.Image.prototype = new gtk.Widget();
gtk.Image.prototype.constructor = gtk.Image;


/**
* Set Image src from given file
* @param {String} filename which is assigned when we call the method.
*/
gtk.Image.prototype.setFromFile = function(fileName, imageWidth, imageHeight){
    var file = fileName || this._defaultImage;
	this._dom.src = file;

    this.resetMinWidth(imageWidth || this._minWidth);
    this.resetMinHeight(imageHeight || this._minHeight);

    this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);
};

/**
* when mouseover the image, the frame will be visible
*/
gtk.Image.prototype.enableDelete = function(){
	gtk.ImageAnimator.init(this);
};

/** @private */
gtk.Image.prototype.resize = function(width, height){
   this._resizeWidth(width, height);
	this._resizeHeight(width, height);
};

/** @private */
gtk.Image.prototype.resetWidth = function(width){
	this._dom.setAttribute("width", width || this._minWidth);
	this.resetMinWidth(width || this._minWidth);
};

/** @private */
gtk.Image.prototype.resetHeight = function(height){
	this._dom.setAttribute("height", height || this._minHeight);
	this.resetMinHeight(height || this._minHeight);
};

/**
* reset the size of image
* @param {number} width
* @param {number} height
*/
gtk.Image.prototype.resetSize = function(width, height){
	 if(width !== null) this.resetWidth(width);
	 if(height !== null) this.resetHeight(height);
};

/**
* reset to the defualt size
*/
gtk.Image.prototype.resetDefaultSize = function(){
	var actualImageSize = this._getActualImageSize();
	this.resetSize(actualImageSize[0], actualImageSize[1]);
};

gtk.Image.prototype._getActualImageSize = function(){
	var imageNode = this.getChildNode();
	var imageObj = new Image();
		 imageObj.src = imageNode.src;
	return [imageObj.width, imageObj.height];
};

gtk.Image.prototype._resizeWidth = function(width){
	this._width = width;
    this._domObj.style.width = width + "px";
		this._dom.width = this._minWidth;
};

gtk.Image.prototype._resizeHeight = function(height){
	this._height = height;
    this._domObj.style.height = height + "px";
		this._dom.height = this._minHeight;
};

/**
 * Set attributes
 * @param {array} array of attributes object
 */
gtk.Image.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.Image.prototype.getChildNode = function(){
	return this._dom;
};

/**
 * Assign property disabled to image
 */
gtk.Image.prototype.disable = function(){
    jsgtk.Dom.showElement(this._disableLayer);
};

/**
 * Assign property enabled to image
 */
gtk.Image.prototype.enable = function(){
    jsgtk.Dom.hideElement(this._disableLayer);
};

/** @private */
gtk.Image.prototype._applyAttribute = function(){
	for(var el in this._attribute)
		this._dom.style[el] = this._attribute[el];
};

gobject.signalNew('closed', gtk.Image, [], null, null);
gobject.signalNew('clicked', gtk.Image, [], null, null);
gobject.signalNew('mouseout', gtk.Image, [], null, null);
gobject.signalNew('mouseover', gtk.Image, [], null, null);

/**@constructor*/
gtk.ImageAnimator = {
	_timeoutId1: null,
	_timeoutId2: null,
	_isHeaderActive: false,
	_isImageMousovered: false,
	_imageObj: null,
	init: function(imageObj){
		this._imageObj = imageObj;

		jsgtk.Dom.showElement(this._imageObj._invisibleOverlay);

		var self = this;

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "mouseover", function(e){
			self._handleImageMouseovered();
			jsgtk.Event.stopBubble(e);
		});

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "mouseout", function(e){
			self._handleImageMouseOut();
			jsgtk.Event.stopBubble(e);
		});

		jsgtk.Event.addDomListener(this._imageObj.header, "mouseover", function(e){
			clearTimeout(self._timeoutId2);
			jsgtk.Event.stopBubble(e)
		});

		jsgtk.Event.addDomListener(this._imageObj.header, "mouseout", function(e){
			self._handleImageMouseOut();
			jsgtk.Event.stopBubble(e)
		});

		jsgtk.Event.addDomListener(this._imageObj._invisibleOverlay, "click", function(){
			self._imageObj.emit("clicked");
		});

		jsgtk.Event.addDomListener(this._imageObj.closeButton, "click", function(){
			self._imageObj.emit("closed");
		});
	},
	_handleImageMouseovered: function(){
		clearTimeout(this._timeoutId2);
		if(this._isHeaderActive) return;
		var self = this;
		self._imageObj._domObj.className += " gtk_image_enable_delete";
		this._timeoutId1 = window.setTimeout(function(){
			jsgtk.Dom.showElement(self._imageObj.header);
			self._imageObj.headerDom.style.top = 18 + "px";
			var i = 18;
			var intervalId = window.setInterval(function(){
				i -= 4;
				self._imageObj.headerDom.style.top = i + "px";
				if(i < 0){
					self._imageObj.headerDom.style.top = 0 + "px";

					self._isHeaderActive = true;
					self._isImageMousovered = true;

	 				clearInterval(intervalId);
				}
			}, 40);
		}, 200);
	},
	_handleImageMouseOut: function(){
		var self = this;
		clearTimeout(this._timeoutId1);
		self._timeoutId2 = window.setTimeout(function(){
			var i = 0;
			var intervalId = window.setInterval(function(){
				i += 4;
				self._imageObj.headerDom.style.top = i + "px";
				if(i >18){
					self._imageObj.headerDom.style.top = 18 + "px";
					jsgtk.Dom.hideElement(self._imageObj.header);

					self._isHeaderActive = false;
					self._isImageMousovered = false;

					self._imageObj._domObj.className = "gtk_image";
					clearInterval(intervalId);
				}
			}, 40);
		}, 50);
	}
};
/**
* @constructor
* @base gtk.Widget
* @param {Interger} columns The numbers of columns that slider can contain
* @param {Interger} spacing The value of spacing to set between each columns
*/
gtk.Slider = function(columns, spacing) {
	gtk.Widget.apply(this);

    var self = this;

	this._width      = this._minWidth  = 300;
	this._height     = this._minHeight = 200;

	this._visibleColumns    = columns || 1;
	this._spacing    = spacing || 1;
	this._marginLeft = 50;

	this._slideWidth = this._getSlideWidth(this._width);

	this._slides     = [];
	this._currentLeft= 0;
	this._arrowWidth = 27;
	this._arrowHeight= 37;
	this._slideWidths;

	this._isMoving = false;

	/** @private */
	this._render = function(){
		this._domObj.className   += " gtk_slider";
		    this._viewport = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slides_viewport", "style": "position: relative;"});
		    this._viewport.style.left  = this._marginLeft + "px";
		    this._viewport.style.overflowY = "hidden";
		        this._slidesContainer = jsgtk.Dom.create({"append": this._viewport, "tag": "div", "class": "gtk_slides_container"});
		        this._slidesContainer.style.height = this._height + "px";

        this._leftArrow = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_leftarrow"});
        this._leftArrowDisabled = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_leftarrow_disabled"});
        this._positionArrowLeft(this._leftArrow);
        this._positionArrowLeft(this._leftArrowDisabled);
        jsgtk.Dom.hideElement(this._leftArrow);

        this._rightArrow = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_rightarrow"});
        this._rightArrowDisabled = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_slider_rightarrow_disabled"});
        this._positionArrowRight(this._rightArrow);
        this._positionArrowRight(this._rightArrowDisabled);
        jsgtk.Dom.hideElement(this._rightArrowDisabled);

 		this.resize(this._width, this._height);
	};

    this._render();

    jsgtk.Event.addDomListener(this._leftArrow, "click", function(e){

        if(!self._isMoving && self._currentLeft !== 0)self._moveRight();
	    jsgtk.Event.stopBubble(e);
	}, false);

    jsgtk.Event.addDomListener(this._rightArrow, "click", function(e){

        if(!self._isMoving){
            var len = self._slides.length >= self._visibleColumns ? self._visibleColumns : 1;
            var left = self._slideWidth * (self._slides.length - len);
            var isMovable = left + self._currentLeft;
            if(isMovable >= 0) self._moveLeft();
        }
        jsgtk.Event.stopBubble(e);
	}, false);
};

gtk.Slider.prototype = new gtk.Widget();
gtk.Slider.prototype.constructor = gtk.Slider;

/**
* Append gtk.Widget(any type)
* @param {gtk.Widget} widget
* @param {boolean} isCloseButtonVisible
*/
gtk.Slider.prototype.appendChild = function(widget, closeButtonVisible){

    var self = this;
    var isCloseButtonVisible = closeButtonVisible === undefined || closeButtonVisible === null ? false : closeButtonVisible;
    var scroller = this._getScroller(widget);

    var slide = this._constructSlide(scroller);
    this._resizeElement(slide, this._slideWidth, this._height - 12);

    var space = this._constructSpace();
    this._resizeElement(space, this._spacing, this._height);

    var closeButton = slide.lastChild;

    this._slides.push({
        "slide"  : slide,
        "space"  : space,
        "widget" : scroller
    });
    scroller.resizeWidthComposite(this._slideWidth);
    scroller.resizeHeightComposite(this._height - 12);

    var slideWidth = this._slideWidth + this._spacing + 2;
    this._setSlideWidths(slideWidth * (this._slides.length));

    this.resize(this._width, this._height);

    var jsgtkDom = jsgtk.Dom;
    var jsgtkEvent = jsgtk.Event;

    if(this._slides.length <= this._visibleColumns) {
        jsgtkDom.hideElement(this._rightArrow);
        jsgtkDom.showElement(this._rightArrowDisabled);
    }else{
        jsgtkDom.hideElement(this._rightArrowDisabled);
        jsgtkDom.showElement(this._rightArrow);
    }

    jsgtk.Event.addDomListener(slide, "mouseover", function(e){
        if(isCloseButtonVisible)
        	jsgtkDom.showElement(this.lastChild);
        jsgtkEvent.stopBubble(e);
    });

    jsgtk.Event.addDomListener(slide, "mouseout", function(e){
        jsgtkDom.hideElement(this.lastChild);
        jsgtkEvent.stopBubble(e);
    });

    jsgtk.Event.addDomListener(closeButton, "mousedown", function(e){
        self._removeSlide(slide, space);
        if(self._slides.length <= self._visibleColumns){
            self._disableArrow();
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.Slider.prototype.resetMinWidth = function(width){
	this._isMinWidthChanged = width ? true : false;
	this._minWidth = width;
    this._width = width;
    this.resizeWidthComposite(this._width);
};

/** @private */
gtk.Slider.prototype.resetMinHeight = function(height){
	this._isMinHeightChanged = height ? true : false;
	this._minHeight = height;
    this._height = height;
    this.resizeHeightComposite(this._height);
};

/**
* clear all gtk.Widget inside gtk.Slider
*/
gtk.Slider.prototype.clearSlides = function(){
    this._slides = [];
    this._slidesContainer.innerHTML = "";
    this._disableArrow();
    this._viewport.scrollLeft = 0;
};

/** @private */
gtk.Slider.prototype._disableArrow = function(){
	var domUtil = jsgtk.Dom;
    domUtil.hideElement(this._rightArrow);
    domUtil.showElement(this._rightArrowDisabled);

    domUtil.hideElement(this._leftArrow);
    domUtil.showElement(this._leftArrowDisabled);
};

/** @private */
gtk.Slider.prototype._removeSlide = function(slide, space){
    var self = this;
    for(var i = this._slides.length; i--;){
        if(this._slides[i].slide === slide) {
            this._slides.splice(i, 1);
            break;
        }
    }

    var width = parseInt(slide.style.width);
    var intervalID = window.setInterval(function(){
        width -= 20;
        slide.style.width = width + "px";
        if(width <= 0) {
            var slideWidth = self._slideWidth + self._spacing + 2;

            self._setSlideWidths(slideWidth * (self._slides.length));

            self._slidesContainer.style.width = self._slideWidths + "px";

            var left = self._currentLeft - slideWidth;

            var slideParent = slide.parentNode;

            if(slideParent.lastChild === space) self._viewport.scrollLeft -= 5;

            if(left >0) self._currentLeft = left;

            slideParent.removeChild(space);
            slideParent.removeChild(slide);

            clearInterval(intervalID);
        }
    }, 5);
};

/** @private */
gtk.Slider.prototype._setSlideWidths = function(width){
    this._slideWidths = width;
};

/** @private */
gtk.Slider.prototype._resizeSlides = function(width, height){
    var slide;

    this._slideWidth = this._getSlideWidth(width);

    var slideWidth = this._slideWidth + this._spacing + 2;
    this._setSlideWidths(slideWidth * (this._slides.length));

    for(var i = 0, len = this._slides.length; i<len; i++){
        slide = this._slides[i];
        this._resizeElement(slide["slide"], this._slideWidth, height - 13);
        this._resizeElement(slide["space"], this._spacing, height);
        slide["widget"].resizeWidthComposite(this._slideWidth);
        slide["widget"].resizeHeightComposite(height - 13);
    }
    this.resize(width, height);

    this._positionArrowLeft(this._leftArrow);
    this._positionArrowLeft(this._leftArrowDisabled);
    this._positionArrowRight(this._rightArrow);
    this._positionArrowRight(this._rightArrowDisabled);
};

/** @private */
gtk.Slider.prototype._getSlideWidth = function(width){
    width = ( (width - this._marginLeft * 2) - ( (this._visibleColumns - 1) * this._spacing) - this._visibleColumns * 2) / this._visibleColumns;
    return width;
};

/** @private */
gtk.Slider.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Slider.prototype._resizeElement = function(element, width, height){
    element.style.width = width + "px";
    element.style.height= height + "px";
};

/** @private */
gtk.Slider.prototype._constructSpace = function(){
    var space = jsgtk.Dom.create({"append": this._slidesContainer, "tag": "div", "style": "float: left;"});
    return space;
};

/** @private */
gtk.Slider.prototype._constructSlide = function(scrolledWindow){
    var slide = jsgtk.Dom.create({"append": this._slidesContainer, "tag": "div", "class": "gtk_slider_slide", "style": "float: left;"});
    slide.appendChild(scrolledWindow._domObj);
	var closeButton = jsgtk.Dom.create({"append": slide, "tag": "div", "class": "gtk_slider_close_button"});
	jsgtk.Dom.hideElement(closeButton);
    return slide;
};

/** @private */
gtk.Slider.prototype._getScroller = function(widget){
    var scroller = new gtk.Scroller();
    scroller.show();
    scroller.appendChild(widget, true);
    scroller.disableHScrolled();

    return scroller;
};

/** @private */
gtk.Slider.prototype._moveRight = function(){
    var self = this;
    var left = this._currentLeft - (this._slideWidth + this._spacing + 2);

    var intervalID = window.setInterval(function(){
        self._currentLeft -= 20;
        self._isMoving = true;
        self._viewport.scrollLeft = self._currentLeft;

        if(self._currentLeft <= left){
            self._viewport.scrollLeft = left;
            self._currentLeft = left;
            clearInterval(intervalID);
            self._enableRightArrow();
            if(self._viewport.scrollLeft === 0) self._disableRightArrow();
            self._isMoving = false;
        }
    }, 10);
};

/** @private */
gtk.Slider.prototype._moveLeft = function(){

    var self = this;
    var left = this._currentLeft + this._slideWidth + this._spacing + 2;

    var intervalID = window.setInterval(function(){
        self._currentLeft += 20;
        self._viewport.scrollLeft = self._currentLeft;
        self._isMoving = true;
        if(self._currentLeft >= left){
            self._viewport.scrollLeft = left;
            self._currentLeft = left;

            self._enableLeftArrow();

            var actualSlideWidth = self._slideWidth + self._spacing + 2;
            var width = Math.floor(self._slideWidths - self._viewport.scrollLeft);
            if(width <= actualSlideWidth * self._visibleColumns) self._disableLeftArrow();
            clearInterval(intervalID);

            self._isMoving = false;
        }
    }, 10);
};

/** @private */
gtk.Slider.prototype._positionArrowLeft = function(leftArrow){
	var leftArrowStyle = leftArrow.style;
    leftArrowStyle.width  = this._arrowWidth + "px";
    leftArrowStyle.height = this._arrowHeight + "px";
    leftArrowStyle.top = this._height / 2 - this._arrowHeight/2 + "px";
    leftArrowStyle.left = "16px";
};

/** @private */
gtk.Slider.prototype._positionArrowRight = function(rightArrow){
	var rightArrowStyle = rightArrow.style;
    rightArrowStyle.width  = this._arrowWidth + "px";
    rightArrowStyle.height = this._arrowHeight + "px";
    rightArrowStyle.top = this._height / 2 - this._arrowHeight/2 + "px";
    rightArrowStyle.right = "8px";
};

/** @private */
gtk.Slider.prototype._enableRightArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._rightArrowDisabled);
    jsgtkDom.showElement(this._rightArrow);
};

/** @private */
gtk.Slider.prototype._disableRightArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._leftArrow);
    jsgtkDom.showElement(this._leftArrowDisabled);
};

/** @private */
gtk.Slider.prototype._enableLeftArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.hideElement(this._leftArrowDisabled);
    jsgtkDom.showElement(this._leftArrow);
};

/** @private */
gtk.Slider.prototype._disableLeftArrow = function(){
	var jsgtkDom = jsgtk.Dom;
    jsgtkDom.showElement(this._rightArrowDisabled);
    jsgtkDom.hideElement(this._rightArrow);
};

/** @private */
gtk.Slider.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width  = this._width + "px";
    this._viewport.style.width = this._width  - this._marginLeft * 2 + 6 + "px";
    this._slidesContainer.style.width = this._slideWidths + "px";
};

/** @private */
gtk.Slider.prototype._resizeHeight= function(height){
    this._height = height;
    this._viewport.style.height= this._height + 25 + "px";
    this._domObj.style.height = this._height + "px";
    this._slidesContainer.style.height = this._height + "px";
};

/** @private */
gtk.Slider.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/** @private */
gtk.Slider.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.Slider.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._resizeSlides(minWidth, this._height);
};

/** @private */
gtk.Slider.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeSlides(this._width, minHeight);
};
/**
 * A single line text entry field
 * @constructor
 * @param {String} maxLength The maximum length of the entry
 */
gtk.Entry = function(maxLength){
	gtk.Widget.apply(this);

	/** 'keypressed' signal */
	this.ENTRY_KEYPRESSED = "keypressed";
	/** 'enterkeypressed' signal */
	this.ENTRY_ENTERKEYPRESSED = "enterkeypressed";
	/** 'focused' signal */
	this.ENTRY_FOCUSED = "focused";
	/** 'blurred' signal */
	this.ENTRY_BLURRED = "blurred";
	/** 'changed' signal */
	this.ENTRY_CHANGED = "changed";
	/** 'clicked' signal */
	this.ENTRY_CLICKED = "clicked";

    var self 				= this;
	this._dom 				= null;
	this._text 				= "";
	this._maxLength 		= maxLength || "";
	this._width 			= this._minWidth = 60;
	this._height 			= this._minHeight = 18;
	this._widthChars;
	this._span;
	this._visible 			= true;
	this._entryFirstValue 	= "";
	this._showError 		= true;

	/** @private */
	this._render = function(){
		var _self = this;
		this._domObj.className += " gtk_entry";
			this._span = jsgtk.Dom.create({append: this._domObj, tag: "span"});
				this._dom = jsgtk.Dom.create({append: this._span, tag: "input", "className":"gtk_entry_input", type: "text", style:"border:0;"});

        this.resize(this._width, this._height);

	};

	this._render();

	this._initEvent();
};

gtk.Entry.prototype = new gtk.Widget();
gtk.Entry.prototype.constructor = gtk.Entry;

/** @private */
gtk.Entry.prototype._initEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;

	jsgtkEvent.addDomListener(this._dom, "keyup", function(e){
        if(e.keyCode === gtk.ENTER_KEY){
        	self.emit("enterkeypressed", self._dom.value, e);
        	return;
        }
        if(self._dom.value != self._entryFirstValue){
			self.emit("changed", self._dom.value);
        }

        self.emit("keypressed", self._dom.value, e);
	    jsgtkEvent.stopBubble(e);
	}, false);

	jsgtkEvent.addDomListener(this._span, "click", function(){
	    self._dom.focus();
	    self.emit("clicked", self._dom.value);
	});

	jsgtkEvent.addDomListener(this._dom, "keypress", function(){
	    self._entryFirstValue = self._dom.value;
	});

	jsgtkEvent.addDomListener(this._dom, "focus", function(e){
	    self.emit("focused", self._dom.value, e);
	    self._domObj.style.border = "1px groove #F9CF06";
	    if(self._label) self._label.style.display = "none";
	});

	jsgtkEvent.addDomListener(this._dom, "blur", function(e){
	    self.emit("blurred", self._dom.value, e);
	    self._domObj.style.border = "";
	    if(self._label) if(!self._dom.value) self._label.style.display = "";
	}, false);
};


/**
 * Focus the cursor in the entry
 */
gtk.Entry.prototype.setFocus = function(){
    this._dom.focus();
};

/**
 * Method to show error or not
 * @param {Boolean} isOnError to show error, false to hide
 */
gtk.Entry.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;
    if(this._showError) this._domObj.className += " gtk_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_entry";
};

/**
 * Method to set tab index value
 * @param {Integer} index integer value to be set
 */
gtk.Entry.prototype.setTabindex = function(index){
    this._dom.tabIndex = index;
};
/**
 * @depricate
 */
gtk.Entry.prototype.setTextSelectable = function(isSelectable){
    if(isSelectable){
		this.activateTextSelectable();
		console.log("gtk.Entry.setTextSelectable() will be depricated, please use gtk.Entry.activateTextSelectable()")
	}else{
		this.deactivateTextSelectable();
		console.log("gtk.Entry.setTextSelectable() will be depricated, please use gtk.Entry.deactivateTextSelectable()")
	}
};

/** make gtk.Entry selectabe*/
gtk.Entry.prototype.activateTextSelectable = function(){this._dom.className = ""};

/** make gtk.Entry unselectable*/
gtk.Entry.prototype.deactivateTextSelectable = function(){this._dom.className = "gtk_entry_non_selectable"};

/**
 * set the entry border to be visible or invisible
 * @param {Boolean} isBorderVisible If true, the entry border is visible. If false, the entry border is invisible
 */
gtk.Entry.prototype.setBorderVisible = function(isBorderVisible){
    this._domObj.style.borderColor = isBorderVisible ? "1px solid grey" : "transparent";
	console.log("gtk.Entry.setBorderVisible() will be depricated, please use gtk.Entry.showBorder() or gtk.Entry.hideBorder()");
};

gtk.Entry.prototype.showBorder = function(){this._domObj.style.borderColor = "1px solid grey"};
gtk.Entry.prototype.hideBorder = function(){this._domObj.style.borderColor = "transparent"};

/**
 * Sets the "text" property to the value of text. The string in text replaces the current contents of the entry.
 * @param {String} text A string to use as the new contents of the entry
 */
gtk.Entry.prototype.setText = function(text){
	this._text = text;
	this._dom.value = text === undefined ? "" : this._text;

	if(this._label) this._removeLabel();

	this.emit("changed");
};

gtk.Entry.prototype._removeLabel = function(){
	this._span.removeChild(this._label);
	this._label = null;
};

/**
 * Returns the value of the "text" property which is a string containing the contents of the entry.
 * @return {String} text the content of the entry as a string
 */
gtk.Entry.prototype.getText = function(){
	return this._dom.value;
};

/**
 * sets the "width-chars" property to the value of n_char
 * @param {interger} nChars Width in chars
 */
gtk.Entry.prototype.setWidthChars = function(nChars){
 	this._widthChars = nChars;
 	this._updateWidthHeight(nChars);
    this._resizeWidth(this._width);
};

/**
 * gets the value of the "widthChars" property which is set by the setWidthChars() method
 * @return {integer} number of chars to request space for
 */
gtk.Entry.prototype.getWidthChars = function(){
	return this._widthChars;
};

/**
 * sets the "max-length" property to the value of max
 * @param {interger} max The maximum length of the entry. The value passed in will be clamped to the range 0-65536.
 */
gtk.Entry.prototype.setMaxLength = function(max){
 	this._maxLength = max;
 	this._dom.setAttribute("maxlength", max);
};

/**
 * retrieves the value of the "max-length" property that specifies the maximum allowed length of the text in the entry.
 * @return {integer} The maximum allowed number of characters
 */
gtk.Entry.prototype.getMaxLength = function(){
	return this._maxLength;
};

/**
 * sets the "visibility" property to the value of visible. If visible is True  the contents of the entry are displayed as plain text. If visible is False, the contents are obscured by replacing the characters with the invisible character
 * @param {boolean} visible If False the contents are obscured using the "invisible-char"
 */
gtk.Entry.prototype.setVisibility = function(visible){
 	this._visible = visible;
 	this._visible? this._dom.setAttribute("type", "text"): this._dom.setAttribute("type", "password");
};

/**
 * retrieves the value of the "visibility" property that determines whether the text in entry is visible.
 * @return {boolean} True if the text is currently visible
 */
gtk.Entry.prototype.getVisibility = function(){
	return this._visible;
};

/**
 * Retrieves the current length of the text in entry
 * @return {interger} Return the current number of characters in GtkEntry, or 0 if there are none.
 */
gtk.Entry.prototype.getTextLength = function(){
	var charLength = this._dom.value.length;
	return charLength;
};

/** @private */
gtk.Entry.prototype._updateWidthHeight = function(size){
	var input = this._dom.cloneNode(true);
	if(size){
		var inputStyle = input.style;
		input.setAttribute("size", size);
		inputStyle.width = "";
	}
	document.body.appendChild(input);
	this._width = input.clientWidth;
    this._minWidth = this._width;
	document.body.removeChild(input);
};

/**
 * Set the displayed label in Entry. When the entry is focus, the label will be remove from entry
 * @param {String} label The label to be displayed in entry
 */
gtk.Entry.prototype.setDisplayLabel = function(label){
	if(this._label) this._label.innerHTML = label || "";
    else this._label = jsgtk.Dom.create({"tag": "span", "append": this._span, "className": "gtk_entry_label", "text": label});
};

/**
 * Method to select text in entry
 */
gtk.Entry.prototype.select = function(){
    this._dom.select();
};

/**
* Entry will be enabled or disabled
* @param {boolean} isEnabled
*/
gtk.Entry.prototype.setEnable = function(isEnabled){
	isEnabled ? this.enable() : this.disable();
};

/**
 *  enable gtk.Entry
 */
gtk.Entry.prototype.enable = function(){
    this._dom.disabled = false;
    this._domObj.style.backgroundColor = "";
    this._dom.style.backgroundColor = "";
};

/**
 * disable gtk.Entry
 */
gtk.Entry.prototype.disable = function(){
    this._dom.disabled = true;
    this._dom.style.width = parseInt(this._domObj.style.width) - 4  + "px";
    this._domObj.style.backgroundColor = "#EDECEC";
    this._dom.style.backgroundColor = "#EDECEC";
};

/** @private */
gtk.Entry.prototype.getMinWidthComposite = function(){
	return this._minWidth;
};

/** @private */
gtk.Entry.prototype.getMinHeightComposite = function(){
	return this._minHeight;
};

/** @private */
gtk.Entry.prototype.resetMinHeightComposite = function(){
	var minSize = this.getSizeRequest();
	this.setHeight(minSize[1]);
};

/** @private */
gtk.Entry.prototype.resetMinWidthComposite = function(){
	var minSize = this.getSizeRequest();
	this.setWidth(minSize[0]);
};

/** @private */
gtk.Entry.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Entry.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - 2 + "px";
	this._dom.style.width = width - 8 + 'px';
	this._width = width;
};
/** @private */
gtk.Entry.prototype._resizeHeight = function(height){
   	this._domObj.style.height = height + "px";
   	this._dom.style.height = height - 2 + "px";
	this._span.style.lineHeight = height + "px";
	this._height = height;
};

/** @private */
gtk.Entry.prototype._resizeHeightByFont = function(height){
    this._height = this._minHeight = height;
   	this._domObj.style.height = height + "px";
   	this._dom.style.height = height - 2 + "px";
	this._span.style.lineHeight = height + "px";
};

/**
 * Reset height by font size
 * @param {Interger} size The size of font in pixel
 */
gtk.Entry.prototype.resizeEntryByFont = function(size){
    this._dom.style.fontSize = size + "px";
	var tmpNode = this._dom.cloneNode(true);
	var tempStyle = tmpNode.style;
	tempStyle.fontSize = size + "px";
	tempStyle.width = "";
	tempStyle.height = "";
	var height = jsgtk.Util.fullHeight(tmpNode);
    this._resizeHeightByFont(height);
};

/** @private */
gtk.Entry.prototype.resizeWidthComposite = function(width){
    this._width = width;
	this._resizeWidth(width);
};

/** @private */
gtk.Entry.prototype.resizeHeightComposite = function(height){
    this._height = height;
	this._resizeHeight(height);
};

gobject.signalNew("keypressed", gtk.Entry, null, null, []);
gobject.signalNew("enterkeypressed", gtk.Entry, null, null, []);
gobject.signalNew("focused", gtk.Entry, null, null, []);
gobject.signalNew("blurred", gtk.Entry, null, null, []);
gobject.signalNew("changed", gtk.Entry, null, null, []);
gobject.signalNew("clicked", gtk.Entry, null, null, []);
/**
 * EntryDropDown is a class to create a text entry dropdown or text entry with auto suggestion.
 * The EntryDropDown store data as a model from gtk.TreeStore
 * @constructor
 * @param {Integer} maximum length of characters
 * @extends gtk.Entry
 * @see gtk.TreeStore
 */
gtk.EntryDropDown = function(maxLength){
	gtk.Entry.apply(this, [maxLength]);

	/** 'selected' signal */
	this.ENTRYDROPDOWN_SELECTED = "selected";

	this._selectedRowText = "";
	this._selectRowIndex = 0;
	this._filteredData = [];
	this._selectedRowValue = null;
	this._hasScroller = false;
	/** @private */
	this._render = function(){
		var _self = this;
		this._domObj.className += " gtk_entry_drop_down";

		this.__dropDown = jsgtk.Dom.create({tag: "div", className: "gtk_entry_drop_down_holder", append: document.body});
		this._hideDropDown();
	};

	this._render();
	this._bindInputEvent();
	this._bindDropDownEvent();
};

gtk.EntryDropDown.prototype = new gtk.Entry();
gtk.EntryDropDown.prototype.constructor = gtk.EntryDropDown;

/**
 * A method to set selected item by any value
 * @param {String} key to search for
 * @param {any type} value that will match the value of given key
 */
gtk.EntryDropDown.prototype.setSelectedItemBy = function(valueMember, value){
	this.emit("_searchModelByValueMember", valueMember, value);
};

/** @private */
gtk.EntryDropDown.prototype._bindDropDownEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;

	jsgtkEvent.addDomListener(this.__dropDown, "click", function(e){
		var node = e.target;
		if(/gtk_entry_drop_down_element/.test(node.className)){
			self.emit("_searchbypath", self._selectedRowNode.id);
        }
        jsgtkEvent.stopBubble(e);
	});

	jsgtkEvent.addDomListener(this.__dropDown, "mouseover", function(e){
		var node = e.target;
        if(/gtk_entry_drop_down_element/.test(node.className)){
            self._deHilightRow(self._selectedRowNode);
		    self._selectedRowNode = node;
		    self._hilightRow(node);
		    self._selectRowIndex = self._getSelectedIndex(node);
        }
        node.className += " gtk_entry_drop_down_hover";
	});

	jsgtkEvent.addDomListener(document.body, "click", function(e){
		self._hideDropDown();
		jsgtkEvent.stopBubble(e);
	});
};

/**
 * Set model to Entry
 * @param {gtk.ListStore} model ListStore model to be set to Entry
 */
gtk.EntryDropDown.prototype.setModel = function(model){
	this._model = model;
	new gtk.EntryDropDownController(this, model);
};

/** @private */
gtk.EntryDropDown.prototype._constructRowNode = function(data){
	this._filteredData  = data;
	var docFragment = document.createDocumentFragment();
    for(var i = 0; i < data.length; i++){
		var dataObj = data[i]
        var element = jsgtk.Dom.create({tag: "div", append: docFragment, text: dataObj["display"], className: "gtk_entry_drop_down_element"});
        element.id = dataObj.path;
	}
	return docFragment;
};

gtk.EntryDropDown.prototype._renderDropDown = function(data){
	var rowsNode = this._constructRowNode(data);
	this._appendToDropDown(rowsNode);
};

/** @private */
gtk.EntryDropDown.prototype._appendToDropDown = function(dropDown){
	this.__dropDown.innerHTML = "";
	if(dropDown.textContent != "")
	    this.__dropDown.appendChild(dropDown);
	else this._hideDropDown();
};

/** @private */
gtk.EntryDropDown.prototype._deHilightRow = function(selectedRow){
	if(selectedRow) selectedRow.className = "gtk_entry_drop_down_element";
};

/** @private */
gtk.EntryDropDown.prototype._hilightRow = function(selectedRow){
	if(selectedRow) selectedRow.className += " gtk_entry_drop_down_selected";
};

/** @private */
gtk.EntryDropDown.prototype._hilightRowByDownArrowKey = function(){
	if(this._selectedRowNode !==  null){
	    if(this._selectedRowNode.nextSibling === null){
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this.__dropDown.childNodes[0];
	        this._selectRowIndex = 0;
	    }
	    else{
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this._selectedRowNode.nextSibling;
	        this._selectRowIndex = this._selectRowIndex + 1;
	    }
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
	else{
	    this._selectedRowNode = this.__dropDown.childNodes[0];
	    this._selectRowIndex = 0;
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
};

/** @private */
gtk.EntryDropDown.prototype._hilightRowByUpArrowKey = function(){
	if(this._selectedRowNode !==  null){
	    if(this._selectedRowNode.previousSibling === null){
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this.__dropDown.lastChild;
	        this._selectRowIndex = this.__dropDown.childNodes.length - 1;
	    }
	    else{
	        this._deHilightRow(this._selectedRowNode);
	        this._selectedRowNode = this._selectedRowNode.previousSibling;
	        this._selectRowIndex = this._selectRowIndex - 1;
	    }
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
	else{
	    this._selectedRowNode = this.__dropDown.lastChild;
	    this._selectRowIndex = this.__dropDown.childNodes.length - 1;
	    this._selectedRowNode.className += " gtk_entry_drop_down_selected";
	}
};

/** @private */
gtk.EntryDropDown.prototype._bindInputEvent = function(){
	var jsgtkEvent = jsgtk.Event;
    var self = this;

    var entry = this._domObj.firstChild.firstChild;
	jsgtkEvent.addDomListener(entry, "keydown", function(e){
        var keyInCode = e.keyCode;
		switch(keyInCode){
			case gtk.DOWN_ARROW_KEY:
			    self._changeSelectedByArrowDown();
				return;
			case gtk.UP_ARROW_KEY:
			    self._changeSelectedByArrowUp();
				return;
			case gtk.ESC_KEY:
                self._hideDropDown();
				return;
			case gtk.TAB_KEY:
                if(self._selectedRowNode) self.emit("_searchbypath", self._selectedRowNode.id);
				return;
			case gtk.ENTER_KEY:
				if(self._selectedRowNode) self.emit("_searchbypath", self._selectedRowNode.id);
				jsgtkEvent.stopBubble(e);
				return;
		}

	});

	this.connect("changed", function(value){
	    self._selectedRowNode = null;
	    self._selectedRowValue = null;
        self._showDropDown();
        self.emit("_modelfiltered", value);
	});
};

/** @private */
gtk.EntryDropDown.prototype._scrollByArroyKey = function(index){
    if(index == 0) this.__dropDown.scrollTop = 0;
    else this.__dropDown.scrollTop = index * 20 - 20;
};

/** @private */
gtk.EntryDropDown.prototype._changeSelectedByArrowUp = function(){
        if(this._filteredData.length > 0){
        	this._scrollByArroyKey(this._selectRowIndex);
            this._showDropDown();
            this._hilightRowByUpArrowKey();
		    this.setText(this._selectedRowNode.textContent);
        }

};

/** @private */
gtk.EntryDropDown.prototype._changeSelectedByArrowDown = function(){
    if(this._filteredData.length > 0){
    	this._scrollByArroyKey(this._selectRowIndex);
        this._showDropDown();
	    this._hilightRowByDownArrowKey();
	    this.setText(this._selectedRowNode.textContent);
    }
};

/** @private */
gtk.EntryDropDown.prototype._getSelectedIndex = function(){
	var tempNodes = this.__dropDown.cloneNode(true).childNodes;
	var found = false;
	for(var i = 0, len = tempNodes.length; i < len; i++){
		if(/gtk_entry_drop_down_selected/.test(tempNodes[i].className))
			break;
	}
	return i;
};

/**
 * Set display default value into EntryDropDown
 * @param {String} text value to be set
 */
gtk.EntryDropDown.prototype.setText = function(text){
    var input = this._domObj.firstChild.firstChild;
	input.value = text === undefined ? "" : text;
};

/**
* Return the text display in gtk.Entry
* @return {string} text
*/
gtk.EntryDropDown.prototype.getText = function(text){
    return this._domObj.firstChild.firstChild.value;
};

/**
 * Set selected value to display by default
 * @param {Integer} index an integer value to be passed
 */
gtk.EntryDropDown.prototype.setSelectedIndex = function(index){
	this.emit("_searchbyindex", index);
};

/** @private */
gtk.EntryDropDown.prototype._setSelectedRowValue = function(value){
	this._selectedRowValue = value;
};

/**
 *  Method to get/return an object of selected row value
 *  @param {Object}, it return selected object value
 */
gtk.EntryDropDown.prototype.getSelectedRowValue = function(){
	return this._selectedRowValue !== null ? this._selectedRowValue : this.getText();
};

/**
 * Get/Return an object valau if exists
 */
gtk.EntryDropDown.prototype.getValue = function(){
    return this.getSelectedRowValue();
};

/** @private */
gtk.EntryDropDown.prototype._hideDropDown = function(){
	this.__dropDown.style.display = "none";
	this._selectedRowNode = null;
	this._filteredData  = [];
};

/** @private */
gtk.EntryDropDown.prototype._showDropDown = function(){
    this.__dropDown.style.width = this._width - 2 + "px";
    this.__dropDown.style.left = "-1px";
    this.__dropDown.style.top = "1px";
	this.__dropDown.style.display = "";
	this._setPosition();
};
/** @private */
gtk.EntryDropDown.prototype._setPosition = function(e){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);
    var header = parseInt(this._height);
    var scrollY = this._getNearestScrollerY();
    this.__dropDown.style.top = top + header + 2 - scrollY + "px";
    this.__dropDown.style.left = left + "px";
};

/** @private */
gtk.EntryDropDown.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode.parentNode != null){
        if(/gtk_scrolled_window/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._hasScroller = true;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};

/** @private */
gtk.EntryDropDown.prototype._addEventMouseWheel = function(scrollObj){
    var _self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    _self._hideDropDown();
	}, false);
};

gobject.signalNew("_searchbyindex", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_searchbypath", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_checkvaluechanged", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_searchModelByValueMember", gtk.EntryDropDown, null, null, []);
gobject.signalNew("_modelfiltered", gtk.EntryDropDown, null, null, []);
gobject.signalNew("selected", gtk.EntryDropDown, null, null, []);
/**
 * PopupWindow is a class to create pop-up window to appear on the top most of the screen.
 * There are five types: Window Default, Window Confirm, Window Warning, Window Info, and Window Error or Critical
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @param {String} type It must be a constant variable to create pop-up window:
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_CONFIRM The flag constant value to define as Window Confirm
 * <br> - gtk.WINDOW_WARNING The flag constant value to define as Window Warning
 * <br> - gtk.WINDOW_INFO The flag constant value to define as Window Info
 * <br> - gtk.WINDOW_ERROR The flag constant value to define as Window Error
 * <br> - gtk.WINDOW_Default The flag constant value to define as Window Default
 * @constructor
 */
gtk.PopupWindow = function(title, textMessage, type){
    switch (type){
		case gtk.WINDOW_CONFIRM:
			return new gtk.PopupWindowConfirm(title, textMessage);
			break;
		case gtk.WINDOW_WARNING:
			return new gtk.PopupWindowWarning(title, textMessage);
			break;
		case gtk.WINDOW_INFO:
			return new gtk.PopupWindowInfo(title, textMessage);
			break;
		case gtk.WINDOW_ERROR:
			return new gtk.PopupWindowError(title, textMessage);
			break;
		default:
			return new gtk.PopupWindowDefault(title, textMessage);
    }
};
/**
 * PopupWindowDefault is a class to create pop-up window type Default
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.Widget
 * @extends gtk.Widget
 */
gtk.PopupWindowDefault = function(title, textMessage){
    gtk.Widget.apply(this);

    this._offsetX;
    this._offsetY;
    this._title     = title;
    this._bodyText  = textMessage;

    this._maxWidth  = 678;
    this._maxHeight = 600;
    this._width     = this._minWidth = 380;
    this._height    = this._minHeight = 130;
    this._iconWidth = 0;
    this._okBtnHandler = null;
    this._closeBtnHandler;

    /** It allows user to connect to a registered signal event "ok" when click OK button */
    this.POPUPWINDOWDEFAULT_OK = "";
    /** It allows user to connect to a registered signal event "closed" when click close button */
    this.POPUPWINDOWDEFAULT_CLOSED = "";

    this._render = function(){

        this._domObj.className += " gtk_popup_window";
        this._domObj.style.position = "absolute";
            this._header =  jsgtk.Dom.create({"append" : this._domObj, "tag": "div", "className": "gtk_popup_window_header"});
                this._titleDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_title", "append": this._header});
                this._titleDiv.innerHTML = this._title;
                this._closeButton = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_button_close", "append": this._header});

            this._body = jsgtk.Dom.create({"append" : this._domObj, "tag": "div", "className": "gtk_popup_window_body"});
                this._bodyDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_body_text", "append": this._body});
                    this._bodyDiv.innerHTML = this._bodyText;

            this._footer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "className": "gtk_popup_window_footer"});
            this._footerButtonOk = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_footer_buttonOk", "append": this._footer});

            this._titleDiv.style.width = this._width - 22 + "px";
    };

    this._render();
};

gtk.PopupWindowDefault.prototype = new gtk.Widget();
gtk.PopupWindowDefault.prototype.constructor = gtk.PopupWindowDefault;

/**
 * A method to set title to the header of the pop-up window
 * @param {String} title The title of the pop-up window to be set
 * type {String}
 */
gtk.PopupWindowDefault.prototype.setTitle = function(title){
    this._title = title;
    this._resizePopupWindow();
};

/**
 * A method to set content to the body of the pop-up window
 * @param {String} textMessage The text content of the body of the pop-up window to be set
 * type {String}
 */
gtk.PopupWindowDefault.prototype.setBody = function(textMessage){
    this._bodyText = textMessage;
    this._bodyDiv.innerHTML = this._bodyText;
    this._resizePopupWindow();
};

/**
 * A method to set focus onto the pop-up window
 */
gtk.PopupWindowDefault.prototype.setFocus = function(){
    this._domObj.focus();
};

/**
 * A method to display the pop-up window after it is instantiated
 */
gtk.PopupWindowDefault.prototype.draw = function(){
        this._drawOkButton();
        this._resizePopupWindow();
        this._setCenterScreen();
        this._disableBackground();
        this._initEvent();
        document.body.appendChild(this._domObj);
};

/** @private */
gtk.PopupWindowDefault.prototype._drawOkButton = function(){
    this._footerButtonOk.innerHTML = "";
    var buttonOk = new gtk.Button("OK");
    buttonOk.resizeWidthComposite(75);
    buttonOk.resizeHeightComposite(22);
    this._footerButtonOk.appendChild(buttonOk.getNode());
};

/** @private */
gtk.PopupWindowDefault.prototype._calculateHeight = function(dom){
    var tempDom = dom.cloneNode(true);
    tempDom.style.display = "none";
    tempDom.style.position = "absolute";

    var height = jsgtk.Util.fullHeight(tempDom);

    return height;
};

/** @private */
gtk.PopupWindowDefault.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    jsgtkEvent.addDomListener(this._header, "mouseover", function(e){
        self._enableDrag();
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(this._header, "mouseout", function(e){
        self._disableDrag();
    });

    this._closeBtnHandler = jsgtkEvent.addDomListener(this._closeButton, "mouseup", function(e){
        self.emit("closed");
        self.close();
    });

    this._okBtnHandler = jsgtkEvent.addDomListener(this._footerButtonOk, "mouseup", function(){
        self.emit("ok");
        self.close();
    });
};

/** @private */
gtk.PopupWindowDefault.prototype._detachOkBtnHandler = function(){
    jsgtk.Event.removeListener(this._okBtnHandler);
    this._okBtnHandler = null;
};

/** @private */
gtk.PopupWindowDefault.prototype._detachCloseBtnHandler = function(){
    jsgtk.Event.removeListener(this._closeBtnHandler);
    this._closeBtnHandler = null;
};

/**
 * A method to close/remove pop-up window
 */
gtk.PopupWindowDefault.prototype.close = function(){
    if(!this._domObj.parentNode) return;
    this._domObj.parentNode.removeChild(this._domObj);
    this._disableDiv.parentNode.removeChild(this._disableDiv);
    this._detachOkBtnHandler();
    this._detachCloseBtnHandler();

};

/** @private */
gtk.PopupWindowDefault.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._header.style.width = width + "px";
        this._bodyDiv.style.width = width - this._iconWidth + "px";
        this._titleDiv.innerHTML = this._title;
        this._titleDiv.style.width = this._width - 32 + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._bodyDiv.style.height = height - 62 + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._getMaxHeight = function(){
    var height = jsgtk.Util.calculateTextWidthHeight(this._bodyText)[1] + 62;
    if(height < this._minHeight) height = this._minHeight;
    return height;
};

/** @private */
gtk.PopupWindowDefault.prototype._getMaxWidth = function(){
    var width = jsgtk.Util.calculateTextWidthHeight(this._bodyText)[0];
    if(width > this._maxWidth - 20) width = this._maxWidth;
    else if(width < this._minWidth) width = this._minWidth;
    return width;
};

/** @private */
gtk.PopupWindowDefault.prototype._resizePopupWindow = function(){
    this._resizeWidth(this._getMaxWidth());
    this._resizeHeight(this._getMaxHeight());
};

/** @private */
gtk.PopupWindowDefault.prototype._calculateOffsetCenter = function(){
    var jsgtkUtil = jsgtk.Util;

    var windowWidth = jsgtkUtil.getWindowWidth();
    var windowHeight = jsgtkUtil.getWindowHeight();

    var scrollY = jsgtkUtil.getScrollY();
    var scrollX = jsgtkUtil.getScrollX();

    var offsetX =(windowWidth - this._width)/2 + scrollX;
    var offsetY =(windowHeight - this._height)/2 + scrollY;
    return [offsetX, offsetY];
};

/** @private */
gtk.PopupWindowDefault.prototype._setCenterScreen = function(){
    var offset = this._calculateOffsetCenter();
    var domObjStyle = this._domObj.style;
    domObjStyle.top = offset[1] + "px";
    domObjStyle.left= offset[0] + "px";
};

/** @private */
gtk.PopupWindowDefault.prototype._disableBackground = function(){
    this._disableDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_disable_body"});
    var widthHeight = jsgtk.Util.getPageSize();

    var disableDivStyle = this._disableDiv.style;
    disableDivStyle.width = widthHeight[0] + "px";
    disableDivStyle.height = widthHeight[1] + "px";
    disableDivStyle.opacity = 0.3;
    disableDivStyle.backgroundColor = "#ADADAD";

    document.body.appendChild(this._disableDiv);
};

/** @private */
gtk.PopupWindowDefault.prototype._getMinimizedTitle = function(titleDom, minimizedWidth){

    var tmpNode = titleDom.cloneNode(true);
    var title = tmpNode.innerHTML;

    var titleWidth = jsgtk.Util.fullWidth(tmpNode);
    if( titleWidth < minimizedWidth){
        return title;
    }

    var charWidth = 0;
    var title = title.replace(/\s+/g,'`');

    for(var i = 0, len = title.length; i < len; i++){
        var character = title.substring(i, i+1);
        tmpNode.innerHTML = character;
        charWidth += jsgtk.Util.fullWidth(tmpNode);

        if(charWidth >= minimizedWidth){
            break;
        }
    }

    var minimizedTitle = title.substring(0, i);
    return minimizedTitle.split("`").join(" ") + "...";
};

/** @private */
gtk.PopupWindowDefault.prototype._enableDrag = function(){
    var minX = 1, maxX =jsgtk.Util.getWindowWidth()-this._width - 17;
    var minY = 0, maxY = jsgtk.Util.getWindowHeight()-this._height - 17;
    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, minX, maxX, minY, maxY);
};

/** @private */
gtk.PopupWindowDefault.prototype._disableDrag = function(){
    var minX = maxX = parseInt(this._domObj.style.left);
    var minY = maxY = parseInt(this._domObj.style.top);
    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, minX, maxX, minY, maxY);
};

gobject.signalNew("ok", gtk.PopupWindowDefault, null, null, []);
gobject.signalNew("closed", gtk.PopupWindowDefault, null, null, []);
/**
 * PopupWindowConfirm is a class to create pop-up window type Confirmation
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.PopupWindowDefault
 * @extends gtk.PopupWindowDefault
 */
gtk.PopupWindowConfirm = function(title, textMessage){
    gtk.PopupWindowDefault.apply(this, [title, textMessage]);
    var self = this;
    this._iconWidth = 95;
    this._cancelBtnHandler = null;
    /** It allows user to connect to a registered signal event "cancelled" when click Cancel button */
    this.POPUPWINDOWCONFIRM_CANCELLED = "";

    this._render = function(){

        var imageDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_confirm_image"});
        this._body.insertBefore(imageDiv,this._bodyDiv);
        this._bodyDiv.style.margin = "0px 0px 0px 66px";
        this._footerButtonCancel = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_footer_buttonConcel"});
    };
    this._render();
};

gtk.PopupWindowConfirm.prototype = new gtk.PopupWindowDefault();
gtk.PopupWindowConfirm.prototype.constructor = gtk.PopupWindowConfirm;

/**
 * A method to display the pop-up window after it is instantiated
 */
gtk.PopupWindowConfirm.prototype.draw = function(){
        this._drawOkButton();
        this._drawCancelButton();
        this._resizePopupWindow();
        this._setCenterScreen();
        this._disableBackground();
        this._initEvent();
        this._attachCancelBtnEvent();
        document.body.appendChild(this._domObj);
};

/** @private */
gtk.PopupWindowConfirm.prototype._attachCancelBtnEvent = function(){
    var self = this;
    this._cancelBtnHandler = jsgtk.Event.addDomListener(this._footerButtonCancel, "click", function(){
		self.emit("cancelled");
        self.close();
    });
};

/** @private */
gtk.PopupWindowConfirm.prototype._drawCancelButton = function(){
        this._footerButtonCancel.innerHTML = "";
        var buttonCancel = new gtk.Button("Cancel");
            buttonCancel.resizeWidthComposite(75);
            buttonCancel.resizeHeightComposite(22);

        this._footerButtonCancel.appendChild(buttonCancel.getNode());
        this._footer.insertBefore(this._footerButtonCancel, this._footerButtonOk);
};

/**
 * A method to close/remove pop-up window
 */
gtk.PopupWindowConfirm.prototype.close = function(){
    if(!this._domObj.parentNode) return;

    this._domObj.parentNode.removeChild(this._domObj);
    this._disableDiv.parentNode.removeChild(this._disableDiv);
    this._detachOkBtnHandler();
    this._detachCancelBtnHandler();
    this._detachCloseBtnHandler();
};

/** @private */
gtk.PopupWindowConfirm.prototype._detachCancelBtnHandler = function(){
    jsgtk.Event.removeListener(this._cancelBtnHandler);
    this._cancelBtnHandler = null;
};

gobject.signalNew("cancelled", gtk.PopupWindowConfirm, null, null, []);
/**
 * PopupWindowError is a class to create pop-up window type Error
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.PopupWindowDefault
 * @extends gtk.PopupWindowDefault
 */
gtk.PopupWindowError = function(title, textMessage){
    gtk.PopupWindowDefault.apply(this, [title, textMessage]);
    this._iconWidth = 95;

    this._render = function(){
        var imageDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_error_image"});
        this._body.insertBefore(imageDiv,this._bodyDiv);
        this._bodyDiv.style.margin = "0px 0px 0px 66px";
    };
    this._render();
};

gtk.PopupWindowError.prototype = new gtk.PopupWindowDefault();
gtk.PopupWindowError.prototype.constructor = gtk.PopupWindowError;
/**
 * PopupWindowInfo is a class to create pop-up window type Info
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.PopupWindowDefault
 * @extends gtk.PopupWindowDefault
 */
gtk.PopupWindowInfo = function(title, textMessage){
    gtk.PopupWindowDefault.apply(this, [title, textMessage]);
    this._iconWidth = 95;

    this._render = function(){
        var imageDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_info_image"});
        this._body.insertBefore(imageDiv,this._bodyDiv);
        this._bodyDiv.style.margin = "0px 0px 0px 66px";
    };
    this._render();
};

gtk.PopupWindowInfo.prototype = new gtk.PopupWindowDefault();
gtk.PopupWindowInfo.prototype.constructor = gtk.PopupWindowInfo;
/**
 * PopupWindowWarning is a class to create pop-up window type Warning
 * @param {String} title The header title of the pop-up window
 * @param {String} textMessage The body text/content of the pop-up window
 * @constructor
 * @base gtk.PopupWindowDefault
 * @extends gtk.PopupWindowDefault
 */
gtk.PopupWindowWarning = function(title, textMessage){
    gtk.PopupWindowDefault.apply(this, [title, textMessage]);
    this._iconWidth = 95;

    this._render = function(){
        var imageDiv = jsgtk.Dom.create({"tag": "div", "className": "gtk_popup_window_warning_image"});
        this._body.insertBefore(imageDiv,this._bodyDiv);
        this._bodyDiv.style.margin = "0px 0px 0px 66px";
    };
    this._render();
};

gtk.PopupWindowWarning.prototype = new gtk.PopupWindowDefault();
gtk.PopupWindowWarning.prototype.constructor = gtk.PopupWindowWarning;
/**
 * NullObject is a class to create a null object
 * @constructor
 */
gtk.NullObject = {
	getMinWidthComposite: function(){},
	getMinHeightComposite: function(){},
	resizeWidthComposite: function(){},
	resizeHeightComposite: function(){}
}
/**
 * Panel is a class to create a container called panel
 * @param {String} title The header title of the panel
 * @param {Integer} padding The integer value of padding of the panel
 * @constructor
 * @base gtk.Widget
 * @extends gtk.Widget
 */
gtk.Panel = function(title, padding) {
	gtk.Widget.apply(this);

	/** 'minimzed' signal */
	this.PANEL_MINIMIZED = "minimized";
	/** 'minimizedpanelmouseovered' signal */
	this.PANEL_MINIMIZEDPANEL_MOUSEOVERED = "minimizedpanelmouseovered";
	/** 'minimizedpanelmouseovered' signal */
	this.PANEL_MINIMIZEDPANEL_MOUSEOUT = "minimizedpanelmouseout";
	/** 'restored' signal */
	this.PANEL_RESTORED = "restored";
	/** 'closed' signal */
	this.PANEL_CLOSED = "closed";

	this._width               = this._minWidth  = 20;
	this._height              = this._minHeight = 20;
	this._title               = title || "gtk Title";
	this._minimizedWidth      = 150;
	this._inconsWidth         = 45;
	this._headerHeight        = 22;
	this._padding             = padding === undefined ? 10 : padding * 2;
	this._bodyHeight          = this._height - this._headerHeight;
	this._child               = null;
	this._isCloseButtonActive = false;
	this._minimizedPanel      = null;
	var self = this;
	this._createTooltip = false;

	/** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		var domObj = this._domObj;

		domObj.id = new UUID().id;
		domObj.className   += " gtk_panel";
        domObj.style.position = "absolute";

		    this._header = jsgtkDom.create({'append': domObj, 'tag': 'div', 'className': 'gtk_panel_header'});
    		        this._domTitle = jsgtkDom.create({"append": this._header, "tag": "div", "text": this._title, 'className': 'gtk_none_selectable gtk_panel_header_title'});

                this._closeButton = jsgtkDom.create({"append": this._header, "tag": "div", "title": "Close Panel","className": "gtk_panel_close_button", "style": "display:none;"});
                this._minimizeButton = jsgtkDom.create({'append': this._header, 'tag': 'div', "title": "Minimize Panel", 'className': 'gtk_panel_minimize_button'});

		    this._body = jsgtk.Dom.create({'append': domObj, 'tag': 'div', 'className': 'gtk_panel_body'});
	};

    this._render();

    jsgtk.Event.addDomListener(this._minimizeButton, "mousedown", function(e){
        self.minimize();
        jsgtk.Event.stopBubble(e);
    }, false);
};

gtk.Panel.prototype = new gtk.Widget();
gtk.Panel.prototype.constructor = gtk.Panel;

/**
 * A method to set the top most given a boolean value
 * @param {Boolean} isTop The boolean value to be topmost of not
 */
gtk.Panel.prototype.setTopMost = function(isTop){
    if(isTop) this._domObj.style.zIndex = "1001";
    else this._domObj.style.zIndex = "0";
};

/**
 * A method to hide the minimized button from the panel
 */
gtk.Panel.prototype.hideMinimizeButton = function(){
    this._minimizeButton.style.display = "none";
};

/** @private */
gtk.Panel.prototype._isNodeExistInMinimizedHolder = function(holder){
    var clonedHolder = holder.cloneNode(true);
    for(var i = 0, len = clonedHolder.childNodes.length; i < len; i++){
        if(this._domObj.id == clonedHolder.childNodes[i].id) return true;
    }
    return false;
}

/** @private */
gtk.Panel.prototype._createMinimizedHolder = function(){
    var id = "gtk_minimized_widget_holder";
    var holder = document.getElementById(id);
    if(!holder) var holder = jsgtk.Dom.create({"append": document.body, "tag": "div", "id": id});
    return holder;
};

/**
 * A method to call setting the panel to minimized state
 */
gtk.Panel.prototype.minimize = function(){
    jsgtk.Dom.hideElement(this._domObj);

    var holder = this._createMinimizedHolder();
    if(this._isNodeExistInMinimizedHolder(holder)){
        jsgtk.Dom.showElement(this._minimizedPanel);
        return;
    }

    this._minimizedPanel = this._createMinimizedPanel();
    holder.appendChild(this._minimizedPanel);

    this._initMinimizedEvents();
};

/** @private */
gtk.Panel.prototype._initMinimizedEvents = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    var restoreButton = this._minimizedPanel.childNodes[2];
    jsgtkEvent.addDomListener(restoreButton, "mousedown", function(){
		self.restore();
    });

    jsgtkEvent.addDomListener(restoreButton, "mouseover", function(){
		restoreButton.setAttribute("title", "Restore Panel");
    });

	var closeButton = this._minimizedPanel.childNodes[1];
	jsgtkEvent.addDomListener(closeButton, "mousedown", function(e){
	    self._minimizedPanel.parentNode.removeChild(self._minimizedPanel);
	    self.emit("closed", self);
	    jsgtkEvent.stopBubble(e);
	});


    jsgtkEvent.addDomListener(this._minimizedPanel, "dblclick", function(e){
        self.restore();
    });
    jsgtk.Event.addDomListener(this._minimizedPanel, "mouseover", function(e){
        self.emit("minimizedpanelmouseovered", self._title);
        jsgtk.Event.stopBubble(e);
    });
    jsgtk.Event.addDomListener(this._minimizedPanel, "mouseout", function(e){
        self.emit("minimizedpanelmouseout", self._title);
        jsgtk.Event.stopBubble(e);
    });
    self.emit('minimized', self);
};

/**
 * A method to justified title of the panel
 * @param {String} justify The contant varaiable of gtk
 */
gtk.Panel.prototype.setTitleJustify = function(justify){
    this._header.childNodes[0].style.textAlign = justify;
};

/** @private */
gtk.Panel.prototype._isPanelRestored = function(id){
    if(!document.getElementById(id)){
        this._domObj.id = id;
        return true;
    }
    return false;
};

/**
 * A method to restore the state of the panel to the previsous state
 */
gtk.Panel.prototype.restore = function(){
    var id = this._domObj.id;
    this._domObj.id = "";
    if(this._isPanelRestored(id)) return;
    this.emit('restored', this);

    jsgtk.Dom.showElement(this._domObj);
    this._resizeDomTitleWidth(this._width - this._inconsWidth);

    jsgtk.Dom.hideElement(this._minimizedPanel);
    this._domObj.id = id;
};

gtk.Panel.prototype.setCloseButton = function(isClose){
	isClose ? this.hideCloseButton() : this.showCloseButton();
};

/** @private */
gtk.Panel.prototype.hideCloseButton = function(){
    if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "5px";
	}
	else{
		this.hideCloseButton();
	}
};

/** @private */
gtk.Panel.prototype.showCloseButton = function(){
    var self = this,
        jsgtkEvent = jsgtk.Event,
        jsgtkDom = jsgtk.Dom;

    jsgtkDom.showElement(this._closeButton);
	this._minimizeButton.style.right = "22px";

	jsgtkEvent.addDomListener(this._closeButton, "click", function(e){
	    self._domObj.parentNode.removeChild(self._domObj);
	    self.emit("closed", self);
	    self._domObj = null;
	    jsgtkEvent.stopBubble(e);
	});
};

/**
 * A method to hide/remove close button from panel
 */
gtk.Panel.prototype.hideCloseButton = function(){
	if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "5px";
	}
};

/**
 * A method to get/return the title of the panel
 * @return {String} The name of the panel
 * @type String
 */
gtk.Panel.prototype.getTitle = function(){
    return this._title;
};

/**
 * A method to append or add a new child into the panel
 * @param {Dom} widget The dom widget to be passed for adding
 */
gtk.Panel.prototype.appendChild = function(widget){
    this._child = widget;
    this._child.setParent(this);
    this._positionChild(widget.getNode());

    this.resizeHeightComposite(this._height);
    this.resizeWidthComposite(this._width);
};

/** @private */
gtk.Panel.prototype._positionChild = function(childNode){
    this._body.appendChild(childNode);
    var childNodeStyle = childNode.style;
    childNodeStyle.left = this._padding / 2 + "px";
    childNodeStyle.top  = this._padding / 2 + "px";
};

/**
 * A method to flush the child inside the panel
 */
gtk.Panel.prototype.flushChild = function(){
    this._body.removeChild(this._child.getNode());
    this._child = null;
};

/**
 * A method to set the panel title to bold
 * @param {Boolean} bold The boolean value to display bold or not
 */
gtk.Panel.prototype.setBoldTitle = function(bold){
	if(bold){
		this._domTitle.style.fontWeight = "bold";
	} else 	this._domTitle.style.fontWeight = "normal";
};

/** @private */
gtk.Panel.prototype._resetTitle = function(title){
    this._domTitle.innerHTML = title;
};

/**
 * A method to set the new title to the panel
 * @param {String} title The string title of the panel
 * @param {String} xalign The constant variable of jsgtk
 */
gtk.Panel.prototype.setTitle = function(title, xalign){
    this._title = title;
    var xalign = xalign || 1.0;
    this._domTitle.innerHTML = this._title;
};

/** @private */
gtk.Panel.prototype._resizeDomTitleWidth = function(width){
    this._domTitle.style.width = width + "px";
};

/** @private */
gtk.Panel.prototype._createMinimizedPanel = function(){
    this._resizeDomTitleWidth(this._minimizedWidth - this._inconsWidth);

    var minimizePanel = this._header.cloneNode(true);
	minimizePanel.className = "gtk_panel_minimize";
    minimizePanel.removeAttribute("style");
    minimizePanel.style.width = this._minimizedWidth + "px";
    minimizePanel.id = this._domObj.id;

    return minimizePanel;
};

/** @private */
gtk.Panel.prototype._getSubElementsHeight = function(){
    return this._headerHeight + this._subHeaderHeight + this._footerHeight;
};

/**
 * A method to resize both width and height the panel
 * @param {Integer} width The integer width value to be set
 * @param {Integer} height The integer height value to be set
 */
gtk.Panel.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Panel.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width    = this._width - 2 + 'px';
	    this._domTitle.style.width = this._width - this._inconsWidth + "px";
    	this._body.style.width      = this._width - 3 + 'px';
};

/** @private */
gtk.Panel.prototype._resizeHeight = function(height){
	this._height = height;
	this._bodyHeight = this._height - this._headerHeight;
	this._domObj.style.height    = this._height - 2 + "px";
	this._body.style.height      = this._bodyHeight + "px";
};

/**
 * A method to get/return body height of the panel
 * @return {Integer} The body height of the panel
 * @type Integer
 */
gtk.Panel.prototype.getBodyHeight = function(){
    return this._bodyHeight;
};

/**
 * A method to get/return the minimum composite width
 * @return {Integer} The minimum composite width of the panel
 * @type Integer
 */
gtk.Panel.prototype.getMinWidthComposite = function(){
    return this._child.getMinWidthComposite() + this._padding;
};

/**
 * A method to get/return the minimum composite height
 * @return {Integer} The minimum composite height of the panel
 * @type Integer
 */
gtk.Panel.prototype.getMinHeightComposite = function(){
    return this._child.getMinHeightComposite() + this._headerHeight + this._padding;
};

/**
 * A method to resize its composite width
 * @param {Integer} The composite width to be set
 */
gtk.Panel.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._child.resizeWidthComposite(minWidth - this._padding);
    this._resizeWidth(minWidth);
};

/**
 * A method to resize its composite height
 * @param {Integer} The composite height to be set
 */
gtk.Panel.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._child.resizeHeightComposite(minHeight - this._padding - this._headerHeight);
    this._resizeHeight(minHeight);
};

gobject.signalNew('minimized', gtk.Panel, [], null, null);
gobject.signalNew('minimizedpanelmouseovered', gtk.Panel,[], null, null);
gobject.signalNew('minimizedpanelmouseout', gtk.Panel, [], null, null);
gobject.signalNew('restored', gtk.Panel, [], null, null);
gobject.signalNew('closed', gtk.Panel, [], null, null);
/**
 * A class to create vertical box or container
 * @constructor
 * @base gtk.Widget
 * @param {Integer} spacing
 * @param {Integer} padding
 */
gtk.VBox = function(spacing, padding) {
	gtk.Widget.apply(this);

	this._spacing = (spacing === undefined || spacing == 0) ? 0 : spacing;
	this._padding = (padding === undefined || padding == 0) ? 0 : padding;
	this._children = [];
	this._width = this._minWidth = 10;
	this._height = this._minHeight = 10;
	this._isSizeLocked = false;

	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " vbox";
	};

	this._render();
	this._initEvent();
};

gtk.VBox.prototype = new gtk.Widget();
gtk.VBox.prototype.constructor = gtk.VBox;

gtk.VBox.prototype._initEvent = function(){
	var self = this;
	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked");
    });
};

/**
 * Return all children widget
 * @return {array} children inside the VBox
 */
gtk.VBox.prototype.getChildren = function(){
	var childWidgets = [], children = this._children, len = children.length;
	for(var i = 0; i < len; i++){
		childWidgets.push(children[i].childWidget);
	}
	return childWidgets;
};

/**
 * Return the first child
 * @return {gtk.Widget} widget first child inside the HBox
 */
gtk.VBox.prototype.getFirstChild = function(){
    return this._children[0]["childWidget"];
};

/**
 * Return the last child
 * @return {gtk.Widget} widget the last child inside the HBox
 */
gtk.VBox.prototype.getLastChild = function(){
    return this._children[this._children.length - 1]["childWidget"];
};

/**
 * Return a next child widget given current widget as reference
 * @return {gtk.Widget} widget the child to be returned
 */
gtk.VBox.prototype.getNextSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === this._children.length - 1) return null;
    return this._children[childIndex + 1]["childWidget"];
};

/**
 * Return a previous child widget given a current widget as reference
 * @return {Object} widget the child to be returned
 */
gtk.VBox.prototype.getPreviousSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === 0) return null;
    return this._children[childIndex - 1]["childWidget"];
};

/**
 * Method to refresh HBox
 * if gtk.HBox.lockSize() has been called, then the size of HBox will not decrease when some child have been remove
 * else vice versa
 */
gtk.VBox.prototype.refresh = function(){
	this._refreshSelf();
	this._refreshChildren();
};

/** @private */
gtk.VBox.prototype._refreshChildren = function(){
	for(var i = this._children.length; i--;){
		this._children[i]['childWidget'].refresh();
	}
};

/**
 * Make sure, the HBox size will not decrease when some children have been remove
 */
gtk.VBox.prototype.lockSize = function(){
    this._isSizeLocked = true;
};

/**
 * HBox size will decrease when some children have been remove
 */
gtk.VBox.prototype.unlockSize = function(){
    this._isSizeLocked = false;
};

/** @private */
gtk.VBox.prototype._refreshSelf = function(){
	var minWidth = this.getMinWidthComposite();
	if(this._isSizeLocked) minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	if(this._isSizeLocked) minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);
};

/**
 * Append child gtk.Widget to HBox
 * @param {Object} widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.appendChild = function(childWidget, packOption){
    childWidget.setParent(this);

    var childObject = this._createChildObject(childWidget, packOption);
	this._children.push(childObject);
	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Method to add a separator between children widget inside HBox
 */
gtk.VBox.prototype.addSeparator = function(){
    var space = this._children[this._children.length - 1]["space"];
    jsgtk.Dom.showElement(space.childNodes[0]);
};

/**
 * Method to clone the child widget
 * @return {Object} widget new cloned widget
 */
gtk.VBox.prototype.cloneWidget = function(){
    return jsgtk.Util.clone(this);
};

/**
 * Insert a child widget infront of give child reference
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, packOption){

    newChildWidget.setParent(this);

    var index = this._isChild(oldChildWidget);
    this._children.splice(index,0, this._createChildObject(newChildWidget, packOption));

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Replace an existing child widget with new child widget
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableHeight: true || false}
 */
gtk.VBox.prototype.replaceChild = function(newChildWidget, oldChildWidget, packOption){
    newChildWidget.setParent(this);

    var index = this._isChild(oldChildWidget);
    if(!index) jsgtk.Util.debug("target child widget not found");

    var child = this._children[index];
    child["element"].insertBefore(newChildWidget.getNode(), oldChildWidget.getNode());
    child["element"].removeChild(oldChildWidget.getNode());

    child["childWidget"]    = newChildWidget;
	child["expand"]         = packOption["expand"]         === undefined ? child["expand"]         : packOption["expand"];
    child["fill"]           = packOption["fill"]           === undefined ? child["fill"]           : packOption["fill"];
	child["padding"]        = packOption["padding"]        === undefined ? child["padding"]        : packOption["padding"];
	child["resizableWidth"] = packOption["resizableWidth"] === undefined ? child["resizableWidth"] : packOption["resizableWidth"];

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Remove a child widget from the HBox
 * @param {Object} widget
 */
gtk.VBox.prototype.removeChild = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");

    this.removeChildByIndex(childIndex);
};

/**
 * Remove a child widget given an index
 * @param {Integer} index value of the child to be removed
 */
gtk.VBox.prototype.removeChildByIndex = function(index){
    if(!this._children[index]) jsgtk.Util.debug("The index is out of range");

    this._resizeHeight(this._height - parseInt(this._children[index]["element"].style.height));

    this._domObj.removeChild(this._children[index]["element"]);
    this._domObj.removeChild(this._children[index]["space"]);

    this._children.splice(index, 1);
};

/**
 * Re-order or relocatte the child widget
 * @param{gtk.Widget} widget
 * @param{integer} position The new position for child in the children list of the box starting from 0
 */
gtk.VBox.prototype.reorderChild = function(childWidget, position) {

    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    if( position >= this._children.length || position < 0 ) jsgtk.Util.debug("Position is undefined or out of range");
    if( childIndex === position ) return;

    var child = this._children[childIndex];
    var targetChild = this._children[position];

    this._children[position] = child;
    this._children[childIndex] = targetChild;

	var tmpNode1 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});
	var tmpNode2 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});

	this._domObj.replaceChild(tmpNode2, targetChild["element"]);
	this._domObj.replaceChild(tmpNode1, child["element"]);

	this._domObj.appendChild(child["element"]);
	this._domObj.appendChild(targetChild["element"]);

	this._domObj.replaceChild(child["element"], tmpNode2);
	this._domObj.replaceChild(targetChild["element"], tmpNode1);
};

/** @private */
gtk.VBox.prototype._createChildObject = function(childWidget, packOption){
    var element = this._createChildNodeHolder(childWidget.getNode());

	var space = this._createSpace();

	this._appendChild(element, space);

    var childObject = {
		"element" : element,
		"childWidget"   : childWidget,
		"space"   : space,
		"expand"  : packOption["expand"] === undefined ? true : packOption["expand"],
		"fill"    : packOption["fill"] === undefined ? true : packOption["fill"],
		"padding" : packOption["padding"] === undefined ? 0 : packOption["padding"],
		"resizableWidth" : packOption["resizableWidth"] === undefined ? true : packOption["resizableWidth"]
	};
	return childObject;
};

/** @private */
gtk.VBox.prototype._isChild = function(child){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === child) return i;
    }
    return false;
};

/** @private */
gtk.VBox.prototype._appendChild = function(element, space){
    this._domObj.appendChild(element);
    this._domObj.appendChild(space);
};

/**
 * Return number of children inside HBox
 * @return {integer} length
 */
gtk.VBox.prototype.getChildrenLength = function(){
    return this._children.length;
};

/** @private */
gtk.VBox.prototype._createChildNodeHolder = function(childNode) {
    var verticleElement = jsgtk.Dom.create({tag: 'div', style: 'position: relative; display: block;'});
    verticleElement.appendChild(childNode);
	return verticleElement;
};

/** @private */
gtk.VBox.prototype._createSpace = function(){
    var space = jsgtk.Dom.create({"tag": 'div', "class": "gtk_vbox_space"});
        var separator = jsgtk.Dom.create({"append": space , "tag": 'div', "class": "gtk_vbox_separator"});
	return space;
};

/** @private */
gtk.VBox.prototype._hideLastSpace = function(){
    this._domObj.childNodes[this._domObj.childNodes.length - 1].style.height = "0px";
};

/**
 * Assign property to enabled
 */
gtk.VBox.prototype.enable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].enable();
    }
};

/**
 * Assign property to disabled
 */
gtk.VBox.prototype.disable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].disable();
    }
};

/** @private */
gtk.VBox.prototype._handleHiddenChild = function(child){
	if(child["childWidget"].isHidden()) {
		jsgtk.Dom.hideElement(child["element"]);
		jsgtk.Dom.hideElement(child["space"]);
	}
	else {
		jsgtk.Dom.showElement(child["element"]);
		jsgtk.Dom.showElement(child["space"]);
	}
};

/** @private */
gtk.VBox.prototype.getMinWidthComposite = function(){
    var maxChildWidth = this._children[0]["childWidget"].getMinWidthComposite();
    var childWidth;
    for(var i = 1, len = this._children.length; i < len; i++){
        childWidth = this._children[i]["childWidget"].getMinWidthComposite();
        maxChildWidth = maxChildWidth < childWidth ? childWidth : maxChildWidth;
    }

	var minWidth = maxChildWidth + this._padding * 2;
	minWidth     = minWidth < this._minWidth ? this._minWidth : minWidth;

    return minWidth;
};

/** @private */
gtk.VBox.prototype._resizeChildrenWidth = function(width){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["element"].style.left = this._padding + "px";
        child["space"].style.left = this._padding + "px";
        if(child["resizableWidth"]){
            child["element"].style.width = width + "px";
            child["childWidget"].resizeWidthComposite(width);
            child["space"].style.width = width + "px";
        }else{
            var childWidth = child["childWidget"].getMinWidthComposite();
            child["element"].style.width = childWidth + "px";
            child["space"].style.width = childWidth + "px";
        }
    }
};

/** @private */
gtk.VBox.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);
    this._resizeChildrenWidth(minWidth - this._padding * 2);
};

gobject.signalNew('clicked', gtk.VBox, [], null, null);
/**
 * VBoxFactory is a class to create a vertical box as a container.
 * @param {Boolean} homogeneous The boolean value whether true or false
 * @param {Integer} spacing The integer value in pixel as spacing to be set
 * @param {Integer} padding The integer value in pixel as padding to be set
 * @see gtk.VBox
 * @constructor
 */
gtk.VBoxFactory = function(homogeneous, spacing, padding) {
    return homogeneous ?
           new gtk.VBoxHomogeneous(spacing, padding) :
           new gtk.VBoxNoneHomogeneous(spacing, padding);
};
/**
 * A class to create horizontal box or container
 * @constructor
 * @base gtk.Widget
 * @param {Integer} spacing
 * @param {Integer} padding
 */
gtk.HBox = function(spacing, padding) {
	gtk.Widget.apply(this);

	this._padding = (padding === undefined || padding == 0) ? 0 : padding;
	this._spacing = (spacing === undefined || spacing == 0) ? 0 : spacing;
	this._children = [];
	this._width = this._minWidth = 10;
	this._height = this._minHeight = 10;
    this._isSizeLocked = false;

	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " hbox";
		domObj.style.width = this._width + 'px';
		domObj.style.height = this._height + 'px';
	};

	this._render();
	this._initEvent();
};

gtk.HBox.prototype = new gtk.Widget();
gtk.HBox.prototype.constructor = gtk.HBox;

gtk.HBox.prototype._initEvent = function(){
	var self = this;
	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked");
    });
};

/**
 * Return all children widget
 * @return {array} children inside the HBox
 */
gtk.HBox.prototype.getChildren = function(){
	var childWidgets = [],
		children = this._children,
	 	len = children.length;
	for(var i = 0; i < len; i++){
		childWidgets.push(children[i].childWidget);
	}
	return childWidgets;
}

/**
 * Return the first child
 * @return {gtk.Widget} widget first child inside the HBox
 */
gtk.HBox.prototype.getFirstChild = function(){
    return this._children[0]["childWidget"];
};

/**
 * Method to set enabled
 */
gtk.HBox.prototype.enable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].enable();
    }
};

/**
 * Method to set disabled
 */
gtk.HBox.prototype.disable = function(){
    for(var i = this._children.length; i--;){
        this._children[i]["childWidget"].disable();
    }
};

/**
 * Return the last child
 * @return {gtk.Widget} widget the last child inside the HBox
 */
gtk.HBox.prototype.getLastChild = function(){
    return this._children[this._children.length - 1]["childWidget"];
};

/**
 * Return a next child widget given current widget as reference
 * @return {gtk.Widget} widget the child to be returned
 */
gtk.HBox.prototype.getNextSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === this._children.length - 1) return null;
    return this._children[childIndex + 1]["childWidget"];
};

/**
 * Return a previous child widget given a current widget as reference
 * @return {Object} widget the child to be returned
 */
gtk.HBox.prototype.getPreviousSibling = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if(childIndex === 0) return null;
    return this._children[childIndex - 1]["childWidget"];
};

/**
 * Method to refresh HBox
 * if gtk.HBox.lockSize() has been called, then the size of HBox will not decrease when some child have been remove
 * else vice versa
 */
gtk.HBox.prototype.refresh = function(){
	this._refreshSelf();
	this._refreshChildren();
};

/** @private */
gtk.HBox.prototype._refreshChildren = function(){
	for(var i = this._children.length; i--;){
		this._children[i]['childWidget'].refresh();
	}
};

/**
 * Make sure, the HBox size will not decrease when some children have been remove
 */
gtk.HBox.prototype.lockSize = function(){
    this._isSizeLocked = true;
};

/**
 * HBox size will decrease when some children have been remove
 */
gtk.HBox.prototype.unlockSize = function(){
    this._isSizeLocked = false;
};

/** @private */
gtk.HBox.prototype._refreshSelf = function(){
	var minWidth = this.getMinWidthComposite();
	if(this._isSizeLocked) minWidth = minWidth < this._width ? this._width : minWidth;
    this.resizeWidthComposite(minWidth);

	var minHeight = this.getMinHeightComposite();
	if(this._isSizeLocked) minHeight = minHeight < this._height ? this._height : minHeight;
    this.resizeHeightComposite(minHeight);
};

/**
 * Append child gtk.Widget to HBox
 * @param {Object} widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.appendChild = function(childWidget, packOption) {

    childWidget.setParent(this);

	this._children.push(this._createChildObject(childWidget, packOption));

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Method to add a separator between children widget inside HBox
 */
gtk.HBox.prototype.addSeparator = function(){
    var space = this._children[this._children.length - 1]["space"];
    jsgtk.Dom.showElement(space.childNodes[0]);
};

/**
 * Method to clone the child widget
 * @return {Object} widget new cloned widget
 */
gtk.HBox.prototype.cloneWidget = function(){
    return jsgtk.Util.clone(this);
};

/**
 * Insert a child widget infront of give child reference
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, packOption){

    childWidget.setParent(this);

    var index = this._isChild(oldChildWidget);
    this._children.splice(index,0, this._createChildObject(newChildWidget, packOption));

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Replace an existing child widget with new child widget
 * @param {gtk.Widget} newWidget
 * @param {gtk.Widget} reference widget
 * @param {Object} packOption {fill: true || false, expand: true || false, padding: any integer, resizableWidth: true || false}
 */
gtk.HBox.prototype.replaceChild = function(newChildWidget, oldChildWidget, packOption){
    newChildWidget.setParent(this);

    var index = this._isChild(oldChildWidget);
    if(!index) jsgtk.Util.debug("target child widget not found");

    var child = this._children[index];
    child["element"].insertBefore(newChildWidget.getNode(), oldChildWidget.getNode());
    child["element"].removeChild(oldChildWidget.getNode());

    child["childWidget"]    = newChildWidget;
	child["expand"]         = packOption["expand"]         === undefined ? child["expand"]         : packOption["expand"];
    child["fill"]           = packOption["fill"]           === undefined ? child["fill"]           : packOption["fill"];
	child["padding"]        = packOption["padding"]        === undefined ? child["padding"]        : packOption["padding"];
	child["resizableHeight"] = packOption["resizableHeight"] === undefined ? child["resizableHeight"] : packOption["resizableHeight"];

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Remove a child widget from the HBox
 * @param {Object} widget
 */
gtk.HBox.prototype.removeChild = function(childWidget){
    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");

    this._resizeWidth(this._width - parseInt(this._children[childIndex]["element"].style.width));

    this._domObj.removeChild(this._children[childIndex]["element"]);
    this._domObj.removeChild(this._children[childIndex]["space"]);

    this._children.splice(childIndex, 1);
};

/**
 * Remove a child widget given an index
 * @param {Integer} index value of the child to be removed
 */
gtk.HBox.prototype.removeChildByIndex = function(index){
    if(!this._children[index]) jsgtk.Util.debug("The index is out of range");

    this._resizeWidth(this._width - parseInt(this._children[index]["element"].style.width));

    this._domObj.removeChild(this._children[index]["element"]);
    this._domObj.removeChild(this._children[index]["space"]);

    this._children.splice(index, 1);
};

/**
 * Re-order or relocatte the child widget
 * @param{gtk.Widget} widget
 * @param{integer} position The new position for child in the children list of the box starting from 0
 */
gtk.HBox.prototype.reorderChild = function(childWidget, position) {

    var childIndex = this._isChild(childWidget);
    if( childIndex === false ) jsgtk.Util.debug("The widget has not yet been packed to vbox");
    if( position >= this._children.length || position < 0 ) jsgtk.Util.debug("Position is undefined or out of range");
    if( childIndex === position ) return;

    var child = this._children[childIndex];
    var targetChild = this._children[position];

    this._children[position] = child;
    this._children[childIndex] = targetChild;

	var tmpNode1 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});
	var tmpNode2 = jsgtk.Dom.create({append: this._domObj, tag: 'div'});

	this._domObj.replaceChild(tmpNode2, targetChild["element"]);
	this._domObj.replaceChild(tmpNode1, child["element"]);

	this._domObj.appendChild(child["element"]);
	this._domObj.appendChild(targetChild["element"]);

	this._domObj.replaceChild(child["element"], tmpNode2);
	this._domObj.replaceChild(targetChild["element"], tmpNode1);
};

/** @private */
gtk.HBox.prototype._createChildObject = function(childWidget, packOption){

    var element = this._createChildNodeHolder(childWidget.getNode());

	var space = this._createSpace();

	this._appendChild(element, space);

	var childObject = {
		"element" : element,
		"childWidget"   : childWidget,
		"space"   : space,
		"expand"  : packOption["expand"] === undefined ? true : packOption["expand"],
		"fill"    : packOption["fill"] === undefined ? true : packOption["fill"],
		"padding" : packOption["padding"] === undefined ? 0 : packOption["padding"],
		"resizableHeight" : packOption["resizableHeight"] === undefined ? true : packOption["resizableHeight"]
	};

	return childObject;
};

/**
 * Return number of children inside HBox
 * @return {integer} length
 */
gtk.HBox.prototype.getChildrenLength = function(){
    return this._children.length;
};

/** @private */
gtk.HBox.prototype._isChild = function(child){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === child) return i;
    }
    return false;
};

/** @private */
gtk.HBox.prototype._appendChild = function(element, space){
    this._domObj.appendChild(element);
    this._domObj.appendChild(space);
};

/** @private */
gtk.HBox.prototype._createChildNodeHolder = function(childNode) {
    var element = jsgtk.Dom.create({tag: 'div', style: 'position: relative; float: left;'});
    element.appendChild(childNode);
	return element;
};

/** @private */
gtk.HBox.prototype._createSpace = function(){
    var space = jsgtk.Dom.create({tag: 'div', "class": "gtk_hbox_space"});
        var separator = jsgtk.Dom.create({"append": space , "tag": 'div', "class": "gtk_hbox_separator"});
	return space;
};

/** @private */
gtk.HBox.prototype._hideLastSpace = function(){
	if(this._domObj.childNodes.length >= 1)
		this._domObj.childNodes[this._domObj.childNodes.length - 1].style.width = "0px";
};

/** @private */
gtk.HBox.prototype._resizeChildrenHeight = function(height){
    var child;
    for(var i = 0, len = this._children.length; i < len; i++){
        child = this._children[i];
        child["element"].style.top = this._padding + "px";
        child["space"].style.top = this._padding + "px";
        if(child["resizableHeight"]){
            child["element"].style.height = height + "px";
            child["childWidget"].resizeHeightComposite(height);
            child["space"].style.height = height + "px";
        }else{
            var childHeight = child["childWidget"].getMinHeightComposite();
            child["element"].style.height = childHeight + "px";
            child["space"].style.height = childHeight + "px";
        }
    }
};

/** @private */
gtk.HBox.prototype.getMinHeightComposite = function(){
    var maxChildHeight = this._children[0]["childWidget"].getMinHeightComposite();
    var childHeight;
    for(var i = 1, len = this._children.length; i < len; i++){
        childHeight = this._children[i]["childWidget"].getMinHeightComposite();
        maxChildHeight = maxChildHeight < childHeight ? childHeight : maxChildHeight;
    }
    var paddings = this._padding * 2;

	var minHeight = maxChildHeight + paddings;
	minHeight	  = minHeight < this._minHeight ? this._minHeight : minHeight;

    return minHeight;
};

/** @private */
gtk.HBox.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;
    this._resizeHeight(minHeight);
    this._resizeChildrenHeight(minHeight - this._padding * 2);
};
gobject.signalNew('clicked', gtk.HBox, [], null, null);
/**
 * HBoxFactory is a class to create horizontal box as a container. It can contain JSGTK widgets.
 * @param {Boolean} homogeneous The boolean value to be set if the box is homogeneous or not
 * @param {Interger} spacing The integer value in pixel as spacing to be set
 * @param {Interger} padding The integer value in pixel as padding to be set
 * @see gtk.HBox
 * @constructor
 */
gtk.HBoxFactory = function(homogeneous, spacing, padding) {
    return homogeneous ?
           new gtk.HBoxHomogeneous(spacing, padding) :
           new gtk.HBoxNoneHomogeneous(spacing, padding);
};
/**
 * The base class of gtk.HPaned and gtk.HPaned
 * @constructor
 * @base gtk.Widget
 * @see gtk.VPaned
 */
gtk.Paned = function(){
	gtk.Widget.apply(this);

	this._minWidth = 200;
	this._minHeight = 200;
	this._width = this._minWidth;
	this._height = this._minHeight;

	/** @private */
	this._render = function(){
		this._domObj.className += " gtk_paned";
	};

	this._render();
};

gtk.Paned.prototype = new gtk.Widget();
gtk.Paned.prototype.constructor = gtk.Paned;

/**
 * Adds the widget specified by child to the top or left of the paned
 * @param {gtk.Widget} widget
 */
gtk.Paned.prototype.pack1 = function(widget){
	this._child1 = widget;
	this._paned1.appendChild(this._child1.getNode());
	this._resizePaned1(true);
};

/**
 * Adds the widget specified by child to the top or left of the paned
 * @param {gtk.Widget} widget
 */
gtk.Paned.prototype.pack2 = function(widget){
	this._child2 = widget;
	this._paned2.appendChild(this._child2.getNode());
	this._resizePaned2(true);
};

/** @private */
gtk.Paned.prototype.resize = function(width, height){
	this._resizePaned1(false);
	this._resizePaned2(false);
};

/** @private */
gtk.Paned.prototype._resizePaned1 = function(isInit){
	this._resizePaned1Width(isInit);
	this._resizePaned1Height(isInit);
};

/** @private */
gtk.Paned.prototype._resizePaned2 = function(isInit){
	this._resizePaned2Width(isInit);
	this._resizePaned2Height(isInit);
};

/** @private */
gtk.Paned.prototype._showGhostDivider = function(){
	var ghostDividerStyle = this._ghostDivider.style;
	ghostDividerStyle.backgroundColor = "#838B8B";
	ghostDividerStyle.opacity = 1;
};

/** @private */
gtk.Paned.prototype._hideGhostDivider = function(){
	var ghostDividerStyle = this._ghostDivider.style;
	ghostDividerStyle.disply = "none";
	ghostDividerStyle.opacity = 0.1;
};

gobject.signalNew("childrendminimize", gtk.Paned, null, null, []);
gobject.signalNew("childrestore", gtk.Paned, null, null, []);
/**
 * VPaned is a class to create vertical pane as a container
 * @base gtk.Paned
 * @constructor
 * @see gtk.Paned
 */
gtk.VPaned = function() {
	gtk.Paned.apply(this);

	this._dividerHeight = 6;
	this._paned1Height = (this._height / 2) - (this._dividerHeight/2);
	this._paned2Height = (this._height / 2) - (this._dividerHeight/2);
	this._isPaned1Restore = true;
	this._isPaned2Restore = true;
    this._topAnchored = 0;
	this._render = function(){
		this._domObj.className += " gtk_vPaned";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		this._domObj.style.position = 'relative';

			this._paned1 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned1'});
			this._paned1.style.width = this._width + 'px';
			this._paned1.style.height = this._paned1Height + 'px';

			this._domDivider = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_dom_divider'});
				this._dom = jsgtk.Dom.create({'append': this._domDivider, 'tag': "div", 'class': 'gtk_paned_divider_image'});

			this._domDivider.style.width = this._width + 'px';
			this._domDivider.style.height = this._dividerHeight + "px";
			this._domDivider.style.top = this._paned1Height + 'px';

			this._ghostDivider = this._domDivider.cloneNode(true);
			this._ghostDivider.className = "gtk_paned_ghost_divider";

			this._paned2 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned2'});
			this._paned2.style.width = this._width + 'px';
			this._paned2.style.height = this._paned1Height + 'px';

		this._domObj.appendChild(this._paned1);
		this._domObj.appendChild(this._domDivider);
		this._domObj.appendChild(this._paned2);
		this._domObj.appendChild(this._ghostDivider);
	};

	this._render();

	this._vBehaviour(0, this._height - this._dividerHeight);
};

gtk.VPaned.prototype = new gtk.Paned();
gtk.VPaned.prototype.constructor = gtk.VPaned;

/**
 * A method to set position of the vertical paned
 * @param {Integer} position The integer value to define the height
 */
gtk.VPaned.prototype.setPosition = function(position){
    var widget1Height = this._child1.getMinHeightComposite();
    this._topAnchored = (widget1Height < position)? position : widget1Height;
};

/**
 * A method to set autoresize given a widget
 * @param {Dom} widget The Dom widget to be passed
 */
gtk.VPaned.prototype.autoResize = function(widget){
	var paned1 = this._paned1;
	var paned2 = this._paned2;
	this._originalPaned1Height = paned1.style.height;
	this._originalPaned1ChildHeight = paned1.childNodes[0].style.height;
	this._originalPaned2Height= paned2.style.height;
	this._originalPaned2ChildHeight = paned2.childNodes[0].style.height;

	if(widget._domObj.parentNode.className == "gtk_paned1" ){
		this._child2.resizeHeightComposite(this._height);
		this._child1.hide();
		paned1.style.height = '0px';
		this._isPaned1Restore = false;
	}

	if(widget._domObj.parentNode.className == "gtk_paned2" ){
		this._child1.resizeHeightComposite(this._height);
		this._child2.hide();
		paned2.style.height = '0px';
		this._isPaned2Restore = false;
	}
	this._domDivider.style.display = 'none';
	this._ghostDivider.style.display = 'none';

	if(!this._isPaned1Restore && !this._isPaned2Restore){
		this.emit('childrendminimize');
	}

};

/**
 * A method to restore its status
 * @param {Dom} widget The Dom widget to be passed
 */
gtk.VPaned.prototype.restoreStatus = function(widget){
	if(widget === this._child1){
		if(this._isPaned2Restore){
			this._paned1.style.height = this._paned1Height + "px";
			this._child1.resizeHeightComposite(this._paned1Height);
            this._child1.show();

			this._paned2.style.height = this._paned2Height + "px";
			this._child2.resizeHeightComposite(this._paned2Height);
            this._child2.show();

			this._domDivider.style.display = 'block';
			this._ghostDivider.style.display = 'block';
		}else{
			this._paned1.style.height = this._height + "px";
			this._child1.resizeHeightComposite(this._height);
			this._child1.show();
		}
		this._isPaned1Restore = true;
	}

	if(widget === this._child2){
		if(this._isPaned1Restore){
			this._paned2.style.height = this._paned2Height + "px";
			this._child2.resizeHeightComposite(this._paned2Height);
            this._child2.show();

			this._paned1.style.height = this._paned1Height + "px";
			this._child1.resizeHeightComposite(this._paned1Height);
            this._child1.show();

			this._domDivider.style.display = 'block';
			this._ghostDivider.style.display = 'block';
		}else{
			this._paned2.style.height = this._height + "px";
			this._child2.resizeHeightComposite(this._height);
			this._child2.show();
		}
		this._isPaned2Restore = true;
	}

	if(this._isPaned1Restore || this._isPaned2Restore) {
		this.emit('childrestore');
	}
};

/** @private */
gtk.VPaned.prototype._resizePaned1Width = function(){
	if(this._width > this._child1.getWidth())
		this._child1.resizeWidthComposite(this._width);
	else this._resizeWidth(this._child1.getWidth());
};

/** @private */
gtk.VPaned.prototype._resizePaned1Height = function(isInit){

	if(isInit) this._initModifyPaned1Height();
	else this._modifyPaned1Height();

	this._resizeHeight(this._paned1Height, this._paned2Height);
	this._topMinY = this._child1.getMinHeightComposite();

	this._vBehaviour( this._topMinY, this._height - this._dividerHeight);
};

/** @private */
gtk.VPaned.prototype._resizePaned2Width = function(){
	if(this._width > this._child2.getWidth())
		this._child2.resizeWidthComposite(this._width);
	else{
		this._resizeWidth(this._child2.getWidth());
	}
};

/** @private */
gtk.VPaned.prototype._resizePaned2Height = function(isInit){

	if(isInit) this._initModifyPaned2Height();
	else this._modifyPaned2Height();

	this._resizeHeight(this._paned1Height, this._paned2Height);
	this._bottomMinY = this._child2.getMinHeightComposite();
	var maxY = this._height - this._dividerHeight - this._bottomMinY;

	this._vBehaviour( this._topMinY, maxY);
};

/** @private */
gtk.VPaned.prototype._initModifyPaned1Height = function(){
	if(this._paned1Height > this._child1.getHeight())
		this._child1.resizeHeightComposite(this._paned1Height);
	else{
		this._paned1Height = this._child1.getHeight();
	}
};

/** @private */
gtk.VPaned.prototype._initModifyPaned2Height = function(){
	if(this._paned2Height > this._child2.getHeight())
		this._child2.resizeHeightComposite(this._paned2Height);
	else
		this._paned2Height = this._child2.getHeight();
};

/** @private */
gtk.VPaned.prototype._modifyPaned1Height = function(){
	this._child1.resizeHeightComposite(this._paned1Height);
	this._paned1Height = this._child1.getHeight();
};

/** @private */
gtk.VPaned.prototype._modifyPaned2Height = function(){
	this._child2.resizeHeightComposite(this._paned2Height);
	this._paned2Height = this._child2.getHeight();
};

/** @private */
gtk.VPaned.prototype._vBehaviour = function(minY, maxY){
	var self = this;
	jsgtk.Util.DomDraggableAdapter.vDragInit(this._ghostDivider, minY, maxY);

	this._ghostDivider.onDragStart = function(){
        this.style.background = "#DFE8F6";
        this.style.opacity = "0.9";
        if(this.firstChild) this.removeChild(this.firstChild);
	};

	this._ghostDivider.onDragEnd = function(x, y){

		var topHeight = y;
		var bottomHeight = self._height - topHeight - self._dividerHeight;

        self.setPosition(topHeight);
		self._resizePaned12Dom(topHeight, bottomHeight);

		self._child1.resizeHeightComposite(topHeight);
		self._child2.resizeHeightComposite(bottomHeight);

        this.style.background = "";
        this.style.opacity = "0.5";
	};
};

/** @private */
gtk.VPaned.prototype._resizePaned12Dom = function(topHeight, bottomHeight){
	this._paned1Height = topHeight;
	this._paned2Height = bottomHeight;
	this._paned1.style.height = topHeight + 'px';
	this._paned2.style.height = bottomHeight + 'px';
};

/** @private */
gtk.VPaned.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
		this._paned1.style.width = this._width + 'px';
		this._domDivider.style.width = this._width + 'px';
		this._ghostDivider.style.width = this._width + 'px';
		this._paned2.style.width = this._width + 'px';
};

/** @private */
gtk.VPaned.prototype._resizeHeight = function(paned1Height, paned2Height){
	this._height = paned1Height + paned2Height + this._dividerHeight;
	this._domObj.style.height = this._height + 'px';
		this._paned1.style.height = paned1Height + 'px';
		this._ghostDivider.style.top = paned1Height + 'px';
		this._paned2.style.height = paned2Height + 'px';
};

/**
 * A method to get or return minimum composite width
 * @return {Integer} The minimum composite width to be passed
 * @type Integer
 */
gtk.VPaned.prototype.getMinWidthComposite = function(){
	var widget1Width = this._child1.getMinWidthComposite();
	var widget2Width = this._child2.getMinWidthComposite();
	var minWidth = this.getSizeRequest()[0] > widget1Width ? this.getSizeRequest()[0] : widget1Width;
	minWidth = minWidth > widget2Width ? minWidth : widget2Width;

	return minWidth;
};

/**
 * A method to get or return minimum composite height
 * @return {Integer} The minimum composite height to be passed
 * @type Integer
 */
gtk.VPaned.prototype.getMinHeightComposite = function(){
	var widget1Height = this._child1.getMinHeightComposite();
	var widget2Height = this._child2.getMinHeightComposite();
	var minHeight = widget1Height + widget2Height;
	minHeight = minHeight > this.getSizeRequest()[1] ? minHeight : this.getSizeRequest()[1];

	return minHeight;
};

/**
 * A method to resize composite width of the widget and its childres
 * @param {Integer} width The integer value as width to be set
 */
gtk.VPaned.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	this._resizeWidth(minWidth);

	this._child1.resizeWidthComposite(minWidth);
	this._child2.resizeWidthComposite(minWidth);

	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};

/**
 * A method to resize composite height of the widget and its childres
 * @param {Integer} width The integer value as height to be set
 */
gtk.VPaned.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;

	var offsetHeight = (minHeight - this.getHeight()) / 2 ;

	this._resizeHeight(minHeight);

	var height1 = this._paned1Height + offsetHeight;
	var height2 = this._paned2Height + offsetHeight;

	if(
		this._paned2Height === this._child2.getMinHeightComposite() ||
		this._paned1Height === this._child1.getMinHeightComposite() ||
		height1 			<  this._child1.getMinHeightComposite() ||
		height2   			<  this._child2.getMinHeightComposite()
	){
		if(this._paned2Height === this._child2.getMinHeightComposite() ||
		   height2 			   <  this._child2.getMinHeightComposite()
		){
		    this._paned1Height = this._paned1Height + offsetHeight * 2;
		    this._paned2Height = this._child2.getMinHeightComposite();
		}

		if(this._paned1Height === this._child1.getMinHeightComposite() ||
		   height1 			   <  this._child1.getMinHeightComposite()
		){
		    this._paned2Height = this._paned2Height + offsetHeight * 2;
   		    this._paned1Height = this._child1.getMinHeightComposite();
		}

	}else{
		this._paned1Height = height1;
		this._paned2Height = height2;
	}

	if(this._topAnchored !== 0){
        var widget1Height = this._child1.getMinHeightComposite();
	    var widget2Height = this._child2.getMinHeightComposite();

	    var height = this._topAnchored + widget2Height;
	    this._paned1Height = height < minHeight ? this._topAnchored : minHeight - widget2Height;
		this._paned2Height = minHeight - this._paned1Height;
	}

    this._paned1Height -= 2;
	this._paned2Height -= 2;

	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};
/**
 * Separate the paned into two vertical paned
 * @constructor
 * @base gtk.Paned
 */
gtk.HPaned = function() {
	gtk.Paned.apply(this);

    var self = this;
	this._dividerWidth    = 6;
	this._paned1Width     = (this._width / 2) - (this._dividerWidth / 2);
	this._paned2Width     = (this._width / 2) - (this._dividerWidth / 2);
	this._isPaned1Restore = true;
	this._isPaned2Restore = true;
	this._isDrawer        = false;
	this._drawerWidth     = 6;
	this._isDrawed        = false;
	this._isDrawedToLeft  = false;
	this._isDrawerLeftEnabled = false;
	this._isDrawerRightEnabled = false;
	this._leftAnchored      = 0;
	this._drawerEventId = null;
	this._isDragged = false;

	/** @private */
	this._render = function(){
		var domObj = this._domObj,
			domObjStyle = this._domObj.style,
			jsgtkDom = jsgtk.Dom;

		domObj.className += " gtk_hPaned";
		domObjStyle.width = this._width + 'px';
		domObjStyle.height = this._height + 'px';

			this._paned1 = jsgtkDom.create({'tag': 'div', 'class': 'gtk_paned1'});
			var paned1Style = this._paned1.style;
			paned1Style.width = this._paned1Width + 'px';
			paned1Style.height = this._height + 'px';

			this._domDivider = jsgtkDom.create({'tag': 'div', 'class': 'gtk_dom_divider'});
			    this._dom = jsgtkDom.create({'append': this._domDivider, 'tag': "div", 'class': 'gtk_paned_divider_image'});

			this._drawer = jsgtkDom.create({"append": domObj, "tag": "div", "class": "gtk_hPaned_drawer"});
			    this._restoreButton = jsgtkDom.create({"append": this._drawer, "tag": "div", "class": "gtk_hPaned_drawer_restore_icon"});
			var drawerStyle = this._drawer.style;
			drawerStyle.width = this._drawerWidth + "px";
			drawerStyle.height= this._height + "px";
			jsgtkDom.hideElement(this._drawer);

			this._draw = jsgtkDom.create({"append": domObj, "tag": "div", "class": "gtk_hPaned_draw"});
			jsgtkDom.hideElement(this._draw);

			var domDividerStyle = this._domDivider.style;
			domDividerStyle.width = this._dividerWidth + 'px';
			domDividerStyle.height =  this._height+ "px";
			domDividerStyle.left = this._paned1Width + 'px';

			this._ghostDivider = this._domDivider.cloneNode(false);
			this._ghostDivider.className = "gtk_paned_ghost_divider";

			this._paned2 = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_paned2'});
			var paned2Style = this._paned2.style;
			paned2Style.width = this._paned1Width + 'px';
			paned2Style.height = this._height + 'px';

		domObj.appendChild(this._paned1);
		domObj.appendChild(this._domDivider);
		domObj.appendChild(this._paned2);
		domObj.appendChild(this._ghostDivider);
	};

	this._render();

	this._hBehaviour(0, this._width - this._dividerWidth);

	jsgtk.Event.addDomListener(this._drawer, "click", function(e){
        self._isDrawed = false;
        if(self._isDrawedToLeft){
            self._child1.show();
            self._paned1.appendChild(self._child1.getNode());
            self.restoreStatus(self._child1, true);
        }else{
            self._child2.show();
            self._paned2.appendChild(self._child2.getNode());
            self.restoreStatus(self._child2, true);
        }

        jsgtk.Dom.hideElement(self._drawer);
	});

};

gtk.HPaned.prototype = new gtk.Paned();
gtk.HPaned.prototype.constructor = gtk.HPaned;

/**
 * set position of the left paned x pixel from the left
 * @param {integer} position number of pixel from the left
 */
gtk.HPaned.prototype.setPosition = function(position){
    var widget1Width = this._child1.getMinWidthComposite();
    this._leftAnchored = (widget1Width < position)? position : widget1Width;
};

/**
 * anble drawer to the right. When user click on the drawer, the right paned will be shrunk
 */
gtk.HPaned.prototype.enableDrawerRight = function(){
    this._enableDrawer(this._child2, "gtk_enabled_drawer_right");
	this._isDrawerRightEnabled = true;
};

/**
 * anble drawer to the right. When user click on the drawer, the left paned will be shrunk
 */
gtk.HPaned.prototype.enableDrawerLeft = function(){
    this._enableDrawer(this._child1, "gtk_enabled_drawer_left");
    this._isDrawerLeftEnabled = true;
};

/** @private */
gtk.HPaned.prototype._enableDrawer = function(widget, className){
    var self = this;
    this._domDivider.className += " gtk_dom_divider_drawer";
    this._domDivider.style.width = 5 + "px";

   	this._dom.className = className;

    var jsgtkEvent = jsgtk.Event;
    /** @private */
    this._drawerEventId = jsgtkEvent.addDomListener(this._ghostDivider, "click", function(e){
        self._isDrawed = true;
        self.autoResize(widget, true);
        jsgtkEvent.stopBubble(e);
    });

    /** @private */
    jsgtkEvent.addDomListener(this._dom, "click", function(e){
        self._isDrawed = true;
        self.autoResize(widget, true);
        jsgtkEvent.stopBubble(e);
    });

    this.resize();

    /** @private */
    jsgtkEvent.addDomListener(this._ghostDivider, "mouseover", function(e){
        if(self._isDragged){
            jsgtk.Event.addDomListener(
                self._drawerEventId["obj"],
                self._drawerEventId["type"],
                self._drawerEventId["callback"]
            );
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.HPaned.prototype._resizeChild1 = function(){
	var width = this._width - this._drawerWidth;
	var paned1     = this._paned1;
	var paned2     = this._paned2;

    if(this._isDrawer){
        this._child1.resizeWidthComposite(this._width - this._drawerWidth);
	    this._child2.hide();

        paned2.style.width = this._drawerWidth + "px";
        paned1.style.width = width + "px";

        this._drawer.style.right = "0px";
        this._isDrawedToLeft = false;

        this._restoreButton.className += " draw_right";
        jsgtk.Dom.showElement(this._drawer);
    }else{
        this._child1.resizeWidthComposite(this._width);
	    this._child2.hide();
	    paned2.style.width = '0px';
    }
	this._isPaned2Restore = false;
};

/** @private */
gtk.HPaned.prototype._resizeChild2 = function(){
	var width = this._width - this._drawerWidth;
	var paned1     = this._paned1;
	var paned2     = this._paned2;

    if(this._isDrawer){
        this._child2.resizeWidthComposite(width);
        this._child1.hide();

        paned1.style.width = this._drawerWidth + "px";
        paned2.style.width = width + "px";

        this._isDrawedToLeft = true;

        this._restoreButton.className += " draw_left";
        jsgtk.Dom.showElement(this._drawer);
    }else{
        this._child2.resizeWidthComposite(this._width);
        this._child1.hide();
        paned1.style.width = '0px';
    }
    this._isPaned1Restore = false;
};

/**
 * shrinken the widget inside each paned
 * @param {gtk.Widget} widget will be shrinken along with its paned
 * @param {boolean} isDrawer if drawer enabled, will show the pooler after paned was shrunk
 */
gtk.HPaned.prototype.autoResize = function(widget, isDrawer){
	this._isDrawer = isDrawer === undefined ? false : isDrawer;

	if(widget._domObj.parentNode.className == "gtk_paned1" )
	    this._resizeChild2();

	if(widget._domObj.parentNode.className == "gtk_paned2" )
	    this._resizeChild1();

	jsgtk.Dom.hideElement(this._domDivider);
    jsgtk.Dom.hideElement(this._ghostDivider);

	if(!this._isPaned1Restore && !this._isPaned2Restore){
        jsgtk.Dom.hideElement(this._drawer);
		this.emit('childrendminimize', this);
	}
};

/** @private */
gtk.HPaned.prototype._restoreWidget1 = function(){
    var drawerWidth = 0;
    if(this._isDrawerRightEnabled){
        if(this._isDrawed){
            jsgtk.Dom.showElement(this._drawer);
            drawerWidth = this._drawerWidth;
        }
    }
    if(this._isPaned2Restore){
		this._paned1.style.width = this._paned1Width + "px";
		this._child1.resizeWidthComposite(this._paned1Width);

		this._paned2.style.width = this._paned2Width + "px";
		this._child2.resizeWidthComposite(this._paned2Width);

		this._domDivider.style.display = 'block';
		if(!this._isDrawerRightEnabled)this._ghostDivider.style.display = 'block';

	}else{
		this._paned1.style.width = this._width - drawerWidth + "px";
		this._child1.resizeWidthComposite(this._width - drawerWidth);
	}
	this._isPaned1Restore = true;
};

/** @private */
gtk.HPaned.prototype._restoreWidget2 = function(){

    var drawerWidth = 0;
    if(this._isDrawerLeftEnabled){
        if(this._isDrawed){
            jsgtk.Dom.showElement(this._drawer);
            drawerWidth = this._dividerWidth;
        }
    }
    if(this._isPaned1Restore){
	    this._paned2.style.width = this._paned2Width + "px";
	    this._child2.resizeWidthComposite(this._paned2Width);

	    this._paned1.style.width = this._paned1Width + "px";
	    this._child1.resizeWidthComposite(this._paned1Width);

	    this._domDivider.style.display = 'block';
	    if(!this._isDrawerLeftEnabled)this._ghostDivider.style.display = 'block';
    }else{
	    this._paned2.style.width = this._width - drawerWidth + "px";
	    this._child2.resizeWidthComposite(this._width - drawerWidth);
    }
    this._isPaned2Restore = true;
};

/**
 * Restore the shrunk widget and its paned
 * @param {gtk.Widget} widget will be shrinken along with its paned
 */
gtk.HPaned.prototype.restoreStatus = function(widget){
	if(widget === this._child1) this._restoreWidget1();
	if(widget === this._child2) this._restoreWidget2();

	if(this._isPaned1Restore || this._isPaned2Restore) {
		this.emit('childrestore', this);
	}
};

/** @private */
gtk.HPaned.prototype._resizePaned1Height = function(){
	if(this._height > this._child1.getHeight())
		this._child1.resizeHeightComposite(this._height);
	else this._resizeHeight(this._child1.getHeight());
};

/** @private */
gtk.HPaned.prototype._resizePaned1Width = function(isInit){

	if(isInit) this._initModifyPaned1Width();
	else this._modifyPaned1Width();

	this._resizeWidth(this._paned1Width, this._paned2Width);
	this._leftMinX = this._child1.getMinWidthComposite();

	this._hBehaviour( this._leftMinX, this._width - this._dividerWidth);
};

/** @private */
gtk.HPaned.prototype._resizePaned2Height = function(){
	if(this._height > this._child2.getHeight())
	    this._child2.resizeHeightComposite(this._height);
	else{
		this._resizeHeight(this._child2.getHeight());
		this._child1.resizeHeightComposite(this._child2.getHeight());
	}
};

/** @private */
gtk.HPaned.prototype._resizePaned2Width = function(isInit){

	if(isInit) this._initModifyPaned2Width();
	else this._modifyPaned2Width();

	this._resizeWidth(this._paned1Width, this._paned2Width);
	this._rightMinX = this._child2.getMinWidthComposite();
	var maxX = this._width - this._dividerWidth - this._rightMinX;

	this._hBehaviour(this._child1.getMinWidthComposite(), maxX);
};

/** @private */
gtk.HPaned.prototype._initModifyPaned1Width = function(){
	if(this._paned1Width > this._child1.getWidth())
		this._child1.resize(this._paned1Width, this._child1.getHeight());
	else{
		this._paned1Width = this._child1.getWidth();
	}
};

/** @private */
gtk.HPaned.prototype._initModifyPaned2Width = function(){
	if(this._paned2Width > this._child2.getWidth()){
	    this._child2.resizeWidthComposite(this._paned2Width);
	}
	else{
		this._paned2Width = this._child2.getWidth();
	}
};

/** @private */
gtk.HPaned.prototype._modifyPaned1Width = function(width){
    if(typeof width != 'undefined') this._child1.resizeWidthComposite(width);
	else this._child1.resizeWidthComposite(this._paned1Width);
};

/** @private */
gtk.HPaned.prototype._modifyPaned2Width = function(width){
    if(typeof width != 'undefined') this._child2.resizeWidthComposite(width);
	else{
	    this._child2.resizeWidthComposite(this._paned2Width);
    	this._paned2Width = this._child2.getWidth();
	}
};

/** @private */
/** @ignore */
gtk.HPaned.prototype._hBehaviour = function(minX, maxX){

	var self = this;

	/** @ignore */
	jsgtk.Util.DomDraggableAdapter.hDragInit(this._ghostDivider, minX, maxX);

	/** @ignore */
    this._ghostDivider.onDragStart = function(){
        self._isDragged = false;
        this.style.background = "#DFE8F6";
        this.style.opacity = "0.9";
        if(this.firstChild) this.removeChild(this.firstChild);
    };

	/** @ignore */
    this._ghostDivider.onDrag = function(){
        if(self._drawerEventId) jsgtk.Event.removeListener(self._drawerEventId);
    };

	/** @ignore */
	this._ghostDivider.onDragEnd = function(x, y){
		var width1 = x;
		var width2 = self._width - width1 - self._dividerWidth;

        self.setPosition(width1);
		self._resizePaned12Dom(width1, width2);

		self._child1.resizeWidthComposite(width1);
		self._child2.resizeWidthComposite(width2);

		self._isDragged = true;

        this.style.background = "";
        this.style.opacity = "0.9";
	};
};

/** @private */
gtk.HPaned.prototype._resizePaned12Dom = function(width1, width2){
	this._paned1Width = width1;
	this._paned2Width = width2;
	this._paned1.style.width = width1 + 'px';
	this._paned2.style.width = width2 + 'px';
};

/** @private */
gtk.HPaned.prototype._resizeWidth = function(paned1Width, paned2Width){
	this._width = paned1Width + paned2Width + this._dividerWidth;
	this._domObj.style.width = this._width + 1 + 'px';
		this._paned1.style.width = paned1Width + 'px';
		this._ghostDivider.style.left = paned1Width + 'px';
		this._paned2.style.width = paned2Width + 'px';
};

/** @private */
gtk.HPaned.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = this._height + 'px';
		this._paned1.style.height = this._height + 'px';
		this._domDivider.style.height = this._height + 'px';
		this._ghostDivider.style.height = this._height + 'px';
		this._paned2.style.height = this._height + 'px';

		this._drawer.style.height = this._height + "px";
};

/** @private */
gtk.HPaned.prototype.getMinWidthComposite = function(){
	var widget1Width = this._child1.getMinWidthComposite();
	var widget2Width = this._child2.getMinWidthComposite();
	var minWidth = widget1Width + widget2Width + this._dividerWidth;
	minWidth = minWidth > this._minWidth ? minWidth : this._minWidth;

	return minWidth;
};

/** @private */
gtk.HPaned.prototype.getMinHeightComposite = function(){
	var widget1Height = this._child1.getMinHeightComposite();
	var widget2Height = this._child2.getMinHeightComposite();
	var minHeight = this._minHeight > widget1Height ? this._minHeight : widget1Height;
	minHeight = minHeight > widget2Height ? minHeight : widget2Height;

	return minHeight;
};

/** @private */
gtk.HPaned.prototype.resizeWidthComposite = function(width){

	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	var offsetWidth = (minWidth - this.getWidth()) / 2 ;

	this.setWidth(minWidth);

	var width1 = this._paned1Width + offsetWidth;
	var width2 = this._paned2Width + offsetWidth;

	if(this._isDrawedToLeft){
	    this._modifyPaned2Width(minWidth - this._drawerWidth);
	    this._resizeWidth(0, minWidth);
	}
	else{
	    this._modifyPaned1Width(minWidth - this._drawerWidth);
        this._resizeWidth(minWidth, 0);
	}

	if(
		this._paned2Width === this._child2.getMinWidthComposite() ||
		this._paned1Width === this._child1.getMinWidthComposite() ||
		width1			   <  this._child1.getMinWidthComposite() ||
		width2			   <  this._child2.getMinWidthComposite()
 	){
		if(this._paned2Width === this._child2.getMinWidthComposite() ||
		   width2			  < this._child2.getMinWidthComposite()
		){
		    this._paned1Width = (this._paned1Width + offsetWidth * 2) - 1;
		    this._paned2Width = (this._child2.getMinWidthComposite()) - 1;
		}
		if(this._paned1Width === this._child1.getMinWidthComposite() ||
		   width1			  <  this._child1.getMinWidthComposite()
		){
		    this._paned2Width = (this._paned2Width + offsetWidth * 2) - 1;
		    this._paned1Width = (this._child1.getMinWidthComposite()) - 1;
		}
	}else{
		this._paned1Width = this._paned1Width + offsetWidth - 1;
		this._paned2Width = this._paned2Width + offsetWidth - 1;
	}

	if(this._leftAnchored !== 0){
	    var widget1Width = this._child1.getMinWidthComposite();
	    var widget2Width = this._child2.getMinWidthComposite();
	    var width = this._leftAnchored + widget2Width;
	    this._paned1Width = width < minWidth ? this._leftAnchored : minWidth - widget2Width;
		this._paned2Width = minWidth - this._paned1Width;
	}

    this._paned1Width -= this._dividerWidth / 2;
    this._paned2Width -= this._dividerWidth / 2;

	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};

/** @private */
gtk.HPaned.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;

	this.setHeight(minHeight);

	this._child1.resizeHeightComposite(minHeight);
	this._child2.resizeHeightComposite(minHeight);

    if(this._isDrawedToLeft) this._resizePaned1Height();
    else this._resizePaned2Height();

	if(this._isPaned1Restore && this._isPaned2Restore) {
        this.resize();
	}
};
/**
* @constructor
* @base gtk.Widget
*/
gtk.JustifiedLabel = function(labelValues) {
	gtk.Widget.apply(this);

	/** 'clicked' signals*/
 	this.JUSTIFIEDLABEL_CLICKED = "clicked";
	/** 'focused' signals*/
	this.JUSTIFIEDLABEL_FOCUSED = "focused";
	/** 'mouseover' signals*/
	this.JUSTIFIEDLABEL_MOUSEOVER = "mouseover";
	/** 'blurred' signals*/
	this.JUSTIFIEDLABEL_BLURRED = "blurred";
	/** 'mouseout' signals*/
	this.JUSTIFIEDLABEL_MOUSEOUT = "mouseout";
	/** 'refreshed' signals*/
	this.JUSTIFIEDLABEL_REFRESHED = "refreshed";
	/** 'cleared' signals*/
	this.JUSTIFIEDLABEL_CLEARED = "cleared";

	this._labelValues    = labelValues;
	this._width          = this._minWidth  = 16;
	this._height         = this._minHeight = 16;
	this._rowHeight      = 20;
	this._seperatorWidth = 80;
	this._applyYellow = false;

	this._render = function(){
		this._domObj.className   += " gtk_justified_label";
	};

	this._render();
};

gtk.JustifiedLabel.prototype = new gtk.Widget();
gtk.JustifiedLabel.prototype.constructor = gtk.JustifiedLabelgtk;

/** @private */
gtk.JustifiedLabel.prototype._clearChild = function(){
    this._domObj.innerHTML = "";
};

/**
 * Set the width of dote separator
 * @param {number} width
 */
gtk.JustifiedLabel.prototype.setSeparatorWidth = function(width){
    this._seperatorWidth = width;
};

/**
 * Set the model
 * @param {gtk.LabelStore} model
 */
gtk.JustifiedLabel.prototype.setModel = function(model){
	this._clearChild();
	new gtk.JustifiedLabelController(this, model);
    this._resize();
	this.emit("refreshed");
};

/**
 * clear model and refresh
 */
gtk.JustifiedLabel.prototype.clearModel = function(){
    this._clearChild();
    this._width =  this._minWidth = 0;
    this._height = this._minHeight = 0;
    this.emit("cleared");
};

/** @private */
gtk.JustifiedLabel.prototype.constructLabelRows = function(data){
    this._labelValues = data;
    var domObj = this._domObj;
    this._rowNode;

    for(var i = 0, len = data.length; i < len; i++){
        var leftLabel = data[i]["leftLabel"];
        var rightLabel = data[i]["rightLabel"];
        var tabindex = data[i]["tabindex"];


        var rowNode = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_justified_label_row"});
        this._rowNode = rowNode;
        rowNode.style.height = this._rowHeight + "px";

            var dotsSeperator  = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_dot_separator_node"});
            var leftLabelNode  = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_left_node"});
                leftLabelNode.innerHTML = leftLabel.getText();

            var rightLabelNode = jsgtk.Dom.create({"append": rowNode, "tag": "div", "class": "gtk_justified_label_row_right_node"});
                rightLabelNode.innerHTML = rightLabel.getText();

        this._handleMouseOver(rowNode, rightLabel.getText());
        if(tabindex !=null)this._setTabIndex(rowNode,tabindex);
    }
    this._resize();
};

/** @private */
gtk.JustifiedLabel.prototype._setTabIndex = function(row, index){
    row.tabIndex = index;
};

/** @private */
gtk.JustifiedLabel.prototype._handleMouseOver = function(rowNode, rightLabel){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    jsgtkEvent.addDomListener(rowNode, "mouseover", function(e){
        self.emit("mouseover", rightLabel);
        self._setYellowBorderLeft(rowNode);
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(rowNode, "mouseout", function(){
    	  self.emit("mouseout", rightLabel);
        self._resetYellowBorderLeft(rowNode);
    });

    jsgtkEvent.addDomListener(rowNode, "click", function(e){
        self.emit("clicked", rightLabel);
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(rowNode, "focus", function(e){
        self.emit("focused", rightLabel);
        self._setYellowBorderLeft(rowNode);
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(rowNode, "blur", function(){
        self._resetYellowBorderLeft(rowNode);
        self.emit("blurred", rightLabel);
    });

};

/** @private */
gtk.JustifiedLabel.prototype._getMaxRowWidth = function(){
    var labelValues = this._labelValues;
    var maxWidth = labelValues[0]["leftLabel"].getMinWidthComposite() +
                   this._seperatorWidth +
                   labelValues[0]["rightLabel"].getMinWidthComposite();

    for(var i = 1, len = this._labelValues.length; i < len; i++){
        var width = labelValues[i]["leftLabel"].getMinWidthComposite() +
                    this._seperatorWidth +
                    labelValues[i]["rightLabel"].getMinWidthComposite();
        maxWidth = width > maxWidth ? width : maxWidth;
    }
    return maxWidth;
};

/** @private */
gtk.JustifiedLabel.prototype._resizeWidth = function(width){
    this._minWidth = this._width = width;
    this._domObj.style.width = width + "px";
};

/** @private */
gtk.JustifiedLabel.prototype._setYellowBorderLeft = function(rowNode){
    rowNode.style.borderLeft = "4px solid #FFCC00";
    rowNode.childNodes[0].style.marginLeft = "8px";
    rowNode.childNodes[1].style.marginLeft = "8px";
};

/** @private */
gtk.JustifiedLabel.prototype._resetYellowBorderLeft = function(rowNode){
    rowNode.style.borderLeft = "0";
    rowNode.childNodes[0].style.marginLeft = "0";
    rowNode.childNodes[1].style.marginLeft = "0";
};

/** @private */
gtk.JustifiedLabel.prototype._resizeHeight = function(height){
    this._minHeight = this._height = height;
    this._domObj.style.height = height + "px";
};

/** @private */
gtk.JustifiedLabel.prototype._resize = function(){
    var width  = this._getMaxRowWidth();
    this._resizeWidth(width);

    var height = this._labelValues.length * this._rowHeight;
    this._resizeHeight(height);
};

/** @private */
gtk.JustifiedLabel.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.JustifiedLabel.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/**
 * Set attributes
 * @param {array} array of attributes object style
 */
gtk.JustifiedLabel.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.JustifiedLabel.prototype._applyAttribute = function(){
    var childLength = this._domObj.childNodes.length;

	for(el in this._attribute){
	    if(el == "backgroundColor"){
	        for(var i = 0; i < childLength; i++){
	            this._domObj.style[el] = this._attribute[el];
                this._domObj.childNodes[i].childNodes[1].style[el] = this._attribute[el];
                this._domObj.childNodes[i].childNodes[2].style[el] = this._attribute[el];
            }
	    }else{
            this._domObj.style[el] = this._attribute[el];
        }
	}
};

/** @private */
gtk.JustifiedLabel.prototype.setWidth = function(width){
    this._width = this._minWidth  = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resizeWidthComposite = function(width){
    this._width = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resetMinWidth = function(width){
    this._width = this._minWidth  = width;
    this._resizeWidth(width);
};

/** @private */
gtk.JustifiedLabel.prototype.resetMinHeight = function(height){
    this._height = this._minHeight  = height;
    this._resizeHeight(height);
};

/** @private */
gtk.JustifiedLabel.prototype.resizeHeightComposite = function(height){
    this._height = height;
    this._resizeHeight(height);
};

gobject.signalNew('clicked', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('focused', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('mouseover', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('blurred', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('mouseout', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('refreshed', gtk.JustifiedLabel, [], null, null);
gobject.signalNew('cleared', gtk.JustifiedLabel, [], null, null);
/**
 * @constructor
 * @base gtk.Widget
 * @param {integer} width of liquidboxWidget
 * @param {integer} space between child widget
 */
gtk.LiquidBox = function(width, space) {
	gtk.Widget.apply(this);

	/** 'refreshed' signal */
	this.LIQUIDBOX_REFRESHED = "refreshed";

	this._width          = this._minWidth = width;
	this._height         = this._minHeight = 10;
    this._spacing        = space || 0;
    this._children       = [];
    this._maxChildWidth  = 0;
    this._maxChildHeight = 0;
    this._columns        = 0;
    this._rows           = 1;

	this._render = function(){
	    var domObj = this._domObj;
		domObj.className += " liquidbox";
	};/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} childWidget
 * @param {object} options {'fill': true || false, 'padding': any number}
 */

	this._render();
};

gtk.LiquidBox.prototype = new gtk.Widget();
gtk.LiquidBox.prototype.constructor = gtk.LiquidBox;

/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} childWidget
 * @param {object} options {'fill': true || false, 'padding': any number}
 */
gtk.LiquidBox.prototype.appendChild = function(childWidget, options){
    childWidget.setParent(this);

    var nodeHolder = this._createChildNodeHolder();
    nodeHolder.appendChild(childWidget.getNode());

    this._children.push({
        "childWidget": childWidget,
        "nodeHolder" : nodeHolder,
        "fill"       : options["fill"] || false,
        "padding"    : options["padding"] || 0
    });

    this._resetMaxChildWidth(childWidget, options["padding"] || 0 );
    this._resetMaxChildHeight(childWidget, options["padding"] || 0);

    this.resizeWidthComposite(this._width);

    var rows = Math.ceil(this._children.length / this._columns);
    if(rows > this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/**
 * Return the length of chidren
 * @return {integer}
 */
gtk.LiquidBox.prototype.getChildrenLength = function(){
	return this._children.length;
};

/**
 * Append child widget into liquidbox
 * @param {gtk.Widget} newChildWidget
 * @param {gtk.Widget} oldChildWidget
 * @param {object} options {'fill': true || false, 'padding': any number}
 */
gtk.LiquidBox.prototype.insertBefore = function(newChildWidget, oldChildWidget, options){
	this.appendChild(newChildWidget, options);

	this._swapChildWidgetsDom(newChildWidget, oldChildWidget);

	this.refresh();
};

/**
 * Refresh the gtk.LiquidBox
 */
gtk.LiquidBox.prototype.refresh = function(){
    this._resetMaxChildrenWidthAndMaxChildrenHeight();
    this.resizeWidthComposite(this._width);
};

/** @private */
gtk.LiquidBox.prototype._swapChildWidgetsDom = function(newChildWidget, oldChildWidget){
	var newChildWidgetObject = this._children[this._getIndexOfChild(newChildWidget)];
	var newChildWidgetNodeHolder = newChildWidgetObject["nodeHolder"];
	this._domObj.removeChild(newChildWidgetNodeHolder);

	var targetChildWidgetObject = this._children[this._getIndexOfChild(oldChildWidget)];
	var targetChildWidgetNodeHolder = targetChildWidgetObject["nodeHolder"];
	this._domObj.insertBefore(newChildWidgetNodeHolder, targetChildWidgetNodeHolder);

	jsgtk.Util.arraySwapElements(this._children, newChildWidgetObject, targetChildWidgetObject);
};

/**
 * Remove child from gtk.LiquidBox by index
 * @param {integer} index
 */
gtk.LiquidBox.prototype.removeByIndex = function(index){
    var childNode = this._children[index]["nodeHolder"];
    childNode.parentNode.removeChild(childNode);

    this._children.splice(index, 1);
    this.resizeWidthComposite(this._width);

    var rows = Math.ceil(this._children.length / this._columns);
    if(rows < this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/** @private */
gtk.LiquidBox.prototype._getIndexOfChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["childWidget"] === childWidget) return i;
    }
    return -1;
}

/**
 * Remove child from gtk.LiquidBox
 * @param {gtk.Widget} widget
 */
gtk.LiquidBox.prototype.removeChild = function(object){
    var index = this._getIndexOfChild(object);
    if(index === -1) jsgtk.Util.debug("The child does not belong to the parent");

    this._children.splice(index, 1);
    this._domObj.removeChild(object.getNode().parentNode);

    this.resizeWidthComposite(this._width);

    var rows = Math.ceil(this._children.length / this._columns);
    if(rows < this._rows) {
        this._rows = rows;
        this.emit("refreshed");
    }
};

/** @private */
gtk.LiquidBox.prototype._resetColumns = function(width){
    this._columns = Math.floor(width / this._maxChildWidth);
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildWidth = function(childWidget, padding){
    var childWidth = childWidget.getMinWidthComposite() + padding * 2;
    this._maxChildWidth = childWidth > this._maxChildWidth ? childWidth : this._maxChildWidth;
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildHeight = function(childWidget, padding){
    var childHeight = childWidget.getMinHeightComposite() + padding * 2;
    this._maxChildHeight = childHeight > this._maxChildHeight ? childHeight : this._maxChildHeight;
};

/** @private */
gtk.LiquidBox.prototype._handleFilledChild = function(child){
    child["childWidget"].resizeWidthComposite(this._maxChildWidth - child["padding"] * 2);
    child["childWidget"].resizeHeightComposite(this._maxChildHeight - child["padding"] * 2);
    child["childWidget"].getNode().style.left = child["padding"] + "px";
    child["childWidget"].getNode().style.top  = child["padding"] + "px";
};

/** @private */
gtk.LiquidBox.prototype._handleNoneFillChild = function(child){
    var childWidth = child["childWidget"].getMinWidthComposite();
    var childHeight = child["childWidget"].getMinHeightComposite();
    child["childWidget"].getNode().style.left = this._maxChildWidth / 2 - childWidth / 2 + "px";
    child["childWidget"].getNode().style.top  = this._maxChildHeight / 2 - childHeight / 2 + "px";
};

/** @private */
gtk.LiquidBox.prototype._resizeChildNodeHolder = function(nodeHolder){
    nodeHolder.style.width  = this._maxChildWidth + "px";
    nodeHolder.style.height = this._maxChildHeight + "px"
};

/** @private */
gtk.LiquidBox.prototype._positionChildNodeHolder = function(nodeHolder, column, rows){
    var vspacings = column > 0 ? this._spacing : 0;
    var hspacings = rows > 0 ? this._spacing : 0;
    nodeHolder.style.left = (this._maxChildWidth + vspacings) * column +"px";
    nodeHolder.style.top  = (this._maxChildHeight + hspacings) * rows + "px";
};

/** @private */
gtk.LiquidBox.prototype._resize = function(){

    var column = 0;
    var rows   = 0;

    var docFragment = document.createDocumentFragment();
    for(var i = 0, len = this._children.length; i < len; i++){

        var child = this._children[i];
        var nodeHolder = child["nodeHolder"];

        docFragment.appendChild(nodeHolder);

        this._positionChildNodeHolder(nodeHolder, column, rows);
        this._resizeChildNodeHolder(nodeHolder);

        if(child["fill"]) this._handleFilledChild(child);
        else this._handleNoneFillChild(child);

        var spacings = this._spacing * (this._children.length);
        this._minHeight = (this._maxChildHeight * (rows+1) + spacings);
        this._resizeHeight(this._minHeight);
        if(column < this._columns - 1){
            column++;
        }else{
            column = 0;
            rows += 1;
        }
    }
    this._domObj.appendChild(docFragment);
};

/** @private */
gtk.LiquidBox.prototype._createChildNodeHolder = function(){
    var nodeHolder = jsgtk.Dom.create({"tag": "div", "class": "gtk_liquid_box_childnode_holder"});
    nodeHolder.style.position = "absolute";
    return nodeHolder;
};

/** @private */
gtk.LiquidBox.prototype._resetMaxChildrenWidthAndMaxChildrenHeight = function(){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        this._resetMaxChildWidth(child["childWidget"], child["padding"]);
        this._resetMaxChildHeight(child["childWidget"], child["padding"]);
    }
};

/** @private */
gtk.LiquidBox.prototype.getMinHeightComposite = function(){
    return this._minHeight + this._spacing * (this._children.length - 1);
};

/** @private */
gtk.LiquidBox.prototype.getMinWidthComposite = function(){
    return this._minWidth + this._spacing * (this._children.length - 1);
};

/** @private */
gtk.LiquidBox.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);

    var spacings = this._spacing * (this._children.length - 1);
    this._resetColumns(minWidth - spacings);
    this._resize();
};

gobject.signalNew('refreshed', gtk.LiquidBox, null, null, []);
/**
 *	Window is a class to create window which can contain a child widget
 * @param {String} type A constant variable value to define type of window.
 * <br>The type of window is optional.
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_TOPLEVEL The flag constant value to define as top level or top most
 * <br> - gtk.WINDOW_POPUP The flag constant value to define as window popup
 * @extends gtk.WindowBase
 * @base gtk.WindowBase
 * @see gtk.WindowBase
 * @constructor
 */
gtk.Window = function(type) {
	gtk.WindowBase.apply(this);

	this._sideBarsWidth = 10;
	this._state = new gtk.StateWindowRestored(this, null);

	this._childWidget = null;

	this._headerHeight = 27;

	this._render = function(){
	    var domObj = this._domObj;
        domObj.className += " gtk_window_new";

            this._ghostWindowDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_ghost_new"});
            jsgtk.Dom.hideElement(this._ghostWindowDom);

            this._leftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_left_bar_new"});
            this._rightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_right_bar_new"});
            this._topBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_top_bar_new"});
            this._bottomBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar gtk_window_bottom_bar_new"});

            this._topLeftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_top_left_bar_new"});
            this._topRightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_top_right_bar_new"});
            this._bottomLeftBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_bottom_left_bar_new"});
            this._bottomRightBarDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "resizeable_bar_corner gtk_window_bottom_right_bar_new"});
	};

	this._render();
};

gtk.Window.prototype = new gtk.WindowBase();
gtk.Window.prototype.constructor = gtk.Window;
/**
 * A method to draw or show the window after instantiated
 */
gtk.Window.prototype.draw = function(){
    document.body.appendChild(this._domObj);
};
/**
 * A method to draw a modal layer
 */
gtk.Window.prototype.drawModal = function(){
    this._showModal();
    document.body.appendChild(this._domObj);
};
/**
 * A method to append or add or replace child into its body
 * @param {Dom} childWidget The child widget or Dom object to be passed
 */
gtk.Window.prototype.appendChild = function(childWidget){
	if(this._childWidget) this._clearChild();

    this._childWidget = childWidget;
    this._bodyDom.appendChild(this._childWidget.getNode());

    this.resizeWidthComposite(this.getMinWidthComposite());
    this.resizeHeightComposite(this.getMinHeightComposite());

    this._initEvent();
};
/**
 * A method to set maximized the window
 */
gtk.Window.prototype.maximize = function(){
	this.emit("maximized", this);

	if(!this.getParent()){
	    this._restoredOption = {
            "width": this._width,
            "height": this._height,
            "left": parseInt(this._domObj.style.left),
            "top": parseInt(this._domObj.style.top)
        };

        this._state.maximize();
	}
};
/**
 * A method to restore window to its previous state
 */
gtk.Window.prototype.restore = function(){
    this._state.restore();

    var id = this._domObj.id;
    this._domObj.id = "";
    if(this._isWindowRestored(id)) return;
    this.emit('restored', this);

    jsgtk.Dom.showElement(this._domObj);
	jsgtk.Dom.showElement(this._minimizeButton);
    this._resizeDomTitleWidth(this._width - this._inconsWidth);

    jsgtk.Dom.hideElement(this._minimizedWindow);
    this._domObj.id = id;
};
/**
 * A method to set minimized the window
 */
gtk.Window.prototype.minimize = function(){
    this.emit('minimized', this);

    this._state.minimize();

    jsgtk.Dom.hideElement(this._domObj);
	jsgtk.Dom.hideElement(this._minimizeButton);

    var holder = this._createMinimizedHolder();
    if(this._isNodeExistInMinimizedHolder(holder)){
        jsgtk.Dom.showElement(this._minimizedWindow);
        return;
    }

    this._minimizedWindow = this._createMinimizedWindow();
    holder.appendChild(this._minimizedWindow);

    this._initMinimizedEvents();
};
/** @private */
gtk.Window.prototype._initEvent = function(){
    var self = this;

    jsgtk.Event.addDomListener(this._bodyDom, "mousedown", function(e){
        jsgtk.Event.stopBubble(e);
    });

    self._resizeLeftBar();
    self._resizeRightBar();
    self._resizeTopBar();
    self._resizeBottomBar();

    self._resizeTopLeftBar();
    self._resizeTopRightBar();
    self._resizeBottomLeftBar();
    self._resizeBottomRightBar();

    jsgtk.Util.DomDraggableAdapter.dragInit(this._domObj, 1, jsgtk.Util.getWindowWidth()-this._width-17, 0, jsgtk.Util.getWindowHeight()-this._height-17);

	/** @private */
	this._domObj.onDrag = function(){
		self._checkAndHideComboDropdown();
    	self._checkAndHideToolTip();
	};
};
/** @private */
gtk.Window.prototype._resizeToLeft = function(x){
    var currentLeft = parseInt(this._domObj.style.left);
    this._domObj.style.left = currentLeft + x + "px";
    this.resizeWidthComposite(parseInt(this._ghostWindowDom.style.width));
    this._resetGhostWidth(parseInt(this._ghostWindowDom.style.width));
};
/** @private */
gtk.Window.prototype._resizeToTop = function(y){
    var currentTop = parseInt(this._domObj.style.top);
    this._domObj.style.top = currentTop + y + "px";
    this.resizeHeightComposite(parseInt(this._ghostWindowDom.style.height));
    this._resetGhostHeight(parseInt(this._ghostWindowDom.style.height));
};
/** @private */
gtk.Window.prototype._resizeToRight = function(x){
    this.resizeWidthComposite(x);
    this._resetGhostWidth(x);
};
/** @private */
gtk.Window.prototype._resizeToBottom = function(y){
    this.resizeHeightComposite(y);
    this._resetGhostHeight(y);
};
/** @private */
gtk.Window.prototype._resizeLeftBar = function(){
    var self = this;
    var minX = parseInt(window.innerWidth);
    jsgtk.Util.DomDraggableAdapter.hDragInit(self._leftBarDom, -minX, self._width - self._minWidth);

    /** @private */
    this._leftBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

    /** @private */
    this._leftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";
    };

    /** @private */
    this._leftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);
        self._resizeToLeft(x);

        self._resizeLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = self._minWidth;
    jsgtk.Util.DomDraggableAdapter.hDragInit(self._rightBarDom, minX);

	/** @private */
    this._rightBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._rightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";
        isResized = true;
    };

	/** @private */
    this._rightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;

        self._resizeToRight(x);

        self._resizeRightBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopBar = function(){
    var self = this;
    var minY = parseInt(window.innerHeight);
    jsgtk.Util.DomDraggableAdapter.vDragInit(self._topBarDom, -minY, self._height - self._minHeight);

	/** @private */
    this._topBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._topBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";
    };

	/** @private */
    this._topBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        self._resizeToTop(y);

        self._resizeTopBar();
    };
};
/** @private */
gtk.Window.prototype._resizeBottomBar = function(){
    var self = this;
    var isResized = false;
    jsgtk.Util.DomDraggableAdapter.vDragInit(self._bottomBarDom, self._minHeight);

	/** @private */
    this._bottomBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

	/** @private */
    this._bottomBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };

	/** @private */
    this._bottomBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;

        self._resizeToBottom(y);

        self._resizeBottomBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopLeftBar = function(){
    var self = this;
    var minX = parseInt(window.innerWidth);
    var maxX = this._width - this._minWidth;
    var minY = parseInt(window.innerHeight);
    var maxY = this._height - this._minHeight;

    jsgtk.Util.DomDraggableAdapter.dragInit(this._topLeftBarDom, -minX, maxX, -minY, maxY);

    /** @private */
    this._topLeftBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

    /** @private */
    this._topLeftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";

        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";
    };

    /** @private */
    this._topLeftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        self._resizeToTop(y);
        self._resizeToLeft(x);

        self._resizeTopLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeTopRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = this._minWidth;
    var maxX = parseInt(window.innerWidth);
    var minY = parseInt(window.innerHeight);
    var maxY = this._height - this._minHeight;

    jsgtk.Util.DomDraggableAdapter.dragInit(this._topRightBarDom, minX, maxX, -minY, maxY);

    /** @private */
    this._topRightBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

    /** @private */
    this._topRightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";

        self._ghostWindowDom.style.top = y + "px";
        self._ghostWindowDom.style.height = self._ghostHeight - y + "px";

        isResized = true;
    };

    /** @private */
    this._topRightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;

        self._resizeToTop(y);
        self._resizeToRight(x);

        self._resizeTopRightBar();
    };
};
/** @private */
gtk.Window.prototype._resizeBottomLeftBar = function(){
    var isResized = false;
    var self = this;
    var minX = parseInt(window.innerWidth);
    var maxX = this._width - this._minWidth;
    var minY = self._minHeight;
    var maxY = parseInt(window.innerHeight);

    jsgtk.Util.DomDraggableAdapter.dragInit(this._bottomLeftBarDom, -minX, maxX, minY, maxY);

    /** @private */
    this._bottomLeftBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

    /** @private */
    this._bottomLeftBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.left = x + "px";
        self._ghostWindowDom.style.width = self._ghostWidth - x + "px";

        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };

    /** @private */
    this._bottomLeftBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;

        self._resizeToLeft(x);
        self._resizeToBottom(y);

        self._resizeBottomLeftBar();
    };
};
/** @private */
gtk.Window.prototype._resizeBottomRightBar = function(){
    var isResized = false;
    var self = this;
    var minX = self._minWidth;
    var maxX = parseInt(window.innerWidth);
    var minY = self._minHeight;
    var maxY = parseInt(window.innerHeight);

    jsgtk.Util.DomDraggableAdapter.dragInit(this._bottomRightBarDom, minX, maxX, minY, maxY);

    /** @private */
    this._bottomRightBarDom.onDragStart = function(x, y, e){
        jsgtk.Dom.showElement(self._ghostWindowDom);
        jsgtk.Event.stopBubble(e);
    };

    /** @private */
    this._bottomRightBarDom.onDrag = function(x, y, e){
        self._ghostWindowDom.style.width = x + "px";
        self._ghostWindowDom.style.height = y + "px";
        isResized = true;
    };

    /** @private */
    this._bottomRightBarDom.onDragEnd = function(x, y){
        jsgtk.Dom.hideElement(self._ghostWindowDom);

        if(!isResized) return;

        self._resizeToRight(x);
        self._resizeToBottom(y);

        self._resizeBottomRightBar();
    };
};
/** @private */
gtk.Window.prototype._resetGhostWidth = function(width){
    this._ghostWidth = width;
};
/** @private */
gtk.Window.prototype._resetGhostHeight = function(height){
    this._ghostHeight = height;
};
/** @private */
gtk.Window.prototype._repositionResizeableBars = function(){
    this._leftBarDom.style.left = "0px";
    this._rightBarDom.style.left = this._width - 3 + "px";
    this._topBarDom.style.top = "0px";
    this._bottomBarDom.style.top = this._height - 3 + "px";

    this._topLeftBarDom.style.left = "0px";
    this._topLeftBarDom.style.top = "0px";

    this._topRightBarDom.style.left = this._width - 3 + "px";
    this._topRightBarDom.style.top = "0px";

    this._bottomLeftBarDom.style.top = this._height - 3 + "px";
    this._bottomLeftBarDom.style.left = "0px";

    this._bottomRightBarDom.style.top = this._height - 3 + "px";
    this._bottomRightBarDom.style.left = this._width - 3 + "px";

    this._ghostWindowDom.style.left = "0px";
    this._ghostWindowDom.style.top = "0px";
};
/** @private */
gtk.Window.prototype._repositionElements = function(){
    this._repositionResizeableBars();
};
/** @private */
gtk.Window.prototype._resizeWidth = function(width){
    this._width = width;
    this._domObj.style.width = width + "px";
        this._bodyDom.style.width = width - this._sideBarsWidth + "px";
        this._ghostWindowDom.style.width = width + "px";

    this._repositionElements();
};
/** @private */
gtk.Window.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = height + "px";
        this._bodyDom.style.height = height - this._headerHeight + "px";
        this._ghostWindowDom.style.height = height + "px";

    this._repositionElements();
};
/** @private */
gtk.Window.prototype.resetMinWidth = function(width){
    this._minWidth = width;
    this._ghostWidth = width;
};
/** @private */
gtk.Window.prototype.resetMinHeight = function(height){
    this._minHeight = height;
    this._ghostHeight = height;
};
/** @private */
gtk.Window.prototype.getMinWidthComposite = function(){
    var childMinWidth = this._childWidget.getMinWidthComposite() + this._sideBarsWidth;
    this.resetMinWidth(childMinWidth);
    return childMinWidth;
};
/** @private */
gtk.Window.prototype.getMinHeightComposite = function(){
    var childMinHeight = this._childWidget.getMinHeightComposite() + this._headerHeight;
    this.resetMinHeight(childMinHeight)
    return childMinHeight;
};
/** @private */
gtk.Window.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();

    minWidth = minWidth < width ? width : minWidth;

    this._childWidget.resizeWidthComposite(minWidth - this._sideBarsWidth);

    this._resizeWidth(minWidth);
};
/** @private */
gtk.Window.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;

    this._childWidget.resizeHeightComposite(minHeight - this._headerHeight - 2);

    this._resizeHeight(minHeight);
};
/**
* @constructor
* @base gtk.Widget
*/
gtk.WindowBase = function() {
	gtk.Widget.apply(this);

	this._minWidth = this._width = this._ghostWidth = this._noneMaximizedWidth = 20;
	this._minHeight = this._height = this._ghostHeight = this._noneMaximizedHeight = 20;
	this._headerHeight = 22;
	this._minimizedWidth      = 150;
	this._inconsWidth         = 62;
	this._minimizedWindow      = null;
	this._childWidget = null;
	this._isCloseButtonActive = false;

	this._enableControl = true;
	this._enableCloseButton = true;
	this._enableMaximizeButton = true;
	this._enableMinimizeButton = true;

	this._isModal = false;
	var self = this;


	this._render = function(){
	    var domObj = this._domObj;
        domObj.className += " gtk_window_base";
		domObj.id = new UUID().id;
        domObj.style.position = "absolute";

            this._headerDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_header_base"});

                this._titleDom = jsgtk.Dom.create({"append": this._headerDom, "text": "Window Title","tag": "div", "class": "gtk_window_title_base"});
                this._minimizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Minimize Window", "class": "gtk_window_minimized_botton_base"});
				this._maximizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Maximize Window", "class": "gtk_window_maximized_botton_base"});
				this._returnToPreviousSizeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Restore Window", "class": "gtk_window_return_size_botton_base"});
				this._closeButton = jsgtk.Dom.create({"append": this._headerDom,"tag": "div", "title": "Close Window", "class": "gtk_window_close_botton_base"});

            this._bodyDom = jsgtk.Dom.create({"append": domObj, "tag": "div", "class": "gtk_window_body_new"});
	};

	this._render();

	jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.showElement(this._maximizeButton);

    jsgtk.Event.addDomListener(this._minimizeButton, "mouseup", function(e){
    	self._checkAndHideComboDropdown();
    	self._checkAndHideToolTip();
    	self._checkAndHideSmartEntryDropdown();
        self.minimize();
    }, false);

	jsgtk.Event.addDomListener(this._maximizeButton, "mouseup", function(e){
		self._checkAndHideComboDropdown();
		self._checkAndHideToolTip();
		self._checkAndHideSmartEntryDropdown();
        self.maximize();
    }, false);

	jsgtk.Event.addDomListener(this._returnToPreviousSizeButton, "mouseup", function(e){
		self._checkAndHideComboDropdown();
		self._checkAndHideToolTip();
		self._checkAndHideSmartEntryDropdown();
        self.restore();
    }, false);

	this.setCloseButton(true);
};

gtk.WindowBase.prototype = new gtk.Widget();
gtk.WindowBase.prototype.constructor = gtk.WindowBase;

/**
 * A method to enable resizeable of the window.
 * The window will be resisizeable.
 */
gtk.WindowBase.prototype.enableResize = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.showElement(this._leftBarDom);
	jsgtkDom.showElement(this._rightBarDom);
	jsgtkDom.showElement(this._topBarDom);
	jsgtkDom.showElement(this._bottomBarDom);
	jsgtkDom.showElement(this._topLeftBarDom);
	jsgtkDom.showElement(this._topRightBarDom);
	jsgtkDom.showElement(this._bottomLeftBarDom);
	jsgtkDom.showElement(this._bottomRightBarDom);
};

/**
 * A method to disable resizeable of the window.
 * It will not allow user to resize the window.
 */
gtk.WindowBase.prototype.disableResize = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.hideElement(this._leftBarDom);
	jsgtkDom.hideElement(this._rightBarDom);
	jsgtkDom.hideElement(this._topBarDom);
	jsgtkDom.hideElement(this._bottomBarDom);
	jsgtkDom.hideElement(this._topLeftBarDom);
	jsgtkDom.hideElement(this._topRightBarDom);
	jsgtkDom.hideElement(this._bottomLeftBarDom);
	jsgtkDom.hideElement(this._bottomRightBarDom);
};

/**
 * A method to disable and hide all control buttons
 */
gtk.WindowBase.prototype.disableControl = function(){
	var jsgtkDom = jsgtk.Dom;
	jsgtkDom.hideElement(this._closeButton);
	jsgtkDom.hideElement(this._minimizeButton);
	jsgtkDom.hideElement(this._maximizeButton);
	jsgtkDom.hideElement(this._returnToPreviousSizeButton);
	this._enableControl=false;
};

/**
 * A method to enable and display all control buttons
 */
gtk.WindowBase.prototype.enableControl = function(){
    this._enableControl = true;
    if(this._enableMinimizeButton)this.enableMinimizeButton();
    if(this._enableMaximizeButton)this.enableMaximizeButton();
    if(this._enableCloseButton) this.enableCloseButton();
};

/**
 * A method to enable and display close button
 */
gtk.WindowBase.prototype.enableCloseButton = function(){
    if(this._enableControl){
        jsgtk.Dom.showElement(this._closeButton);
        if(this._enableMaximizeButton){
            this._minimizeButton.style.right = "40px"
            this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "24px";
        }else this._minimizeButton.style.right = "24px"
        this._enableCloseButton = true;
    }
};

/**
 * A method to enable and display minimize button
 */
gtk.WindowBase.prototype.enableMinimizeButton = function(){
    if(this._enableControl){
         jsgtk.Dom.showElement(this._minimizeButton);
         if((!this._enableCloseButton)&&(!this._enableMaximizeButton)) this._minimizeButton.style.right = "5px";
         else if(this._enableCloseButton && this._enableMaximizeButton) this._minimizeButton.style.right = "40px";
         else this._minimizeButton.style.right = "24px";
         this._enableMinimizeButton = true;
     }
};

/**
 * A method to enable and display maximize button
 */
gtk.WindowBase.prototype.enableMaximizeButton = function(){
   if(this._enableControl){
       jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
       if(this._enableCloseButton){
            this._minimizeButton.style.right = "40px"
            this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "24px";
       }
       this._enableMaximizeButton = true;
   }
};

/**
 * A method to disable and hide close button
 */
gtk.WindowBase.prototype.disableCloseButton = function(){
    jsgtk.Dom.hideElement(this._closeButton);

    if(this._enableMaximizeButton){
        this._maximizeButton.style.right =this._returnToPreviousSizeButton.style.right= "5px";
        this._minimizeButton.style.right = "24px"
    }else this._minimizeButton.style.right = "5px"

    this._enableCloseButton = false;
};

/**
 * A method to disable and hide minimize button
 */
gtk.WindowBase.prototype.disableMinimizeButton = function(){
    jsgtk.Dom.hideElement(this._minimizeButton);
    this._enableMinimizeButton = false;
};

/**
 * A method to disable and hide maximize button
 */
gtk.WindowBase.prototype.disableMaximizeButton = function(){
    jsgtk.Dom.hideElement(this._maximizeButton);
    jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);

    if(this._enableCloseButton)this._minimizeButton.style.right = "24px"
    else this._minimizeButton.style.right = "5px"

    this._enableMaximizeButton = false;
};

/**
 * A method to show maximize header
 */
gtk.WindowBase.prototype.showMaximizedHeader = function(){
	jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.hideElement(this._maximizeButton);
};

/**
 * A method to show restore header
 */
gtk.WindowBase.prototype.showRestoredHeader = function(){
	jsgtk.Dom.hideElement(this._returnToPreviousSizeButton);
	jsgtk.Dom.showElement(this._maximizeButton);
};

/**
 * A method to show minimize header
 */
gtk.WindowBase.prototype.showMinimizedHeader = function(){
    jsgtk.Dom.showElement(this._returnToPreviousSizeButton);
    jsgtk.Dom.hideElement(this._maximizeButton);
};

/**
 * A method to set state of the window
 * @param {String} state The constant value to be passed.
 * <br>These are the constant values to be passed:
 * <br> - gtk.WINDOW_RESTORED The flag constant value to set as restored state
 * <br> - gtk.WINDOW_MAXIMIZED The flag constant value to set as maximized state
 * <br> - gtk.WINDOW_MINIMIZED The flag constant value to set as minimixed state
 */
gtk.WindowBase.prototype.setState = function(state){
    this._state = state;
};

/**
 * A method to get or return the current state of window
 * @return {String} The constant state value of the window
 * <br>These are the constant values to be returned:
 * <br> - gtk.WINDOW_RESTORED The flag constant value to set as restored state
 * <br> - gtk.WINDOW_MAXIMIZED The flag constant value to set as maximized state
 * <br> - gtk.WINDOW_MINIMIZED The flag constant value to set as minimixed state
 */
gtk.WindowBase.prototype.getState = function(){
    return this._state;
};

gtk.WindowBase.prototype.setTitle = function(title){
    this._titleDom.innerHTML = title;
};

/**
 * A method to set the title of the window to be bold
 * @param {Boolean} bold The boolean value true is bold, fasle it not bold
 */
gtk.WindowBase.prototype.setBoldTitle = function(bold){
	if(bold){
		this._titleDom.style.fontWeight = "bold";
	} else 	this._titleDom.style.fontWeight = "normal";
};

/**
 * A method to set position of the window
 * @param {String} position The constant value to be passed.
 * <br>This is the constant value to be passed:
 * <br> - gtk.JUSTIFY_CENTER The flag constant value to set as center screen
 */
gtk.WindowBase.prototype.setPosition = function(position){
    if(position == gtk.JUSTIFY_CENTER) this._positionCenter();
};

/** @private */
gtk.WindowBase.prototype._positionCenter = function(){
    var pageSize = jsgtk.Util.getPageSize();
    var left = (pageSize[0] - this._width )/2;
    var top = (pageSize[1] - this._height )/2;
    this.move(left, top);
};

/** @private */
gtk.WindowBase.prototype._clearChild = function(){
	this._bodyDom.removeChild(this._childWidget.getNode());
};

/**
 * A method to move or set position of window given left and top values in pixel
 * @param {Integer} left The left value in pixel
 * @param {Integer} top The top value in pixel
 */
gtk.WindowBase.prototype.move = function(left, top){
	var domObjStyle = this._domObj.style;
    domObjStyle.top = top + "px";
    domObjStyle.left = left + "px";
};

/** @private */
gtk.WindowBase.prototype._initMinimizedEvents = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    var restoreButton = this._minimizedWindow.childNodes[3];

    jsgtkEvent.addDomListener(restoreButton, "mouseup", function(e){
		self.restore();
		jsgtkEvent.stopBubble(e);
    });

	var closeButton = this._minimizedWindow.childNodes[4];
	jsgtkEvent.addDomListener(closeButton, "mouseup", function(e){
	    self._minimizedWindow.parentNode.removeChild(self._minimizedWindow);
	    self.emit("closed", self);
	    self._checkAndRemoveComboDropdown();
	    self._checkAndRemoveToolTip();
	    self._checkAndRemoveSmartEntryDropdown();
	    jsgtkEvent.stopBubble(e);
	});

    jsgtkEvent.addDomListener(this._minimizedWindow, "mouseup", function(e){
        self.emit("minimizedmouseovered", self._title);
        self._checkAndHideComboDropdown();
        self._checkAndHideToolTip();
        self._checkAndHideSmartEntryDropdown();
        self.restore();
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(this._minimizedWindow, "dblclick", function(e){
        self.restore();
    });
};

/** @private */
gtk.WindowBase.prototype._createMinimizedHolder = function(){
    var id = "gtk_minimized_widget_holder";
    var holder = document.getElementById(id);
    if(!holder) var holder = jsgtk.Dom.create({"append": document.body, "tag": "div", "id": id});
    return holder;
};

/** @private */
gtk.WindowBase.prototype._isNodeExistInMinimizedHolder = function(holder){
    var clonedHolder = holder.cloneNode(true);
    for(var i = 0, len = clonedHolder.childNodes.length; i < len; i++){
        if(this._domObj.id == clonedHolder.childNodes[i].id) return true;
    }
    return false;
};

/** @private */
gtk.WindowBase.prototype._createMinimizedWindow = function(){
    this._resizeDomTitleWidth(this._minimizedWidth - this._inconsWidth);

    var minimizeWindow = this._headerDom.cloneNode(true);
	minimizeWindow.className = "gtk_window_base_minimize";
    minimizeWindow.removeAttribute("style");
    minimizeWindow.style.width = this._minimizedWidth + "px";
    minimizeWindow.id = this._domObj.id;

    return minimizeWindow;
};

/** @private */
gtk.WindowBase.prototype._resizeDomTitleWidth = function(width){
    this._titleDom.style.width = width + "px";
};

/** @private */
gtk.WindowBase.prototype._hideCloseButton = function(){
    if(this._closeButton){
		this._closeButton.style.display = "none";
		this._minimizeButton.style.right = "21px";
		this._maximizeButton.style.right = "5px";
	}
};

/** @private */
gtk.WindowBase.prototype._showCloseButton = function(){
    var self = this,
        jsgtkEvent = jsgtk.Event,
        jsgtkDom = jsgtk.Dom;

    jsgtkDom.showElement(this._closeButton);
	this._minimizeButton.style.right = "41px";

	jsgtkEvent.addDomListener(this._closeButton, "mouseup", function(e){
	    self.emit("closed", self);
	    self._closeModal();
	    self._domObj.parentNode.removeChild(self._domObj);
	    self._domObj = null;
	    self._checkAndRemoveComboDropdown();
	    self._checkAndRemoveToolTip();
	    self._checkAndRemoveSmartEntryDropdown();
	    jsgtkEvent.stopBubble(e);
	});
};

/**
 * A method to hide minimize button
 */
gtk.WindowBase.prototype.hideMinimizeButton = function(){
    this._minimizeButton.style.display = "none";
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveComboDropdown = function(){
    var comboDropDown = document.getElementsByClassName("gtk_combobox_holder");
    if(comboDropDown !== null){
		var comboDropDownLength = comboDropDown.length;
		if(comboDropDownLength > 0){
			for(var i = 0; i < comboDropDownLength; i++){
				var childTobeRemoved = comboDropDown[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveSmartEntryDropdown = function(){
    var smartEntryDropDown = document.getElementsByClassName("gtk_smart_entry_drop_down_widget_holder");
    if(smartEntryDropDown !== null){
		var smartEntryDropDownLength = smartEntryDropDown.length;
		if(smartEntryDropDownLength > 0){
			for(var i = 0; i < smartEntryDropDownLength; i++){
				var childTobeRemoved = smartEntryDropDown[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideSmartEntryDropdown = function(){
    var smartEntryDropDown = document.getElementsByClassName("gtk_smart_entry_drop_down_widget_holder");
    if(smartEntryDropDown !== null){
		var smartEntryDropDownLength = smartEntryDropDown.length;
		if(smartEntryDropDownLength > 0){
			for(var i = 0; i < smartEntryDropDownLength; i++){
				var childTobeHidden = smartEntryDropDown[i];
				childTobeHidden.style.display = "none";
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideComboDropdown = function(){
    var comboDropDown = document.getElementsByClassName("gtk_combobox_holder");
    if(comboDropDown !== null){
    	var comboDropDownLength = comboDropDown.length;
		if(comboDropDownLength > 0){
			for(var i = 0; i < comboDropDownLength; i++){
				var childTobeHidden = comboDropDown[i];
				childTobeHidden.style.display = "none";
			}
		}
    }
};

/** @private */
gtk.WindowBase.prototype._checkAndRemoveToolTip = function(){
    var toolTip = document.getElementsByClassName("gtk_widget gtk_bubble");
    if(toolTip !== null){
		var toolTipLength = toolTip.length;
		if(toolTipLength > 0){
			for(var i = 0; i < toolTipLength; i++){
				var childTobeRemoved = toolTip[0];
				childTobeRemoved.parentNode.removeChild(childTobeRemoved);
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._checkAndHideToolTip = function(){
    var toolTip = document.getElementsByClassName("gtk_widget gtk_bubble");
    if(toolTip !== null){
		var toolTipLength = toolTip.length;
		if(toolTipLength > 0){
			for(var i = 0; i < toolTipLength; i++){
				var childTobeHidden = toolTip[i];
				childTobeHidden.style.display = "none";
			}
		}
	}
};

/** @private */
gtk.WindowBase.prototype._isWindowRestored = function(id){
    if(!document.getElementById(id)){
        this._domObj.id = id;
        return true;
    }
    return false;
};

/**
 * A method to set or show close button give a boolean value
 * @param {Boolean} isCloseButtonActive The boolean value true is to show, or false to hide
 */
gtk.WindowBase.prototype.setCloseButton = function(isCloseButtonActive){
	this._isCloseButtonActive = isCloseButtonActive;
	if(this._isCloseButtonActive) this._showCloseButton();
	else this._hideCloseButton();
};

/**
 * A method to close or remove window
 */
gtk.WindowBase.prototype.close = function(){
    this.emit("closed", self);
    this._closeModal();
    this._domObj.parentNode.removeChild(this._domObj);
    this._checkAndRemoveComboDropdown();
    this._checkAndRemoveToolTip();
    this._checkAndRemoveSmartEntryDropdown();

};

/** @private */
gtk.WindowBase.prototype._showModal = function(){
    if(!this._isModal)this._createModal();
    this._domObj.style.zIndex = "8001";
};

/** @private */
gtk.WindowBase.prototype._closeModal = function(){
	if(typeof this._modal !== 'undefined')
    	this._modal.parentNode.removeChild(this._modal);
	this._domObj.style.zIndex = "8000";
};

/** @private */
gtk.WindowBase.prototype._hideModal = function(){
    this._modal.style.display = "none";
    this._domObj.style.zIndex = "8000";
};

/** @private */
gtk.WindowBase.prototype._createModal = function(){
    this._modal = jsgtk.Dom.create({"append":document.body, "tag": "div", "id": "gtk_window_modal"});
    var widthHeight = jsgtk.Util.getPageSize();
    this._modal.style.width = widthHeight[0] + "px";
    this._modal.style.height = widthHeight[1] + "px";
    this._isModal = true;
};
gobject.signalNew('maximized', gtk.WindowBase, [], null, null);
gobject.signalNew('minimized', gtk.WindowBase, [], null, null);
gobject.signalNew('minimizedmouseovered', gtk.WindowBase, [], null, null);
gobject.signalNew('restored', gtk.WindowBase, [], null, null);
gobject.signalNew('closed', gtk.WindowBase, [], null, null);
/**
 * ComboBoxController is a core class of gtk.ComboBox to control all logic matters
 * to perform any specific action.
 * @constructor
 * @see gtk.ComboBox
 */
gtk.ComboBoxController = function(view){
	this._model = null;
	var self = this;

	this._render = function(model, parentNode){
		var mashedUpdata = this.__mashupModel(model.getChildren());
		view._renderDropDown(mashedUpdata, parentNode);
	};

	view.connect("_rendersubnodes", function(path, parentNode){
		var childModel = self._model.getChildByPath(path);
		self._render(childModel, parentNode);
	});

	view.connect("_search", function(value){
		var model = self._model.search(value);
		var path = model ? model.getPath() : "";
		view._hilightRow(path);
	});

	view.connect("_itemselected", function(path){
	    var childModel = self._model.getChildByPath(path);
	    view._setSelectedItem(childModel.getValue())
	});

	view.connect("_valueMembersearched", function(valueMember, value){
	    var childModel = self._model._searchByValueMember(valueMember, value);
	    view._setSelectedItem(childModel.getValue())
	});
};

/** @private */
gtk.ComboBoxController.prototype.setModel = function(model){
	this._model = model;
	this._render(this._model, null);
};

/** @private */
gtk.ComboBoxController.prototype.__mashupModel = function(nodes){
    var data = [], node, hasChild;
    for(var i = 0, len = nodes.length; i < len; i++){
        node = nodes[i];
        hasChild = node.getChildren().length  > 0;
        data.push({"display": node.getValue()["display"], "hasChildNodes": hasChild, "path": node.getPath(), "valueObject": node.getValue()});
    }
    return data;
};
/**
 * ComboBox is a class to create a combobox dropdown list that contains many optional choices
 * The combobox dropdown list stores data as model from gtk.TreeStore.
 * @see gtk.TreeStore
 * @constructor
 * @base gtk.Widget
 */
gtk.ComboBox = function(){
    gtk.Widget.apply(this);

	/** 'clicked' signal */
	this.COMBOBOX_CLICKED = "clicked";
	/** 'changed' signal */
	this.COMBOBOX_CHANGED = "changed";

    this._minWidth = this._width = 150;
    this._minHeight = this._height = 30;

	this._isToogled = false;
	this._vBarWidth = 25;
	this._isDropDownCreated = false;
	this._selectedItem;

	this._render = function(){
	    this._domObj.className += " gtk_combobox";
			this._entryHolder = jsgtk.Dom.create({tag: "div", append: this._domObj, className: "gtk_combobox_input_holder"});
				this._entry = jsgtk.Dom.create({tag: "input", append: this._entryHolder, className: "gtk_combobox_input"});

		    this._gradient = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_button_gradient"});

			this._activeLayer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_combobox_active_layer"});

			this._labelContainer = jsgtk.Dom.create({'append': this._domObj, 'tag': 'div','class': 'gtk_combobox_label_container'});
    			var table = jsgtk.Dom.create({append: this._labelContainer, tag: "table"});
	    			var tr = jsgtk.Dom.create({append: table, tag: "tr"});
	    				var td = jsgtk.Dom.create({append: tr, tag: "td"});
		    				this._labelDom = jsgtk.Dom.create({"append": td, "text": "- Please Select -", "tag": "div", 'class': 'gtk_combobox_label'});

			var arrowImge = jsgtk.Dom.create({'append': this._domObj, 'tag': 'div','class': 'gtk_combobox_image_container'});
            arrowImge.style.width = this._vBarWidth + 'px';
    			var table = jsgtk.Dom.create({append: arrowImge, tag: "table"});
	    			var tr = jsgtk.Dom.create({append: table, tag: "tr"});
	    				var td = jsgtk.Dom.create({append: tr, tag: "td"});
	    					jsgtk.Dom.create({append: td, tag: "div", "class": "gtk_combobox_image"});

			this.__dropDown = jsgtk.Dom.create({tag: "div", append: document.body, className: "gtk_combobox_holder"});
			jsgtk.Dom.hideElement(this.__dropDown);
	};

	this._render();
	this._initEvent();
	this._dropDownRendere = new gtk.SmartEntryDropDownRendere(this);
	this._controller = new gtk.ComboBoxController(this);
};

gtk.ComboBox.prototype = new gtk.Widget();
gtk.ComboBox.prototype.constructor = gtk.ComboBox;

/**
 * A method to to set model onto the widget
 * @param {Object} model The object model value from gtk.TreeStore
 */
gtk.ComboBox.prototype.setModel = function(model){
	this._controller.setModel(model);
};

/**
 * A method to set selected item by any value member
 * @param {String} valueMember The name of value member assigned
 * @param {String or Integer} value The field value in accordance with its value member
 */
gtk.ComboBox.prototype.setSelectedItemBy = function(valueMember, value){
    if(value) this.emit("_valueMembersearched", valueMember, value);
};

/**
 * A method to get/return object of selected item
 * @return {Object}, The object value of the selected item
 * @type Object
 */
gtk.ComboBox.prototype.getSelectedItem = function(){
    return this._selectedItem;
};

/**
 * A method to set its tab index number
 * @param {Interger} index The tab index value to be set
 */
gtk.ComboBox.prototype.setTabindex = function(index){
    this._domObj.tabIndex = index;
};

/** @private */
gtk.ComboBox.prototype._addEventMouseWheel = function(scrollObj){
    var self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    self._deactivate();
	}, false);
};

/**
 * A method to set displaying error or not
 * @param {Boolean} isOnError The boolean value to be passed: true is to show error, false is to hide
 */
gtk.ComboBox.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;

    if(this._showError) this._domObj.className += " gtk_combobox_on_error";
    else this._domObj.className = "gtk_widget gtk_combobox";
};

/**
 * A method to get/return string value of selected item
 * @return {String} The text value of selected item
 * @type String
 */
gtk.ComboBox.prototype.getActiveText = function(){
    return this._labelDom.innerHTML;
};

/**
 * A method to reload or refresh its contents
 */
gtk.ComboBox.prototype.refresh = function(){
    this._isDropDownCreated = false;
    this._deactivate();
    this._clearDropDown();
	this._labelDom.innerHTML = "- Please Select -";
};

/** @private */
gtk.ComboBox.prototype._setSelectedItem = function(value){
    if(this._selectedItem === value) return;
    this._selectedItem = value;
    this._labelDom.innerHTML = value["display"];
    this.emit("changed", value);
};

/** @private */
gtk.ComboBox.prototype._renderDropDown = function(children, parentNode){
	var rowNodes = this._dropDownRendere._constructRowNodes(children, parentNode);
	this._appendDropDown(rowNodes);
    this._isDropDownCreated = true;
};

/** @private */
gtk.ComboBox.prototype._clearDropDown = function(){
	this.__dropDown.innerHTML = "";
};

/** @private */
gtk.ComboBox.prototype._handleRowNodeSelected = function(){
	var selectedRowNode = this._dropDownRendere.getSelectedRowNode();
	this.emit("_itemselected", selectedRowNode.id);
	this._deactivate();
};

/** @private */
gtk.ComboBox.prototype.__bindInputEvent = function(){
	var jsgtkEvent = jsgtk.Event, self = this;
	var timeoutId;
	this.__inputKeyupId = jsgtkEvent.addDomListener(this._entry, "keydown", function(e){

		switch(e.keyCode){
			case gtk.DOWN_ARROW_KEY:
				self.__fireDownArrowKey();
				return;
			case gtk.UP_ARROW_KEY:
				self.__fireUpArrowKey();
				return
			case gtk.ENTER_KEY:
				self.__fireEnterKey();
				return;
			case gtk.TAB_KEY:
				self.__fireEnterKey();
				jsgtk.Event.stopDefault(e);
				return;
		}

		if(self._entry.value){

			clearTimeout(timeoutId);

			timeoutId = setTimeout(function(){

                var text = self._entry.value;
                var selectedRowNode = self._dropDownRendere.getSelectedRowNode();

			    if(selectedRowNode) {
                    var isMatched = self._compareText(selectedRowNode.innerHTML, text);
                    if(!isMatched) self._handleSearch(text);
                }else self._handleSearch(text);
			}, 200);
		}
	});
};

/** @private */
gtk.ComboBox.prototype._handleSearch = function(text){
    var lastCharacter = text[text.length - 1];
    this.emit("_search", lastCharacter);
    this._entry.value = lastCharacter;
};

/** @private */
gtk.ComboBox.prototype._initEvent = function(){
	var self = this;

	jsgtk.Event.addDomListener(this._domObj, "mousedown", function(e){
		if(self._isToogled)
		    self._deactivate();
		else
            self._activate();

	    if(!self._isDropDownCreated) {
	        self.emit("clicked");
	        self._showLoading();
	    }

		jsgtk.Event.stopBubble(e);
		jsgtk.Event.stopDefault(e);
	});

	jsgtk.Event.addDomListener(this.__dropDown, "mousedown", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className)){
			self._handleRowNodeSelected();
		}
		jsgtk.Event.stopBubble(e);
	});

	jsgtk.Event.addDomListener(this.__dropDown, "mouseover", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._dropDownRendere.hilightRowNode(node);
	});

	jsgtk.Event.addDomListener(this._entry, "blur", function(e){
		self._deactivate();
		jsgtk.Event.stopBubble(e);
	});

	this.__bindInputEvent();
};

/** @private */
gtk.ComboBox.prototype._compareText = function(text1, text2){
    var re = new RegExp("^" + text2.toLowerCase());
    var result = text1.toLowerCase().match(re);
    return result;
};

/** @private */
gtk.ComboBox.prototype._hilightRow = function(path){
	this._dropDownRendere.hilightRowById(path)
};

/** @private */
gtk.ComboBox.prototype.setFocus = function(){
	this._dropDownRendere._deHilightSelectedRow();
	this._entry.focus();
}

/** @private */
gtk.ComboBox.prototype.lostFocus = function(){
	this._entry.blur();
};

/** @private */
gtk.ComboBox.prototype._appendDropDown = function(dropDown){
	this._hideLoading();
	this.__dropDown.appendChild(dropDown);
};

/** @private */
gtk.ComboBox.prototype._showLoading = function(){
	this.__dropDown.className += " gtk_combobox_loading";
};
/** @private */
gtk.ComboBox.prototype._hideLoading = function(){
	this.__dropDown.className = "gtk_combobox_holder";
};
/** @private */
gtk.ComboBox.prototype.__removeInputEvent = function(){
	var jsgtkEvent = jsgtk.Event;
	if(this.__inputKeypressId) jsgtkEvent.removeListener(this.__inputKeypressId);
	if(this.__inputKeyupId) jsgtkEvent.removeListener(this.__inputKeyupId);
};

/** @private */
gtk.ComboBox.prototype.__fireDownArrowKey = function(){
	this._dropDownRendere.hilightNextRowNode();
};

/** @private */
gtk.ComboBox.prototype.__fireUpArrowKey = function(){
	this._dropDownRendere.hilightPreviousRowNode();
};

/** @private */
gtk.ComboBox.prototype.__fireEnterKey = function(){
	if(this._dropDownRendere.getSelectedRowNode()){
		this._handleRowNodeSelected();

		this._dropDownRendere._deHilightSelectedRow();
	}
};

/** @private */
gtk.ComboBox.prototype._setPosition = function(){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);
    var header = parseInt(this._domObj.style.height);
    var scrollY = this._getNearestScrollerY();

    this.__dropDown.style.top = top + header + 2 - scrollY + "px";
    this.__dropDown.style.left = left + 1 + "px";
};

/** @private */
gtk.ComboBox.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode.parentNode != null){
        if(/gtk_scroller_base/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};

/** @private*/
gtk.ComboBox.prototype._activate = function(){
	this._isToogled = true;
	this.setFocus();
	jsgtk.Dom.showElement(this.__dropDown);
    jsgtk.Dom.showElement(this._activeLayer);
    this._domObj.className += " gtk_combobox_active";
	this._setPosition();
};

/** @private*/
gtk.ComboBox.prototype._deactivate = function(){
	this._isToogled = false;
	this.lostFocus();
	jsgtk.Dom.hideElement(this.__dropDown);
    jsgtk.Dom.hideElement(this._activeLayer);
    this._domObj.className = "gtk_widget gtk_combobox";
};

/** @private */
gtk.ComboBox.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
		this._labelContainer.style.width = width - this._vBarWidth - 9 + "px";
		this.__dropDown.style.width = width + "px";
};

/** @private */
gtk.ComboBox.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = height + 'px';
};

gobject.signalNew("_rendersubnodes", gtk.ComboBox, null, null, []);
gobject.signalNew("_setselectedvalues", gtk.ComboBox, null, null, []);
gobject.signalNew("_search", gtk.ComboBox, null, null, []);
gobject.signalNew("_itemselected", gtk.ComboBox, null, null, []);
gobject.signalNew("_valueMembersearched", gtk.ComboBox, null, null, []);
gobject.signalNew("changed", gtk.ComboBox, null, null, []);
gobject.signalNew("clicked", gtk.ComboBox, null, null, []);
/**
 * TreeIter is an assistant class to create model or structure of a model
 * @constructor
 * @base gtk.Object
 */
gtk.TreeIter = function(path){
	gtk.Object.apply(this);

	this._modelType = gtk.TREE_ITER;
 	this.path = path ? path : "0";
 	this.hasNode = false;
 	this.children = [];
 	this._value;
};

gtk.TreeIter.prototype = new gtk.Object();
gtk.TreeIter.prototype.constructor = gtk.TreeIter;

gtk.TreeIter.prototype.getChldBy = function(valueMember, value){
    var strValue = String(this._value[valueMember]).toLowerCase();
    var strToCompare = String(value).toLowerCase();

	if(strValue === strToCompare) return this;
	for(var i = this.children.length; i--;){
		return this.children[i].getChldBy(valueMember, value);
	}
};

gtk.TreeIter.prototype.setRow = function(value){
	this._value = value;
};

gtk.TreeIter.prototype.getValue = function(){
	return this._value;
};

gtk.TreeIter.prototype.getModelType = function(){
	return this._modelType;
};

/**
 * Returns a copy of the treeiter.
 * return {object} treeiter
 */
gtk.TreeIter.prototype.copy = function(){
 	return this;
};

gtk.TreeIter.prototype.getChildByPath = function(path){
    var paths = path.split(":");

    var child = this._getChildByPath( parseInt(paths[0]) );

    paths.splice(0,1);
    if(paths.length === 1) return child;

    return child.getChildByPath(paths.join(":"));
};

gtk.TreeIter.prototype._getChildByPath = function(path){
	for(var i = this.children.length; i--;){
		var childPath = this.children[i].path.split(":");
		if(parseInt(childPath[childPath.length - 2]) === path) return this.children[i];
	}
	return null;
};

gtk.TreeIter.prototype.updateValue = function(value){
    this._value["value"] = value;
    this.emit("valueupdated", this.path, value);
};

gtk.TreeIter.prototype.getPath = function(){
	return this.path;
};

/**
* getChildren() method
* @param {array} children: array of children of gtk.TreeIter type
*/
gtk.TreeIter.prototype.getChildren = function(){
	return this.children;
};

/**
* add() method
* @param{object} child: object of of gtk.TreeIter type
*/
gtk.TreeIter.prototype.add = function(child){
	this.children.push(child);
};

/*
* remove() method : remove a child
* @param {object} child: child of gtk.TreeIter type
*/
gtk.TreeIter.prototype.remove = function(child){
	for(var i = this.children.length; i--;){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
* getChild() method
* @param {integer} index : index of child
* @return {object} child: child of gtk.TreeIter type
*/
gtk.TreeIter.prototype.getChild = function(index){
	return this.children[index] || null;
};

/** @private */
gtk.TreeIter.prototype._search = function(text){
	var re = new RegExp("^" + text.toLowerCase());

    var children = this.getChildren();
    var len = children.length;
    if(len > 0){
		for(var i = 0 ; i < len; i++){
			var child = children[i];
	    	this._child = child._search(text);
	    	if(this._child) return this._child;
		}
	}
	else{
    	var row = this.getValue()["display"];
	    var result = row.toLowerCase().match(re);
		if(result)
			return this;
	}
};

gobject.signalNew('valueupdated', gtk.TreeIter, [], null, null);
/**
 * @constructor
 * @param {array} headers columns of header
 * @base gtk.Object
 */
gtk.ListStore = function(headers){
 	gtk.Object.apply(this);
	this._modelType = gtk.ListStore;
 	this.children   = new Array();
	this._headers   = headers || [];
 };

gtk.ListStore.prototype = new gtk.Object();
gtk.ListStore.prototype.constructor = gtk.ListStore;

/**
 * appends a new row to the liststore
 * @param {object} json object
 * @return {gtk.TreeIter}
 * @see gtk.TreeIter
 */
gtk.ListStore.prototype.append = function(row){
    var self = this;
	row["value"] = this._filterRow(row["value"]);
	var iter = new gtk.TreeIter(this.children.length.toString() + ':');
	iter.setRow(row);
	this.add(iter);
	iter.connect("valueupdated", function(path, value){
        self.emit("rowupdated", path, value);
	});
	return iter;
};

/** @private */
gtk.ListStore.prototype.appendToHeader = function(header){
    this._headers.push(header);
};

/**
 * appends new rows to the liststore
 * @param {array} arrow of json object
 * @return {array} array of gtk.TreeIter object
 * @see gtk.TreeIter
 */
gtk.ListStore.prototype.appendRows = function(rows){
    if(rows.length > 0){
        var payloadRows = [];

        for(var i = 0, len = rows.length; i < len; i++){
            payloadRows.push(this.append(rows[i]));
        }

        this.emit("rowsupdated", payloadRows);
    }
	return payloadRows;
};

/**
 * Return gtk.TreeIter object
 * @param {string} path which is id of each gtk.TreeIter object
 * @return {gtk.TreeIter}
 */
gtk.ListStore.prototype.getChildByPath = function(path){
    for(var i = this.children.length; i--;){
        var child = this.children[i];
        if(child.getPath() === path) return child;
    }
};

/** @private */
gtk.ListStore.prototype.getModelType = function(){
	return this._modelType;
};

/** @private */
gtk.ListStore.prototype.getColumnsWidth = function(){
    var widths = [];
    for(var i = 0, len = this._headers.length; i < len; i++){
        widths.push(this._headers[i].width);
    }
    return widths;
};

/** @private */
gtk.ListStore.prototype.getWidths = function(){
    var width = 0;
    for(var i = this._headers.length; i--;){
        width += this._headers[i].width;
    }
    return width;
};

/** @private */
gtk.ListStore.prototype._filterRow = function(row){
	var filters = [];
	for(var i = 0, len = this._headers.length; i < len; i++){
		if(row[i] === null || row[i] === "" || row[i] === " " || row[i] === undefined){
			filters.push('N/A');
			continue;
		}
		filters.push(row[i]);
 	}
 	return	filters;
};

/**
 * Return all gtk.TreeIter
 * @return {array}
 */
gtk.ListStore.prototype.getChildren = function(){
	return this.children;
};

/** @private */
gtk.ListStore.prototype.getHeaders = function(){
	return this._headers;
};

/** @private */
gtk.ListStore.prototype.add = function(child){
	this.children.push(child);
};

/**
 * Remove gtk.TreeIter from gtk.ListStor
 * @param {gtk.TreeIter}
 */
gtk.ListStore.prototype.remove = function(child){
	for(var i = this.children.length; i--;){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
 * Return gtk.TreeIter by index
 * @param {integer} index
 */
gtk.ListStore.prototype.getChild = function(index){
	return this.children[index] || null;
};

/** @private */
gtk.ListStore.prototype._getColumn = function(valueMember){
    for(var i = 0, len = this._headers.length; i < len; i++){
        if(this._headers[i]["title"] === valueMember) return i;
    }
    return 0;
};

/**
 * @private
 */
gtk.ListStore.prototype.search = function(valueMember, value){
    var column = this._getColumn(valueMember);
    var re = new RegExp("^" + value.toString().toLowerCase());
    var children = this.children;
    var result = "";
    for(var i = 0, len = children.length; i < len; i++){
        result = children[i].getValue().value[column].toString().toLowerCase().match(re);
        if(result){
            return children[i];
        }
    }
    return children[0];
};

/** @private */
gtk.ListStore.prototype.filter = function(value, valueMember){
    var column = this._getColumn(valueMember);
    var filteredChildren = [];
    var re = new RegExp("^" + value.toString().toLowerCase());
    var children = this.children;
    var result = "";

    for(var i = 0, len = children.length; i < len; i++){
        result = children[i].getValue().value[column].toString().toLowerCase().match(re);
        if(result){
            filteredChildren.push(children[i]);
        }
    }
    return filteredChildren;
};

gobject.signalNew('rowsupdated', gtk.ListStore, [], null, null);
gobject.signalNew('rowupdated', gtk.ListStore, [], null, null);
/**
 * @constructor
 * @base gtk.Widget
 * @param {gtk.ListStor} model
 * @link gtk.ListStor
 */
gtk.ListView = function(model){
	gtk.Widget.apply(this);

	/** 'rowselected' signal */
	this.LISTVIEW_ROWSELECTED = 'rowselected';
	/** 'rowclicked' signal */
	this.LISTVIEW_ROWCLICKED = 'rowclicked';
	/** 'rowover' signal */
	this.LISTVIEW_ROWOVER = 'rowover';
	/** 'rowout' signal */
	this.LISTVIEW_ROWOUT = 'rowout';
	/** 'rowdoubleclicked' signal */
	this.LISTVIEW_ROWDOUBLECLICKED = 'rowdoubleclicked';

	this._width            = this._minWidth = 400;
	this._height           = this._minHeight = 100;
    this._titleHeight      = 19;
    this._cellHeight       = 18;
	this._bodyHeight       = this._height - this._cellHeight;
	this._columnsWidth     = [];

	this._headerVisible    = true;
	this._leftPadding      = 4;
	this._model            = model || null;
	this._selectedRow      = null;
	this._emptyRows        = [];
	this._rowIndex         = -1;
	this._border           = 2;
	this._selectedRowPath  = "";
	this._hilightedRowNode    = null;
	this._parentWidth      = this._width;
	this._valueMember      = "";
	this._createTooltip    = false;
	this._selectedRowIndex = 0;

	this._render = function(){
		this._domObj.className   += " gtk_listview";
            this._tableDom = jsgtk.Dom.create({"append": this._domObj, "tag": "TABLE", "BORDER": "0","CELLPADDING": "6", "class": "gtk_listview_table"});
                this._headerDom = jsgtk.Dom.create({"append": this._tableDom, "tag": "THEAD", "class": "gtk_listview_table_head"});
                this._bodyDom   = jsgtk.Dom.create({"append": this._tableDom, "tag": "TBODY", "class": "gtk_listview_table_body"});
                this._footerDom = jsgtk.Dom.create({"append": this._tableDom, "tag": "TFOOT", "class": "gtk_listview_table_footer"});

            this._vGhost = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_listview_vghost"});
            jsgtk.Dom.hideElement(this._vGhost);

            this._headerDomGhost = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_listview_table_head_ghost"});
            jsgtk.Dom.hideElement(this._headerDomGhost);

        this._hiddenTextBox = jsgtk.Dom.create({"append":  document.body, "tag": "input", "type": "text",  "class": "gtk_listview_hiddent_textbox"});

        this._grayOutBackground   = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_gray_out"});
        this._loadingImgContainer = jsgtk.Dom.create({"append": this._domObj, "tag": "div", "class": "gtk_loading_bar_container_listview"});
            this._loadingImg      = jsgtk.Dom.create({"append": this._loadingImgContainer, "tag": "div", "class": "gtk_loading_bar_listview"});

        jsgtk.Dom.hideElement(this._grayOutBackground);
        jsgtk.Dom.hideElement(this._loadingImgContainer);
	};
	this._render();

	this._model = model;
	new gtk.ListViewController(this, this._model);

	this._enableSortable();

	this._keyControllEvent();
};

gtk.ListView.prototype = new gtk.Widget();
gtk.ListView.prototype.constructor = gtk.ListView;

/**
 * Show loading bar
 */
gtk.ListView.prototype.showLoading = function(){
    jsgtk.Dom.showElement(this._grayOutBackground);
    jsgtk.Dom.showElement(this._loadingImgContainer);
};


/**
 * Hide loading bar
 */
gtk.ListView.prototype.hideLoading = function(){
    jsgtk.Dom.hideElement(this._grayOutBackground);
    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * Return number of none empty row
 * @return {integer}
 */
gtk.ListView.prototype.getNumberOfActiveRow = function(){
    var rowNodes = this._bodyDom.childNodes, count = 0;
    for(var i = 0, len = rowNodes.length; i < len; i++){
        var emptyRowId = new RegExp("^" + "empty_");
        if(!rowNodes[i].id.match(emptyRowId)) count++;
    }
    return count;
};

/**
 * set the next row of current selected row selected
 */
gtk.ListView.prototype.setNextSelectedRow = function(){
    var nextRow = this._hilightedRowNode ? this._hilightedRowNode.nextSibling : this._bodyDom.firstChild;
    this._rowIndex = this._getIndex(nextRow);
    this._hilightCell(this._rowIndex);
    this._emitSelectedRowSignalToController(nextRow.id);
};

/**
 * set selected row by index
 * @param {integer} index
 */
gtk.ListView.prototype.setSelectedRowByIndex = function(index){
    var childLen = this._model.children.length;
    this._rowIndex = index > childLen ? childLen-1 : index;
    this._hilightCell(this._rowIndex);
    var path = this._bodyDom.childNodes[this._rowIndex].id;
    this._emitSelectedRowSignalToController(path);
    this.emit("rowclicked", this.getSelectedRow());
};

/**
 * set selected row by key value
 * @param {any type} key
 * @param {any type} value
 */
gtk.ListView.prototype.setSelectedRowByValue = function(valueMember, value){
    this.emit("searchcontenttocontroller", valueMember, value);
    this.emit("rowclicked", this.getSelectedRow());
};

/**
 * filter row by key value
 * @param {any type} key
 * @param {any type} value
 */
gtk.ListView.prototype.filterRowsByValue = function(valueMember, value){
    this.emit("filterrowstocontroller", valueMember, value);
};

/** @private */
gtk.ListView.prototype.hilightSelectedRow = function(path){
    this._rowIndex = this._getRowIndexById(path);
    this._hilightCell(this._rowIndex);

    this._scrollTableBodyByIndex(this._rowIndex);
};

/**
 * set model for gtk.ListView
 * @param {gtk.ListStor} model
 */
gtk.ListView.prototype.setModel = function(model){
    this._model = model;
    this._refreshView();
    this.emit("refreshmodel", model);

    this.resizeHeightComposite(this._height + 2);
    this.resizeWidthComposite(this._parentWidth);
};

/** @private */
gtk.ListView.prototype._getRowById = function(id){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i].id === id) return rows[i];
    }
    return rows[0];
};

/** @private */
gtk.ListView.prototype._getRowIndexById = function(id){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i].id === id) return i;
    }
    return 0;
};

/** @private */
gtk.ListView.prototype.getCellHeight = function(){
    return this._cellHeight;
};

/**
 * enale row selection by using arrow key
 */
gtk.ListView.prototype.activateArrowKey = function(){
    this._hiddenTextBox.focus();
};

/** @private */
gtk.ListView.prototype._scrollTableBodyByIndex = function(rowIndex){
    var bodyHeight = this._height - this._cellHeight;
    var currentScrollTop = rowIndex * this._cellHeight;
    var offsetTop = currentScrollTop - bodyHeight;
    this._bodyDom.scrollTop += offsetTop + 36;
};

/** @private */
gtk.ListView.prototype._refreshView = function(){
    this._headerDom.innerHTML = "";
    this._bodyDom.innerHTML = "";
    this._hilightedRowNode = null;
};

/** @private */
gtk.ListView.prototype._hilightCell = function(index){
    if(this._hilightedRowNode) jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": ""});
    this._hilightedRowNode = this._bodyDom.childNodes[index];
    jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": "#FFFF66"});

    if(index !== this._selectedRowIndex){
        var selectedRowNode = this._bodyDom.childNodes[this._selectedRowIndex];
        jsgtk.Dom.applyAttributes(selectedRowNode, {"backgroundColor": ""});
    }
};

/** @private */
gtk.ListView.prototype._hilightCellByArrowKey = function(index){
    if(this._hilightedRowNode){
        jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": ""});
    }
    this._hilightedRowNode = this._bodyDom.childNodes[index];
    jsgtk.Dom.applyAttributes(this._hilightedRowNode, {"backgroundColor": "#FFFFCC"});

    if(index !== this._selectedRowIndex){
        var selectedRowNode = this._bodyDom.childNodes[this._selectedRowIndex];
        jsgtk.Dom.applyAttributes(selectedRowNode, {"backgroundColor": "#FEFA89"});
    }
};

/** @private */
gtk.ListView.prototype._emitSelectedRowSignalToController = function(path){
    var emptyRowId = new RegExp("^" + "empty_");

    if(!path.match(emptyRowId) && this._selectedRowPath !== path) {
        this.emit("_rowselectedtocontroller", path);
        this._selectedRowPath = path;
    }
};

/** @private */
gtk.ListView.prototype._enableSortable = function(){
    jsgtk.Dom.SortTable(this._tableDom);
};

/** @private */
gtk.ListView.prototype.updateRowValue = function(path, value){
    var childNodes = this._getRowById(path).childNodes;
    for(var i = 0, len = childNodes.length; i < len; i++){
        childNodes[i].firstChild.innerHTML = value[i];
    }
};

/** @private */
gtk.ListView.prototype._constructRowColumns = function(tr, columns, columnsData){
    for(var j = 0, len = columns; j < len; j++){
        var td = jsgtk.Dom.create({"append": tr, "tag": "TD", "class": "gtk_listview_table_body_td"});
            var cell = jsgtk.Dom.create({"append": td, "tag": "DIV", "class": "gtk_listview_table_body_td_cell"});
            if(columnsData[j]) cell.innerHTML = columnsData[j];
            cell.style.height =  this._cellHeight + "px";
            if( j === len - 1) jsgtk.Dom.hideElement(cell);
    }
};

/** @private */
gtk.ListView.prototype._constructBodyRow = function(index, path, columns, columnsData){
    var tr = jsgtk.Dom.create({"tag": "TR", "class": "gtk_listview_table_body_tr", "id": path});

    if(index % 2 === 0) tr.className += " gtk_listview_table_body_tr_even";
    else tr.className += " gtk_listview_table_body_tr_odd";

    this._constructRowColumns(tr, columns, columnsData);

    return tr;
};

/** @private */
gtk.ListView.prototype._storeEmptyRows = function(tr, path){
    var emptyRowId = new RegExp("^" + "empty_");
    if(path.match(emptyRowId)) this._emptyRows.push(tr);
};

/** @private */
gtk.ListView.prototype._resetSelectedRowPath = function(){
    this._selectedRowPath = "";

};

/** @private */
gtk.ListView.prototype.renderEmptyRows = function(children, columns){
    this.constructBody(children, columns);
};

/** @private */
gtk.ListView.prototype.renderRows = function(children, columns){
    this.constructBody(children, columns);
};

/** @private */
gtk.ListView.prototype.constructBody = function(children, columns){
    this._resetSelectedRowPath();
	var docFragment = document.createDocumentFragment();
	for(var i = 0, len = children.length; i < len; i++){
	    var path = children[i]["path"];

	    var tr = this._constructBodyRow(i, path, columns, children[i]["rows"]);
        docFragment.appendChild(tr);

	    this._handleRowEvent(tr, path);

	    this._storeEmptyRows(tr, path);
	}

	this._bodyDom.appendChild(docFragment);
	this._enableSortable();
};

/** @private */
gtk.ListView.prototype.refreshRows = function(){
    this._bodyDom.innerHTML = "";
    this._hilightedRowNode = null;
};

/** @private */
gtk.ListView.prototype.clearEmptyRows = function(){
    for(var i = 0, len = this._emptyRows.length; i < len; i++){
        if(this._emptyRows[i].parentNode) this._bodyDom.removeChild(this._emptyRows[i]);
    }
    this._emptyRows = [];
};

/** @private */
gtk.ListView.prototype.constructHeader = function(headers){
	var tr = jsgtk.Dom.create({"append": this._headerDom, "tag": "TR", "class": "gtk_listview_table_head_tr"});

	var docFragment = document.createDocumentFragment();

	for(var i = 0, len = headers.length; i < len; i++){
	    var td = jsgtk.Dom.create({"append": docFragment, "tag": "TD", "width": headers[i].width, "height": this._titleHeight, "class": "gtk_listview_table_head_td"});
	        var div = jsgtk.Dom.create({"append": td, "tag": "div", "width": headers[i].width, "class": "gtk_listview_table_head_td_div"});

                var textSpan = jsgtk.Dom.create({"append": div, "tag": "span", "text": headers[i].title, "style" : "float: left;", "class": "gtk_listview_table_head_td_div_span"});
	            var sortArrow = jsgtk.Dom.create({"append": div, "tag": "div", "style" : "float: left;", "class": "gtk_listview_table_head_td_div_sortarrow arrowdown"});
	            jsgtk.Dom.hideElement(sortArrow);

	            var vbar = jsgtk.Dom.create({"append": div, "tag": "div", "class": "gtk_listview_table_head_td_div_vbar"});
	            vbar.style.left = headers[i].width - this._leftPadding + "px";

	            this._handleColumnResizeEvent(i, vbar);
	            this._handleCellEvent(i, div, sortArrow);

	            if( i === len - 1){
	                jsgtk.Dom.hideElement(vbar);
                    div.removeChild(sortArrow);
	            }
    }
    tr.appendChild(docFragment);
};

/** @private */
gtk.ListView.prototype._getIndex = function(row){
    var rows = this._bodyDom.childNodes;
    for(var i = 0, len = rows.length; i < len; i++){
        if(rows[i] === row) return i;
    }
};

/** @private */
gtk.ListView.prototype._increaseColumnWidth = function(offsetWidth){

    var secondLastColumnMinColWidth = this._model.getColumnsWidth()[this._columnsWidth.length - 2];

    var width = this._columnsWidth[this._columnsWidth.length - 2] - offsetWidth;
    width = width < secondLastColumnMinColWidth ? secondLastColumnMinColWidth : width;

    if(width > secondLastColumnMinColWidth){
        this._columnsWidth[this._columnsWidth.length - 2] = width;
        this._resizeWidth(this._width);
    }else this._resizeWidth(this._width + offsetWidth);
};

/** @private */
gtk.ListView.prototype._decreaseColumnWidth = function(offsetWidth){
    this._columnsWidth[this._columnsWidth.length - 2] -= offsetWidth;
    this._resizeWidth(this._width);
};

/** @private */
gtk.ListView.prototype._isSecondLastColumn = function(index){
    return index === this._columnsWidth.length - 2;
};

/** @private */
gtk.ListView.prototype._updateColumWidth = function(index, offsetWidth){

    this._columnsWidth[index] += offsetWidth;

    if(this._isSecondLastColumn()) {
        this._resizeWidth(this._width + offsetWidth);
        return;
    }

    if(offsetWidth > 0)
        this._increaseColumnWidth(offsetWidth);
    else
        this._decreaseColumnWidth(offsetWidth);
};

/** @private */
gtk.ListView.prototype._getGhostPosition = function(index){
    var left = 0;
    for(var i = 0; i <= index; i++){
        left += this._columnsWidth[i];
    }
    return left;
};

/**
 * Return model of gtk.ListView
 * @return {gtk.ListStor} model
 */
gtk.ListView.prototype.getModel = function(){
	return this._model;
};

/** @private */
gtk.ListView.prototype.resetMinHeight = function(minHeight){
    this._minHeight = minHeight;
    this._resizeHeight(minHeight);
};

/** @private */
gtk.ListView.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.ListView.prototype.setMinColumnsWidth = function(columnsWidth, width){
    this._columnsWidth = [];

    for(var i = 0, len = columnsWidth.length; i < len; i++){
        this._columnsWidth.push(columnsWidth[i]);
    }
    this._minWidth = width;
    this._width = this._minWidth;
};

/** @private */
gtk.ListView.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.ListView.prototype._resizeHeight = function(height){
    this._height = height;
    this._domObj.style.height = this._height + "px";
    this._vGhost.style.height = this._height + "px";

    this._tableDom.style.height = this._height + "px";
    this._bodyHeight = this._height - this._cellHeight;
    this._bodyDom.style.height = this._bodyHeight + "px";
};

/** @private */
gtk.ListView.prototype._resizeHeaderWidth = function(){
    var headerNodes = this._headerDom.firstChild.childNodes;
    for(var i = 0, len = this._columnsWidth.length; i < len; i++){
        headerNodes[i].width = this._columnsWidth[i];
        headerNodes[i].firstChild.style.width = this._columnsWidth[i] + "px";
        headerNodes[i].firstChild.lastChild.style.left = this._columnsWidth[i] - this._leftPadding + "px";
    }
};

/** @private */
gtk.ListView.prototype._resizeWidth = function(width){
    this._width = width > this._width ? width : this._width;
    this._domObj.style.width = this._width - 2 + "px";
    this._resizeHeaderWidth();
};

/** @private */
gtk.ListView.prototype.setSelectedRow = function(data){
    this._selectedRow = data;
};

/**
 * Return the selected row's value
 * @return {object}
 */
gtk.ListView.prototype.getSelectedRow = function(data){
    return this._selectedRow;
};

/** @private */
gtk.ListView.prototype.setParentWidth = function(width){
    this._parentWidth = width;
};

/** @private */
gtk.ListView.prototype.resizeHeightComposite = function(height){
    if(height >= this._minHeight){
        this._resizeHeight(height - 2);
        this.emit("_renderemptyrows", height - this._cellHeight, this._cellHeight);
    }
};

/** @private */
gtk.ListView.prototype.resizeWidthComposite = function(width){
    if(width >= this._minWidth){

        var offsetWidth = Math.ceil((width - this._width) / (this._columnsWidth.length - 1));

        for(var i = this._columnsWidth.length - 1; i--;) this._columnsWidth[i] += offsetWidth;

        this._width = width;
        this._resizeWidth(width);
    }
};

/** @private */
gtk.ListView.prototype._handleRowEvent = function(row, path){
    var self = this;
    var jsgtkEvent = jsgtk.Event;
    jsgtkEvent.addDomListener(row, "click", function(e){

        self.activateArrowKey();
        self._emitSelectedRowSignalToController(path);
        self.hideTooltip();

        self._rowIndex = self._getIndex(this);
        self._selectedRowIndex = self._rowIndex;

        if(!self._isEmptyRowClicked(self._rowIndex)){
            self.emit("rowclicked", self.getSelectedRow());
            self._hilightCell(self._rowIndex);
        }
        jsgtkEvent.stopBubble(e);
    });
   jsgtkEvent.addDomListener(row, "mouseover", function(e){
        var rowOver = e.target.parentNode.parentNode;
        self.emit("rowover",e,rowOver);
        jsgtkEvent.stopBubble(e);
    });
    jsgtkEvent.addDomListener(row, "mouseout", function(e){
        self.emit("rowout");
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(row, "dblclick", function(e){
        self._selectedRowIndex = self._rowIndex;
        self.hideTooltip();

        if(!self._isEmptyRowClicked(self._rowIndex)){
            self.emit("rowdoubleclicked", self.getSelectedRow());
        }
        jsgtkEvent.stopBubble(e);
    });
};

/** @private */
gtk.ListView.prototype._isEmptyRowClicked = function(selectedIndex){
    var isEmptyRow = false;
    if(this._model !== null){
        var LastChildIndex = this._model.children.length - 1;
        if(LastChildIndex < selectedIndex){
            isEmptyRow = true;
        }
    }
    return isEmptyRow;
};

/** @private */
gtk.ListView.prototype._showVGhost = function(index){
    var left = this._getGhostPosition(index);
    this._vGhost.style.left = left - 4 + "px";
    jsgtk.Dom.showElement(this._vGhost);
};

/** @private */
gtk.ListView.prototype._handleColumnResizeEvent = function(i, dom){
    var j = i;
    var self = this;
    var jsgtkDom =  jsgtk.Dom;

    jsgtk.Event.addDomListener(dom, "mouseover", function(){

        self._showVGhost(j);

        var minX = self._getGhostPosition(j - 1) + self._model.getColumnsWidth()[j];

       jsgtk.Util.DomDraggableAdapter.hDragInit(self._vGhost, minX, null);

        var currentX;

        self._vGhost.onDragStart = function(x){
            self._vGhost.className = "gtk_listview_vghost_visible";
            currentX = x;

            self._headerDomGhost.style.cursor = "col-resize";
            jsgtkDom.showElement(self._headerDomGhost);
        };

        self._vGhost.onDragEnd = function(x, y){
            self._vGhost.className = "gtk_listview_vghost";
            var offsetWidth = x - currentX;

            self._updateColumWidth(j, offsetWidth);
            currentX = x;

            jsgtkDom.hideElement(self._vGhost);

            jsgtkDom.hideElement(self._headerDomGhost);
	    };
    });
};

/** @private */
gtk.ListView.prototype._handleCellEvent = function(i, cell, sortArrow){
    var index = i;
    var jsgtkEvent = jsgtk.Event;
    var jsgtkDom =  jsgtk.Dom;

    jsgtkEvent.addDomListener(cell, "mouseover", function(e){
       jsgtkDom.showElement(sortArrow);
        jsgtkEvent.stopBubble(e);
    });

    jsgtkEvent.addDomListener(cell, "mouseout", function(e){
        jsgtkDom.hideElement(sortArrow);
        jsgtkEvent.stopBubble(e);
    });

    var isDecending = true;
    jsgtkEvent.addDomListener(cell, "click", function(e){
        if(isDecending){
            sortArrow.id = "gtk_listview_table_head_td_div_sortarrow_arrowup";
            isDecending = false;
        }else{
            sortArrow.id = "gtk_listview_table_head_td_div_sortarrow_arrowdown";
            isDecending = true;
        }
    });
};

/** @private */
gtk.ListView.prototype._keyControllEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

	jsgtkEvent.addDomListener(this._hiddenTextBox, "keyup", function(e){
	    var topIndex = self._model.children.length;
        if(e.keyCode === 13){
	        var path = self._bodyDom.childNodes[self._rowIndex].id;
	        self._emitSelectedRowSignalToController(path);
	        self._hilightCell(self._rowIndex);
	        self._selectedRowIndex = self._rowIndex;

	        if(!self._isEmptyRowClicked(self._rowIndex)){
                self.emit("rowclicked", self.getSelectedRow());
            }
	    }

	    if(e.keyCode === 40){
            if(self._rowIndex === topIndex - 1) self._rowIndex = -1;
            self._rowIndex++;

            if(!self._isEmptyRowClicked(self._rowIndex)){
                self._hilightCellByArrowKey(self._rowIndex);
            }
        }

        if(e.keyCode === 38){
            if(self._rowIndex <= 0) self._rowIndex = topIndex;
            self._rowIndex--;

            if(!self._isEmptyRowClicked(self._rowIndex)){
                self._hilightCellByArrowKey(self._rowIndex);
            }
        }
        self._scrollTableBodyByIndex(self._rowIndex);

        jsgtkEvent.stopBubble(e);
    });
};

gobject.signalNew('refreshmodel', gtk.ListView, [], null, null);
gobject.signalNew('_rowselectedtocontroller', gtk.ListView, [], null, null);
gobject.signalNew('_renderemptyrows', gtk.ListView, [], null, null);
gobject.signalNew('searchcontenttocontroller', gtk.ListView, [], null, null);
gobject.signalNew('filterrowstocontroller', gtk.ListView, [], null, null);
gobject.signalNew('rowselected', gtk.ListView, [], null, null);
gobject.signalNew('rowclicked', gtk.ListView, [], null, null);
gobject.signalNew('rowover', gtk.ListView, [], null, null);
gobject.signalNew('rowout', gtk.ListView, [], null, null);
gobject.signalNew('rowdoubleclicked', gtk.ListView, [], null, null);
/**
 * @constructor
 * @base gtk.Widget
 */
gtk.MultiPanelContainer = function() {
	gtk.Widget.apply(this);

	this._width           = this._minWidth  = 20;
    this._height          = this._minHeight = 20;
    this._loadingImgWidth = 150;
    this._activeChild     = null;
    this._children        = [];

	/** @private */
	this._render = function(){
		var jsgtkDom = jsgtk.Dom;
		var domObj = this._domObj;
		domObj.className   += " gtk_MultiPanelContainer";
		    this._grayText = jsgtk.Dom.create({"append": domObj, "tag": "table", "class": "gtk_MultiPanelContainer_table"});
		        var tr = jsgtk.Dom.create({"append": this._grayText, "tag": "tr"});
                    var td = jsgtk.Dom.create({"append": tr, "tag": "td", "class": "gtk_MultiPanelContainer_tr_gray_text","text": "Text"});

            this._loadingDom = jsgtk.Dom.create({"append": domObj, "tag": "table", "class": "gtk_MultiPanelContainer_loading"});
		        var tr = jsgtk.Dom.create({"append": this._loadingDom, "tag": "tr"});
                    var td = jsgtk.Dom.create({"append": tr, "tag": "td", "class": "gtk_MultiPanelContainer_tr_loading"});
                        this._loadingImg = jsgtk.Dom.create({"append": td, "tag": "div", "class": "gtk_MultiPanelContainer_tr_loaing_img"});
            jsgtk.Dom.hideElement(this._loadingDom);
	};

    this._render();
};

gtk.MultiPanelContainer.prototype = new gtk.Widget();
gtk.MultiPanelContainer.prototype.constructor = gtk.MultiPanelContainer;

/**
 * show loading bar
 */
gtk.MultiPanelContainer.prototype.showLoading = function(){
    jsgtk.Dom.showElement(this._loadingDom);
    jsgtk.Dom.hideElement(this._grayText);
};

/**
 * hide loading bar
 */
gtk.MultiPanelContainer.prototype.hideLoading = function(){
    jsgtk.Dom.showElement(this._grayText);
    jsgtk.Dom.hideElement(this._loadingDom);
};

/** @private */
gtk.MultiPanelContainer.prototype._resetMinWidthMinHeight = function(childWidget){
    var childWidth = childWidget.getMinWidthComposite();
    this._minWidth = childWidth > this._minWidth ? childWidth : this._minWidth;

    var childHeight = childWidget.getMinHeightComposite();
    this._minHeight = childHeight > this._minHeight ? childHeight : this._minHeight;
};

/**
 * Set text inside gtk.MultiPanelContainer
 * @param {string} text
 */
gtk.MultiPanelContainer.prototype.setText = function(text){
    this._grayText.rows[0].firstChild.innerHTML = text;
};

/**
 * Return gtk.Widget whic is active inside gtk.MultiPanelContainer
 * @param {gtk.Widget || null} gtk.Widget
 */
gtk.MultiPanelContainer.prototype.getActiveChild = function(){
    var isActive = false;
    for(var i = 2, len = this._domObj.childNodes.length; i < len; i++){
        var node = this._domObj.childNodes[i].firstChild;
        if(node.style.display !== "none") isActive = true;
    }
    if(isActive) return this._activeChild;
    else return null;

}

/**
 * Remove gtk.Widget (child) from gtk.MultiPanelContainer
 * @param {gtk.Widget} childWiget
 */
gtk.MultiPanelContainer.prototype.removeChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        if(child["childWidget"] === childWidget) {
            child["holder"].parentNode.removeChild(child["holder"]);
            this._children.splice(i, 1);
            return;
        }
    }
};

/**
 * Append gtk.Widget to gtk.MultiPanelContainer
 * @param {gtk.Widget} childWidget
 */
gtk.MultiPanelContainer.prototype.appendChild = function(childWidget){
    this._resetMinWidthMinHeight(childWidget);

    var holder = this._createChildHolder();
    holder.appendChild(childWidget.getNode());

    this.setActiveChild(childWidget);

    var childObject = {
        "holder": holder,
        "childWidget": childWidget
    };

    this._children.push(childObject);
    this._domObj.appendChild(holder);

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);

    if(this._children.length > 1) this._minimizePreviousChild();

    var self = this;
    childWidget.connect("restored", function(panel){
        for(var i = self._children.length; i--;){
            var child = self._children[i];
            if(child["childWidget"] ===  panel){
                self.setActiveChild(panel);
                child["holder"].style.zIndex = 2;
                jsgtk.Dom.showElement(child["holder"]);
            }else{
                child["childWidget"].minimize();
                child["holder"].style.zIndex = 1;
            }
        }
    });

    childWidget.connect("closed", function(panel){
        self.removeChild(panel)
    });
};

/**
 * Set the gtk.Widget (child) active
 * @param {gtk.Widget} childWidget
 */
gtk.MultiPanelContainer.prototype.setActiveChild = function(childWidget){
    this._activeChild = childWidget;
};

/** @private */
gtk.MultiPanelContainer.prototype._minimizePreviousChild = function(){
    this._children[this._children.length - 2]["childWidget"].minimize();
    jsgtk.Dom.hideElement(this._children[this._children.length - 2]["holder"]);
};

/** @private */
gtk.MultiPanelContainer.prototype._getIndexOfChild = function(childWidget){
    for(var i = 0, len = this._children.length; i < len; i++){
        var child = this._children[i];
        if(child["childWidget"] === childWidget) return i;
    }
    return null;
};

/**
 * Replace existing childWidget with new childWidget
 * @param {gtk.Widget} newChildWidget
 * @param {gtk.Widget} oldChildWidget
 */
gtk.MultiPanelContainer.prototype.replaceChild = function(newChildWidget, oldChildWidget){
    var index = this._getIndexOfChild(oldChildWidget);
    if(!index) return;

    this._domObj.removeChild(this._children[index]["holder"]);

    this._children.splice(index, 1);
    this.appendChild(newChildWidget);
};

/**
 * Return children widget
 * @return {array} children widget
 */
gtk.MultiPanelContainer.prototype.getChildren = function(){
    var children = []
    for(var i = 0, len = this._children.length; i < len; i++){
        children.push(this._children[i]["childWidget"]);
    }
    return children;
};

/** @private */
gtk.MultiPanelContainer.prototype._positionLoadingImg = function(width){
    this._loadingImg.style.left = width / 2 - this._loadingImgWidth / 2 + "px";
}

/** @private */
gtk.MultiPanelContainer.prototype._resizeChildrenWidth = function(width){
    for(var i = this._children.length; i--;){
        var child = this._children[i];
        child["childWidget"].resizeWidthComposite(width);
        child["holder"].style.width = width + "px";
    }
};

/** @private */
gtk.MultiPanelContainer.prototype._resizeChildrenHeight = function(height){
    for(var i = this._children.length; i--;){
        var child = this._children[i];
        child["childWidget"].resizeHeightComposite(height);
        child["holder"].style.height = height + "px";
    }
};

/** @private */
gtk.MultiPanelContainer.prototype._createChildHolder = function(){
    var holder = jsgtk.Dom.create({"tag": "div", "class": "gtk_multiPanelContainer_widget_holder"});
    return  holder;
};

/** @private */
gtk.MultiPanelContainer.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this._resizeWidth(minWidth);
    this._positionLoadingImg(minWidth);
    if(this._children.length > 0) this._resizeChildrenWidth(minWidth);
};

/** @private */
gtk.MultiPanelContainer.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this._resizeHeight(minHeight);
    if(this._children.length > 0) this._resizeChildrenHeight(minHeight);
};
/**
 * @constructor
 * @base gtk.Widget
 */
gtk.NoteBook = function() {
	gtk.Widget.apply(this);

	/** 'tabchanged' signal */
	this.NOTEBOOK_TABCHANGED = "tabchanged";

	this._minWidth        = 200;
	this._minHeight       = 200;
	this._width           = this._minWidth;
	this._height          = this._minHeight;
	this._tabHeight       = 32;
    this._hBarHeight      = 3;
    this._loadingImgWidth = 16;
    this._extraWidthRight = 10;
    this._border          = 2;
    this._vspaceWidth     = 2;
    this._numberTabs      = 0;
	this._tabs = new gtk.Tabs(this._width, this._tabHeight, this._vspaceWidth);

    var contentsHeight = this._height - this._tabHeight - this._hBarHeight;
	this._contents = new gtk.Contents(this._width, contentsHeight);

	this._render = function(){
		this._domObj.className += " gtk_notebook";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		    var hBar = jsgtk.Dom.create({'tag' : 'div', 'className': 'gtk_hbar'});
		this._appendChild(this._contents, this._tabs, hBar);
	};
	this._initEvent();
	this._render();
};

gtk.NoteBook.prototype = new gtk.Widget();
gtk.NoteBook.prototype.constructor = gtk.NoteBook;

/** @private */
gtk.NoteBook.prototype._initEvent = function(){
    var self = this;
    this._tabs.connect('active', function(id){
		var content = self._contents.activateContent(id);
		self.emit("tabchanged", self.getActiveTab());
	});
    jsgtk.Event.addDomListener(this._domObj, 'keyup', function(e){
	    if(e.keyCode == gtk.RIGHT_ARROW_KEY){
		    var activeTabIndex = self._tabs.getIndex(self._tabs._activeTab);
            if(activeTabIndex < self.getNumberTabs()-1)self.setActiveTab(activeTabIndex+1);
	    }
	    else if(e.keyCode == gtk.LEFT_ARROW_KEY){
	       var activeTabIndex = self._tabs.getIndex(self._tabs._activeTab);
            if(activeTabIndex > 0)self.setActiveTab(activeTabIndex -1);
	    }
	});
	jsgtk.Event.addDomListener(this._domObj, 'focus', function(e){
	   self._tabs._setFocus(self._tabs._activeTab);
	});
	jsgtk.Event.addDomListener(this._domObj, 'blur', function(e){
	   self._unFocus(self._tabs._activeTab);
	});
};

/**
 * Turn border on || off
 * @param {boolean}
 */
gtk.NoteBook.prototype.setBorderVisible = function(visible){
    if(visible) this._contents.showBorder();
    else this._contents.hideBorder();
};


/**
 * Update the content of selected tab
 * @param {object} pageObj {"content": gtk.Widget, "padding": 5}
 */
gtk.NoteBook.prototype.updatePage = function(pageObj){
	var index = this._tabs.getIndex(pageObj['tab']);
	this._contents.updateContent(pageObj["content"], index);

	this._contents.activate(index);

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * Appends a page to the notebook
 * @param {object} tabObj {"tab": label1, "padding": 55}
 * @param {object} contentObj {"content": content2, "padding": 5}
 */
gtk.NoteBook.prototype.appendPage = function(tabObj, contentObj){
	contentObj["content"] = contentObj["content"] || new gtk.Blank();

    var id = new UUID();
	tabObj["id"] = id;
    contentObj["id"] = id;

	this._contents.addChild(contentObj);
	this._tabs.addTab(tabObj);

	this._tabs.activateDefault();
	this._contents.activateDefault();

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);

    return {
        "tab": tabObj["tab"],
        "content": contentObj["content"],
		"isBlanked": contentObj.type === gtk.BLANK_WIDGET ? true : false
    };
};

/**
 * Insert a page to the notebook at index
 * @param {object} tabObj {"tab": label1, "padding": 55}
 * @param {object} contentObj {"content": content2, "padding": 5}
 * @param {integer} index
 */
gtk.NoteBook.prototype.insertAt = function(tabObj, contentObj,index){

	contentObj["content"] = contentObj["content"] || new gtk.Blank();

    var id = new UUID();
	tabObj["id"] = id;
    contentObj["id"] = id;

	this._contents.insertAt(contentObj, index);
	this._tabs.insertAt(tabObj, index);

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);

   return {
        "tab": tabObj["tab"],
        "content": contentObj["content"],
		"isBlanked": contentObj.type === gtk.BLANK_WIDGET ? true : false
    };
};

/**
 * Remove tabe by index
 * @param {intger} index of tab to be removed
 */
gtk.NoteBook.prototype.removeTab = function(index){
    var len = this._tabs.getChildren().length;
    if(len == 1) jsgtk.Util.debug("The tab can not be removed");
    if(index < 0 || index >= len) jsgtk.Util.debug("The index out of range");

    this._tabs.removeTab(index);
    this._contents.removeChild(index);

    if(index === 0) this._activatePreviousTab(index + 1);
    if(this._tabs.getActiveTab() === this._tabs.getChildren()[index]) this._activatePreviousTab(index);
    else this._activateActiveTab();
};

/** @private */
gtk.NoteBook.prototype.setTabindex = function(index){
    this._tabs.getNode().tabIndex = index;
};

/**
 * Return number of tabs
 * @return {intger}
 */
gtk.NoteBook.prototype.getNumberTabs = function(){
    return this._tabs._tabs.length;
};

/** @private */
gtk.NoteBook.prototype.showLoading = function(activeTab){
    this._tabs.showLoading(activeTab["tab"]);
};

/** @private */
gtk.NoteBook.prototype.hideLoading = function(activeTab){
    this._tabs.hideLoading(activeTab["tab"]);
};

/** @private */
gtk.NoteBook.prototype._activateActiveTab = function(){
    this._tabs.activateByObject(this._tabs.getActiveTab());
    this._contents.activateByObject(this._contents.getActiveContent());
};

/** @private */
gtk.NoteBook.prototype._activatePreviousTab = function(index){
    this._tabs.activate(index - 1);
    this._contents.activate(index -1);
};

/**
 * activate the tab by tab
 * @param {gtk.Widget} tab
 */
gtk.NoteBook.prototype.setActiveTabByTab = function(tab){
    var index = this._tabs.getIndex(tab);
    this.setActiveTab(index);
};

/**
 * activate the tab by index
 * @param {integer} index
 */
gtk.NoteBook.prototype.setActiveTab = function(index){
    this._tabs.activate(index);
    this._contents.activate(index);
    this.emit("tabchanged", this.getActiveTab());
};

/** @private */
gtk.NoteBook.prototype.setFocusTab = function(index){
    this._tabs.setFocus(index);
};

/**
 * Return all tabs objects
 * @return {array} array of tabs of object
 */
gtk.NoteBook.prototype.getTabs = function(){
    var tabs = [];
    for(var i = 0, len = this._tabs.getChildren().length; i < len; i++){
        tabs.push({
            "tab": this._tabs.getChildren()[i]["tab"],
            "content": this._contents.getChildren()[i]["content"],
			"isBlanked": this._contents.getChildren()[i]["content"].type === gtk.BLANK_WIDGET ? true : false
        });
    }
    return tabs;
};

/**
 * Return active tab
 */
gtk.NoteBook.prototype.getActiveTab = function(){
    return {
        "tab": this._tabs.getActiveTab(),
        "content": this._contents.getActiveContent(),
		"isBlanked": this._contents.getActiveContent().type === gtk.BLANK_WIDGET ? true : false
    };
};

/** @private */
gtk.NoteBook.prototype._appendChild = function(contents, tabs, hBar){
	this._domObj.appendChild(tabs._domObj);
	this._domObj.appendChild(hBar);
	this._domObj.appendChild(contents._domObj);
};


/** @private */
gtk.NoteBook.prototype._resizeHeight = function(height){
    this._domObj.style.height = height - this._border + 'px';
	this._height = height;
};

/** @private */
gtk.NoteBook.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this._border + 'px';
	this._width = width;
};

gtk.NoteBook.prototype._getLoadingImgWidths = function(){
    var children = this._tabs.getChildren();
    var width = 0;
    for(var i = 0, len = children.length; i < len; i++){
        pageObj = children[i];
        width += pageObj["padding"] > this._loadingImgWidth ? pageObj["padding"] : this._loadingImgWidth;
    }
    return width;
};

/** @private */
gtk.NoteBook.prototype.getMinWidthComposite = function(){
	var minWidth = this._minWidth;

	var tabsWidth = this._tabs.getMinWidthComposite();
	var contentsWidth = this._contents.getMinWidthComposite();

	if(tabsWidth > minWidth) minWidth = tabsWidth;
	if(contentsWidth > minWidth) minWidth = contentsWidth;
    this._getLoadingImgWidths();
	return minWidth + this._extraWidthRight + this._getLoadingImgWidths();
};

/** @private */
gtk.NoteBook.prototype.getMinHeightComposite = function(){
	var minHeight = this._minHeight - this._tabs.getHeight();
	var contentsHeight = this._contents.getMinHeightComposite();

	if(contentsHeight > minHeight) minHeight = contentsHeight;

	return minHeight + this._tabs.getHeight();
};

/** @private */
gtk.NoteBook.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._contents.resizeHeightComposite(minHeight - this._tabs.getHeight() - this._border);

    this._resizeHeight(minHeight);
};

/** @private */
gtk.NoteBook.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	this._tabs.resizeWidthComposite(minWidth - 1);
	this._contents.resizeWidthComposite(minWidth - this._border);

    this._resizeWidth(minWidth);
};

gobject.signalNew('tabchanged', gtk.NoteBook, [], null, null);
/**
 * Scroller is a class to create a scroller with horizontal and vertical scrollbars
 * @constructor
 * @base gtk.Widget
 */
gtk.Scroller = function() {
	gtk.Widget.apply(this);

	/** 'scrolled' signal */
	this.SCROLLER_SCROLLED = "scrolled";

	this._width          = this._minWidth  = 100;
	this._height         = this._minHeight = 50;
	this._childResizable = false;
	this._isScrolledX    = true;
	this._isScrolledY    = true;
	this._vScrolledBar   = null;
	this._child          = null;
	this._scrollYDefault = 0;
	this._attribute		= null;
	this._padding	= 0;

	this._render = function(){
        this._domObj.className       += " gtk_scroller_base";
        this._domObj.style.width      = this._width + "px";
        this._domObj.style.height     = this._height + "px";
	};
	this._render();
	this._initEvent();
};

gtk.Scroller.prototype = new gtk.Widget();
gtk.Scroller.prototype.constructor = gtk.Scroller;

/**
 * Append child to scroller
 * @param {gtk.Widget} widget
 * @param {boolean} resizable true child expand false vice versa
 * @param {integer} padding
 */
gtk.Scroller.prototype.appendChild = function(widget, resizable, padding){
    this._childResizable = resizable === undefined || resizable === null ? false : resizable;

   this._padding = (typeof padding !== "undefined") ? padding : 0;

	this._child = widget;
	this._domObj.appendChild(widget._domObj);

	if(typeof padding !== "undefined") this.setAttribute({padding: padding + "px"});

    this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);

	this._child.setParent(this);
};

/** @private */
gtk.Scroller.prototype.add = function(widget, resizable){
	this.appendChild(widget, resizable);
	console.log("gtk.Scroller.add(), will be depricated in next release, please use gtk.Scroller.appendChild() instead");
};

/**
 * Scroll to top
 */
gtk.Scroller.prototype.scrollToTop = function(){
    this.scrollTo(null, 0);
};

/**
 * Scroll to bottom
 */
gtk.Scroller.prototype.scrollToBottom = function(){
	var childHeight = this._child.getMinHeightComposite();

    this.scrollTo(null, this._height > childHeight ? this._height : childHeight);
};

/**
 * Scroll to left
 */
gtk.Scroller.prototype.scrollToLeft = function(){
    this.scrollTo(0);
};

/**
 * Scroll to right
 */
gtk.Scroller.prototype.scrollToRight = function(){
    this.scrollTo(this._width);
};

/**
 * Scroll to right x pixel, Scroll to bottom y pixel
 * @param {integer} x
 * @param {integer} y
 */
gtk.Scroller.prototype.scrollTo = function(x , y){
    this._domObj.scrollTop = y;
    this._domObj.scrollLeft = x;
};

/**
 * Return child widget gtk.Widget
 * @return {gtk.Widget}
 */
gtk.Scroller.prototype.getChildren = function(){
	return this._child;
};

/** @private */
gtk.Scroller.prototype._initEvent = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this._domObj, "scroll", function(e){
        var scrollY = _self._domObj.scrollTop;
        var scrollHeight = _self._domObj.scrollHeight;
        var minScrolledY = scrollHeight-(scrollHeight - _self._domObj.scrollTop);
        if(scrollY > 0 && minScrolledY >= 15){
            _self.emit("scrolled");
        }
    });
};

/**
 * Refresh gtk.Scroller and its child widget
 */
gtk.Scroller.prototype.refresh = function(){
    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);
};

/**
 * Show loading bar
 */
gtk.Scroller.prototype.showLoading = function(){
    if(this._isScrolledY) this._domObj.style.overflowY = "hidden";
    if(this._isScrolledX) this._domObj.style.overflowX = "hidden";

    if(!this._loadingImgContainer){
		this._loadingImgContainer = this._constructLoading();
        this._appendLoading();
    }else this._loadingImgContainer.style.display = "table";
};


/**
 * Hide loading bar
 */
gtk.Scroller.prototype.hideLoading = function(){
    if(this._isScrolledY) this._domObj.style.overflowY = "auto";
    if(this._isScrolledX) this._domObj.style.overflowX = "auto";

    jsgtk.Dom.hideElement(this._loadingImgContainer);
};

/**
 * Remove event from scroller
 */
gtk.Scroller.prototype.removeScrolledEvent = function(){
    jsgtk.Event.removeListener(this._onScroll);
};

/** @private */
gtk.Scroller.prototype.getScrolledDom = function(){
    return this._domObj;
};

/**
 * Set style to the scroller
 * @param {object} attribute {'height': ..px, 'border': '...', 'fontColor': ..}
 */
gtk.Scroller.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();

	this.resizeWidthComposite(this._minWidth);
    this.resizeHeightComposite(this._minHeight);
};

/** @private */
gtk.Scroller.prototype._applyAttribute = function(){
	for(var el in this._attribute){
		if(el == "padding") this._padding = parseInt(this._attribute[el]);
		this._domObj.style[el] = this._attribute[el];
	}

};

/**
 * Disable horizontal scrollbar
 */
gtk.Scroller.prototype.disableHScrolled = function(){
    this._domObj.style.overflowX = "hidden";
    this._isScrolledX = false;
};

/**
 * Disable vertical scrollbar
 */
gtk.Scroller.prototype.disableVScrolled = function(){
    this._domObj.style.overflowY = "hidden";
    this._isScrolledY = false;
};

/**
 * Enable horizontal scrollbar
 */
gtk.Scroller.prototype.enableHScrolled = function(){
    this._domObj.style.overflowX = "auto";
    this._isScrolledX = true;
};

/**
 * Enable vertical scrollbar
 */
gtk.Scroller.prototype.enableVScrolled = function(){
    this._domObj.style.overflowY = "auto";
    this._isScrolledY = true;
    this._width = this._width - 200;
    this._resize(this._width, this._height);
};

/** @private */
gtk.Scroller.prototype.getScrollHeight = function(){
    return this._child.getHeight();
};

/** @private */
gtk.Scroller.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Scroller.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width            = width + "px";
};

/** @private */
gtk.Scroller.prototype._resizeHeight = function(height){
    this._height = height;
	this._domObj.style.height            = height + "px";
	if(!this._isScrolledY && this._child !== null){
	    this._child._domObj.style.height = this._height - 2 + "px";
	}
};

/** @private */
gtk.Scroller.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/** @private */
gtk.Scroller.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/** @private */
gtk.Scroller.prototype.resizeWidthComposite = function(width){
	var _width = width - this._padding*2;

	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = _width > minWidthComposite ? _width : minWidthComposite;

	this._resizeWidth(minWidth);

	if(this._childResizable){
	    var childMinWidth = this._child.getMinWidthComposite();
	    if(minWidth > childMinWidth) this._child.resizeWidthComposite(minWidth);
	    else this._child.resizeWidthComposite(childMinWidth);
	}
	if(this._child instanceof gtk.ListView) this._child.setParentWidth(minWidth);
};

/** @private */
gtk.Scroller.prototype.resizeHeightComposite = function(height){
	var _height = height - this._padding*2;

	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = _height > minHeightComposite ? _height : minHeightComposite;

	this._resizeHeight(minHeight);

    if(this._childResizable){
	    var childMinHeight = this._child.getMinHeightComposite();
	    if(minHeight > childMinHeight) this._child.resizeHeightComposite(minHeight);
	    else this._child.resizeHeightComposite(childMinHeight);
	}
};

gobject.signalNew("scrolled", gtk.Scroller, null, null, []);
/**
 * VScroller is a class to create a vertical scroller. The vertical scroller has only vertical scrollbar
 * @base gtk.Scroller
 * @see gtk.Scroller
 * @constructor
 */
gtk.VScroller = function() {
	gtk.Scroller.apply(this);
	this._scollerWidth = 18;
	this.disableHScrolled();
};

gtk.VScroller.prototype = new gtk.Scroller();
gtk.VScroller.prototype.constructor = gtk.VScroller;

/**
 * A method to get width of the scroll window
 * @return {Integer} The minimum composite width of the scroll
 * @type Integer
 */
gtk.VScroller.prototype.getMinWidthComposite = function(){
	var childMinWidth = this._child.getMinWidthComposite();
	return this._minWidth > childMinWidth ? this._minWidth : childMinWidth;
};

/**
 * A method to resize width of the scroll window
 * @param {Integer} width The composite width value in pixel to be set
 */
gtk.VScroller.prototype.resizeWidthComposite = function(width){
	var _width = width - this._padding*2;

	var minWidth = this.getMinWidthComposite();
	minWidth = _width > minWidth ? _width : minWidth;

	this._resizeWidth(minWidth);

    this._child.resizeWidthComposite(minWidth - this._scollerWidth);

	if(this._child instanceof gtk.ListView) this._child.setParentWidth(minWidth);
};
/**
 * Factory of all gtk.SmartEntry
 * @param {string} type of gtk.SmartEntry : gtk.SMART_ENTRY_DROPDOWN_FIXED_MODEL, gtk.SMART_ENTRY_DROPDOWN_DYNAMIC_MODEL
 * @constructor
 */
gtk.SmartEntryFactory = function(type){
	var smartEntry;
	switch (type){
		case gtk.SMART_ENTRY_DROPDOWN_FIXED_MODEL:
			smartEntry = new gtk.SmartEntryDropDownFacade(new gtk.SmartEntryDropDownFixedModel());
			break;
		case gtk.SMART_ENTRY_DROPDOWN_DYNAMIC_MODEL:
			smartEntry = new gtk.SmartEntryDropDownFacade(new gtk.SmartEntryDropDownDynamicModel());
			break;
	}
	return smartEntry;
};

/**
 * SmartEntryDropDownFacade is an assistant class to create Smart Entry
 * @param {string} smartEntryDropDown The object of smart entry dropdown
 * @constructor
 * @base gtk.Object
 */
gtk.SmartEntryDropDownFacade = function(smartEntryDropDown){
    gtk.Object.apply(this);

    var vscroller = new gtk.VScroller();
    vscroller.getNode().style.border = "1px solid #D5D5D5";
    vscroller.appendChild(smartEntryDropDown, true);

    vscroller.setModel       = function(model){smartEntryDropDown.setModel(model);};
    vscroller.showError      = function(isOnError){smartEntryDropDown.showError(isOnError);};
    vscroller.getValues      = function(){return smartEntryDropDown.getValues();}
    vscroller.getItems       = function(){return smartEntryDropDown.getItems()};
    vscroller.createFixedTag = function(obj){smartEntryDropDown.createFixedTag(obj);};
    vscroller.createTag      = function(obj){smartEntryDropDown.createTag(obj);};
    vscroller.setLabel       = function(text){smartEntryDropDown.setLabel(text);};
    vscroller.setFocus       = function(){smartEntryDropDown.setFocus();};

    smartEntryDropDown.connect("smartentrychanged", function(value){
        vscroller.emit("smartentrychanged", value);
    })

    return vscroller;
};

gobject.signalNew("smartentrychanged", gtk.Scroller, null, null, []);
/**
 * A widget with an entry which is able to create many tags
 * @constructor
 * @base gtk.Widget
 */
gtk.SmartEntryDropDown = function(){
    gtk.Widget.apply(this);
    /** 'smartentrychanged' signal */
	this.SMARTENTRYDROPDOWN_SMARTENTRYCHANGED = "smartentrychanged";

    this._minWidth 		= this._width 	= 150;
    this._minHeight 		= this._height = 10;
    this._entryWidth 	= 100;
    this._textBoxError	= false;
    this._ismouseMoved  = false;
    this._isParentSignalHandled = false;

	this._render = function(){
	    this._domObj.className += " gtk_smart_entry_drop_down_widget";
	    	this._holder = jsgtk.Dom.create({tag: "div", append: this._domObj, className: "gtk_smart_entry_drop_down_area"});
	    		this._tagHolder = jsgtk.Dom.create({tag: "div", append: this._holder, className: "gtk_smart_entry_drop_down_tag_group"});

			    this._entryHolder = jsgtk.Dom.create({tag: "div", append: this._holder, className: "gtk_smart_entry_drop_down_input_holder"});
				    this._labelEntry = jsgtk.Dom.create({tag: "span", append: this._entryHolder, className: "gtk_smart_entry_drop_down_label"});
				    this._entry = jsgtk.Dom.create({tag: "input", append: this._entryHolder, className: "gtk_smart_entry_drop_down_input"});

		this.__dropDown = jsgtk.Dom.create({tag: "div", "append": document.body, className: "gtk_smart_entry_drop_down_widget_holder"});
			jsgtk.Dom.hideElement(this.__dropDown);
	};

	this._render();
};

gtk.SmartEntryDropDown.prototype = new gtk.Widget();
gtk.SmartEntryDropDown.prototype.constructor = gtk.SmartEntryDropDown;
/**
 * Set mode to gtk.SmartEntryDropDown
 * @param {gtk.TreeStore} mode
 */
gtk.SmartEntryDropDown.prototype.setModel = function(model){
	this._controller.setModel(model);
};
/**
 * Show Error
 * @param {boolean} isOnError
 */
gtk.SmartEntryDropDown.prototype.showError = function(isOnError){
	 this._textBoxError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : this._textBoxError;

    if(this._textBoxError) this._domObj.className += " gtk_smart_entry_onerror";
    else this._domObj.className = "gtk_widget gtk_smart_entry_drop_down_widget";
};
/** @private */
gtk.SmartEntryDropDown.prototype._renderDropDown = function(children, parentNode){
	var rowNodes = this._dropDownRendere._constructRowNodes(children, parentNode);
	this._appendDropDown(rowNodes);
};
/** @private */
gtk.SmartEntryDropDown.prototype._renderTag = function(){
	var tag = this._tagRendere._createTag(null, this._dropDownRendere.getSelectedRowNode());
	this._tagHolder.appendChild(tag);
};
/** @private */
gtk.SmartEntryDropDown.prototype._resetEntry = function(){
	this._entry.focus();
	this._entry.value = "";
	this._entry.style.width = this._entryWidth + 5 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype._clearDropDown = function(){
	this.__dropDown.innerHTML = "";
};
/** @private */
gtk.SmartEntryDropDown.prototype._handleRowNodeSelected = function(){
	var selectedRowNode = this._dropDownRendere.getSelectedRowNode();
	this._entry.value = selectedRowNode.firstChild.nodeValue;
	this._renderTag();
	this.__hideDropDown();
	this._resetEntry();
	this.__showLabel();
};
/** @private */
gtk.SmartEntryDropDown.prototype._handleParentSignal = function(){
    var self = this;
    this._isParentSignalHandled = true;
    var parentWidget = this.getParent();
    parentWidget.connect("scrolled", function(){
        self.__hideDropDown();
    });
};
/** @private */
gtk.SmartEntryDropDown.prototype._resetDropDownPosition = function(){
    var top = jsgtk.Util.pageY(this._domObj);
    var left = jsgtk.Util.pageX(this._domObj);

    var header = parseInt(this._holder.offsetHeight);

    var scrollY = this._getNearestScrollerY();

    this.__dropDown.style.top = top + header + 5 - scrollY + "px";
    this.__dropDown.style.left = left + 1 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype._getNearestScrollerY = function(){
    var scrollY = 0;
    var parentNode = this._domObj.parentNode;
    while(parentNode != null){
        if(/gtk_scroller_base/.test(parentNode.className)){
            scrollY = parentNode.scrollTop;
            this._addEventMouseWheel(parentNode);
            break;
        }
        parentNode = parentNode.parentNode;
    }
    return scrollY;
};
/** @private */
gtk.SmartEntryDropDown.prototype._addEventMouseWheel = function(scrollObj){
    var self = this;
    jsgtk.Event.addDomListener(scrollObj, 'scroll', function(e){
	    self.__hideDropDown();
	}, false);
};
/** @private */
gtk.SmartEntryDropDown.prototype._initEvent = function(){
	var self = this;

	jsgtk.Event.addDomListener(this._domObj, "click", function(e){
		self.setFocus();
		if(!self._isParentSignalHandled) self._handleParentSignal();
		jsgtk.Event.stopBubble(e);
	});
	this.__bindInputEvent();

	jsgtk.Event.addDomListener(this.__dropDown, "click", function(e){
		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._handleRowNodeSelected();

		jsgtk.Event.stopBubble(e);
	});

	jsgtk.Event.addDomListener(this.__dropDown, "mouseover", function(e){

	    if(!self._ismouseMoved) return;

		var node = e.target;
		if(jsgtk.Util.isMatched("gtk_smart_entry_drop_down_element", node.className))
			self._dropDownRendere.hilightRowNode(node);
		self._ismouseMoved = false;
	});

	jsgtk.Event.addDomListener(this.__dropDown, "mousemove", function(e){
        self._ismouseMoved = true;
	});

	jsgtk.Event.addDomListener(document.body, "click", function(e){
		self.__hideEntry();
		self.__hideDropDown();
	});
};
/** @private */
gtk.SmartEntryDropDown.prototype._hilightRow = function(path){
	this._dropDownRendere.hilightRowById(path);
};
/**
 * Set gtk.SmartEntryDropDown focus
 */
gtk.SmartEntryDropDown.prototype.setFocus = function(){
	this.__showEntry();

	this._tagRendere.unSelectTags();
	this._dropDownRendere._deHilightSelectedRow();

	this._entry.focus();
}
/** @private */
gtk.SmartEntryDropDown.prototype.__showEntry = function(){
	this._entryHolder.style.display = "inline-block";
	this._entry.parentNode.style.visibility = "visible";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideEntry = function(){
	this._entry.parentNode.style.visibility = "hidden";
};
/** @private */
gtk.SmartEntryDropDown.prototype._appendDropDown = function(dropDown){
	this.__hideLoading();
	this.__dropDown.appendChild(dropDown);
};
/** @private */
gtk.SmartEntryDropDown.prototype.__showLoading = function(){
	this.__dropDown.className += " gtk_smart_entry_drop_down_loading";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideLoading = function(){
	this.__dropDown.className = "gtk_smart_entry_drop_down_widget_holder";
};
/** @private */
gtk.SmartEntryDropDown.prototype._isCtrlAKeyPressed = function(e){
	return (e.which == gtk.A_KEY || e.which == gtk.CAP_A_KEY) && e.ctrlKey;
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireDownArrowKey = function(){
	this._dropDownRendere.hilightNextRowNode();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireUpArrowKey = function(){
	this._dropDownRendere.hilightPreviousRowNode();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireLeftArrowKey = function(){
	if(!this._tagRendere.isTagExisted()) return;

	var tag = this._tagRendere.isPreviousTagExisted();
	if(!tag) return;

	if(this._tagRendere.isFixedTag(tag)) return;

	this._tagRendere.unSelectTag();
	this._tagRendere.selectTag(tag || this._tagRendere.getSelectedTag());

	this.__hideEntry();
    this.__hideDropDown();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireRightArrowKey = function(){
	if(!this._tagRendere.isTagExisted() || !this._tagRendere.getSelectedTag()) return;

	var nextTag = this._tagRendere.isNextTagExisted();

	if(!nextTag){
		this.__showEntry();
		return;
	}

	this._tagRendere.unSelectTag();
	this._tagRendere.selectTag(nextTag || this._tagRendere.getSelectedTag());
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireEnterKey = function(){
	if(this._dropDownRendere.getSelectedRowNode()){
		this._handleRowNodeSelected();

		this._dropDownRendere._deHilightSelectedRow();
	}
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireBackSpace = function(value){
    var tags = this._tagHolder.childNodes;

	var entry = this._entry;
	if(entry.value == "") this.__showLabel();

    var len = tags.length;
    if(len < 1) return;

    var tag = tags[len-1];

    if(len > 0 && value == "" && tag.childNodes[0].className != "gtk_smart_entry_drop_down_fixed_tag"){

        if(tag.selected){

            tag.parentNode.removeChild(tag);
			this.__showEntry();
        	this._tagRendere.unSelectTags();
        }
        else{
    		this._tagRendere.unSelectTags();
            tag.selected = true;
            tag.className += " gtk_smart_entry_drop_down_tag_selected";
			this.__hideEntry();
            this.__hideDropDown();
        }
    }
    return;
};
/** @private */
gtk.SmartEntryDropDown.prototype.__fireDeleteKey = function(){
	var tags = this._tagHolder.childNodes;
    var len = tags.length;
    var entry = this._entry;

    if(len > 0 && entry.value == ""){
        for(var i = len-1 ; i >= 0 ; i--){
        	var tag = tags[i];
            if(tag.selected){
				this._tagHolder.removeChild(tag);
			}
        }
		this._tagRendere.unSelectTag();
        this.__showEntry();
        entry.focus();
    }
};
/** @private */
gtk.SmartEntryDropDown.prototype._selectAllTags = function(){
	this._tagRendere.selectTags();
	this.__hideEntry();
	this.__hideDropDown();
};
/** @private*/
gtk.SmartEntryDropDown.prototype.__showDropDown = function(){
	jsgtk.Dom.showElement(this.__dropDown);
};
/** @private*/
gtk.SmartEntryDropDown.prototype.__hideDropDown = function(){
	jsgtk.Dom.hideElement(this.__dropDown);
};
/** @private */
gtk.SmartEntryDropDown.prototype.__hideLabel = function(){
	this._labelEntry.style.visibility = "hidden";
};
/** @private */
gtk.SmartEntryDropDown.prototype.__showLabel = function(){
	this._labelEntry.style.visibility = "";
};
/** @private */
gtk.SmartEntryDropDown.prototype.getValues = function(){
	var children = this._holder.childNodes[0].childNodes,
	len = children.length;
	var values = [];
	for(var i = 0; i < len; i++){
		values.push(children[i].childNodes[0].innerHTML);
	}
	return values;
};
/** @private */
gtk.SmartEntryDropDown.prototype.getItems = function(){
	var children = this._holder.childNodes[0].childNodes,
	len = children.length;
	var values = [];
	for(var i = 0; i < len; i++){
		values.push(children[i]["valueObject"]);
	}
	return values;
};
/**
 * Method to create a smartEntry fixed tag
 * @param {String} fixedTag the value of fixed tag to be created
 */
gtk.SmartEntryDropDown.prototype.createFixedTag = function(obj){
    var tagHolder = jsgtk.Dom.create({"tag": "div", "append": this._tagHolder, "className": "gtk_smart_entry_drop_down_tag_holder"});
    var tag  = jsgtk.Dom.create({"tag": "span", "append": tagHolder , "text": obj.value, "className": "gtk_smart_entry_drop_down_fixed_tag"});
	tagHolder.object = obj;
};
/**
 * Create tag in smartEntry to a specific order
 * @param {integer} index The index of tag to be created
 * @param {String} value The value of tag to be created
 */
gtk.SmartEntryDropDown.prototype.createTag = function(obj){
	var tag = this._tagRendere._createTag(obj["display"]);
	this._tagHolder.appendChild(tag);
    tag.valueObject = obj;
};
/**
 * Set text to be displayed inside entry
 * @param {string} label
 */
gtk.SmartEntryDropDown.prototype.setLabel = function(label){
	this._labelEntry.innerHTML = "";
	this._labelEntry.appendChild(document.createTextNode(label));
	this.__updateEntryWidth();
};
/** @private */
gtk.SmartEntryDropDown.prototype.__updateEntryWidth = function(){
	var tempSpan = this._labelEntry.cloneNode(true);
	var width = jsgtk.Util.fullWidth(tempSpan);
	if(width < 100) width = 100;
	this._entryWidth = width;
	this._entry.style.width = width + 10 + "px";
	this._entry.parentNode.style.width = width + 10 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype.resizeWidthComposite = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
	this.__dropDown.style.width = width - 2 + "px";
};
/** @private */
gtk.SmartEntryDropDown.prototype.resizeHeightComposite = function(height){
	this._height = height;
	this._domObj.style.height = height - 5 + 'px';
};
gobject.signalNew("_rendersubnodes", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("_setselectedvalues", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("_search", gtk.SmartEntryDropDown, null, null, []);
gobject.signalNew("smartentrychanged", gtk.SmartEntryDropDown, null, null, []);
/**
 * UploadButton is a class to create an upload button
 * @param {String} label The label or text display inside the upload button
 * @param {String} stock The stock value to be set
 * @base gtk.Button
 * @constructor
 */
gtk.UploadButton = function(label){
    gtk.Button.apply(this, [label]);

    var self = this;
    this._imageShapeDom = null;

    /** A registered signal name '<b>filechanged</b>' when file is changed */
    this.UPLOADBUTTON_FILECHANGED 	= "";
    /** A registered signal name '<b>mouseover</b>' */
    this.UPLOADBUTTON_MOUSEOVER 	= "";
    /** A registered signal name '<b>mouseout</b>' */
    this.UPLOADBUTTON_MOUSEOUT 	= "";

    /** @private */
    this._render = function(){
	    this._domObj.className += " gtk_upload_button";
	    this._domObj.style.display = "";

	    this._domImage = jsgtk.Dom.create({"tag": "div", "append": this._domObj, "className": "gtk_uploadbutton_image"});
	    this._className = this._domObj.className;
	    this._inputFile = jsgtk.Dom.create({"tag": "input", "type": "file", "append": this._domObj, "className": "gtk_upload_button_input_file"});
	    this._inputFile.style.width = this._width + "px";
	    this._inputFile.style.height = this._height + "px";
	};
   this._render();

   jsgtk.Event.addDomListener(this._inputFile, "change", function(){
		self.emit("filechanged", self._inputFile.value);
	});

	jsgtk.Event.addDomListener(this._domObj, "mouseover", function(){
		self.emit("mouseover");
	});

	jsgtk.Event.addDomListener(this._domObj, "mouseout", function(){
		self.emit("mouseout");
	});
};

gtk.UploadButton.prototype = new gtk.Button();
gtk.UploadButton.prototype.constructor = gtk.UploadButton;

/**
 * A method to set the name of upload button
 * @param {String} name The name of upload button
 */
gtk.UploadButton.prototype.setName = function(name){
    this._inputFile.setAttribute("name", name);
};

/**
 * A method to set the text of upload button
 * @param {String} strName The name of upload button
 */
gtk.UploadButton.prototype.setText = function(strName){
    if(strName != null || typeof strName !== 'undefined') {
		this.setTextLabel(strName);
		this._inputFile.style.width = this._width - 2 + "px";
	}
};

/**
 * A method to get an upload object as Dom
 * @return {Dom} The Dom object of upload button
 * @type Dom
 */
gtk.UploadButton.prototype.getUploadNode = function(){
    console.log("gtk.UploadButton.getUploadNode() will be depricated, please use gtk.UploadButton.getChildNode()");
    return this.getChildNode();
};

/**
 * A method to get object of the upload input file
 * @return {Dom} The object of upload file
 * @type Dom
 */
gtk.UploadButton.prototype.getChildNode = function(){
    return this._inputFile;
};

/**
 * A method to show loading image inside the button
 */
gtk.UploadButton.prototype.showLoading = function(){
	this._labelDom.style.display = "none";
	this._imageDom.style.display = "none";
	this._inputFile.style.display = "none";
	if(!this._loading) this._loading = jsgtk.Dom.create({tag:"div", className: "gtk_button_loading", append: this._domObj});
};

/**
 * A method to transform or chagne upload button to image
 * @param {gtk.Image} imageObj The gtk.Image object to display to be set
 */
gtk.UploadButton.prototype.setAsImage = function(imageObj){
	if(this._imageShapeDom !== null) {
		this._domImage.innerHTML = "";
		this._imageShapeDom = null;
	}
	this._imageShapeDom = imageObj;
	this._domObj.className += " gtk_button_with_image_shape";
	this._gradient.style.display = "none";
	this.setTextLabel("");
	this._domImage.appendChild(imageObj.getNode());
	imageObj.resetSize(imageObj._width, imageObj._height);

	this._width	= this._minWidth	= imageObj._width;
	this._height	= this._minHeight	= imageObj._height;

	this.resizeWidthComposite(imageObj._width);
	this.resizeHeightComposite(imageObj._height);
};

/** @private */
gtk.UploadButton.prototype._getActualImageSize = function(imageObj){
	var imageNode = imageObj.getChildNode();
	var tmpImgObj = new Image();
		 tmpImgObj.src = imageNode.src;
	return [tmpImgObj.width, tmpImgObj.height];
};

/** @private */
gtk.UploadButton.prototype._getPaddingAndBorderWidth = function(){
	var attribute = this._attribute;
	var paddingWidth 	= 0;
	var borderWidth	= 0;
	for(var el in attribute){
		if(el == "padding") paddingWidth = parseInt(attribute[el]) * 2;
		if(el == "border") borderWidth = parseInt(attribute[el]) * 2;
	}
	return [paddingWidth, borderWidth];
};

/**
 * A method to set attributes and apply styles
 * @param {Object} attribute The object literal that contain JavaScript styles
 */
gtk.UploadButton.prototype.setAttribute = function(attribute){
	this._attribute 	= attribute;
	var paddingWidth 	= 0;
	var borderWidth	= 0;
	for(var el in attribute){
		this._domObj.style[el] = attribute[el];
		if(el == "padding") paddingWidth = parseInt(attribute[el]);
		if(el == "border") borderWidth = parseInt(attribute[el]);
	}
	this._domImage.style.top = paddingWidth + "px";
	this._domImage.style.left = paddingWidth + "px";

	this.resizeWidthComposite(this._width);
	this.resizeHeightComposite(this._height);
};

/**
 * A method to disable the control of upload button
 */
gtk.UploadButton.prototype.disable = function(){
	this._inputFile.disabled = false;
	if(this._domImage.childNodes.length > 0){
			this._domObj.className += " gtk_button_disabled gtk_upload_button_disabled";
			this._imageShapeDom.getChildNode().style.opacity = 0.50;
	}
	else this._domObj.className += " gtk_button_disabled";

	this._gradient.className += " gradisable";
	this._inputFile.style.display = "none";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "default"});

};

/**
 * A method to set enabled to the button
 */
gtk.UploadButton.prototype.enable = function(){
	this._inputFile.disabled = false;
	var uploadBtnClass = "gtk_widget gtk_container gtk_button gtk_upload_button";
	if(this._domImage.childNodes.length > 0) {
			uploadBtnClass = "gtk_widget gtk_container gtk_upload_button";
			this._imageShapeDom.getChildNode().style.opacity = 1;
	}
	this._domObj.className = uploadBtnClass;
	this._gradient.className = "gtk_button_gradient";
	this._inputFile.style.display = "";
	if(this._widget1 !== null) this._widget1.setStyle({cursor: "default"});

};

/**
 * A method to hide loading image of upload button
 */
gtk.UploadButton.prototype.hideLoading = function(){
	this._labelDom.style.display = "";
	this._imageDom.style.display = "";
	this._inputFile.style.display = "";
	if(this._loading){
		this._domObj.removeChild(this._loading);
		this._loading = null;
	}
};

/** @private */
gtk.UploadButton.prototype._resizeHeight = function(height){
	this._domObj.style.height = this._height + "px";
	this._domObj.childNodes[0].style.height = this._height / 2 + "px";
	this._height = height;
};

/** @private */
gtk.UploadButton.prototype._resizeWidth = function(width){
	this._domObj.style.width = width - this.borderWidth + "px";
	if(typeof this._domImage !== 'undefined'){
		if(this._domImage.childNodes.length > 0) {
			this._domObj.style.width = width + "px";
		}
	}
	this._width = width;
};

/**
 * A method to resize the width of the upload button
 * @param {Interger} width The length of width to be resized.
 */
gtk.UploadButton.prototype.resizeWidthComposite = function(width){
	this.setWidth(width);
	this.resize(width, this._height);
	this._inputFile.style.width = width + "px";
	this._domObj.style.width = width + "px";

	if(typeof this._domImage !== 'undefined'){
		if(this._domImage.childNodes.length > 0) {
			this._imageShapeDom.resetSize(width, this._height);
			this._domObj.style.top = "0px";
		}
	}
};

/**
 * A method to resize the height of upload Button
 * @param {Interger} height The length of width to be resized.
 */
gtk.UploadButton.prototype.resizeHeightComposite = function(height){
	this.setHeight(height);
	this.resize(this._width, height);
	var inputFileStyle = this._inputFile.style;
	inputFileStyle.height = height + "px";
	inputFileStyle.fontSize = height + "px";

	this._domObj.style.height = height + "px";

	if(this._domImage.childNodes.length > 0) {
		this._imageShapeDom.resetSize(this._width, height);
		this._domObj.style.top = "0px";
	}
};

gobject.signalNew("filechanged", gtk.UploadButton, null, null, []);
gobject.signalNew("mouseover", gtk.UploadButton, null, null, []);
gobject.signalNew("mouseout", gtk.UploadButton, null, null, []);
/**
 * Frame is a class to create a frame box to display text or image
 * @param {String} label The label or text display as title of the frame
 * @base gtk.Widget
 * @constructor
 */
gtk.Frame = function(label){
	gtk.Widget.apply(this);

	this._label = label || " ";
    this._labelXAlign = 0;
    this._labelYAlign = 0.5;
    this._child = null;

	this._minWidth = this._minContainerWidth = this._width = 1;
	this._minHeight = this._minContainerHeight = this._height = 1;
	this._childOffsetWidth = 18;
	this._childOffsetHeight = 39;
	this._attribute;
	/** @private */
	this._render = function(){
		var domObj = this._domObj;
		domObj.className += ' gtk_frame';
		    this._labelDom = jsgtk.Dom.create({'append': domObj, 'tag' : 'div'});
		        this._label = new gtk.Label(label);
    		    this._label.setJustify(gtk.JUSTIFY_CENTER);
    		    this._labelDom.appendChild(this._label.getNode());

		this._minWidth = this._minContainerWidth = this._width = this._label.getWidth() + 2;
		this._minHeight = this._minContainerHeight = this._height = this._label.getHeight() + 24;

		    this._frameArea = jsgtk.Dom.create({"append": domObj,"tag":"div", "className": "gtk_frame_area"});
		    var frameStyle = this._frameArea.style;
		    frameStyle.width = this._width - 2 + "px";
		    frameStyle.height = this._height - 10 + "px";
			    this._widgetContainer = jsgtk.Dom.create({"append": this._frameArea, 'tag': 'div', 'style': 'position: relative'});
			    this._widgetContainer.style.left = "7px";

		domObj.style.height = this._height + "px";
		domObj.style.width = this._width + "px";
		this.setLabelAlign(this._labelXAlign, 0.5);
	};

	this._render();
};

gtk.Frame.prototype = new gtk.Widget();
gtk.Frame.prototype.constructor = gtk.Frame;

/**
 * A method to add or append child inside its body
 * @param {Dom} widget The Dom widget to be set
 */
gtk.Frame.prototype.appendChild = function(widget){
	if(typeof widget != 'undefined'){
        if(this._child !== null){
            this._widgetContainer.removeChild(this._child.getNode());
        }
        this._child = widget;
        this._widgetContainer.appendChild(widget.getNode());

        var size = this._getSize();
        this.resize(size[0], size[1]);
    }
};

/**
 * A method to remove a child from its body
 * @param {Dom} widget The Dom widget to be remove
 */
gtk.Frame.prototype.removeChild = function(widget){
        this._widgetContainer.removeChild(widget.getNode());
        this._child = gtk.NullObject;
};

/**
 * A method to add the child widget into Frame
 * @param {gtk.widget} widget A child widget to be displayed in frame
 */
gtk.Frame.prototype.add = function(widget){
    this.appendChild(widget);
	console.log("gtk.Frame.add() will be deprecated, please use appendChild()");
};

/**
 * A method to set the text of the label as specified by label. If label is None the current label is removed
 * @param {String} label A string to be used as the label text
 */

gtk.Frame.prototype.setLabel = function(label){
    this._label.setText(label);
    this._resetMinWidthMinHeight(this._label);
    this.setLabelAlign(this._labelXAlign, this._labelYAlign);
};

/**
 * A method to return the text in the label widget
 * @return {String} The text in the label, or None if there is no label widget.
 * @type String
 */
gtk.Frame.prototype.getLabel = function(){
    return this._label.getText();
};

/**
 * A method to set the label widget (usually to something other than a gtk.Label widget) for the frame
 * @param {gtk.Widget} widget The new label widget
 */
gtk.Frame.prototype.setLabelWidget = function(widget){
    this.setLabel(widget.getText());
    this._labelWidget = widget;
	this._domObj.style.width = 	this._labelWidget._minWidth + 2 + "px";
	this._frameArea.style.width = this._labelWidget._minWidth + "px";
};

/**
 * A method to retrieve the label widget for the frame
 * @return {Dom} The label widget, or null if there is no label widget
 * @type Dom
 */
gtk.Frame.prototype.getLabelWidget = function(){
    return this._labelWidget;
};

/**
 * A method to set the alignment of the frame's label widget and decoration (defaults are 0.0 and 0.5) as specified by xalign and yalign
 * @param {Double} xAlign The horizontal alignment of the label widget along the top edge of the frame (in the range of 0.0 to 1.0)
 * @param {Double} yAlign The vertical alignment of the decoration with respect to the label widget (in the range 0.0 to 1.0)
 */
gtk.Frame.prototype.setLabelAlign = function(xAlign, yAlign){
    this._labelXAlign = xAlign;
    this._labelYAlign = yAlign;
    var left = (this._width - this._label.getWidth()) * xAlign -1;
    if(left < 1) left = 1;
    this._labelDom.style.left = left + "px";
};

/**
 * A method to get or return an array containing the X and Y alignment of the frame's label widget and decoration
 * @return {Array} An Array containing the x and y alignments of the frame's label widget
 * @type Array
 */
gtk.Frame.prototype.getLabelAlign = function(){
    return [this._labelXAlign, this._labelYAlign];
};

/** @private */
gtk.Frame.prototype._getSize = function(){
    var width  = this._child.getMinWidthComposite();
    var height = this._child.getMinHeightComposite();
    width      = this._minContainerWidth  > width  ? this._minContainerWidth  : width;
    height     = this._minContainerHeight > height ? this._minContainerHeight : height;
    return [width + this._childOffsetWidth, height + this._childOffsetHeight];
};

/** @private */
gtk.Frame.prototype._resetMinWidthMinHeight = function(label){
    if(this._minWidth < label.getWidth()) this._minWidth = label.getWidth();
    if(this._minHeight < label.getHeight()) this._minHeight = label.getHeight();
};

/** @private */
gtk.Frame.prototype._setLabelXAlign = function(){
    this._labelXAlign = (this._width/2) - parseInt(this._labelDom.style.width)/2;
};

/** @private */
gtk.Frame.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
    this._resizeChildren(width, height);
    this.setLabelAlign(this._labelXAlign, this._labelYAlign);
};

/** @private */
gtk.Frame.prototype._resizeWidth = function(width){
    if(this._minWidth > width) return;

    if(this._width < width) this._width = width;
    else width = this._width;

    this._domObj.style.width = width - 2 + "px";
        this._frameArea.style.width = width - 4 + "px";
            this._widgetContainer.style.width = width - this._childOffsetWidth + 'px';
};

/** @private */
gtk.Frame.prototype._resizeHeight = function(height){
    this._domObj.style.height = height - 2 + "px";
        this._frameArea.style.height = height - 12 + "px" ;
            this._widgetContainer.style.height = height - this._childOffsetHeight + 'px';
    this._height = height;
};

/** @private */
gtk.Frame.prototype._resizeChildren = function(width, height){
    this._child.resizeWidthComposite(width - this._childOffsetWidth);
    this._child.resizeHeightComposite(height - this._childOffsetHeight);
};

/**
 * A method to get the minimum width of frame
 * @return {Integer} The minimum width of frame
 */
gtk.Frame.prototype.getMinWidthComposite = function(){
	var minWidth = this._child.getMinWidthComposite();
    if(this._minWidth < minWidth)
		this._minWidth = minWidth;
	return this._minWidth + this._childOffsetWidth;
};

/**
 * A method to get the minimum height of frame
 * @return {Integer} The minimum height of frame
 */
gtk.Frame.prototype.getMinHeightComposite = function(){
	var minHeight = this._child.getMinHeightComposite();
    if(this._minHeight < minHeight)
		this._minHeight = minHeight;
	return this._minHeight + this._childOffsetHeight;
};

/**
 * A method to resize the width of frame
 * @param {Interger} width The length of width to be resize.
 */
gtk.Frame.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this.setWidth(minWidth);
	this.resize(minWidth, this._height);
};

/**
 * A method to resize the height of frame
 * @param {Interger} height The length of width to be resize.
 */
gtk.Frame.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite? height : minHeightComposite;
	this.resize(this._width, minHeight);
};
/**
 * A method to set and apply style given an object literal of an object
 * @param {Array} The array of attributes object styles
 */
gtk.Frame.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.Frame.prototype._applyAttribute = function(){
	for(var el in this._attribute){
	   if(el == "border"){
	    this._frameArea.style.border = this._attribute[el];
	   }else if(el == "backgroundColor"){
	        this._domObj.style[el] = this._attribute[el];
	   }
	   else{
	        this._label.getNode().childNodes[0].childNodes[0].childNodes[0].childNodes[0].style[el] = this._attribute[el];
	   }
	}
};
/**
 * Form is a class to create form widget
 * @param {Object} attributes The attributes of theform
 * @constructor
 * @base gtk.Widget
 */
gtk.Form = function(attributes){
    gtk.Widget.apply(this);

    this._child = null;

    /** @private */
    this._render = function(){
        this._domObj.className += " gtk_form";
        this._form = jsgtk.Dom.create({"tag":"form", "append": this._domObj});
        this.setAttribute(attributes);
    };

    this._render();
};

gtk.Form.prototype = new gtk.Widget();
gtk.Form.prototype.constructor = gtk.Form;

/**
 * A method to set the attributes to form
 * @param {Object} attributes The attributes to be set as form's attributes
 */
gtk.Form.prototype.setAttribute = function(attributes){
    var dom = this._form;
    for(var property in attributes){
        dom.setAttribute(property, attributes[property]);
    }
};

/**
 * A method to add a widget to Form
 * @param {Widget} widget A child widget to be added to Form
 */
gtk.Form.prototype.add = function(widget){
	if(!this._child){
		this._child = widget;
		if(widget._domObj) this._form.appendChild(widget.getNode());

		var size = this._getChildSize(widget);
		this.resize(size[0], size[1]);
	}
};

/** @private */
gtk.Form.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);

    if(this._child){
        this._child.resizeWidthComposite(width);
        this._child.resizeHeightComposite(height);
    }
};

/** @private */
gtk.Form.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = this._width + 'px';
};

/** @private */
gtk.Form.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = this._height - 2 + 'px';
};

/** @private */
gtk.Form.prototype._getChildSize = function(widget){
    var width    = widget.getWidth();
    var height   = widget.getHeight();

    return [width, height];
};

/**
 * A method to get the minimum width of Form
 * @return The minimum width of Form
 * @type Integer
 */
gtk.Form.prototype.getMinWidthComposite = function(){
    this._minWidth = this._child ? this._child.getMinWidthComposite() : this._width;
	return this._minWidth;
};

/**
 * A method to get the minimum height of Form
 * @return The minimum height of Form
 * @type Integer
 */
gtk.Form.prototype.getMinHeightComposite = function(){
    this._minHeight = this._child ? this._child.getMinHeightComposite() : this._height;
	return this._minHeight;
};

/**
 * A method to resize the width of Form
 * @param {Interger} width The length of width to be resize.
 */
gtk.Form.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this.resize(minWidth, this._height);
};

/**
 * A method to resize the height of Form
 * @param {Interger} height The length of width to be resize.
 */
gtk.Form.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this.resize(this._width, minHeight);
};
/**
 * UnorderList is a class to create an ordered or unordered list
 * @base gtk.Widget
 * @constructor
 */
gtk.UnorderList = function(){
	gtk.Widget.apply(this);
	this._minWidth = this._width = 150;
	this._minHeight = this._height = 16;

	/** A registered signal name 'clicked' */
	this.UNORDERLIST_CLICKED = '';

	this._render = function(){
	    this._domObj.className += " gtk_unorderlist";
	        this.__ul = jsgtk.Dom.create({tag: "ul", append: this._domObj});
	};

	this._domObj.style.width = this._width + "px";
	this._domObj.style.height = this._height + "px";

	this._render();
	this._initEvents();
};

gtk.UnorderList.prototype = new gtk.Widget();
gtk.UnorderList.prototype.constructor = gtk.UnorderList;

/**
 * A method to set modal data or reset modal data
 * @param {Object} model The model to be set as gtk.ListModel
 */
gtk.UnorderList.prototype.setModel = function(model){
    this.__model = model;
    this.__constroller = new gtk.UnorderListController(this, model);
};

/** @private */
gtk.UnorderList.prototype._renderList = function(model, parent){
    var docFragment = parent || document.createDocumentFragment();
    if(model.children != null){
        var children = model.children;
        var childLength = children.length;
    }else{
        var children = model;
        var childLength = children.length;;
    }

    for(var i = 0;i < childLength; i++){
        var child = children[i];
        if(child.children == null){
            var li = jsgtk.Dom.create({tag: "li", style: child.style, text: child.value, append: docFragment});
        }
        else{
            var li = jsgtk.Dom.create({tag: "li", style: child.style, text: child.value, append: docFragment});
            var ul = jsgtk.Dom.create({tag: "ul", style: child.style, append: li, style: "list-style-type: " + child.type});
            this._renderList(child.children, ul);
        }
    };

    return docFragment;
};

/** @private */
gtk.UnorderList.prototype._initEvents = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this.__ul, 'click', function(){
        _self.emit("clicked", _self._getItemsValue());
    });
};

/** @private */
gtk.UnorderList.prototype._appendList = function(list){
    if(this.__ul.childNodes) this.__ul.innerHTML = "";
    this.__ul.appendChild(list);
};

/**
 * A method to set type orderedlist or unorderedlist
 * @param {String} type The type style of list
 */
gtk.UnorderList.prototype.setListType = function(type){
    this.__ul.style.listStyleType = type;
};

/**
 * A method to set line height or spacing between each list
 * @param {Integer} height The line height of paragraph or sentence
 */
gtk.UnorderList.prototype.setLineHeight = function(height){
    this.__ul.style.lineHeight = height + "px";
    this._calculateDomHeight();
    this._resize();
};

/** @private */
gtk.UnorderList.prototype._getItemsValue = function(){
    var childObj = this.__ul.childNodes;
    var itemLength = childObj.length;
    var itemsValue = [];
    for(var i = 0; i< itemLength; i++){
        itemsValue.push(childObj[i].textContent);
    }
    return itemsValue;
};

/** @private */
gtk.UnorderList.prototype._calculateDomHeight = function(){
    var temp = this._domObj.childNodes[0].cloneNode(true);
    temp.style.width = this._width - 14 + "px";
    this._height = this._minHeight = jsgtk.Util.fullHeight(temp);
    return this._height;
};

/**
 * A method to get minimum width of the list
 * @return {Integer} The minimum width of the list
 * @type Integer
 */
gtk.UnorderList.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};

/**
 * A method to get minimum height of the list
 * @return {Integer} The minimum height of the list
 * @type Integer
 */
gtk.UnorderList.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};

/** @private */
gtk.UnorderList.prototype._resizeHeight = function(height){
    this._domObj.style.height = height + "px";
};

/** @private */
gtk.UnorderList.prototype._resizeWidth = function(width){
    this._domObj.style.width = width + "px";
};

/** @private */
gtk.UnorderList.prototype._resize = function(){
    this._resizeWidth(this._width);
    this._resizeHeight(this._height);
};

/**
 * A method to resize composite width of the list
 * @param {Integer} width The composite width to be set
 */
gtk.UnorderList.prototype.resizeWidthComposite = function(width){
    this._width = this._minWidth = this._width > width ? this._width : width;
    this._calculateDomHeight();
    this._resize();
};

/**
 * A method to resize composite height of the list
 * @param {Integer} width The composite height to be set
 */
gtk.UnorderList.prototype.resizeHeightComposite = function(height){
    this._height = this._minHeight = this._height > height ? this._height : height;
    this._resize();
};

/**
 * A method to set change to pointer or hand
 */
gtk.UnorderList.prototype.setPointerCursor = function(){
    this.__ul.style.cursor = "pointer";
    this.__ul.style.cursor = "hand";
};

gobject.signalNew("clicked", gtk.UnorderList, null, null, []);
/**
 * Accordion is a class to create an accordion widget
 * @param {Boolean} The boolean value true or false is to define whether all children have the same width or height
 * @base gtk.Widget
 * @constructor
 */
gtk.Accordion = function(homogeneous){
    gtk.Widget.apply(this);

    this._minWidth  = this._width  = 200;
    this._minHeight = this._height = 30;
    this._accordionHeaderHeight = 22;
    this._children = [];
    this._homogeneous = homogeneous || true;
    this._activeIndex = null;

    /** A registered signal name 'activated' */
    this.ACCORDION_ACTIVATED = '';
    /** A registered signal name 'deactivated' */
    this.ACCORDION_DEACTIVATED = '';

    this._render = function(){
        this._domObj.className += " gtk_accordion";
    };
    this._render();
};

gtk.Accordion.prototype = new gtk.Widget();
gtk.Accordion.prototype.constructor = gtk.Accordion;

/**
 * Method to add or append content to to each tab of accordion
 * @param {String} title The title of the accordion tab
 * @param {Object} childWidget The DOM object of the child content
 */
gtk.Accordion.prototype.appendChild = function(title, childWidget){
    childWidget.setParent(this);

    var childObject = this._createChildObject({"title": title, "childWidget": childWidget});
    this._children.push(childObject);

    this._appendAccordionNodeHolder(childObject["accordionNodeHolder"]);

    this.resizeWidthComposite(this._width);
    this.resizeHeightComposite(this._height);

    this.activate(0);
};

/**
 * Method to set deactivation unto any tab of accordion
 * @param {Integer} index The index value of the accordion tab
 */
gtk.Accordion.prototype.deActivate = function(index){
    this._activeIndex = null;
    for(var i = 0, len = this._children.length; i < len; i++){
        if(i === index) {
            jsgtk.Dom.hideElement(this._children[index]["childNodeHolder"]);
            var imgNode = this._children[index]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_collapseimg";
        }
    }
};

/**
 * Method to set activation unto any tab of accordion
 * @param {Integer} The index value of the accordion tab
 */
gtk.Accordion.prototype.activate = function(index){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(i === index) {
            jsgtk.Dom.showElement(this._children[index]["childNodeHolder"]);
            var imgNode = this._children[index]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_expandimg";
            this._activeIndex = index;
        }
        else {
            jsgtk.Dom.hideElement(this._children[i]["childNodeHolder"]);
            var imgNode = this._children[i]["headerNodeHolder"].childNodes[1];
            imgNode.className = "gtk_accordion_collapseimg";
        }
    }
};

/** @private */
gtk.Accordion.prototype._getAccordionIndex = function(headerNodeHolder){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["headerNodeHolder"] === headerNodeHolder) return i;
    }
};

/** @private */
gtk.Accordion.prototype._createAccordionNodeHolder = function(headerNodeHolder, childNodeHolder){
    var accordionNodeHolder = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_node_holder"});
    accordionNodeHolder.appendChild(headerNodeHolder);
    accordionNodeHolder.appendChild(childNodeHolder);

    return accordionNodeHolder;
}

/** @private */
gtk.Accordion.prototype._createHeaderNodeHolder = function(title){
    var headerNode = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_header"});
        var titleNode = jsgtk.Dom.create({"append": headerNode, "text": title,"tag": "div", "className": "gtk_accordion_title"});
        var imgNode = jsgtk.Dom.create({"append": headerNode, "tag": "span", "className": "gtk_accordion_collapseimg"});

    this._initEvent(headerNode);

    return headerNode;
};

/** @private */
gtk.Accordion.prototype._createChildNodeHolder = function(childWidget){
    var childNodeHolder = jsgtk.Dom.create({"tag": "div", "className": "gtk_accordion_child_node_holder"});
    childNodeHolder.appendChild(childWidget.getNode());

    return childNodeHolder;
};

/** @private */
gtk.Accordion.prototype._createChildObject = function(childProperties){
    var headerNodeHolder     = this._createHeaderNodeHolder(childProperties["title"]);
    var childNodeHolder     = this._createChildNodeHolder(childProperties["childWidget"]);
    var accordionNodeHolder = this._createAccordionNodeHolder(headerNodeHolder, childNodeHolder);

    var childObject = {
        "accordionNodeHolder": accordionNodeHolder,
        "headerNodeHolder": headerNodeHolder,
        "childNodeHolder": childNodeHolder,
        "childWidget": childProperties["childWidget"]
    };

    return childObject;
};

/** @private */
gtk.Accordion.prototype._appendAccordionNodeHolder = function(accordionNodeHolder){
    this._domObj.appendChild(accordionNodeHolder);
};

/** @private */
gtk.Accordion.prototype._getAccordionsHeaderHeight = function(){
    return this._children.length * this._accordionHeaderHeight;
};

/** @private */
gtk.Accordion.prototype._resizeChildrenNodeWidth = function(minWidth){
    for(var i = this._children.length; i--;)
        this._children[i]["childWidget"].resizeWidthComposite(minWidth);
};

/** @private */
gtk.Accordion.prototype._resizeChildrenNodeHeight = function(minHeight){
    for(var i = this._children.length; i--;)
        this._children[i]["childWidget"].resizeHeightComposite(minHeight);
};

/**
 * Method to get minimum height of childNodeHolder
 * @return {Integer} The minimum height of it
 * @type Integer
 */
gtk.Accordion.prototype.getMinHeightComposite = function(){
    var minHeight = 0
    for(var i = this._children.length; i--;){
        var childWidgetMinHeight = this._children[i]["childWidget"].getMinHeightComposite()
        minHeight = minHeight < childWidgetMinHeight ? childWidgetMinHeight : minHeight;
    }

    minHeight += this._children.length * this._accordionHeaderHeight;

    this._minHeight = this._minHeight < minHeight ? minHeight : this._minHeight;
    return this._minHeight;
};

/**
 * Method to get minimum width of childNodeHolder
 * @return {Integer} The minimum width of it
 * @type Integer
 */
gtk.Accordion.prototype.getMinWidthComposite = function(){
    for(var i = this._children.length; i--;){
        var childWidgetMinWidth = this._children[i]["childWidget"].getMinWidthComposite();
        this._minWidth = this._minWidth < childWidgetMinWidth ? childWidgetMinWidth : this._minWidth;
    }
    return this._minWidth;
};

/**
 * Method to resize width composite of it
 * @param {Integer} width The new width needs setting to the widget
 */
gtk.Accordion.prototype.resizeWidthComposite = function(width){
    var minWidth = this.getMinWidthComposite();
    minWidth = minWidth < width ? width : minWidth;
    this._resizeWidth(minWidth);
    this._resizeChildrenNodeWidth(minWidth);
};

/**
 * Method to resize height composite of it
 * @param {Integer} height The new height needs setting to the widget
 */
gtk.Accordion.prototype.resizeHeightComposite = function(height){
    var minHeight = this.getMinHeightComposite();
    minHeight = minHeight < height ? height : minHeight;
    this._resizeHeight(minHeight);
    this._resizeChildrenNodeHeight(minHeight - this._getAccordionsHeaderHeight());
};

/** @private */
gtk.Accordion.prototype._initEvent = function(headerNode){
    var self = this;
    jsgtk.Event.addDomListener(headerNode, "click", function(e){
        var index = self._getAccordionIndex(headerNode);

        if(index === self._activeIndex) {
        	self.deActivate(index);
        	self.emit("deactivated", index);
        }
        else {
        	self.activate(index);
        	self.emit("activated", index);
        }

    }, false);
};

gobject.signalNew("activated", gtk.Accordion, null, null, []);
gobject.signalNew("deactivated", gtk.Accordion, null, null, []);
/**
 * Blank is a class to create a blank widget
 * @base gtk.Widget
 * @constructor
 */
gtk.Blank = function(){
	gtk.Widget.apply(this);
	this.type = gtk.BLANK_WIDGET;
	this._render = function(){
		this._domObj.className += " gtk_blank";
	};

	this._render();
	this.showLoading();
};

gtk.Blank.prototype = new gtk.Widget();
gtk.Blank.prototype.constructor = gtk.Blank;
/**
 * Bubble is a class to create Bubble or Tooltip
 * @param {Object} dom DOM object of its child content
 * @param {Object} callerId DOM object of parent to show/display bubble or tooltip
 * @param {String} title The title of the Bubble
 * @param {Integer} tailDistance An integer value for tail of bubble
 * @param {Boolean} isVertical The boolean value true or false to define if it is horizontal or vertical
 * @param {Integer} tailLength An integer value of tail length
 * @base gtk.Widget
 * @constructor
 */
gtk.Bubble = function(dom, callerId, title, tailDistance, isVertical, tailLength){
    gtk.Widget.apply(this);

    this._domContent = dom;

    this._callerDom = (typeof callerId == "object") ? callerId : document.getElementById(callerId);

    this._isVertical = isVertical || false;
    this._tailDistance = tailDistance;
    this._title = title || "";
    this._closeButton = null;
    this._bubbleShape = null;
    this._contentWidth = 0;
    this._contentHeight = 0;
    this._minHeight = 25;
    this._minWidth = 100;
    this._titleHeight = 0;
    this._tailSide = "left";
    this._tailWidth = tailLength || 0;
    this._tailDistance = tailDistance || 30;

    /** A registered signal name 'closed' */
    this.BUBBLE_CLOSED = "";

    dom.className = "gtk_bubble_content";
    this._contentWidth = this._calculateDomWidth(dom);
    this._contentHeight = this._calculateDomHeight(dom);

	this._render = function(dom, callerId, title, tailDistance, isVertical, tailLength){
	    var bubbleDom = this._domObj;
	    bubbleDom.className += " gtk_bubble";
	    bubbleDom.setAttribute("id", "gtk_bubble");
	    bubbleDom.style.position = "absolute";

        this._closeButton = jsgtk.Dom.create({"tag": "p", "append": bubbleDom, "className": "gtk_bubble_close_button"});
        this._closeButton.innerHTML = " ";

        this._spanTitle = this._createTitle(bubbleDom);

        var callerDomProperties = this._getDomProperties(this._callerDom);

        this._tailSide = callerDomProperties.tailSide;

		this._positionBubbleTail(callerDomProperties);

        this._bubbleShape = new mylabs.Shape("bubble", 0, 0, this._contentWidth, this._getBubbleHeight(), this._tailSide, this._tailDistance);

        var domObjStyle = bubbleDom.style;
        domObjStyle.height = this._contentHeight + "px";
        domObjStyle.width = this._contentWidth + "px";

		document.body.appendChild(bubbleDom);
        this._setContent(this._domContent);

        bubbleDom.insertBefore(this._bubbleShape.getNode(), bubbleDom.firstChild);
    };

	this._render(dom, callerId, title, tailDistance, isVertical, tailLength);

	this._initEvents();
};

gtk.Bubble.prototype = new gtk.Widget();
gtk.Bubble.prototype.constructor = gtk.Bubble;

/** @private */
gtk.Bubble.prototype._initEvents = function(){
    var _self = this;
    jsgtk.Event.addDomListener(this._closeButton, 'click', function(e){
        _self.close();
        e.cancelBubble = true;
    });
};

/** @private */
gtk.Bubble.prototype._setContent = function(content){
    if(content){
        this._domContent.className = "gtk_bubble_content";
        this._domObj.appendChild(content);
    }
};

/** @private */
gtk.Bubble.prototype._setContentWidth = function(width){
    this._contentWidth = width > this._minWidth ?  width : this._minWidth;
};

/** @private */
gtk.Bubble.prototype._getContentWidth = function(){
    return this._contentWidth;
};

/** @private */
gtk.Bubble.prototype._setContentHeight = function(height){
    this._contentHeight = height;
};

/** @private */
gtk.Bubble.prototype._getContentHeight = function(){
    return this._contentHeight;
};

/** @private */
gtk.Bubble.prototype._getBubbleHeight = function(){
    var bubbleHeight = this._domContent ? this._getContentHeight() + this._titleHeight : this._titleHeight + 10;

    if(bubbleHeight > this._minHeight){
        this._setContentHeight(bubbleHeight);
    }else{
        this._setContentHeight(this._minHeight);
        bubbleHeight = this._getContentHeight();
    }
    return bubbleHeight;
};

/** @private */
gtk.Bubble.prototype._createTitle = function(domBubble){
    if(this._title){
        this._updateTailWidth();
        var spanTitle = jsgtk.Dom.create({"append": domBubble, 'tag': "span", "className": "gtk_bubble_title"});
        spanTitle.innerHTML = this._title;
        spanTitle.style.width = this._getContentWidth() + "px";
        this._titleHeight = this._calculateDomHeight(domBubble.childNodes[1]);

        return spanTitle;
    }
};

/** @private */
gtk.Bubble.prototype._updateTailWidth = function(){
    var height = this._titleHeight? this._titleHeight : 0;

    this._tailWidth = this._getContentHeight() + height < 70 ? 10 : 20;
};

/**
 * Method to set position of bubble or tooltip
 * @param {Object} caller The DOM object of caller or parent
 */
gtk.Bubble.prototype.setPosition = function(caller){
	this._callerDom = (typeof caller == "object") ? caller : document.getElementById(caller);
	var callerDomProperties = this._getDomProperties(this._callerDom);
	this._positionBubbleTail(callerDomProperties);
	this._resize();
};

/**
 * Method to set position of bubble or tooltip by mouse position
 * @param {Event} e The event to be passed
 */
gtk.Bubble.prototype.setPositionByMouse = function(e){
    this._domObj.style.left = e.clientX + 5 + "px";
    this._domObj.style.top = e.clientY - 12 + "px";
};

/** @private */
gtk.Bubble.prototype._positionBubbleTail = function(callerDomProperties){
	var domBubble = this._domObj;
    var domStyle = domBubble.style;
    this._updateTailWidth();

    if(callerDomProperties.tailSide == "right"){
        domStyle.left = callerDomProperties.left - (this._tailWidth) - this._getContentWidth() + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4  +"px";
        domStyle.top = callerDomProperties.top + parseInt(callerDomProperties.height / 2) - this._tailDistance - (this._tailWidth/2)+ "px";
    }
    else if(callerDomProperties.tailSide == "left"){
        domStyle.left = callerDomProperties.width + callerDomProperties.left + "px";
        domStyle.paddingLeft = this._tailWidth + "px";
        this._closeButton.style.left = this._getContentWidth() + this._tailWidth - 16 +"px";
        domStyle.top = callerDomProperties.top + parseInt(callerDomProperties.height / 2) - this._tailDistance - (this._tailWidth/2)+ "px";
    }
    else if(callerDomProperties.tailSide == "up"){
        domStyle.top = callerDomProperties.top + callerDomProperties.height  + "px";
        domStyle.left = callerDomProperties.width/2 + callerDomProperties.left - this._tailDistance - 8 + "px";
        this._spanTitle.style.top = this._tailWidth + "px";
        if(this._domContent) this._domContent.style.top = this._tailWidth + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4 + "px";
        this._closeButton.style.top = this._tailWidth - 4 + "px";
    }
    else{
        var contentHeight = this._getContentHeight();
        domStyle.top = callerDomProperties.top - this._titleHeight - this._tailWidth - contentHeight  + "px";
        domStyle.left = callerDomProperties.width/2 - this._tailDistance + callerDomProperties.left - 8 + "px";
        this._closeButton.style.left = this._getContentWidth() - this._tailWidth + 4 + "px";
        this._closeButton.style.top = -4 + "px";
    }
};

/** @private */
gtk.Bubble.prototype._initEvents = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._closeButton, 'click', function(e){
        self.close();
        e.cancelBubble = true;
    });
};


/**
 * Method to show the bubble after initiated
 */
gtk.Bubble.prototype.show = function(){
    if(this._intervalID) { clearInterval(this._intervalID); }
    this.setPosition(this._callerDom);
    this._domObj.style.display = "";
};

/**
 * Method to set a newa title of the bubble
 * @param {String} title The string title is set to bubble
 */
gtk.Bubble.prototype.setTitle = function(title){
    if(!this._spanTitle){
        this._spanTitle = jsgtk.Dom.create({"tag": "span", "className": "gtk_bubble_title"});
        this._spanTitle.className += this._tailSide == "left" ? " leftTailBubble": " rightTailBubble";
        var svgDom = document.getElementsByTagName("path")[0].parentNode;
        this._domObj.insertBefore(this._spanTitle, svgDom);
    }

    this._title = title;
    this._spanTitle.innerHTML = title;
    var titleHeight = this._calculateDomHeight(this._spanTitle);
    this._resize();
};

/**
 * Method to get or return the title of the bubble
 * @return {String} The title of of the bubble if applicable
 */
gtk.Bubble.prototype.getTitle = function(){
    return this._title;
};

/**
 * Method to close or remove the bubble
 */
gtk.Bubble.prototype.close = function(){
    if(this._showConfirm){
        var quit = window.confirm(this._message);
        if(quit){
            jsgtk.Event.trigger(this, "confirm");
        }
    }

    this._removeModal();
    if(this._domObj.parentNode !== null) this._domObj.parentNode.removeChild(this._domObj);

    this._showConfirm = false;
    this.emit("closed");
};

/** @private */
gtk.Bubble.prototype._removeModal = function(){
    var modal = document.getElementById("gtk_bubble_modal");
    if(modal){ document.body.removeChild(modal); }
};

/**
 * Method to add Dom object into the bubble
 * @param {Dom} content The Dom object to be passed
 */
gtk.Bubble.prototype.addContent = function(content){
    this.removeContent();
    content.className = "gtk_bubble_content";
    this._domContent = content;
    this._contentWidth = this._calculateDomWidth(content);
    this._contentHeight = this._calculateDomHeight(content);
    this._domObj.style.width = this._contentWidth + "px";
    this._domObj.style.height = this._contentHeight + "px";
    this._domObj.appendChild(content);
    this._resize();
};

/**
 * Method to remove all of its contents
 */
gtk.Bubble.prototype.removeContent = function(){
    var childNodes = this._domObj.childNodes;
    var count = childNodes.length;
    var content ;
    for(var i = 0; i <count; i++){
        if(childNodes[i].className == "gtk_bubble_content"){
            content = childNodes[i];
        }
    }

    if(content){
        this._domObj.removeChild(content);
    }
};

 /**
 * Method to set content as tooltip
 * @param {DOm} domContent The DOM content value
 */
gtk.Bubble.prototype.setTooltip = function(domContent){
    if(domContent){
        this.addContent(domContent);
    }
    if(this._title) this.removeTitle();
    this._closeButton.style.display = "none";
};

/** @private */
gtk.Bubble.prototype._resize = function(){
    var nodeStyle = this._domObj.style;
    nodeStyle.height = "";
    nodeStyle.width = this._contentWidth + "px";
    var bubbleHeight = this._calculateDomHeight(this._domObj);
    nodeStyle.height = bubbleHeight + "px";

    callerTop = jsgtk.Util.pageY(this._callerDom);
    callerHeight = this._callerDom.clientHeight;
    this._contentHeight = this._calculateDomHeight(this._domContent);

    if(this._spanTitle){
    	this._titleHeight = this._calculateDomHeight(this._spanTitle);
    	this._spanTitle.style.width = this._contentWidth + "px";
    }


    var callerProperties = this._getDomProperties(this._callerDom);
    this._tailSide = callerProperties.tailSide;

    this._bubbleShape.remove();

    this._bubbleShape = new mylabs.Shape("bubble", 0, 0, this._contentWidth , bubbleHeight, this._tailSide, this._tailDistance);

    this._domObj.insertBefore(this._bubbleShape.getNode(), this._domObj.firstChild);

    if(this._tailSide == "right"){
        nodeStyle.left = callerProperties.left - (this._tailWidth + 6) - this._getContentWidth() + "px";
        nodeStyle.paddingLeft = "";
        this._closeButton.style.left = this._contentWidth - this._tailWidth + 6 + "px";
    }
    else if(this._tailSide == "up"){
        this._domContent.style.top = this._tailWidth + "px";
        nodeStyle.paddingLeft = "0px";
        this._closeButton.style.left = this._contentWidth - 16 + "px";
        this._closeButton.style.top = this._tailWidth - 6 + "px";
        this._spanTitle.style.top = this._tailWidth + "px";
    }
    else if(this._tailSide == "down"){
        this._domObj.style.top = callerTop - bubbleHeight - callerHeight + "px";
        this._closeButton.style.left = this._contentWidth - 16 + "px";
        this._closeButton.style.top = "0px";
        this._spanTitle.style.top = "0px";
        nodeStyle.paddingLeft = "0px";
    }
    else{
        this._closeButton.style.left = this._contentWidth + this._tailWidth - 16 + "px";
    }
};

 /**
  * Method to get the content from server then append to bubble
  * @param {String} url The URL of resource site
  * @param {Function} callback The method which will be executed after get response from server. Callback function should return a dom content
  */
gtk.Bubble.prototype.loadContent = function(url, callback){

    this._divLoader = jsgtk.Dom.create({"tag": "div", "className": "gtk_bubble_table"});
    var divLoaderStyle = this._divLoader.style;
    divLoaderStyle.height = MylabsSvgVml.type == "SVG" ? "70px" : "120px";
    divLoaderStyle.width = "300px";

    this._progressBarContainer = jsgtk.Dom.create({"append": this._divLoader, "tag": "span", "className": "gtk_bubble_progress_bar_container"});
        this._progressBar = jsgtk.Dom.create({"append": this._progressBarContainer, "tag": "div"});

    this.addContent(this._divLoader);

    this._updateTailWidth();
    if(this._tailSide == "right"){
        this._domObj.style.paddingLeft = "0px";
    }
    else if(this._tailSide == "down"){
        this._domObj.style.paddingLeft = "0px";
    }
    else if(this._tailSide == "up"){
        var top = this._tailWidth + "px";
        this._divLoader.style.top = top;
        this._closeButton.style.top = top;
        this._spanTitle.style.top = top;
    }
    else{
        this._domObj.style.paddingLeft = this._tailWidth + "px";
    }

    this._ajax(url, callback);

};

 /** @private */
gtk.Bubble.prototype._updateProgressBar = function(state){
    var self = this;
    this._barWidth = 0;
    var bar = this._progressBar;
    var x = window.setInterval(
        function(){
            self._barWidth += 10;
            bar.style.width = self._barWidth + "px";
            if( self._barWidth== 180) { clearInterval(x); }
        },
        40
    );
};

/** @private */
gtk.Bubble.prototype._appendContent = function(callback){
    var self = this;
    var time = 0, count = 50;
    var error = false;
    var y = window.setInterval(
        function(){

            time += count;
            if(self._barWidth == 180){
                if(parseFloat(self._progressBar.style.width) < 195) self._progressBar.style.width = (parseFloat(self._progressBar.style.width) + 0.035) + "px";
                if(self._response.state == 4){
                    clearInterval(y);
                    self._progressBar.style.width = "100%";
                    self._domObj.removeChild(self._divLoader);

                    var domContent = callback(eval( "(" + self._response.text + ")" ));
                    self.addContent(domContent);
                }
            }

            if(time > 15000 && !error){
                error = true;
                var errorSpan = jsgtk.Dom.create({"tag":"span"});
                errorSpan.innerHTML = "<i>This is taking longer than usual. </i><br/> Try again";
                self._divLoader.appendChild(errorSpan);

                self._divLoader.style.textAlign = "center";
            }
        },
        count
    );
};

/** @private */
gtk.Bubble.prototype._updateContent = function(content){
    var container = jsgtk.Dom.create({tag: "div"});
    container.innerHTML = content;
    this._domObj.removeChild(this._divLoader);
    this.addContent(container);
};

/** @private */
gtk.Bubble.prototype._ajax = function(url, callback){
	var method = "GET";
	var request;
	var self = this;

	if(window.XMLHttpRequest){
	    request = new XMLHttpRequest();
	}
	else if (window.ActiveXObject){
	    request = new ActiveXObject("Msxml2.XMLHTTP");
	    if (!request){
		    request = new ActiveXObject("Microsoft.XMLHTTP");
	    }
	}
	this._updateProgressBar();
	this._appendContent(callback);
	this._response = {};
	/** @private */
	request.onreadystatechange = function() {
		if (request.readyState == 4){
	        self._response = {
			    state: 4,
			    text: request.responseText
			};
		}
	};

	request.open(method, url, true);
	request.send(null);
};

/**
 * Method to remove title from bubble
 */
gtk.Bubble.prototype.removeTitle = function(){
    this._title = "";
    this._domObj.removeChild(this._spanTitle);
    this._titleHeight = 0;
    this._resize();
    this._spanTitle = null;
};

/** @private */
gtk.Bubble.prototype._getDomProperties = function(callerDom){
    var callerTemp = callerDom.cloneNode(true);
    var side, windowSize,
        widthFromLeft = jsgtk.Util.pageX(callerDom),
        heightFromTop = jsgtk.Util.pageY(callerDom),
        callerX = jsgtk.Util.fullWidth(callerTemp),
        callerY = jsgtk.Util.fullHeight(callerTemp);

    if(this._isVertical){
        windowSize = jsgtk.Util.windowHeight();
        side = (windowSize - heightFromTop - callerY > this._contentHeight + this._titleHeight + this._tailWidth) ? "up" : "down";
    }
    else{
        windowSize = document.body.clientWidth;
        side = (windowSize - (widthFromLeft + callerX + this._tailWidth) <  this._contentWidth) ? "right" : "left";
    }

    return {
        tailSide: side,
        left: widthFromLeft,
        top: heightFromTop,
        windowSize: windowSize,
        width: callerX,
        height: callerY
    };
};

/**
 * Method to show the modal modal window cover on top of page
 * @param {Function} callback The callback method
 * @param {String} msg The confirm message display in the confirmation popup box
 */
gtk.Bubble.prototype.confirmClose = function(callback, msg){
    var self = this;
    this._showConfirm = true;
    this._message = msg || "Are you sure you want to quit?";

    jsgtk.Event.addListener(this, "confirm", callback);
    document.onkeypress = function(e){
        e = (e)?e:((event)?event:null);
        if(e.keyCode == 27){
            self.close();
        }
    };
};

/**
 * Method to show the modal modal window cover on top of page
 */
gtk.Bubble.prototype.showModal = function(){
    this.show();

    var modal = jsgtk.Dom.create({"append": document.body, "tag": 'div', "id": "gtk_bubble_modal"});
    var pageSize = jsgtk.Util.getPageSize();
    modal.style.width = pageSize[0] + "px";
    modal.style.height = pageSize[1] + "px";
    this._domObj.style.display = "";
};

/** @private */
gtk.Bubble.prototype._calculateDomHeight = function(dom){
    if(dom){
        var tempDom = dom.cloneNode(true);
        tempDom.style.visibility = "hidden";
        tempDom.style.position = "absolute";
        var marginTop = parseInt(jsgtk.Util.getStyle(tempDom, "marginTop")) || 0;
        var marginBottom = parseInt(jsgtk.Util.getStyle(tempDom, "marginBottom")) || 0;
        var height = jsgtk.Util.fullHeight(tempDom) + marginTop + marginBottom;
        height = this._minHeight > height ? this._minHeight : height;
        return height;
    }
    return 0;
};

/** @private */
gtk.Bubble.prototype._calculateDomWidth = function(dom){
    if(dom){
        var tempDom = dom.cloneNode(true);
        tempDom.style.visibility = "hidden";
        tempDom.style.position = "absolute";
        var marginLeft = parseInt(jsgtk.Util.getStyle(tempDom, "marginLeft")) || 0;
        var marginRight = parseInt(jsgtk.Util.getStyle(tempDom, "marginRight")) || 0;
        var width = jsgtk.Util.fullWidth(tempDom) + marginLeft + marginRight;
        return width;
    }
    return 0;
};

/**
 * Method to hide the bubble with in an interval of time. When the user mouseover the bubble, it won't hide. When User mouseout the bubble, the bubble will be hide
 * @param {Integer} time The time interval which set to hide bubble. The default interval is 500.
 */
gtk.Bubble.prototype.hide = function(time){
    var self = this;
    if(!this._autoHide){
		this._domObj.style.display = "none";
		return;
	};
    this._intervalID = null;
    var timeout = time || 500;
    var modal = document.getElementById("gtk_bubble_modal");
    if(modal){
         return;
    }else{
        jsgtk.Event.addDomListener(this._domObj, "mouseover", function(){
            self._domObj.style.display = "";
            clearInterval(self._intervalID);
            jsgtk.Event.addDomListener(self._domObj, "mouseout", function(){

                if(!document.getElementById("gtk_bubble_modal")){ self._domObj.style.display = "none"; }
            });
        });

        var displayNone = function(){
            self._domObj.style.display = "none";
            clearInterval(self._intervalID);
        };

        this._intervalID = setInterval(displayNone, timeout);
    }
};

/** Method to set display auto hideElement
 *  @param {Boolean} The boolean true or false for auto display or hide
 */
gtk.Bubble.prototype.setAutoHide = function(autoHide){
	this._autoHide = autoHide;
};

/**
 * Method to set fixed position
 */
gtk.Bubble.prototype.setFixedPosition = function(){
	this._domObj.style.position = "fixed";
};

gobject.signalNew("closed", gtk.Bubble, null, null, []);
/**
* Factory class for gtk.Table
* @constructor
* @param {integer} rows number of row of table
* @param {integer} columns number of column of table
* @param {boolean} homogeneous
* <br> - homogeneous = true will return gtk.TableHomogenoues
* <br> - homogeneous = false will return gtk.TableNoneHomogenoues
* @param {boolean} resizeable
* <br> - resizeable = true will make all children expand to fill table
* <bt> - resizeable = false all children will be packed to the most top and most left of table
* @param {integer} spacing between each child
*/
gtk.TableFactory = function(rows, columns, homogeneous, resizeable, spacing) {
	return homogeneous ?
	       new gtk.TableHomogenoues(rows, columns, resizeable, spacing) :
	       new gtk.TableNoneHomogenoues(rows, columns, resizeable, spacing);
};
/**
* @constructor
* @param {integer} rows number of row of table
* @param {integer} columns number of column of table
* @param {boolean} resizeable
* <br> - resizeable = true will make all children expand to fill table
* <bt> - resizeable = false all children will be packed to the most top and most left of table
* @param {integer} spacing between each child
*/
gtk.TableHomogenoues = function(rows, columns, resizeable, spacing) {
	gtk.TableBase.call(this, rows, columns, resizeable, spacing);

	this._render = function(){
	    var domObj            = this._domObj;
		domObj.className     += " gtk_table_homogenouse";
        this._children        = this._initChildren(this._rows, this._columns);

		this._initWidthsHeights();
	};

	this._render();
};

gtk.TableHomogenoues.prototype = new gtk.TableBase();
gtk.TableHomogenoues.prototype.constructor = gtk.TableHomogenoues;

/**
 * Attach each gtk.Widget to gtk.Table
 * @param {gtk.Widget} child
 * @param {intger} left x1
 * @param {integer} right x2
 * @param {integer} top y1
 * @param {integer} bottom y2
 * @param {object} xoptions {align: gtk.JUSTIFY_LEFT || gtk.JUSTIFY_RIGHT, fill: false || true}
 * @param {object} yoptions {align: gtk.JUSTIFY_TOP || gtk.JUSTIFY_BOTTOM, fill: false || true}
 * @param {integer} xpadding
 * @param {integer} ypadding
 */
gtk.TableHomogenoues.prototype.attach = function(child, left, right, top, bottom, xoptions, yoptions, xpadding, ypadding){

    var xcells = 1;

    for(var i = left + 1; i < right; i++){
        this._children[i][top].minWidth   = child.getMinWidthComposite();
        this._children[i][top].minHeight  = child.getMinHeightComposite();
        xcells += 1;
    }
    if(left + 1 < right){
        for(var i = left; i < right; i++){
            var ycells = 1;
            for(var j = top + 1; j < bottom; j++){
                this._children[i][j].minWidth   = child.getMinWidthComposite();
                this._children[i][j].minHeight  = child.getMinHeightComposite();
                ycells += 1;
            }
        }
    }else{

        var ycells = 1;
        for(var i = top + 1; i < bottom; i++){
            this._children[left][i].minWidth   = child.getMinWidthComposite();
            this._children[left][i].minHeight  = child.getMinHeightComposite();
            ycells += 1;
        }
    }

    var targetChild = this._children[left][top];
    targetChild.widget   = child;
    targetChild.xoptions = xoptions;
    targetChild.yoptions = yoptions;
    targetChild.xpadding = xpadding;
    targetChild.ypadding = ypadding;
    targetChild.xcells   = xcells;
    targetChild.ycells   = ycells;
    targetChild.left     = left;
    targetChild.right    = right;
    targetChild.top      = top;
    targetChild.bottom   = bottom;
    targetChild.minWidth = child.getMinWidthComposite();
    targetChild.minHeight= child.getMinHeightComposite();

    this._setMaxChildWidthHeightHomo(targetChild);
    this._placeChildrenHomo(this._maxChildWidth, this._maxChildHeight);
    this.resize(this._maxChildWidth * this._columns, this._maxChildHeight * this._rows);
};
/** @private */
gtk.TableHomogenoues.prototype._placeChildrenNoneHomo = function(){
    var child;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._rows; i++){
        for(var j = 0; j < this._columns; j++){
            child = this._children[j][i];
            var width = 0;
            var height = 0;
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({"append": docFragment, "tag": "div", "style": "position: absolute;"});

                for(var k = child.left; k < child.right; k++){
                    width += this._widths[k]["width"];
                }
                var left = 0; //how far from the left has to place the wrapper
                for(var k = 0; k < child.left; k++){
                    left += this._widths[k]["width"];
                }

                for(var k = child.top; k < child.bottom; k++){
                    height += this._heights[k]["height"];
                }
                var top = 0;//how far from the top has to place the wrapper
                for(var k = 0; k < child.top; k++){
                    top += this._heights[k]["height"];
                }

                childWrapper.style.width  = width + "px";
                childWrapper.style.height  = height + "px";
                childWrapper.style.left = left + "px";
                childWrapper.style.top = top + "px";

                childWrapper.appendChild(child.widget._domObj);

                this._positionWidgetByXoptionsNoneHomo(child, width);
                this._positionWidgetByYoptionsNoneHomo(child, height);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableHomogenoues.prototype._positionWidgetByXoptionsNoneHomo = function(child, width){
    var xpadding = child.xpadding;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = xpadding + "px";
        child.widget.resizeWidthComposite(width - xpadding * 2 - spacing);
    }else{
        var left;
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = width - (child.minWidth + child.xpadding) - spacing;
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (width-spacing) / 2) - (child.minWidth / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
        child.widget.resizeWidthComposite(child.minWidth);
    }
};
/** @private */
gtk.TableHomogenoues.prototype._positionWidgetByYoptionsNoneHomo = function(child, height){
    var ypadding = child.ypadding;
     var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = ypadding + "px";
        child.widget.resizeHeightComposite(height - ypadding * 2 - spacing);
    }else{
        var top;
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = height - (child.minHeight + child.ypadding) - spacing;
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (height-spacing) / 2) - (child.minHeight / 2);
                child.widget._domObj.style.top = top + 'px';
                break;
        }
        child.widget.resizeHeightComposite(child.minHeight);
    }
};
/** @private */
gtk.TableHomogenoues.prototype._placeChildrenHomo = function(childWidth, childHeight){
    var children = this._children;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._columns; i++){
        for(var j = 0; j < this._rows; j++){
            var child = children[i][j];
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({'append': docFragment, 'tag': 'div', 'style': 'position: absolute;'});
                childWrapper.style.width  = childWidth  * child.xcells + 'px';
                childWrapper.style.height = childHeight * child.ycells + 'px';
                childWrapper.style.top    = child.top * childHeight + 'px';
                childWrapper.style.left   = child.left * childWidth + 'px';
                childWrapper.appendChild(child.widget._domObj);

                var width  = this._positionWidgetByXoptionsHomo(child, childWidth);
                var height =this._positionWidgetByYoptionsHomo(child, childHeight);

                child.widget.resizeWidthComposite(width - 2);
                child.widget.resizeHeightComposite(height - 2);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableHomogenoues.prototype._positionWidgetByYoptionsHomo = function(child, childHeight){
    var height;
    var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = child.ypadding + 'px';
        height  = childHeight  * child.ycells - child.ypadding * 2 - spacing;
    }else{
        var top;
        height  = child.widget.getMinHeightComposite();
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = (childHeight * child.ycells) - (height + child.ypadding);
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (childHeight * child.ycells) / 2) - (height / 2);
                child.widget._domObj.style.top = top + "px";
                break;
        }
    }
    return height;
};
/** @private */
gtk.TableHomogenoues.prototype._positionWidgetByXoptionsHomo = function(child, childWidth){
    var width;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = child.xpadding + 'px';
        width  = childWidth * child.xcells - child.xpadding * 2 - spacing;
    }else{
        var left;
        width  = child.widget.getMinWidthComposite();
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = (childWidth * child.xcells) - (width + child.xpadding);
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (childWidth * child.xcells) / 2) - (width / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
    }
    return width;
};
/** @private */
gtk.TableHomogenoues.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};
/** @private */
gtk.TableHomogenoues.prototype._getWidths = function(){
    var width = 0;
    for(var i = 0; i < this._columns; i++){
        width += this._widths[i]["width"];
    }
    return width;
};

/** @private */
gtk.TableHomogenoues.prototype._getHeights = function(){
    var height = 0;
    for(var i = 0; i < this._rows; i++){
        height += this._heights[i]["height"];
    }
    return height;
};
/** @private */
gtk.TableHomogenoues.prototype._getResizeableColumns = function(){
    var columns = 0;
    for(var i = this._widths.length; i--;){
        if(this._widths[i]["resizeable"]) columns += 1;
    }
    return columns;
};
/** @private */
gtk.TableHomogenoues.prototype.getMinWidthComposite = function(){
    return this._maxChildWidth * this._columns;
};
/** @private */
gtk.TableHomogenoues.prototype.getMinHeightComposite = function(){
    return this._maxChildHeight * this._rows;
};
/** @private */
gtk.TableHomogenoues.prototype.resizeWidthComposite = function(width){
    var widthComposite = this.getMinWidthComposite();
   	widthComposite     = width > widthComposite ? width : widthComposite;

    width = width > widthComposite ? width : widthComposite;
    var maxChildWidth  = (width / this._columns) + this._spacing;
    var maxChildHeight = this._height / this._rows;
    if(this._resizeable) this._placeChildrenHomo(maxChildWidth, maxChildHeight);
    this.resize(maxChildWidth * this._columns, maxChildHeight * this._rows);
};
/** @private */
gtk.TableHomogenoues.prototype.resizeHeightComposite = function(height){
   	var heightComposite = this.getMinHeightComposite();
   	heightComposite     = height > heightComposite ? height : heightComposite;

	height = height > heightComposite ? height : heightComposite;
    var maxChildHeight = height / this._rows + this._spacing;
    var maxChildWidth  = this._width / this._columns;
	if(this._resizeable) this._placeChildrenHomo(maxChildWidth, maxChildHeight);
    this.resize(maxChildWidth * this._columns, maxChildHeight * this._rows);
};
/**
* @constructor
* @param {integer} rows number of row of table
* @param {integer} columns number of column of table
* @param {boolean} resizeable
* <br> - resizeable resolve to true will make all children expand to fill table
* <br> - resizeable resolbe to false all children will be packed to the most top and most left of table
* @param {integer} spacing between each child
*/
gtk.TableNoneHomogenoues = function(rows, columns, resizeable, spacing) {
	gtk.TableBase.call(this, rows, columns, resizeable, spacing);

	this._render = function(){
	    var domObj            = this._domObj;
		domObj.className     += " gtk_table_none_homogenouse";
        this._children        = this._initChildren(this._rows, this._columns);

		this._initWidthsHeights();
	};

	this._render();
};

gtk.TableNoneHomogenoues.prototype = new gtk.TableBase();
gtk.TableNoneHomogenoues.prototype.constructor = gtk.TableNoneHomogenoues;

/**
 * Attach each gtk.Widget to gtk.Table
 * @param {gtk.Widget} child
 * @param {intger} left x1
 * @param {integer} right x2
 * @param {integer} top y1
 * @param {integer} bottom y2
 * @param {object} xoptions {align: gtk.JUSTIFY_LEFT || gtk.JUSTIFY_RIGHT, fill: false || true}
 * @param {object} yoptions {align: gtk.JUSTIFY_TOP || gtk.JUSTIFY_BOTTOM, fill: false || true}
 * @param {integer} xpadding
 * @param {integer} ypadding
 */
gtk.TableNoneHomogenoues.prototype.attach = function(child, left, right, top, bottom, xoptions, yoptions, xpadding, ypadding){

    var xcells = 1;

    for(var i = left + 1; i < right; i++){
        this._children[i][top].minWidth   = child.getMinWidthComposite();
        this._children[i][top].minHeight  = child.getMinHeightComposite();
        xcells += 1;
    }
    if(left + 1 < right){
        for(var i = left; i < right; i++){
            var ycells = 1;
            for(var j = top + 1; j < bottom; j++){
                this._children[i][j].minWidth   = child.getMinWidthComposite();
                this._children[i][j].minHeight  = child.getMinHeightComposite();
                ycells += 1;
            }
        }
    }else{

        var ycells = 1;
        for(var i = top + 1; i < bottom; i++){
            this._children[left][i].minWidth   = child.getMinWidthComposite();
            this._children[left][i].minHeight  = child.getMinHeightComposite();
            ycells += 1;
        }
    }

    var targetChild = this._children[left][top];
    targetChild.widget   = child;
    targetChild.xoptions = xoptions;
    targetChild.yoptions = yoptions;
    targetChild.xpadding = xpadding;
    targetChild.ypadding = ypadding;
    targetChild.xcells   = xcells;
    targetChild.ycells   = ycells;
    targetChild.left     = left;
    targetChild.right    = right;
    targetChild.top      = top;
    targetChild.bottom   = bottom;
    targetChild.minWidth = child.getMinWidthComposite();
    targetChild.minHeight= child.getMinHeightComposite();

    this._resetWidths(child, left, right, xoptions);
    this._resetHeights(child, top, bottom, yoptions);

    this._placeChildrenNoneHomo();

    var maxWidth  = this._getWidths();
    var maxHeight = this._getHeights();

    this.resize(maxWidth, maxHeight);
    this._minWidth = maxWidth;
    this._minHeight= maxHeight;
};

/** @private */
gtk.TableNoneHomogenoues.prototype._placeChildrenNoneHomo = function(){
    var child;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._rows; i++){
        for(var j = 0; j < this._columns; j++){
            child = this._children[j][i];
            var width = 0;
            var height = 0;
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({"append": docFragment, "tag": "div", "style": "position: absolute;"});

                for(var k = child.left; k < child.right; k++){
                    width += this._widths[k]["width"];
                }
                var left = 0; //how far from the left has to place the wrapper
                for(var k = 0; k < child.left; k++){
                    left += this._widths[k]["width"];
                }

                for(var k = child.top; k < child.bottom; k++){
                    height += this._heights[k]["height"];
                }
                var top = 0;//how far from the top has to place the wrapper
                for(var k = 0; k < child.top; k++){
                    top += this._heights[k]["height"];
                }

                childWrapper.style.width  = width + "px";
                childWrapper.style.height  = height + "px";
                childWrapper.style.left = left + "px";
                childWrapper.style.top = top + "px";

                childWrapper.appendChild(child.widget._domObj);

                this._positionWidgetByXoptionsNoneHomo(child, width);
                this._positionWidgetByYoptionsNoneHomo(child, height);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByXoptionsNoneHomo = function(child, width){
    var xpadding = child.xpadding;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = xpadding + "px";
        child.widget.resizeWidthComposite(width - xpadding * 2 - spacing);
    }else{
        var left;
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = width - (child.minWidth + child.xpadding) - spacing;
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (width-spacing) / 2) - (child.minWidth / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
        child.widget.resizeWidthComposite(child.minWidth);
    }
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByYoptionsNoneHomo = function(child, height){
    var ypadding = child.ypadding;
     var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = ypadding + "px";
        child.widget.resizeHeightComposite(height - ypadding * 2 - spacing);
    }else{
        var top;
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = height - (child.minHeight + child.ypadding) - spacing;
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (height-spacing) / 2) - (child.minHeight / 2);
                child.widget._domObj.style.top = top + 'px';
                break;
        }
        child.widget.resizeHeightComposite(child.minHeight);
    }
};
/** @private */
gtk.TableNoneHomogenoues.prototype._placeChildrenHomo = function(childWidth, childHeight){
    var children = this._children;
    this._domObj.innerHTML = "";
    var docFragment = document.createDocumentFragment();
    for(var i = 0; i < this._columns; i++){
        for(var j = 0; j < this._rows; j++){
            var child = children[i][j];
            if(child.widget){
                var childWrapper = jsgtk.Dom.create({'append': docFragment, 'tag': 'div', 'style': 'position: absolute;'});
                childWrapper.style.width  = childWidth  * child.xcells + 'px';
                childWrapper.style.height = childHeight * child.ycells + 'px';
                childWrapper.style.top    = child.top * childHeight + 'px';
                childWrapper.style.left   = child.left * childWidth + 'px';
                childWrapper.appendChild(child.widget._domObj);

                var width  = this._positionWidgetByXoptionsHomo(child, childWidth);
                var height =this._positionWidgetByYoptionsHomo(child, childHeight);

                child.widget.resizeWidthComposite(width - 2);
                child.widget.resizeHeightComposite(height - 2);
            }
        }
    }
    this._domObj.appendChild(docFragment);
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByYoptionsHomo = function(child, childHeight){
    var height;
    var spacing = child.bottom === this._rows ? 0 : this._spacing;
    if(child.yoptions.fill){
        child.widget._domObj.style.top = child.ypadding + 'px';
        height  = childHeight  * child.ycells - child.ypadding * 2 - spacing;
    }else{
        var top;
        height  = child.widget.getMinHeightComposite();
        switch(child.yoptions.align){
            case gtk.JUSTIFY_TOP :
                child.widget._domObj.style.top = child.ypadding + "px";
                break;
            case gtk.JUSTIFY_BOTTOM :
                top = (childHeight * child.ycells) - (height + child.ypadding);
                child.widget._domObj.style.top = top + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                top = ( (childHeight * child.ycells) / 2) - (height / 2);
                child.widget._domObj.style.top = top + "px";
                break;
        }
    }
    return height;
};
/** @private */
gtk.TableNoneHomogenoues.prototype._positionWidgetByXoptionsHomo = function(child, childWidth){
    var width;
    var spacing = child.right === this._columns ? 0 : this._spacing;
    if(child.xoptions.fill){
        child.widget._domObj.style.left = child.xpadding + 'px';
        width  = childWidth * child.xcells - child.xpadding * 2 - spacing;
    }else{
        var left;
        width  = child.widget.getMinWidthComposite();
        switch(child.xoptions.align){
            case gtk.JUSTIFY_LEFT :
                child.widget._domObj.style.left = child.xpadding + "px";
                break;
            case gtk.JUSTIFY_RIGHT :
                left = (childWidth * child.xcells) - (width + child.xpadding);
                child.widget._domObj.style.left = left + "px";
                break;
            case gtk.JUSTIFY_CENTER :
                left = ( (childWidth * child.xcells) / 2) - (width / 2);
                child.widget._domObj.style.left = left + 'px';
                break;
        }
    }
    return width;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.getMinWidthComposite = function(){
    return this._minWidth;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.getMinHeightComposite = function(){
    return this._minHeight;
};
/** @private */
gtk.TableNoneHomogenoues.prototype.resizeWidthComposite = function(width){
    var widthComposite = this.getMinWidthComposite();
   	widthComposite     = width > widthComposite ? width : widthComposite;

    var columns = this._getResizeableColumns();
    var offsetWidth = (widthComposite - this._getWidths()) / columns;
    for(var i = 0; i < this._columns; i++){
        if(this._widths[i]["resizeable"]) this._widths[i]["width"] += offsetWidth;
    }
    if(this._resizeable) this._placeChildrenNoneHomo();
    this.resize(widthComposite, this._height);
};
/** @private */
gtk.TableNoneHomogenoues.prototype.resizeHeightComposite = function(height){
   	var heightComposite = this.getMinHeightComposite();
   	heightComposite     = height > heightComposite ? height : heightComposite;

    var rows = this._getResizeableRows();
    var offsetHeight = (heightComposite - this._getHeights()) / rows;
    for(var i = 0; i < this._rows; i++){
        if(this._heights[i]["resizeable"]) this._heights[i]["height"] += offsetHeight;
    }
    if(this._resizeable) this._placeChildrenNoneHomo();
    this.resize(this._width, heightComposite);
};
/**
 * A widget that displays the contents of a gtk.TextBuffer. A class inherit from gtk.Container
 * @constructor
 */
gtk.TextView = function(){
	gtk.Widget.apply(this);
	/** 'focused' signal */
	this.TEXTVIEW_FOCUSED = "focused";
	/** 'blurred' signal */
	this.TEXTVIEW_BLURRED = "blurred"
	/** 'enterkeypressed' signal */
	this.TEXTVIEW_ENTERKEYPRESSED = "enterkeypressed";

	this._editable 		= true;
	this._justification = "left";
	this._focus 		= false;
	this._showError		= true;

	this._minWidth 		= this._width 	= 100;
	this._minHeight 	= this._height 	= 50;

	/** @ private */
	this._render = function(){
	    var className = this._domObj.className;
		this._domObj = jsgtk.Dom.create({"tag": "textarea"});
		this._domObj.className = className + ' gtk_text_view';
	};

	this._render();

	this._initEvent();
};

gtk.TextView.prototype = new gtk.Widget();
gtk.TextView.prototype.constructor = gtk.TextView;
/** @private */
gtk.TextView.prototype._initEvent = function(){
	var self = this;
	var jsgtkEvent = jsgtk.Event;
	jsgtkEvent.addDomListener(this._domObj, "focus", function(){
	    self._focus = true;
	    self.emit("focused", self._focus);
	});

	jsgtkEvent.addDomListener(this._domObj, "blur", function(){
	    self._focus = false;
	    self.emit("blurred", self._focus);
	});
};
/**
 * Sets the "editable" property to the value of setting. If setting is True the text in textview is editable by default
 * @param {boolean} setting If True the text is editable by default
 */
gtk.TextView.prototype.setEditable = function(setting){
    this._editable = setting;

    setting? this._domObj.removeAttribute("readonly"):this._domObj.setAttribute("readonly", "readonly");
};
/**
 * Show error
 * @param {boolean} isOnError
 */
gtk.TextView.prototype.showError = function(isOnError){
	this._showError = (typeof isOnError !== 'undefined' && typeof isOnError === 'boolean') ? isOnError : true;

    if(this._showError) this._domObj.className += " gtk_text_view_onerror";
    else this._domObj.className = "gtk_widget gtk_container gtk_text_view";
};
/**
 * Returns the value of the "editable" property
 * @return True if text is editable by default
 */
gtk.TextView.prototype.getEditable = function(){
    return  this._editable;
};
/**
 * Sets the "justification" property to the value of justification. The value of justification must be one of: gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT, gtk.JUSTIFY_CENTER or gtk.JUSTIFY_FILL
 * @param {String} the text justification
 */
gtk.TextView.prototype.setJustification = function(justification){
    this._justification = justification;
    this._domObj.style.textAlign = justification;
};
/**
 * Returns the value of the "justification" property. the default justification of paragraphs in text_view. The value of "justification" must be one of: gtk.JUSTIFY_LEFT, gtk.JUSTIFY_RIGHT, gtk.JUSTIFY_CENTER or gtk.JUSTIFY_FILL.
 * @return The default justification
 */
gtk.TextView.prototype.getJustification = function(){
    return this._justification;
};
/**
 * Retrieve the value inside TextView
 * @return The value of TextView
 */
gtk.TextView.prototype.getValue = function(){
    return this._domObj.value;
};
/**
 * Set the default value inside TextView
 * @param {String} value The value to be set into TextView
 */
gtk.TextView.prototype.setValue = function(value){
    this._domObj.value = value;
};
/** @private */
gtk.TextView.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};
/**
 * Allow user to create custom event when enterkey is pressed
 */
gtk.TextView.prototype.allowEnterKey = function(){
    var self = this;
    jsgtk.Event.addDomListener(this._domObj, "keypress", function(e){
	    if(e.keyCode == gtk.ENTER_KEY){
	        self.emit("enterkeypressed", self._domObj.value);
	        jsgtk.Event.stopDefault(e);
	    }
	});
};
/**
 * Focus the cursor on the textview
 */
gtk.TextView.prototype.focus = function(){
    this._domObj.focus();
};
/** @private */
gtk.TextView.prototype._resizeWidth = function(width){
    this._domObj.style.width = width - 3 + 'px';
    this._width = width;
};
/** @private */
gtk.TextView.prototype._resizeHeight = function(height){
    this._domObj.style.height = height - 4 + 'px';
    this._height = height;
};
/** @private */
gtk.TextView.prototype.resizeWidthComposite = function(width){
    this.setWidth(width);
    this.resize(this._width, this._height);
};
/** @private */
gtk.TextView.prototype.resizeHeightComposite = function(height){
    this.setHeight(height);
    this.resize(this._width, this._height);
};
gobject.signalNew("focused", gtk.TextView, null, null, []);
gobject.signalNew("blurred", gtk.TextView, null, null, []);
gobject.signalNew("enterkeypressed", gtk.TextView, null, null, []);
/**
 * Dom is a class to create a text or an HTML container. It can contain any HTML string value.
 * @param {Integer} width The integer width value in pixel to be set
 * @param {Integer} height The integer height value in pixel to be set
 * @constructor
 * @base gtk.Widget
 */
gtk.Dom = function(width, height) {
	gtk.Widget.apply(this);

	/** 'clicked' signal */
	this.DOM_CLICKED = "clicked";
	/** 'doubleclicked' signal */
	this.DOM_DOUBLECLICKED = "doubleclicked";
	/** 'mouseover' signal */
	this.DOM_MOUSEOVER = "mouseover";
	/** 'mouseup' signal */
	this.DOM_MOUSEUP = "mouseup";
	/** 'mouseout' signal */
	this.DOM_MOUSEOUT = "mouseout";

	this._attribute = null;

	this._width  = this._minWidth  = this._initWidth	= width  || 16;
	this._height = this._minHeight = this._initHeight	= height || 16;

	this._dom = null;

	this._render = function(){
	    var domObj = this._domObj;
		domObj.className   += " gtk_dom";
	};

    this._render();
    this._initEvent();
};

gtk.Dom.prototype = new gtk.Widget();
gtk.Dom.prototype.constructor = gtk.Dom;

/**
* Append the child widget to gtk.Dom
* @param {gtk.Widget} dom The Dom object to be added
*/
gtk.Dom.prototype.appendChild = function(dom){
    if(this._dom) this._domObj.replaceChild(dom, this._dom);
    else this._domObj.appendChild(dom);

    this._dom = dom;

    this._resetSize();
};

/**
 * Accept any HTML string
 * @param {String} value The HTML value to be set
 */
gtk.Dom.prototype.setInnerHTML = function(value){
	var newDom = jsgtk.Dom.create({'tag': 'div'});
	newDom.innerHTML = value;
	this.appendChild(newDom);
};

/**
 * Return HTML string
 * @return {String} The HTML string value inside the Dom
 * @type String
 */
gtk.Dom.prototype.getValue = function(){
    return this._dom.innerHTML;
};

/** @private */
gtk.Dom.prototype._resetSize = function(){
	var size = this._getChildSize();
	this._minWidth  = size[0];
	this._minHeight = size[1];
	this.resize(size[0], size[1]);
};

/** @private */
gtk.Dom.prototype._initEvent = function(){
    var self = this;
    var jsgtkEvent = jsgtk.Event;

    jsgtkEvent.addDomListener(this._domObj, "click", function(e){
		self.emit("clicked", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "dblclick", function(e){
		self.emit("doubleclicked", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseover", function(e){
		self.emit("mouseover", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseout", function(e){
		self.emit("mouseout", self.getValue());
	}, false);

	jsgtkEvent.addDomListener(this._domObj, "mouseup", function(e){
		self.emit("mouseup", self.getValue());
	}, false);
};

/** @private */
gtk.Dom.prototype.resize = function(width, height){
    this._resizeWidth(width);
    this._resizeHeight(height);
};

/** @private */
gtk.Dom.prototype._resizeWidth = function(width){
	this._width = width;
	this._domObj.style.width = width + 'px';
	this._dom.style.width = width + 'px';
};

/** @private */
gtk.Dom.prototype._resizeHeight = function(height){
	this._height = height;
	this._domObj.style.height = height + 'px';
	this._dom.style.height = height + "px";
};

/**
 * Set style or apply attribute
 * @param {Object} attribute A collection of object literal of styles
 */
gtk.Dom.prototype.setAttribute = function(attribute){
	this._attribute = attribute;
	this._applyAttribute();
};

/** @private */
gtk.Dom.prototype._applyAttribute = function(){
	for(var el in this._attribute){
	    this._domObj.style[el] = this._attribute[el];
	}
};

/** @private */
gtk.Dom.prototype._getChildSize = function(){
	var jsgtkUtil = jsgtk.Util;
	var tmpNode = this._dom.cloneNode(true);
	tmpNode.style.width = this._width + "px";
	var height   = jsgtkUtil.fullHeight(tmpNode) ? jsgtkUtil.fullHeight(tmpNode) : this._height;
	return [this._width, height];
};

/** @private */
gtk.Dom.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
	this._resizeWidth(minWidth);
};

/** @private */
gtk.Dom.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeHeight(minHeight);
};

gobject.signalNew("clicked", gtk.Dom, null, null, []);
gobject.signalNew("doubleclicked", gtk.Dom, null, null, []);
gobject.signalNew("mouseover", gtk.Dom, null, null, []);
gobject.signalNew("mouseup", gtk.Dom, null, null, []);
gobject.signalNew("mouseout", gtk.Dom, null, null, []);
/**
 * Contents is a class to create contents for NoteBook
 * @param {Integer} width The width of the widget to be set
 * @param {Integer} height The height of the widget
 * @constructor
 * @base gtk.Widget
 */
gtk.Contents = function(width, height) {
	gtk.Widget.apply(this);

	this._width = width;
	this._height = height;
	this._children = [];
	this._activeContent = null;
	this._render = function(){
		this._domObj.className += " gtk_contents";
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
	};

	this._render();
};

gtk.Contents.prototype = new gtk.Widget();
gtk.Contents.prototype.constructor = gtk.Contents;

/**
 * Method to hide border of widget
 */
gtk.Contents.prototype.hideBorder = function(){
    this._domObj.style.border = "0px";
};

/**
 * Method to show border of widget
 */
gtk.Contents.prototype.showBorder = function(){
    var domStyle = this._domObj.style;
    domStyle.borderBottom = "1px solid #EFC169";
    domStyle.borderLeft   = "1px solid #EFC169";
    domStyle.borderRight  = "1px solid #EFC169";
};

/**
 * Method to set activated as default
 */
gtk.Contents.prototype.activateDefault = function(){
	this._activeContent = this._children[0]["content"];
	this._domObj.appendChild(this._activeContent.getNode());
};

/**
 * Method to set activated by index value
 * @param {Integer} index The index value to be set
 */
gtk.Contents.prototype.activate = function(index){
	var content = this._children[index]["content"];
	this._domObj.innerHTML = "";
	this._domObj.appendChild(content.getNode());
	this._activeContent = content;
	return content;
};

/**
 * Method to set activated by object value
 * @param {Object} content The object value to be passed
 */
gtk.Contents.prototype.activateByObject = function(content){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["content"] === content) break;
    }
    this.activate(i);
};

/**
 * Method to set activated its content
 * @param {Integer} id The id value as integer
 */
gtk.Contents.prototype.activateContent = function(id){
	var content;
	for(var i = 0, len = this._children.length; i < len; i++){
		content = this._children[i]["content"];
		if(id === this._children[i]["id"]) break;
	}
	this._domObj.innerHTML = "";
	this._domObj.appendChild(content._domObj);
	this._activeContent = content;
	return content;
};

/**
 * Method to get/return active content
 * @return {Object} The active object value
 * @type Object
 */
gtk.Contents.prototype.getActiveContent = function(){
    return this._activeContent;
};

/**
 * Method to update its content
 * @param {Object} content The object value of the content
 * @param {Integer} index The integer value of the content tab index
 */
gtk.Contents.prototype.updateContent = function(content, index){
    this._children[index]["content"] = content;
    content.setParent(this);
};

/**
 * Method to add child object into its body
 * @param {Object} contentObj The object value of the content
 */
gtk.Contents.prototype.addChild = function(contentObj){
    contentObj["padding"] = contentObj["padding"] === undefined || contentObj["padding"] === null ? 1 : contentObj["padding"];
    contentObj["content"].setParent(this);
	this._children.push(contentObj);
};

/**
 * Method to insert an object value at a specific position
 * @param {Object} contentObj The object value of the content
 * @param {Integer} index The index value of the content tab
 */
gtk.Contents.prototype.insertAt = function(contentObj, index){
	this._children.splice(index, 0, contentObj);
};

/**
 * Method to remove child from it body given an index value
 * @param {Integer} index The index value to be set
 */
gtk.Contents.prototype.removeChild = function(index){
    jsgtk.Util.removeArrayElements(this._children, index);
    this._domObj.removeChild(this._domObj.childNodes[0]);
};

/**
 * Method to get or return all children
 * @return {Object} The array objects of its contents or children
 * @type Object
 */
gtk.Contents.prototype.getChildren = function(){
    return this._children;
};

/** @private */
gtk.Contents.prototype._resizeChildrenHeight = function(height){

    for(var i = 0, len = this._children.length; i < len; i++){
        var childWiget = this._children[i]["content"];
        var padding    = this._children[i]["padding"];
        var childHeight = height - padding * 2;
        childWiget.resizeHeightComposite(childHeight);
        childWiget.getNode().style.top = padding + "px";
    }
};

/** @private */
gtk.Contents.prototype._resizeChildrenWidthComposite = function(width){
    for(var i = this._children.length; i--;){
        var childWiget = this._children[i]["content"];
        var padding    = this._children[i]["padding"];
        var childWidth = width - padding * 2;
        childWiget.resizeWidthComposite(childWidth);
        childWiget.getNode().style.left = padding + "px";
    }
};

/**
 * Method to get/return minimum composite width
 * @return {Integer} The minimum composite width of the content
 * @type Integer
 */
gtk.Contents.prototype.getMinWidthComposiste = function(){
	var children = this._children;
	var minWidth = this._minWidth;

	for(var i = children.length; i--;){
		var width = children[i]["content"].getMinWidthComposite() + children[i]["padding"] * 2;
		if( width > minWidth & width != undefined ) minWidth = width;
	}

	this._minWidth = minWidth;
	return this._minWidth;
};

/**
 * Method to get or return minimum composite height
 * @return {Integer} The minimum composite height of the content
 * @type Integer
 */
gtk.Contents.prototype.getMinHeightComposite = function(){
	var children = this._children;
	var minHeight = this._minHeight;
	for(var i = children.length; i--;){
		var height = children[i]["content"].getMinHeightComposite() + children[i]["padding"] * 2;
		if( height > minHeight & height != undefined ) minHeight = height;
	}

	this._minHeight = minHeight;
	return this._minHeight;
};

/**
 * Method to resize width
 * @param {Integer} width The width to be passed
 */
gtk.Contents.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;
    this._resizeWidth(minWidth);
    this._resizeChildrenWidthComposite(minWidth);
};

/**
 * Method to resize height
 * @param {Integer} height The height to be passed
 */
gtk.Contents.prototype.resizeHeightComposite = function(height){
	var minHeightComposite = this.getMinHeightComposite();
	var minHeight = height > minHeightComposite ? height : minHeightComposite;
	this._resizeHeight(minHeight);
    this._resizeChildrenHeight(minHeight);
};
/**
 * Tabs is a class to create tab for NoteBook
 * @constructor
 * @param {Integer} width The width of the widget
 * @param {Integer} height The height of the widget
 * @param {Integer} vspaceWidth The vertical space
 * @base gtk.Widget
 * @see gtk.NoteBook
 * @see gtk.Contents
 */
gtk.Tabs = function(width, height, vspaceWidth) {
	gtk.Widget.apply(this);

	this._children        = [];
	this._width           = width;
	this._height          = height;
	this._vspaceWidth     = vspaceWidth;
	this._activeTabDom    = null;
	this._activeTab       = "";
	this._border          = 2;
    this._loadingImgWidth = 16;
    this._tabs = [];
	this._activeTabId 	  = null;

	/** A registered signal name 'active' */
	this.TABS_ACTIVE 	= "";

	this._render = function(){
		this._domObj.className   += " gtk_tabs";
		this._domObj.style.width  = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
	};

	this._render();
};

gtk.Tabs.prototype = new gtk.Widget();
gtk.Tabs.prototype.constructor = gtk.Tabs;

/**
 * Method to show loading given tab object
 * @param {Object} tab The object value of the tab content
 */
gtk.Tabs.prototype.showLoading = function(tab){
    var pageObj     = this._getPage(tab);
    var page        = tab.getNode().parentNode.parentNode;
    var loadingImg  = page.lastChild;
    var pageWrapper = page.firstChild;

    if(jsgtk.Dom.isShowed(loadingImg)) return;

    var extraWidth = this._loadingImgWidth > pageObj["padding"] / 2 ? this._loadingImgWidth : 0;

    var width = parseInt(page.style.width);
    page.style.width = width + extraWidth + "px";
    pageWrapper.style.width = width + extraWidth + "px";

    loadingImg.style.left = pageObj["padding"] / 2 + tab.getMinWidthComposite() + "px";
    jsgtk.Dom.showElement(loadingImg);
};

/**
 * Method to hide loading given tab object
 * @param {Object} tab The object value of the tab content
 */
gtk.Tabs.prototype.hideLoading = function(tab){
    var pageObj     = this._getPage(tab);
    var page        = tab.getNode().parentNode.parentNode;
    var loadingImg  = page.lastChild;
    var pageWrapper = page.firstChild;

    if(!jsgtk.Dom.isShowed(loadingImg)) return;

    var extraWidth = this._loadingImgWidth > pageObj["padding"] / 2 ? this._loadingImgWidth : 0;

    var width = parseInt(page.style.width);
    page.style.width = width - extraWidth + "px";
    pageWrapper.style.width = width - extraWidth + "px";

    jsgtk.Dom.hideElement(loadingImg);
};

/**
 * Method to set activated as default
 */
gtk.Tabs.prototype.activateDefault = function(){
    this.activate(0);
};

/** @private */
gtk.Tabs.prototype._deactivateTab = function(){
    this._activeTabDom.className = "gtk_page";
    jsgtk.Dom.applyAttributes(this._activeTabDom.firstChild, {"borderBottom": "1px solid #D5D5D5"});
};

/** @private */
gtk.Tabs.prototype._activeateTab = function(index){
    this._activeTabDom = this._children[index]["tab"]._domObj.parentNode.parentNode;
    this._activeTabDom.className = "tab_active";
    this._activeTab = this._children[index]["tab"];
    this._activeTabId = this._children[index]["id"];
    jsgtk.Dom.applyAttributes(this._activeTabDom.firstChild, {"borderBottom": "0px"});
};

/**
 * Method to set activated given an index value
 * @param {Integer} index The index value of the tab
 */
gtk.Tabs.prototype.activate = function(index){
    if(this._activeTabDom) this._deactivateTab();
	this._activeateTab(index);
};

/**
 * Method to set activated given an object value
 * @param {Object} label The object value
 */
gtk.Tabs.prototype.activateByObject = function(label){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === label) break;
    }
    this.activate(i);
};

/**
 * Method to get or return an index value
 * @return {Integer} tab The index value of the tab
 */
gtk.Tabs.prototype.getIndex = function(tab){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === tab) return i;
    }
};

/**
 * Method to add new tab given an object value
 * @param {Object} pageObj The object value
 */
gtk.Tabs.prototype.addTab = function(pageObj){
    pageObj["padding"] = pageObj["padding"] || 0;
	this._children.push(pageObj);
	this._domObj.appendChild(this._createPage(pageObj));
	this._domObj.appendChild(this._createVerticalSpace());
};

/**
 * Method to insert a tab in the specific value
 * @param {Object} pageObj The object value
 * @param {Integer} index The index value
 */
gtk.Tabs.prototype.insertAt = function(pageObj, index){
	this._children.splice(index, 0, pageObj);
    var domSibbling = this._domObj.childNodes[index * 2];
    var vspace = this._createVerticalSpace();
    this._domObj.insertBefore(vspace, domSibbling);
    this._domObj.insertBefore(this._createPage(pageObj), vspace);
};

/**
 * Method to remove tab from body given an index value
 * @param {Integer} index The index value to be removed
 */
gtk.Tabs.prototype.removeTab = function(index){
    jsgtk.Util.removeArrayElements(this._children, index);
    var tabIndex = index * 2;
    this._domObj.removeChild(this._domObj.childNodes[tabIndex]);
    this._domObj.removeChild(this._domObj.childNodes[tabIndex]);
};

/** @private */
gtk.Tabs.prototype._resizePage = function(pageObj, pageNode, wrapper){

	var tabWidget = pageObj["tab"];
	var size = tabWidget.getSize();

	pageNode.style.width  = size[0] + this._border + pageObj["padding"] + 'px';
	pageNode.style.height = size[1] + 12 + 'px';

    jsgtk.Util.detect.init();
    if(jsgtk.Util.detect.OS === "Windows") 	pageNode.style.top = 6 + 'px';
    else pageNode.style.top = 5 + 'px';

		wrapper.style.width = size[0] + this._border + pageObj["padding"] + 'px';
		wrapper.style.height = size[1] + 11 + 'px';

		    var labelNode = tabWidget.getNode();
		    labelNode.style.left = pageObj["padding"] / 2 + "px";
}

/** @private */
gtk.Tabs.prototype._createPage = function(pageObj){
	var pageNode = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_page','tabindex':this._tabindex});
	    this._tabs.push(pageNode)
		var wrapperNode = jsgtk.Dom.create({'append': pageNode, 'tag': 'div', 'class': 'gtk_tabs_wrapper'});
	    wrapperNode.appendChild(pageObj["tab"].getNode());

	    var loadingImg = jsgtk.Dom.create({'append': pageNode, 'tag': 'div', 'class': 'gtk_page_loading_img'});
        jsgtk.Dom.hideElement(loadingImg);

    this._resizePage(pageObj, pageNode, wrapperNode);
    this._tabindex++;
    this._initPageEvents(pageNode, pageObj);
	return pageNode;
};

/** @private */
gtk.Tabs.prototype._initPageEvents = function(page, pageObj){
    var self = this;
    jsgtk.Event.addDomListener(page, 'mousedown', function(e){
        if(self._activeTabDom === page) return;

	    if(self._activeTabDom) self._deactivateTab();

	    self._activeateTab(self.getIndex(pageObj["tab"]));

		self.emit('active', pageObj["id"]);
	});

};

/** @private */
gtk.Tabs.prototype._setFocus = function(tab){
    tab.className +=" gtk_label_focus";
};

/** @private */
gtk.Tabs.prototype._unFocus = function(tab){
    tab.className +="gtk_widget gtk_label";
};

/** @private */
gtk.Tabs.prototype._createVerticalSpace = function(){
	var vspace = jsgtk.Dom.create({'tag': 'div', 'class': 'gtk_vspace'});
	vspace.style.width = this._vspaceWidth + "px";
	vspace.style.height = "100%";
	return vspace;
};

/**
 * Method to get or return the active tab
 * @return {Object} The active object value
 * @type Object
 */
gtk.Tabs.prototype.getActiveTab = function(){
    return this._activeTab;
};

/**
 * Method to get/return the active tab
 * @return {Integer} The index value
 * @type Integer
 */
gtk.Tabs.prototype.getActiveTabId = function(){
	return this._activeTabId;
}

/**
 * Method to get/return all children belong to the class
 * @return {Object} The array of objects
 * @type Object
 */
gtk.Tabs.prototype.getChildren = function(){
    return this._children;
};

/** @private */
gtk.Tabs.prototype._getPage = function(tab){
    for(var i = 0, len = this._children.length; i < len; i++){
        if(this._children[i]["tab"] === tab) return this._children[i];
    }
};

/**
 * Method to resize width
 * @return {Integer} The width to be passed
 */
gtk.Tabs.prototype.getMinWidthComposite = function(){
	var tabs = this._children;
	if(tabs.length > 0){
		var minWidth = 0;
		for(var i = tabs.length; i--;){
			minWidth += tabs[i]["tab"].getMinWidthComposite() + this._border + tabs[i]["padding"];
		}
		var extraWidth = this._children.length * (this._border + this._vspaceWidth) ;
		this._minWidth = minWidth + extraWidth;
		return this._minWidth;
	}
};

/**
 * Method to resize width
 * @param {Integer} The width to be passed
 */
gtk.Tabs.prototype.resizeWidthComposite = function(width){
	var minWidthComposite = this.getMinWidthComposite();
	var minWidth = width > minWidthComposite ? width : minWidthComposite;

	this._resizeWidth(minWidth);
};

gobject.signalNew('active', gtk.Tabs, [], null, null);
/**
 * Menu is a class to create a list of menu items
 * @base gtk.Widget
 * @constructor
 * @see gtk.MenuItem
 */
gtk.Menu = function() {
	gtk.Widget.apply(this);
    var self = this;
	this._menuItems        = [];
	this._separatorDoms     = [];
	this._minWidth         = 20;
	this._minHeight        = 20;
	this._menuItemActive   = false;
	this._menuItemDeactive = true;
	this._isActive         = false;
	this._width            = this._minWidth;
	this._height           = this._minHeight;
	this._selectedItem     = null;
	this._cellHeight       = 22;
	this._zIndex           = 9000;
	this._index            = -1;
	this._currentHilighted = null;

	/** A registered signal name 'menuselected' */
	this.MENU_MENUSELECTED		= "";
	/** A registered signal name 'menudeactivated' */
	this.MENU_MENUDEACTIVATED	= "";
	/** A registered signal name 'menuactivated' */
	this.MENU_MENUACTIVATED		= "";

	this._render = function(){
		this._domObj.className += " gtk_menu_shell";
		    this._hiddenEntry = jsgtk.Dom.create({"append": this._domObj, "tag": "input", "type": "text", "class": "gtk_menu_shell_hidden_textbox"});
		this._domObj.style.width = this._width + 'px';
		this._domObj.style.height = this._height + 'px';
		this._domObj.style.zIndex = this._zIndex;
		this._domObj.style.position = "absolute";
		this.hide();
	};

	this._render();

	jsgtk.Event.addDomListener(this._hiddenEntry, "keyup", function(e){
	    var topIndex = self._menuItems.length - 1;

	    if(e.keyCode === 13){
	        var item = self._menuItems[self._index];
	        if(!item.getSubMenu())item.activateItem(item.getText());
	    }

        if(e.keyCode === 40){
            if(self._index === topIndex) self._index = -1;
            self._index++;
            self._hilightCell(self._index);
            return;
        }

        if(e.keyCode === 38){
            if(self._index <= 0) self._index = topIndex + 1;
            self._index--;
            self._hilightCell(self._index);
            return;
        }

        if(e.keyCode === 37 || e.keyCode === 39){
            var item = self._menuItems[self._index];
            if(item === undefined) return;
            if(!item.getSubMenu()){
                self.emit("menudeactivated");
                return;
            }
            item.showSubMenu();
        }

        jsgtk.Event.stopBubble(e);
	});
};

gtk.Menu.prototype = new gtk.Widget();
gtk.Menu.prototype.constructor = gtk.Menu;

/**
 * A method to add or append a menu item
 * @param {Dom} menuItem The Dom object of the menu item to be set
 */
gtk.Menu.prototype.append = function(menuItem){
	var self = this;
	var domItem = menuItem.getNode();
	var separatorMargin = 0;
	this._domObj.appendChild(domItem);

	this._menuItems.push(menuItem);
	this._minWidth  = this._getMaxChildWidth();

	if(this._separatorDoms.length > 0){
		separatorMargin = (3 * this._separatorDoms.length) ;
	}

	this._minHeight = (this._cellHeight * this._menuItems.length) + separatorMargin;

	this.resize(this._minWidth, this._minHeight);

	menuItem.connect("menumouseover", function(item){
        self._hideSubMenu(menuItem);
        self._index = self._getIndex(item);
        self._hilightCell(self._index);
        self._hiddenEntry.focus();
	});

	menuItem.connect("menuactivated", function(){
	    self.show();
	    self._hiddenEntry.focus();
	});

	menuItem.connect("menuselected", function(text){
	    self.emit("menuselected", text);
	    self._isActive = false;
	    self.hideMenuComposite();
	});
};

/** @private */
gtk.Menu.prototype._hilightCell = function(index){
    if(this._currentHilighted) jsgtk.Dom.applyAttributes(this._currentHilighted, {"backgroundColor": ""});
    this._currentHilighted = this._menuItems[index].getNode();
    jsgtk.Dom.applyAttributes(this._currentHilighted, {"backgroundColor": "#FFDD00"});
};

/** @private */
gtk.Menu.prototype._getIndex = function(item){
    for(var i = 0, len = this._menuItems.length; i < len; i++){
        if(this._menuItems[i] === item) return i;
    }
};

/**
 * A method to set selected on any menu item
 * @param {Dom} menuItem The Dom object of the menu item to be set
 */
gtk.Menu.prototype.selectItem = function(menuItem){
	for (var i = 0; i < this._menuItems.length; i++){
			if(this._menuItems[i] === menuItem) this._index = i;
	}
	this._hilightCell(this._index);
};

/**
 * A method to hide or deactivate the menu
 */
gtk.Menu.prototype.deactivate = function(){
	this.hide();
};

/**
 * A method to add a separator amongst each menu items
 */
gtk.Menu.prototype.addSeparator = function(){
	var separator = jsgtk.Dom.create({'append':this._domObj, 'tag' : 'div'});
	separator.className += ' separator';
	this._separatorDoms.push(separator);
};

/** @private */
gtk.Menu.prototype._getMaxChildWidth = function(){
	var maxWidth = 0;
	for(var i = this._menuItems.length; i--;){
	    var itemWidth = this._menuItems[i].getMinWidthComposite();
		if(itemWidth > maxWidth) maxWidth = itemWidth;
	}
	return maxWidth;
};

/** @private */
gtk.Menu.prototype.resize = function(width,height){
	this._resizeWidth(width);
    this._resizeHeight(height);

    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        item.resizeWidthComposite(width);
    }
};

/** @private */
gtk.Menu.prototype._hideSubMenu = function(menuItem){
    var subMenu;
    for(var i = 0, len = this._menuItems.length; i < len; i++){
        var item = this._menuItems[i];
        if(item !== menuItem){
            subMenu = item.getSubMenu();
            if(subMenu) subMenu.hideMenuComposite();
        }
    }
};

/**
 * A method to hide the menu item
 */
gtk.Menu.prototype.hideMenuComposite = function(){
    var subMenu;
    this.hide();
    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        subMenu = item.getSubMenu();
        if(subMenu) subMenu.hideMenuComposite();
    }
};

/**
 * A method to show menu item
 */
gtk.Menu.prototype.showMenuComposite = function(){
    var subMenu;
    this.show();
    for(var i = this._menuItems.length; i--;){
        var item = this._menuItems[i];
        subMenu = item.getSubMenu();
        if(subMenu) subMenu.showMenuComposite();
    }
};

/**
 * A method to set anchor of the menu
 * @param {Integer} x The x-axis value
 * @param {Integer} y The y-axis value
 * @param {Integer} index The index value of the menu item
 */
gtk.Menu.prototype.anchorTo = function(x, y, index){
    if(!this._isActive){
        this.show();
        this._hiddenEntry.focus();

        index = index || 0;
        this.show();
        var left,top;

        this._zIndex += index;
        this._domObj.style.zIndex = this._zIndex;

        var windowWidth = jsgtk.Util.getWindowWidth();
        var newX = windowWidth - x;

        if(this._width > newX) left = x - this._width;
        else left = x;

        var windowHeight = jsgtk.Util.getWindowHeight();
        var newY = windowHeight - y;

        if(this._height > newY) top = y - this._height;
        else top = y;

        this._domObj.style.top = top + "px";
        this._domObj.style.left = left + "px";

        this._isActive = true;
    }else{
        this.hideMenuComposite();
        this._isActive = false;
    }
};

/**
 * A method to resize its composite width
 * @param {Integer} width The integer width value to be set
 */
gtk.Menu.prototype.resizeWidthComposite = function(width){
    var minWidth = this._minWidth < width ? width : this._minWidth;
    this.resize(minWidth, this._height);
};

/**
 * A method to resize its composite height
 * @param {Integer} height The integer height value to be set
 */
gtk.Menu.prototype.resizeHeightComposite = function(height){
    var minHeight = this._minHeight < height ? height : this._minHeight;
    this.resize(this._width, minHeight);
};

gobject.signalNew("menuactivated", gtk.Menu, null, null, []);
gobject.signalNew("menudeactivated", gtk.Menu, null, null, []);
gobject.signalNew("menuselected", gtk.Menu, null, null, []);
/**
 * MenuItem is a class to create a menu item
 * @param {gtk.Image} img The gtk.Image object to be set
 * @param {String} text The string text of the menu item
 * @base gtk.Item
 * @constructor
 * @see gtk.Menu
 */
gtk.MenuItem = function(img, text){
	gtk.Item.apply(this);
    var self = this;
    this._width            = this._minWidth  = 50;
    this._height           = this._minHeight = 22;
    this._imageWidgetWidth = 16;
	this._imageWidget      = img || null;
    this._labelWidget      = new gtk.Label( text || "Item");
    this._labelWidget.show();

	this._toggle           = false;
	this._domMenuItemLabel = null;
	this._isSubMenu        = false;
	this._subMenu          = null;
	this._callBack		   = function(){};
	this._leftPadding      = 5;
	this._rightPadding     = 10;
	this._labelPadding     = 5;

	/** A registered signal name 'menuselected' */
	this.MENUITEM_MENUSELECTED	= "";
	/** A registered signal name 'menumouseover' */
	this.MENUITEM_MENUMOUSEOVER		= "";
	/** A registered signal name 'menuactivated' */
	this.MENUITEM_MENUACTIVATED		= "";

	this._render = function(){
		this._domObj.className += ' menu_item';
		this._domObj.style.width = this._width + "px";
		this._domObj.style.height = this._height + "px";

		this._appendChildren();
	};

	this._render();

	jsgtk.Event.addDomListener(this._domObj,'mouseover',function(e){
	    self.emit("menumouseover", self);
	    jsgtk.Event.stopBubble(e);
	    if(self._subMenu){
	        self._subMenu.show();
			var position = self._getSubMenuPosition(this, self._subMenu);
		    self._subMenu.anchorTo(position[0]-3, position[1]-3, 1);
	    }
	}, false);

	jsgtk.Event.addDomListener(this._domObj, 'click', function(e){
	    if(!self._subMenu) self.activateItem(self._labelWidget.getText());
		jsgtk.Event.stopBubble(e);
	}, false);
};

gtk.MenuItem.prototype = new gtk.Item();
gtk.MenuItem.prototype.constructor = gtk.MenuItem;

/**
 * A method to display the sub menu
 */
gtk.MenuItem.prototype.showSubMenu = function(){
    if(this._subMenu){
        this._subMenu.show();
		var position = this._getSubMenuPosition(this._domObj, this._subMenu);
	    this._subMenu.anchorTo(position[0]-3, position[1]-3, 1);
    }
};

/** @private */
gtk.MenuItem.prototype._getSubMenuPosition = function(dom, subMenu){
    var parentNode = dom.parentNode;
    var menuItems = parentNode.childNodes;

    var left = ( parseInt(parentNode.style.left) || 0) + parseInt(dom.style.width) + 6;
    var windowWidth = jsgtk.Util.getWindowWidth();
    var subMenuWidth = subMenu.getMinWidthComposite();
    var newLeft = windowWidth - left;
	if(newLeft < subMenuWidth){
        left = (parseInt(parentNode.style.left) || 0) - subMenuWidth;
    }

    var height = 0;
    for(var i = 0; i < menuItems.length; i++){
        if(this._domObj === menuItems[i]){
            break;
        }
        height += parseInt(menuItems[i].style.height) ? parseInt(menuItems[i].style.height) : 0;
    }
    var parentTop  = parseInt(parentNode.style.top);
    var windowHeight = jsgtk.Util.getWindowHeight();
    var subMenuHeight = subMenu.getMinHeightComposite();
    var top = height + parentTop;
    var newTop = windowHeight - top;
	if(newTop < subMenuHeight){
		top = (windowHeight || 0) - subMenuHeight;
	}

	return [left, top];
};

/** @private */
gtk.MenuItem.prototype._appendChildren = function(){
    var width = this._labelWidget.getMinWidthComposite() + this._imageWidgetWidth + this._leftPadding + this._rightPadding + this._labelPadding;
    this._minWidth = width > this._minWidth ? width : this._minWidth;
    this.resize(this._minWidth, this._minHeight);

    if(this._imageWidget) {
        var imageNode = this._imageWidget.getNode();
        imageNode.style.left = this._leftPadding + "px";
        imageNode.style.top  = "2px";
        this._domObj.appendChild(imageNode);
    }

    var labelNode = this._labelWidget.getNode();
    this._domObj.appendChild(labelNode);
    labelNode.style.position = "absolute";
    labelNode.style.left = this._leftPadding + this._imageWidgetWidth + this._labelPadding + "px";
    labelNode.style.top  = "1px";
};

/**
 * A method to set the menu as sub menu
 * @param {Dom} menu The Dom object to be set
 */
gtk.MenuItem.prototype.setSubMenu = function(menu){
	var self = this;
	this._isSubMenu = true;
	this._subMenu = menu;

	this._subMenu.getNode().className += ' sub_menu';
	document.body.appendChild(this._subMenu.getNode());

	this._imageSubMenu = jsgtk.Dom.create({'append' : this._domObj, 'tag' : 'img', 'src' : '../../src/images/test/toSubmenu.png'});
	this._imageSubMenu.className = 'image_sub_menu';
	this._imageSubMenu.style.left = parseInt(this._domObj.style.width) - 10 + "px";

	this._subMenu.connect("menudeactivated", function(){
        self.emit("menuactivated");
        self._subMenu.hide();
	});

	this._subMenu.connect("menuselected", function(text){
    	self.activateItem(text);
	});
};

/** @private */
gtk.MenuItem.prototype._setSubMenuImage = function(imagePath){
	this._imageSubMenu.src = imagePath;
};

/**
 * A method to get or return menu text
 * @return {String} The string text of the menu item
 * @type String
 */
gtk.MenuItem.prototype.getText = function(){
    return this._labelWidget.getText();
};

/**
 * A method to get or return the string text of sub menu
 * @return {String} The string text of the sub menu item
 * @type String
 */
gtk.MenuItem.prototype.getSubMenu = function(){
    return this._subMenu;
};

/** @private */
gtk.MenuItem.prototype.resize = function(width, height){
	this._resizeWidth(width);
	this._resizeHeight(height);
};

/** @private */
gtk.MenuItem.prototype.removeSubMenu = function(){
	document.body.removeChild(this._subMenu.getNode());
	this._domObj.removeChild(this._imageSubMenu);
	this._subMenu = null;
};

/** @private */
gtk.MenuItem.prototype.activateItem = function(text){
    this.emit("menuselected", text);
};

/**
 * A method to resize width composite of the menu
 * @param {Integer} width The width composite of the menu
 */
gtk.MenuItem.prototype.resizeWidthComposite = function(width){
    var minWidth = width > this._minWidth ? width : this._minWidth;
    this.resize(minWidth, this._height);
};

gobject.signalNew("menumouseover", gtk.MenuItem, null, null, []);
gobject.signalNew("menuselected", gtk.MenuItem, null, null, []);
gobject.signalNew("menuactivated", gtk.MenuItem, null, null, []);
/**
 * Item is an assistant class to create menu item
 * @constructor
 * @base gtk.Widget
 * @see gtk.Menu
 * @see gtk.MenuItem
 */
gtk.Item = function(){
	gtk.Widget.apply(this);
	var self = this;
	this._toggle = false;

	/** A registered signal name 'selected' */
	this.ITEM_SELECTED 		= "";
	/** A registered signal name 'deselected' */
	this.ITEM_DESELECTED	= "";
	/** A registered signal name 'toggled' */
	this.ITEM_TOGGLED		= "";

	this._render = function(){
		this._domObj.className += ' item';
	};

	this._render();
};

gtk.Item.prototype = new gtk.Widget();
gtk.Item.prototype.constructor = gtk.Item;

/**
 * A method to set menu selected
 */
gtk.Item.prototype.select = function(){
	this.emit('selected');
};

/**
 * A method to set menu deselected
 */
gtk.Item.prototype.deselect = function(){
	this.emit('deselected');
};

/**
 * A method to set toggled of the menu
 */
gtk.Item.prototype.toggle = function(){
	this.emit('toggled');
	if(this._toggle) this._toggle = false;
	else this._toggle = true;
};

gobject.signalNew('selected', gtk.Button, null, null, []);
gobject.signalNew('deselected', gtk.Button, null, null, []);
gobject.signalNew('toggled', gtk.Button, null, null, []);

/**
 * TreeStore is a helper or model or subordinate class used to create a model for other class applying in
 * MVC (Model View Controller) and MVP (Model View Presenter) as development concept.
 * @constructor
 */
gtk.TreeStore = function(header){
 	gtk.Object.apply(this);

	this._modelType = gtk.TREESTORE;
	this._header = header || "Header";
 	this.children = [];
};

gtk.TreeStore.prototype = new gtk.Object();
gtk.TreeStore.prototype.constructor = gtk.TreeStore;

/**
 * Method to append a new row to the liststore
 * @param {Object} parent An object instantiated from gtk.TreeIter
 * @param {Object} row A tuple or list containing ordered column values to be set in the new row
 * @return {object} A gtk.TreeIter pointing to the new row
 * @type Object
 */
gtk.TreeStore.prototype.append = function(parent, row){
	if(parent){
		var path = parent.getPath() + parent.getChildren().length.toString() + ':';
		var iter = new gtk.TreeIter(path);
		iter.setRow(row);
		iter.hasNode = row["hasChild"] || false;
		parent.add(iter);
		parent.hasNode = true;
	}else{
		var iter = new gtk.TreeIter(this.children.length.toString() + ':');
		iter.setRow(row);
		iter.hasNode = row["hasChild"] || false;
		this.add(iter);
	}
	return iter;
};

/**
 * Method to append a new node
 * @param {Object} parent An object instantiated from gtk.TreeIter
 * @param {Object} row A tuple or list containing ordered column values to be set in the new row
 * @return {Object} All children models
 * @type Object
 */
gtk.TreeStore.prototype.appendNodes = function(parent, rows){
    if(rows.length === 0) return;
    var childrenModel = [];
    for(var i = 0, len = rows.length; i < len; i++){
        childrenModel.push(this.append(parent, rows[i]));
    }
    this.emit("nodesappended", parent);
    return childrenModel;
};

/**
 * Method to get a model type or name
 * @return {String} A type of model name
 * @type String
 */
gtk.TreeStore.prototype.getModelType = function(){
	return this._modelType;
};

/**
 * Method to get a child given its value member
 * @param {String} valueMember A name of its valued member
 * @param {String} value A value to be set
 * @return {Object} A model of the child
 * @type Object
 */
gtk.TreeStore.prototype.getChldBy = function(valueMember, value){
	for(var i = this.children.length; i--;){
		var childModel = this.children[i].getChldBy(valueMember, value);
		if(childModel) return childModel;
	}
};

/**
 * Method to get a child given its index value
 * @param {Integer} index An index value to be passed
 * @return {Object} An object of a child
 * @type Object
 */
gtk.TreeStore.prototype.getChildByIndex = function(index){
	if(index < 0 || index > this.children.length) return null;
	return this.children[index];
};

/**
 * Method to get a child given its path value
 * @param {String} path A path value to be looked for
 * @return {Object} An object of a child
 * @type Object
 */
gtk.TreeStore.prototype.getChildByPath = function(path){
    var paths = path.split(":");
    var child = this._getChildByPath( parseInt(paths[0]) );

    paths.splice(0,1);
    if(paths.length === 1) return child;

    return child.getChildByPath(paths.join(":"));
};

/** @private */
gtk.TreeStore.prototype._getChildByPath = function(path){
	for(var i = this.children.length; i--;){
		var childPath = this.children[i].path.split(":");
		if(parseInt(childPath[0]) === path) return this.children[i];
	}
	return null;
};

/**
 * Method to get all children
 * @return {Array} An array of objects of a children
 * @type Array
 */
gtk.TreeStore.prototype.getChildren = function(){
	return this.children;
};

/**
 * Method to push or append a child to be the last child in an array
 * @param {Object} childModel The object of a child to be passed
 */
gtk.TreeStore.prototype.add = function(child){
	this.children.push(child);
};

/** @private */
gtk.TreeStore.prototype._searchByValueMember = function(valueMember, value, children){
	var child;

	var children = children || this.children;
	for(var i = 0, len = children.length; i < len; i++){
		child = children[i];
		if(child._value[valueMember] === value){
			return child;
		}
	}

	for(var i = 0, len = children.length; i < len; i++){
		child = children[i];
		if(child.hasNode) {
			child = this._searchByValueMember(valueMember, value, child.getChildren());
			if(child) return child;
		}
	}
};

/**
 * Method to search and filter child/children
 * @param {String} value The object of a child to be passed
 * @return {Object} A found object from all children
 * @type Object
 */
gtk.TreeStore.prototype.filter = function(value){
    if(value.length === 0) return null;

    var matchedChildren = [];
    var re = new RegExp("^" + value.toLowerCase());
    for(var i = 0, len = this.children.length; i < len; i++){
    	var child = this.children[i];
    	result = child.getValue()["display"].toLowerCase().match(re);
    	if(result) matchedChildren.push(child);
    }
    return matchedChildren;
};

/**
 * Method to remove or delete a child from the list of children
 * @param {Object} child The object of a child to be removed
 */
gtk.TreeStore.prototype.remove = function(child){
	for(var i = 0, len = this.children.length; i < len; i++){
		if(this.children[i] === child){
			this.children.splice(i, 1);
			break;
		}
	}
};

/**
 * Method to get a child given an index value
 * @param {Integer} index The integer value as an index of the to-be-wanted child
 * @return {Object} The object of the child to be returned
 * @type Object
 */
gtk.TreeStore.prototype.getChild = function(index){
	return this.children[index] || null;
};

/**
 * Method to search through all children given text and all children
 * @param {String} text The string value to be wanted to search for
 * @param {Array} children The array of children objects
 * @return {Object} The object of the found child
 * @type Object
 */
gtk.TreeStore.prototype.search = function(text, children){
    var re = new RegExp("^" + text.toLowerCase());
    var children = children || this.children;
    var result = "";

    for(var i = 0, len = children.length; i < len; i++){
    	var child = children[i];
		result = child._search(text);
		if(result) return result;
    }
};

gobject.signalNew('nodesappended', gtk.TreeStore, [], null, null);
